TControlBar *bar      = 0;
TControlBar *selector = 0;

Int_t   gEventCounter = -1;
RecoRootEvent *recoev = 0;
const char *defaultfilename= "output/reco.root";
TChain *gRecoTree = 0;
TString gEnergyPlots = "EnergyPlots.ps";
Bool_t gDumpfilenames = kTRUE;

enum EAllCanvasPadID {
		kDirGraphPadID          = 1,
		kHmaxShapeGraphPadID    = 2,
		kFocalPlanePadID        = 3,
                kFocalPlaneHistos1PadID = 4
};




//______________________________________________________________________________
void ERecoEventViewer() {
    bar = new TControlBar("vertical", "Euso Reco Event Viewer",10,10);
   
    bar->AddButton("How to run        ", "ERecoHelp()","Instructions for running this macro"); 
    bar->AddButton("Select Event      ", "ERecoSelectevent()","Select an event"); 
    bar->AddButton("Open File         ", "ERecoOpenfile()","Open a new file"); 
    bar->AddButton("Start Viewer      ", "recotree->StartViewer()", "Start Viewer");
    bar->AddButton("Browser           ", "TBrowser b","Open a Browser");
    bar->AddButton("Next Event        ", "ERecoNextevent()","Load next event"); 
    bar->AddButton("Prev Event        ", "ERecoPrevevent()","Load previous event"); 
    bar->AddSeparator();
    bar->AddButton("Dir. Module Graph ", "ERecoDirectionGetGraph()","Get Fit Graph from direction module"); 
    bar->AddButton("Hmax Module Graph ", "ERecoHmaxshapeGetGraph()","Get Fit Graph from hmax shape module"); 
    bar->AddButton("Direction  Plots  ", "ERecoDirectionPlots()", "Draw direction plots for all events");
    bar->AddButton("Hmax  Plots       ", "ERecoHmaxPlots()", "Draw Hmax for all events");
    bar->AddButton("Xmax  Plots       ", "ERecoXmaxPlots()", "Draw Xmax for all events");
    bar->AddButton("Energy Plots      ", "ERecoEnergyPlots()", "Draw Error of Hmax for all events");
    bar->AddButton("Single Canvas ON/OFF","ERecSinglecanvas()","Toggle single canvas ON/OFF");
    bar->AddSeparator();
    bar->AddButton("Quit Root        ",  ".q","Quit root");
    bar->Show();
    gROOT->SaveContext();
    new TMinuit();
    if (gMinuit) gMinuit->SetPrintLevel(0);
//    ERecoInit();
    BuildRecoChain("RecoChain.list");
}

//______________________________________________________________________________
void ERecoInit() {
    // reading ListOfRecoFiles and preparing the TChain
    TFile *f = new TFile(defaultfilename);
    if ( f->IsZombie() ) {
        delete f;
        Printf(Form("Unable to open default file %s. Opening browser...", defaultfilename));
        ERecoOpenfile();
    }
    ERecoGettree();
}

//______________________________________________________________________________
void BuildRecoChain(const string& FileList)
{
    FILE* CHFILE = fopen(FileList.c_str(),"r");
    if(CHFILE!=0) {
       gRecoTree = new TChain("recotree");
       char RootFileName[200];
       int ifiles=0;
       while((fgets(RootFileName,sizeof(RootFileName),CHFILE)) !=NULL) {
	      ifiles++;
	      char root_file_name[250];
	      sscanf(RootFileName,"%s",root_file_name);
	      gRecoTree->Add(root_file_name);
	      if (gDumpfilenames) {
	          if(ifiles==1) cout << "The following files to be analyzed: " << endl;
 	          cout << ifiles<<") " << root_file_name << endl; 
	      }
       }
    }
    if (gRecoTree) cout << "Total number of events is " << gRecoTree->GetEntries() << " in " << ifiles << " files " << endl;
}

//______________________________________________________________________________
void ERecoGettree() {
    TTree *rtree = (TTree*)gDirectory->Get("recotree");
    if ( rtree ) {
        Printf("recotree found with %d events stored.", (Int_t)rtree->GetEntries());
        ERecoLoadevent();
    }
}
//______________________________________________________________________________
void ERecoOpenfile() {

    if ( recoev ) {
        delete recoev;
        recoev = 0;
    }

    if ( selector ) {
        delete selector;
        selector = 0;
    }

    TFile *f = 0;
    delete f;
    TGFileInfo* info = new TGFileInfo();
    while ( info->fFilename == 0 || !f || f->IsZombie() ) {
        TGFileDialog* filedialog = new TGFileDialog(gClient->GetRoot(), 0, kFDOpen, info);
        if ( info->fFilename ) 
        f = new TFile( info->fFilename );
    }

    if ( f && !f->IsZombie() ) {
        Printf("%s successfully opened.", info->fFilename);
        ERecoLoadevent();
    } 
    else {
        Printf("Unable to open %s",info->fFilename);
    }
}

//______________________________________________________________________________
Bool_t ERecoLoadevent( Int_t i=0) {

    TTree *rtree = (TTree*)gDirectory->Get("recotree");
    
    if ( !rtree) {
        Printf("loadevent(): unable to retrieve recotree from TFile. Exiting");
        return;
    }

    // associate the tree and the event
    recoev = NULL;
    rtree->SetBranchAddress("events",&recoev);
    

    cout << Form("Entry %d, %d bytes read", i, rtree->GetEntry(i)) << endl;


    if ( selector ) selector->Hide();

    gEventCounter = i;
    
 
    RecoTrackDirection *track = &recoev->GetRecoTrackDirection();
    if (recoev->GetRecoTrackDirection().GetQuality() == -1) {
        cout << Form("Event %d is not reconstructed\n",i);
        return kFALSE;
    }
    return kTRUE;
    
}

//______________________________________________________________________________
void ERecoNextevent() {

    TTree *rtree = (TTree*)gDirectory->Get("recotree");
    if ( (gEventCounter+1) >= 0 && (gEventCounter+1) < (Int_t)rtree->GetEntries() ) {
         if ( ERecoLoadevent( gEventCounter+1 ) ) ERecoRefresh();
    }
    else
        Printf("nextevent: Last event reached.");

}
//______________________________________________________________________________
void ERecoPrevevent() {

   TTree *rtree = (TTree*)gDirectory->Get("recotree");
   if ( (gEventCounter-1) >= 0 && (gEventCounter-1) < (Int_t)rtree->GetEntries() ) {
        if ( ERecoLoadevent( gEventCounter-1 ) ) ERecoRefresh();
    }
    else
        Printf("prevevent: First event reached.");

}

//______________________________________________________________________________
void ERecoSelectevent( Int_t first=0) {
    Int_t maxNumButtons = 15;
    Int_t numButtons;

    if ( selector ) delete selector;
    if ( !selector ) {
        selector = new TControlBar("vertical", "Event Selector",10,10);


        TTree *rtree = (TTree*)gDirectory->Get("recotree");
        Long64_t nEv = (Long64_t)rtree->GetEntries();

        if ((nEv-first)/maxNumButtons > 0)
            numButtons = maxNumButtons;
        else
            numButtons = (nEv-first)%maxNumButtons;
        
        if ( first > 0 ) 
            selector->AddButton("Prev",Form("selectevent(%d)",
                        first-maxNumButtons > 0 ? first-maxNumButtons : 0),"Test not stable");
            
        for( Long64_t i(0); i<numButtons; i++) {
            selector->AddButton(Form("Event %d",i+first),Form("loadevent(%d)",i+first),"Test not stable");
        } 
        
        if ( first+maxNumButtons < nEv )
           selector->AddButton("Next",Form("selectevent(%d)",first+maxNumButtons),"");
        selector->AddButton("Hide","selector->Hide()","Test");
        gROOT->SaveContext();
    } 

    selector->Show();
    ERecoRefresh();
}
//______________________________________________________________________________
void ERecoRefresh() {
    ERecoSinglecanvas("refresh");
}

//______________________________________________________________________________
void ERecoSinglecanvas(TString opt="")
{
    TCanvas *c, *c1;
    if ( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") ) {
        Bool_t paintDirGraph      = kFALSE;
        Bool_t paintHmaxShapeGraph = kFALSE;
        
        TPad *dummy;
        if ( dummy = (TPad*)c_all->FindObject(Form("c_all_%d",kDirGraphPadID)) ) {
            paintDirGraph = (dummy->FindObject("BetaCheck") != 0);
        }
        if ( dummy = (TPad*)c_all->FindObject(Form("c_all_%d",kHmaxShapeGraphPadID)) ) {
            paintHmaxShapeGraph = (dummy->FindObject("fTemp_HMaxShapeModule") != 0);
        }
        if (opt == "")                 delete c;
        if ( paintDirGraph )           ERecoDirectionGetGraph();
        if ( paintHmaxShapeGraph )     ERecoHmaxshapeGetGraph();
    } 
    else {
        if (opt == "") {
            c = new TCanvas("c_all","c_all");
            c->Divide(2,2);
        }
        if ( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_dir_graph") ) {
            if (opt == "") delete c1;
            ERecoDirectionGetGraph();
        }
        if ( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_hmaxshape_graph") ) {
            if (opt == "") delete c1;
            ERecoHmaxshapeGetGraph();
        }
        
     }	



}

//______________________________________________________________________________
void ERecoDirectionGetGraph() {
    TCanvas *c;
    if ( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") ) {
        c->cd(kDirGraphPadID);
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_dir_graph");
        if (!c) c = new TCanvas("c_dir_graph","c_dir_graph");
        c->Clear();
        c->cd();
    }

    TMultiGraph *graph = recoev->GetRecoTrackDirection().GetCheckInfo();
    if (graph) graph->Draw("alp");
}

//______________________________________________________________________________
void ERecoHmaxshapeGetGraph() {
    TCanvas *c;
    if ( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") ) {
        c->cd(kHmaxShapeGraphPadID);
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_hmaxshape_graph");
        if (!c) c = new TCanvas("c_hmaxshape_graph","c_dir_graph");
        c->Clear();
        c->cd();
    }

    TH1F *graph = recoev->GetRecoHmaxByShapeMethod().GetHisto();
    if (graph) graph->Draw("");
}

//______________________________________________________________________________
void ERecoDirectionPlots() {
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_DirectionPlots")) ) {
        c = new TCanvas("c_DirectionPlots","c_DirectionPlots");
        c->Divide(2,2);        
    
    } 
    
    TH2F *h1_dirPlots = (TH2F*)gROOT->FindObject("h1_dirPlots");
    if (!h1_dirPlots) 
        h1_dirPlots = new TH2F("h1_dirPlots","",30,0,90,30,0,90);
    else
        h1_dirPlots->Reset();

    TH2F *h2_dirPlots = (TH2F*)gROOT->FindObject("h2_dirPlots");
    if (!h2_dirPlots) 
        h2_dirPlots = new TH2F("h2_dirPlots","",30,0,360,30,0,360);
    else
        h2_dirPlots->Reset();

    TH1F *h3_dirPlots = (TH1F*)gROOT->FindObject("h3_dirPlots");
    if (!h3_dirPlots) 
        h3_dirPlots = new TH1F("h3_dirPlots","",30,0,20);
    else
        h3_dirPlots->Reset();

    TProfile *h4_dirPlots = (TProfile*)gROOT->FindObject("h4_dirPlots");
    if (!h4_dirPlots) 
        h4_dirPlots = new TProfile("h4_dirPlots","",30,0,90);
    else
        h4_dirPlots->Reset();
    // Save info
    h1_dirPlots->SetXTitle("simulated  #Theta, [deg]");
    h1_dirPlots->SetYTitle("reconstructed #Theta, [deg]");
    h2_dirPlots->SetXTitle("simulated  #phi, [deg]");
    h2_dirPlots->SetYTitle("reconstructed #phi, [deg]");
    h3_dirPlots->SetXTitle("direction error, [deg]");
    h3_dirPlots->SetYTitle("Entries");
    h4_dirPlots->SetXTitle("mean direction error, [deg]");
    h4_dirPlots->SetYTitle("simulated #Theta, [deg]");
                 
    TString    cut  = "fTrackDirection.fQuality!=-1";
    TString var1 = "fTrackDirection.fTheta*TMath::RadToDeg():fTruth.fTheta*TMath::RadToDeg()>>h1_dirPlots";
    TString var2 = "fTrackDirection.fPhi*TMath::RadToDeg():fTruth.fPhi*TMath::RadToDeg()>>h2_dirPlots";        
    TString var3 = "fTrackDirection.fErrorDirection*TMath::RadToDeg()>>h3_dirPlots";
    TString var4 =
         "fTrackDirection.fErrorDirection*TMath::RadToDeg():fTruth.fTheta*TMath::RadToDeg()>>h4_dirPlots";
    c->cd(1);
    recotree->Draw(var1,cut);
    c->cd(2);
    recotree->Draw(var2,cut);
    c->cd(3);
    recotree->Draw(var3,cut);
    c->cd(4);
    recotree->Draw(var4,cut,"prof");       
}
//______________________________________________________________________________
void ERecoHmaxPlots() {
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_HmaxPlots")) ) {
        c = new TCanvas("c_HmaxPlots","c_HmaxPlots");
        c->Divide(2,2);        
    
    } 
    
    TH2F *h1_hmaxPlots = (TH2F*)gROOT->FindObject("h1_hmaxPlots");
    if (!h1_hmaxPlots) 
        h1_hmaxPlots = new TH2F("h1_hmaxPlots","",30,0,90,30,0,30);
    else
        h1_hmaxPlots->Reset();
    
    TH2F *h1p_hmaxPlots = (TH2F*)gROOT->FindObject("h1p_hmaxPlots");
    if (!h1p_hmaxPlots) 
        h1p_hmaxPlots = new TH2F("h1p_hmaxPlots","",30,0,90,30,0,30);
    else
        h1p_hmaxPlots->Reset();

    TH2F *h2_hmaxPlots = (TH2F*)gROOT->FindObject("h2_hmaxPlots");
    if (!h2_hmaxPlots) 
        h2_hmaxPlots = new TH2F("h2_hmaxPlots","",30,-10,30,30,-10,30);
    else
        h2_hmaxPlots->Reset();
    
    TH2F *h2p_hmaxPlots = (TH2F*)gROOT->FindObject("h2p_hmaxPlots");
    if (!h2p_hmaxPlots) 
        h2p_hmaxPlots = new TH2F("h2p_hmaxPlots","",30,-10,30,30,-10,30);
    else
        h2p_hmaxPlots->Reset();
    
    h1_hmaxPlots->SetXTitle("simulated  #Theta, [deg]");
    h1_hmaxPlots->SetYTitle("reconstructed H_{max}, [km]");
    h2_hmaxPlots->SetXTitle("simulated  H_{max}, [km]");
    h2_hmaxPlots->SetYTitle("reconstructed H_{max}, [km]");
    h1p_hmaxPlots->SetXTitle("simulated  #Theta, [deg]");
    h1p_hmaxPlots->SetYTitle("reconstructed H_{max}, [km]");
    h2p_hmaxPlots->SetXTitle("simulated  H_{max}, [km]");
    h2p_hmaxPlots->SetYTitle("reconstructed H_{max}, [km]");
    
    h1_hmaxPlots->SetLineColor(kRed);
    h1p_hmaxPlots->SetLineColor(kBlue);
    h2_hmaxPlots->SetLineColor(kRed);
    h2p_hmaxPlots->SetLineColor(kBlue);
    
    TString opt = "";
    TString Cut = "fTrackDirection.fQuality!=-1 ";
    TString Shape_Cut = "&& fRecoHmaxByShapeMethod.fQuality!=-1&&fRecoHmaxByShapeMethod.fHmax>0";
    TString Proton_Cut = "&& fRecoHmaxForProton.fHmax!=-1";
    TString var1 = "fRecoHmaxByShapeMethod.fHmax:fTruth.fTheta*TMath::RadToDeg()>>h1_hmaxPlots";
    TString var2 = "fRecoHmaxForProton.fHmax:fTruth.fTheta*TMath::RadToDeg()>>h1p_hmaxPlots";
    TString var3 = "fRecoHmaxByShapeMethod.fHmax:fTruth.fMaxPosZ>>h2_hmaxPlots";
    TString var4 = "fRecoHmaxForProton.fHmax:fTruth.fMaxPosZ>>h2p_hmaxPlots";
    c->cd(1);
    recotree->Draw(var1,Cut+Shape_Cut,opt);
    c->cd(2);
    recotree->Draw(var2,Cut+Proton_Cut,opt);
    c->cd(3);
    recotree->Draw(var3,Cut+Shape_Cut,opt);
    c->cd(4);
    recotree->Draw(var4,Cut+Proton_Cut,opt);
    
}
//______________________________________________________________________________
void ERecoEnergyPlots() {
    gErrorIgnoreLevel = 1;
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_EnergyPlots")) ) {
        c = new TCanvas("c_EnergyPlots","c_EnergyPlots");
    }
    c->Print(gEnergyPlots+"[");
    ERecoEnergyPlot("SimVsRec");
    ERecoEnergyPlot("Error1d");
    ERecoEnergyPlot("Error2d");
    ERecoEnergyPlot("Error2dVsThetaFoV");
    c->Print(gEnergyPlots+"]");
}

//______________________________________________________________________________
void ERecoEnergyPlot(Option_t *opt ) {
    TString option(opt);
    TCut Quality = "fRecoEnergy.fEnergy>0 && fRecoEnergy.fQuality>0 && fRecoEnergy.fQuality<15 && TMath::Abs(fRecoEnergy.fEnergyMCError/fTruth.fEnergy)<1";
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_EnergyPlots")) ) return;
    c->Clear();
    if (option == "SimVsRec") {
        gStyle->SetOptStat(0); 
        cout << "\n=====> Doing simulated vs reconstructed plot\n";
	TH2F *h = new TH2F("h","Simulated vs reconstructed",100,13,15,100,13,15);
        gRecoTree->Draw("TMath::Log10(fRecoEnergy.fEnergy):TMath::Log10(fTruth.fEnergy)>>h",Quality,"box");
	h->GetXaxis()->SetTitle("Reconstructed Log10(Energy/MeV)");
	h->GetYaxis()->SetTitle("Simulated Log10(Energy/MeV)");
	TLine *line = new TLine(13,13,15,15);
	line->Draw();
        c->Print(gEnergyPlots);
	delete h;
	cout << "=====> done\n";
	return;
    }
    else if (option == "Error1d") {
        gStyle->SetOptStat(0); 
	gStyle->SetOptFit(1); 
        cout << "\n=====> Doing 1d relative error plot\n";
	TH1 *h = 0;
	// Delta E/E in enegy bins
	Double_t Erange[5] = {13.,13.5,13.7,14.2,14.6};
	TGraphErrors *fEmean  = new TGraphErrors;
	TGraphErrors *fEsigma = new TGraphErrors;
	for (Int_t i(0); i<4; i++) {
	     TCut cut = Form("TMath::Log10(fTruth.fEnergy)>%f && TMath::Log10(fTruth.fEnergy)<%f",Erange[i],Erange[i+1]);
	     h = new TH1F("h",Form("Relative Reconstruction Error in [%.3f,%.3f] log10(Energy)",Erange[i],Erange[i+1]),100,-2,2);
             gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy>>h",Quality&&cut);
	     h->Fit("gaus","","",-0.5,0.5);
	     h->GetXaxis()->SetTitle("Relative Reconstruction Error");
             c->Print(gEnergyPlots);
	     Double_t pars[3];
	     TF1* f = h->GetFunction("gaus");
	     f->GetParameters(&pars[0]);
	     fEmean->SetPoint(i,0.5*(Erange[i]+Erange[i+1]),pars[1]);
	     fEmean->SetPointError(i,0.5*(Erange[i+1]-Erange[i]), f->GetParError(1));
	     fEsigma->SetPoint(i,0.5*(Erange[i]+Erange[i+1]),pars[2]);
	     fEsigma->SetPointError(i,0.5*(Erange[i+1]-Erange[i]),f->GetParError(2));
	     delete h;
	}
	c->Clear();
	fEmean->Draw("APE");
	fEmean->SetMarkerStyle(21);
        fEmean->GetYaxis()->SetTitle("Mean Relative Energy Resolution");
        fEmean->GetXaxis()->SetTitle("Log10(Energy/MeV)");	
	c->Print(gEnergyPlots);
	c->Clear();
	fEsigma->Draw("APE");
	fEsigma->SetMarkerStyle(21);
        fEsigma->GetYaxis()->SetTitle("Sigma Relative Energy Resolution");
        fEsigma->GetXaxis()->SetTitle("Log10(Energy/MeV)");		
	c->Print(gEnergyPlots);
	cout << "=====> done\n";
	return;         
    }
    else if (option == "Error2d") {
        gStyle->SetOptStat(1); 
        cout << "\n=====> Doing 2d relative error plot\n";
	TH1 *h = 0;
	// Delta E/E vs Log10(E)
	h = new TH2F("h","Relative Reconstruction Error",100,13,15,100,-2,2);
        gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy:TMath::Log10(fTruth.fEnergy)>>h",Quality);
	h->GetXaxis()->SetTitle("Simulated Log10(Energy/MeV)");
	h->GetYaxis()->SetTitle("Relative Reconstruction Error");
        c->Print(gEnergyPlots);
	delete h;
	// Delta E/E vs Log10(E) prof
	h = new TProfile("h","Relative Reconstruction Error",100,13,15);
	gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy:TMath::Log10(fTruth.fEnergy)>>h",Quality,"prof");
	h->SetMinimum(-2);
	h->SetMaximum(2);
	c->Print(gEnergyPlots);
	delete h;
	// Delta E/E vs Theta
	Double_t Erange[5] = {13.,13.5,13.7,14.2,14.6};
	for (Int_t i(0); i<4; i++) {
	     TCut cut = Form("TMath::Log10(fTruth.fEnergy)>%f && TMath::Log10(fTruth.fEnergy)<%f",Erange[i],Erange[i+1]);
	     h = new TProfile("h",Form("Relative Reconstruction Error in [%.3f,%.3f] log10(Energy)",Erange[i],Erange[i+1]),100,0,90);
	     gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy:TMath::RadToDeg()*fTruth.fTheta>>h",Quality&&cut,"prof");
      	     h->GetXaxis()->SetTitle("Simulated Theta/deg");
	     h->GetYaxis()->SetTitle("Relative Reconstruction Error");
     	     h->SetMinimum(-2);
	     h->SetMaximum(2);
	     c->Print(gEnergyPlots);
             delete h;
	     h = new TH2F("h",Form("Relative Reconstruction Error in [%.3f,%.3f] log10(Energy)",Erange[i],Erange[i+1]),100,0,90,100,-2,2);
	     gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy:TMath::RadToDeg()*fTruth.fTheta>>h",Quality&&cut,"box");
      	     h->GetXaxis()->SetTitle("Simulated Theta/deg");
	     h->GetYaxis()->SetTitle("Relative Reconstruction Error");
	     TLine *line = new TLine(0,0,90,0);
	     line->Draw();
	     line->SetLineColor(kRed);
	     c->Print(gEnergyPlots);
	     delete line;
	     delete h;
        }
	cout << "=====> done\n";
	return;         
    }
    // Delta E/E vs ThetaFoVMax
    else if (option == "Error2dVsThetaFoV") {
        gStyle->SetOptStat(1); 
        cout << "\n=====> Doing 2d relative error vs ThetaFoV plots\n";
	Double_t Erange[5] = {13.,13.5,13.7,14.2,14.6};
	Double_t Trange[5] = {0,7.5,15,22.5,30};
	for (Int_t i(0); i<4; i++) {
	     TCut cut = Form("TMath::Log10(fTruth.fEnergy)>%f && TMath::Log10(fTruth.fEnergy)<%f",Erange[i],Erange[i+1]);
	     c->Clear();
	     c->Divide(2,2);   
	     TH1F* h[4];
	     TH1F* htmp=0;
	     for (Int_t j(0); j<4; j++) {
	          c->cd(j+1);
	          htmp = new TH1F("htmp",Form("Relative Reconstruction Error in [%.3f,%.3f] log10(Energy) [%3.f,%3.f] ThetaFoVMax",Erange[i],Erange[i+1],Trange[j],Trange[j+1]),100,-2,2);
		  TCut cut1 = Form("TMath::RadToDeg()*fRecoEnergy.fThetaFoVMax > %f && TMath::RadToDeg()*fRecoEnergy.fThetaFoVMax <%f",Trange[j],Trange[j+1]);
	          gRecoTree->Draw("(fRecoEnergy.fEnergyMCError)/fTruth.fEnergy>>htmp",Quality&&cut&&cut1);
		  h[j] = (TH1F*)htmp->Clone();
		  h[j]->SetName(Form("h%d",j));
		  h[j]->Draw();
	          h[j]->GetXaxis()->SetTitle("Relative Reconstruction Error");
		  delete htmp;
	     }
	     c->Print(gEnergyPlots);
             for (Int_t j(0); j<4; j++) delete h[j];	
        }
	cout << "=====> done\n";
    }
    else return;
    
}
//______________________________________________________________________________
void ERecoEnergyPlotsOld() {
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_EnergyPlots")) ) {
        c = new TCanvas("c_EnergyPlots","c_EnergyPlots");
        c->Divide(1,2);        
    
    } 
    
    // do clever binning
    Int_t Nx(100);
    Double_t Xbins[101];
    Xbins[0] = 0.; 
    for (Int_t i(1);i<=Nx;i++) {
        Xbins[i] = TMath::Power(10, 19 + 2.8*i/(Nx+1));
    }
    TH1F *h1_EnergyPlots = (TH1F*)gROOT->FindObject("h1_EnergyPlots");
    if (!h1_EnergyPlots) 
        h1_EnergyPlots = new TH1F("h1_EnergyPlots","",Nx,Xbins);
    else
        h1_EnergyPlots->Reset();
    
    TH1F *h1p_EnergyPlots = (TH1F*)gROOT->FindObject("h1p_EnergyPlots");
    if (!h1p_EnergyPlots) 
        h1p_EnergyPlots = new TH1F("h1p_EnergyPlots","",Nx,Xbins);
    else
        h1p_EnergyPlots->Reset();
    TString Cut = "fTrackDirection.fQuality!=-1";
    TString Shape_Cut = "&& fRecoHmaxByShapeMethod.fQuality!=-1&&fRecoHmaxByShapeMethod.fHmax>0";
    TString Proton_Cut = "&& fRecoHmaxForProton.fHmax!=-1";
    TString var1 = "fRecoEnergy.fEnergy>>h1_EnergyPlots";
    TString var2 = "fRecoEnergy.fEnergyP>>h1p_EnergyPlots";
    c->cd(1);
    recotree->Draw(var1,Cut+Shape_Cut);
//     h1_EnergyPlots->Fit("gaus","","",1.e20,3.e21);
    c->cd(2);
    recotree->Draw(var2,Cut+Proton_Cut);
//     h1p_EnergyPlots->Fit("gaus","","",1.e20,3.e21);    
    h1_EnergyPlots->SetXTitle("reconstructed energy, [eV]");
    h1_EnergyPlots->SetYTitle("Entries");
    h1p_EnergyPlots->SetXTitle("reconstructed energy, [eV]");
    h1p_EnergyPlots->SetYTitle("Entries");   
}


//______________________________________________________________________________
void ERecoXmaxPlots() {
    TCanvas *c;
    if ( ! (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_XmaxPlots")) ) {
        c = new TCanvas("c_XmaxPlots","c_XmaxPlots");
        c->Divide(1,2);        
    
    } 
    
    TH1F *h1_XmaxPlots = (TH1F*)gROOT->FindObject("h1_XmaxPlots");
    if (!h1_XmaxPlots) 
        h1_XmaxPlots = new TH1F("h1_XmaxPlots","",30,400,1200);
    else
        h1_XmaxPlots->Reset();
    
    TH1F *h1p_XmaxPlots = (TH1F*)gROOT->FindObject("h1p_XmaxPlots");
    if (!h1p_XmaxPlots) 
        h1p_XmaxPlots = new TH1F("h1p_XmaxPlots","",30,400,1200);
    else
        h1p_XmaxPlots->Reset();
    TString Cut = "fTrackDirection.fQuality!=-1 ";
    TString Shape_Cut = "&& fRecoHmaxByShapeMethod.fQuality!=-1&&fRecoHmaxByShapeMethod.fHmax>0";
    TString Proton_Cut = "&& fRecoHmaxForProton.fHmax!=-1";
    TString var1 = "fRecoXmax.fXmax>>h1_XmaxPlots";
    TString var2 = "fRecoXmax.fXmaxProton>>h1p_XmaxPlots";
    c->cd(1);
    recotree->Draw(var1,Cut+Shape_Cut);
    c->cd(2);
    recotree->Draw(var2,Cut+Proton_Cut);
    h1_XmaxPlots->SetXTitle("reconstructed Xmax, [eV]");
    h1_XmaxPlots->SetYTitle("Entries");
    h1p_XmaxPlots->SetXTitle("reconstructed Xmax, [eV]");
    h1p_XmaxPlots->SetYTitle("Entries");   
}
