// Author: A.Thea, D.Naumov   Aug,  9 2004

///*****************************************************************************
//
// EventViewer macro:
// 
// ETreeEventViewer is an interface for the classes of eventviewer library made to 
// display multiple events: 3D shower viewer, some histos related to the shower.
// With time we will implement also detector info for many events
//
// See also: EEventViewer macro.
// 
//*****************************************************************************/

Bool_t axis = kTRUE;


ETreePainter* treepainter = NULL;
TTree *etree = NULL;
TTree *stree = NULL;
TChain *chain = NULL;
Bool_t SAVED = false;  // to know if stree has been already saved
EGViewer* pad_shower = 0;
EGViewer* pad_atmo_photons = 0;
EGViewer* pad_p_on_pupil = 0;

const char *defaultfilename= "";
const char *defaultStree= "";



// !!!!!!!!!!!!!!!!! WARNING : !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//
//	- GammaA-B   :  - should modify ETreePainter (X1 pb) when building Stree
//
//	- other than Zeta_A, Zeta_Mono, Gamma_A : change _air_ into _back_   (scattered ckov)
//
// !!!!!!!!!!!!!!!!! WARNING : !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



TCut cut1 = "H_max_last_bin && X_max_last_bin &&L_max_last_bin && H_fluomax_last_bin &&X_fluomax_last_bin && L_fluomax_last_bin";
TCut cut2 = "X1>0.002";  // because some pb below (shower generation)
TCut cut3 = "";
TCut cut4OR = "";
TCut cut5OR = "";
TCut drawcut;



//__________________________________________________________________________________________________________________________
void ETreeEventViewer() {
   
   bar = new TControlBar("vertical", "ETree Event Viewer",10,10);
   
   bar->AddButton("How to run        ",                 "help()","Instructions for running this macro"); 
   bar->AddButton("Open File         ",                 "openfile()","Open a new file"); 
   bar->AddButton("Open Stree        ",                 "openstree()","Open a stree file"); 
   bar->AddButton("Browser           ",                 "TBrowser b", "Open a Browser"); 
   bar->AddButton("STREE Viewer",                       "simpletreeviewer()","Simple Results Tree Viewer"); 
   bar->AddButton("Cut summary",                        "CutSummary()",""); 
   bar->AddButton("Shower - Truth Analysis",            "ShowerAnalysis()","Shower analysis");
   bar->AddButton("Photons at creation Analysis",       "AtmoPhotonsAnalysis()","Analysis of Photons created by shower");
   bar->AddButton("Photons on pupil analysis",          "PhotonsOnPupilAnalysis()","Show Tree Atmo Histos");
   bar->AddButton("Quit Root        ",                  ".q","Quit root");
   bar->Show();
   
   gROOT->SaveContext();
   Bool_t def = OpenDefaultStree();
   if(!def) openstree();
   
   drawcut += cut1;
   drawcut += cut2;
   drawcut += cut3;
   drawcut = drawcut || cut4OR;
   drawcut = drawcut || cut5OR;
}

//__________________________________________________________________________________________________________________________
void ShowerAnalysis() {
   bar = new TControlBar("vertical", "Shower and truth analysis",10,10);
   
   bar->AddButton("Truth Histograms",                   "treetruthhistos()","Show Tree  Truth Histos");
   bar->AddButton("Shower energy - xmax correlations",  "treeshowercorrel()","Show Tree shower Correlations");
   bar->AddButton("Shower theta correlations",          "shower_theta_correl()","Show Tree shower Correlations");
   bar->Show();
   
   gROOT->SaveContext();
   if(!pad_shower) pad_shower = new EGViewer();
   else pad_shower->Clear();
   pad_shower->SetName("Shower Analysis");
   pad_shower->SetTitle("Shower Analysis");
}

//__________________________________________________________________________________________________________________________-
void AtmoPhotonsAnalysis() {
   bar = new TControlBar("vertical", "Photons at creation analysis",10,10);
   
   bar->AddButton("Electrons / Photons correlations",   "phot_elec_correl()","");
   bar->AddButton("Crash ground",                       "crash_ground()","");
   bar->AddButton("Xmax shift",                         "Xmax_shift()","");
   bar->AddButton("photons Histo",                      "treeatmohistos()","");
   bar->AddButton("photons / theta correlations",       "atmo_theta_correl()","");
   bar->AddButton("photons / energy correlations",      "atmo_energy_correl()","");
   bar->Show();

   gROOT->SaveContext();
   if(!pad_atmo_photons) pad_atmo_photons = new EGViewer();
   else pad_atmo_photons->Clear();
   pad_atmo_photons->SetName("Photons in atmosphere Analysis");
   pad_atmo_photons->SetTitle("Photons in atmosphere Analysis");
}

//__________________________________________________________________________________________________________________________-
void PhotonsOnPupilAnalysis() {
   bar = new TControlBar("vertical", "Photons on pupil analysis",10,10);
   
   bar->AddButton("Photons on pupil Histograms",        "treepupilhistos()","");
   bar->AddButton("Pupil / theta Correlations",         "pupil_theta_correl()","");
   bar->AddButton("Pupil / energy Correlations",        "pupil_energy_correl()","");
   bar->AddButton("Transmissions",                      "transmission()","");
   bar->AddButton("Miscellaneous",                      "Miscellaneous()","");
   bar->Show();

   gROOT->SaveContext();
   if(!pad_p_on_pupil) pad_p_on_pupil = new EGViewer();
   else pad_p_on_pupil->Clear();
   pad_p_on_pupil->SetName("Photons on pupil Analysis");
   pad_p_on_pupil->SetTitle("Photons on pupil Analysis");
}

//__________________________________________________________________________________________________________________________-
void help() {
   //
  
   new TCanvas("chelp","Help to run demos",200,10,700,600);

   welcome = new TPaveText(.1,.8,.9,.97);
   welcome->AddText("Welcome to ESAF Tree Event Viewer");
   welcome->SetTextFont(32);
   welcome->SetTextColor(4);
   welcome->SetFillColor(24);
   welcome->Draw();

   hdemo = new TPaveText(.05,.05,.95,.7);
   hdemo->SetTextAlign(12);
   hdemo->SetTextFont(52);
   hdemo->AddText("- Demo for visualization of the shower development in atmosphere");
   hdemo->AddText("  and signal observed by the EUSO detector");
   hdemo->AddText("- Click left mouse button to execute one demo");
   hdemo->AddText("- Click left mouse button on Animation On/Off to start/stop animation");
   hdemo->AddText(" ");
   hdemo->SetAllWith("....","color",2);
   hdemo->SetAllWith("....","font",72);
   hdemo->SetAllWith("....","size",0.03);

   hdemo->Draw();
}

//__________________________________________________________________________________________________________________________-
void CutSummary() {
    //
    if(!stree) Printf("NO STREE : open stree before");
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kTruthPadID);
        gPad->Clear();
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_truth");
        if (!c) c = new TCanvas("c_truth","c_truth");

        c->Clear();
        c->cd();
    }
    TPaveText* hdemo = new TPaveText(.05,.05,.95,.95);
    hdemo->SetTextAlign(12);
    hdemo->SetTextFont(52);
    hdemo->SetTextColor(4);
    TText* temptext = 0;
    
    hdemo->AddText("");
    temptext = hdemo->AddText("Summarize cut effects on nb of events:");
    temptext->SetTextColor(1);
    
    hdemo->AddText("");
    temptext = hdemo->AddText(Form("Cut1 =  %s ",cut1.GetTitle()));
    temptext->SetTextColor(4);
    stree->Draw("Energy>>httemp",cut1,"goff");
    TH1F* hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Nb of events                  %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
   
    hdemo->AddText("");
    temptext = hdemo->AddText(Form("Cut2 =  %s ",cut2.GetTitle()));
    temptext->SetTextColor(4);
    stree->Draw("Energy>>httemp",cut2,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Nb of events      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
    stree->Draw("Energy>>httemp",cut1+cut2,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Cumulated nb      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
        
    hdemo->AddText("");
    temptext = hdemo->AddText(Form("Cut3 =  %s ",cut3.GetTitle()));
    temptext->SetTextColor(4);
    stree->Draw("Energy>>httemp",cut3,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Nb of events      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
    stree->Draw("Energy>>httemp",cut1+cut2+cut3,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Cumulated nb      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
        
    hdemo->AddText("");
    temptext = hdemo->AddText(Form("Cut4OR =  %s ",cut4OR.GetTitle()));
    temptext->SetTextColor(4);
    stree->Draw("Energy>>httemp",cut4OR,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Nb of events      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
    stree->Draw("Energy>>httemp",cut1+cut2+cut3||cut4OR,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Cumulated nb      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
        
    hdemo->AddText("");
    temptext = hdemo->AddText(Form("Cut5OR =  %s ",cut5OR.GetTitle()));
    temptext->SetTextColor(4);
    stree->Draw("Energy>>httemp",cut5OR,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Nb of events      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
    stree->Draw("Energy>>httemp",cut1+cut2+cut3||cut4OR||cut5OR,"goff");
    hhtemp = (TH1F*)gROOT->FindObject("httemp");
    temptext = hdemo->AddText(Form("        Cumulated nb      %g",hhtemp->GetEntries()));
    temptext->SetTextColor(2);
        
    hdemo->AddText("");
    hdemo->Draw();

}

//__________________________________________________________________________________________________________________________-
void gettree() {
    etree = (TTree*)gDirectory->Get("etree");
    if( etree ) {
        Printf("etree with %d events found.", (Int_t)etree->GetEntries());
	if(treepainter) delete treepainter;
	treepainter = new ETreePainter( etree );
    }
    
    stree = (TTree*)gDirectory->Get("stree"); 
    if(!stree) {
      Warning("gettree()","Fail to generate stree");
      return;
    }
    savesimpletree();
}

//__________________________________________________________________________________________________________________________-
void getstree() {
    stree = (TTree*)gDirectory->Get("stree");
    if( stree ) {
        Printf("%d events found.", (Int_t)stree->GetEntries());
	if(treepainter) delete treepainter;
	treepainter = new ETreePainter( stree );
	SAVED = true;
    }
    else Warning("getstree()","stree not found");
}

//__________________________________________________________________________________________________________________________-
Bool_t checktree( Option_t *opt = "default" ) {
    TString option(opt);
    Bool_t rtn = true;
    if((opt == "etree") && !etree ) {
        Warning("I do not find etree");
	rtn = false;
    }
    else if((opt == "stree") && !stree ) {
        Warning("I do not find stree");
	rtn = false;
	//help();
    
    }
    else if((opt == "chain") && !chain ) {
        Warning("I do not find chain");
	rtn = false;
    }
    else if((opt == "default")&&!etree&&!chain&&!stree) {
        Warning("checktree()","NO TREE FOUND (no chain, etree or stree)");
	rtn = false;
    }
    
    return rtn;
}

//__________________________________________________________________________________________________________________________-
void getchain() {
  //    chain = (TChain*)gDirectory->Get("etree");
    if(chain) {
        Printf("chain etree with %d events found.\n", (Int_t)chain->GetEntries());
	if(treepainter)
           delete treepainter;
	treepainter = new ETreePainter( chain );
    }
    
    stree = (TTree*)gDirectory->Get("stree"); 
    if(!stree) {
      Warning("getchain()","Fail to generate stree");
      return;
    }
    savesimpletree();
}

//__________________________________________________________________________________________________________________________-
void openfile() {
    TFile *f = 0;
    delete f;
    TGFileInfo* info = new TGFileInfo();
    while ( info->fFilename == 0 || !f || f->IsZombie() ) {
        TGFileDialog* filedialog = new TGFileDialog(gClient->GetRoot(), 0, kFDOpen, info);
        if( info->fFilename ) 
        f = new TFile( info->fFilename );
    }

    if( f && !f->IsZombie() ) {
        Printf("%s successfully opened.", info->fFilename);
	gettree();
	}
    else 
        Printf("Unable to open %s",info->fFilename);    
}

//__________________________________________________________________________________________________________________________-
void openstree() {
    TFile *f = 0;
    delete f;
    TGFileInfo* info = new TGFileInfo();
    while ( info->fFilename == 0 || !f || f->IsZombie() ) {
        TGFileDialog* filedialog = new TGFileDialog(gClient->GetRoot(), 0, kFDOpen, info);
        if( info->fFilename ) 
        f = new TFile( info->fFilename );
    }

    if( f && !f->IsZombie() ) {
        Printf("%s successfully opened.", info->fFilename);
        Warning("openstree()","SHOWER DRAWINGS NOT USABLE in this configuration");
	getstree();
    }
    else 
        Printf("Unable to open %s",info->fFilename);    
}

//__________________________________________________________________________________________________________________________-
Bool_t OpenDefaultStree() {
    TFile *f = new TFile(defaultStree);

    if( f && !f->IsZombie() ) {
        Printf("Default Stree %s successfully opened. ", defaultStree);
	getstree();
	return true;
    }
    else {
        Printf("Unable to open default Stree %s",defaultStree);
	return false;
    }
}

//__________________________________________________________________________________________________________________________-
void treetruthhistos() {
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treetruthhistos()","No Tree found");
	return;
    }
    
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;

    
    if(!pad_shower) pad_shower = new EGViewer();
    TH1* histo = 0;
    
    TCanvas *c = pad_shower->FindCanvas("c_treetruth_histo");
    if(!c ) c = pad_shower->AddTab("c_treetruth_histo","c_treetruth_histo");
    TCanvas *c2 = pad_shower->FindCanvas("c_treetruth_histo2");
    if(!c2 ) c2 = pad_shower->AddTab("c_treetruth_histo2","c_treetruth_histo2");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    treepainter->DrawHistos("Energy",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Energy");
    histo->SetTitle("Energy distribution in log scale");
    histo->SetXTitle("log10(Energy) (MeV)");

    pad2->cd();
    pad2->SetLogy(1);
    treepainter->DrawHistos("X1",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_X1");
    histo->SetTitle("Depth of first interaction point");
    histo->SetXTitle("X1 (g/cm^{2})");
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_X1","expo");
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    textpos = histo->GetBinLowEdge(1) + 0.05*(histo->GetBinLowEdge(histo->GetXaxis()->GetLast()) - histo->GetBinLowEdge(1));
    histo->Fit("mypol_X1","WOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.2g  exp(%.2g X_{1})",exp(par[0]),par[1]);
    t->DrawLatex(textpos,pow(10,log10(histo->GetMaximum())*0.15),txt);

    pad3->cd();
    treepainter->DrawHistos("Phi",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Phi");
    histo->SetTitle("Azimuth angle");
    histo->SetXTitle("phi (deg)");

    pad4->cd();
    treepainter->DrawHistos("Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Theta");
    histo->SetTitle("Local #theta");
    histo->SetXTitle("#theta (deg)");
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Theta","[0]*sin(2*(x*3.14/180))");
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    textpos = min + 0.3*(max-min);
    histo->Fit("mypol_Theta","WOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.2f sin(2x) fit",par[0]);
    t->DrawText(textpos,histo->GetMaximum()*0.84,txt);
    

/////////////////////////////
    c2->Clear();
    c2->cd();

    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmax");
    histo->SetTitle("Height of maximum");
    histo->SetXTitle("Hmax (km)");
    
    pad21->cd();
    pad21->SetLogy(1);
    treepainter->DrawHistos("Xmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmax");
    histo->SetTitle("Depth of maximum");
    histo->SetXTitle("Xmax (g/cm^{2})");
    min = histo->GetBinCenter(histo->GetMaximumBin()) + 10;
    max = histo->GetBinLowEdge(histo->GetXaxis()->GetLast()) + histo->GetBinWidth(histo->GetXaxis()->GetLast());
    mypol = new TF1("mypol_Xmax","expo",min,max);   // fit the Xmax distribution tail (10 g/cm2 after maximum)
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    textpos = histo->GetBinLowEdge(1) + 0.05*(histo->GetBinLowEdge(histo->GetXaxis()->GetLast()) - histo->GetBinLowEdge(1));
    histo->Fit("mypol_Xmax","WOQR");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.2g  exp(%.2g X_{max})",exp(par[0]),par[1]);
    t->DrawLatex(textpos,pow(10,log10(histo->GetMaximum())*0.15),txt);
    
    pad31->cd();
    treepainter->DrawHistos("Xmax_X1",drawcut);
    pad31->SetLogy(1);
    histo = (TH1*)gPad->GetPrimitive("h_Xmax_X1");
    histo->SetTitle("Depth of maximum from depth of first interaction");
    histo->SetXTitle("Xmax - X1 (g/cm^{2})");
    
    pad41->cd();
    pad41->SetLogy(1);
    treepainter->DrawHistos("Nemax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nemax");
    histo->SetTitle("Number of electrons at maximum");
    histo->SetXTitle("Nemax");
}

//__________________________________________________________________________________________________________________________-
void treeshowercorrel() {
    
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treeshowercorrel()","No Tree found");
	return;
    }
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    if(!pad_shower) pad_shower = new EGViewer();

    TCanvas *c = pad_shower->FindCanvas("c_treeshower_correl");
    if(!c ) c = pad_shower->AddTab("c_treeshower_correl","c_treeshower_correl");
    
    TCanvas *c2 = pad_shower->FindCanvas("c_treeshower_correl_F");
    if(!c2 ) c2 = pad_shower->AddTab("c_treeshower_correl_F","c_treeshower_correl_F");


    // *************** PLOTS FOR BEHAVIOR ********************
    
    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    
    pad1->cd();
    treepainter->DrawHistos("Hmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmax_Theta");
    histo->SetTitle("Shower Height of maximum vs/ local #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Hmax (km)");
    histo->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_Hmax_Theta",my_logcos,min,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,-10.);
    mypol->SetParLimits(0,0.01,10.);
    mypol->SetParLimits(1,-10.,-0.1);
    textpos = min + 0.1*(max-min);
    histo->Fit("mypol_Hmax_Theta","WROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.3g  -  %.3g * log(cos#theta)",par[0],-par[1]);
    t->DrawLatex(textpos,histo->GetMaximum()*0.64,txt);
    */
    
    pad2->cd();
    treepainter->DrawHistos("Xmax_Energy",drawcut);
    profXmax = (TProfile*)gPad->GetPrimitive("h_Xmax_Energy");
    profXmax->SetTitle("Shower Depth of maximum vs/ Energy");
    profXmax->SetXTitle("log10(Energy) (MeV)");
    profXmax->SetYTitle("Xmax (g/cm^{2})");
    profXmax->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<profXmax->GetNbinsX(); m++) {
        if(profXmax->GetBinContent(m+1) && fabs(min) == 0) min = profXmax->GetBinLowEdge(m+1);
	if(profXmax->GetBinContent(profXmax->GetNbinsX() - m) && fabs(max) == 0) max = profXmax->GetBinLowEdge(profXmax->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    textpos = min + 0.03*(max-min);
    /*
    TF1* mypol = new TF1("mypol_Xmax_Energy","pol1",min,max);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    profXmax->Fit("mypol_Xmax_Energy","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Xmax = %.3g  +  %.3g * log_{10}(E/MeV) g/cm^{2}",par[0],par[1]);
    t->DrawLatex(textpos,profXmax->GetMaximum()*0.54,txt);
    */

    pad3->cd();
    treepainter->DrawHistos("Nemax_Energy",drawcut);
    profNemax = (TProfile*)gPad->GetPrimitive("h_Nemax_Energy");
    profNemax->SetTitle("Number of electrons at maximum vs/ Energy");
    profNemax->SetXTitle("log10(Energy) (MeV)");
    profNemax->SetYTitle("log10(Nemax)");
    profNemax->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<profNemax->GetNbinsX(); m++) {
        if(profNemax->GetBinContent(m+1) && fabs(min) == 0) min = profNemax->GetBinLowEdge(m+1);
	if(profNemax->GetBinContent(profNemax->GetNbinsX() - m) && fabs(max) == 0) max = profNemax->GetBinLowEdge(profNemax->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nemax_Energy",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    profNemax->Fit("mypol_Nemax_Energy","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Ne_{max} = %.3e  *  (E/MeV)",pow(10,par[0]));
    t->DrawLatex(textpos,profNemax->GetMaximum()*0.54,txt);

    pad4->cd();
    treepainter->DrawHistos("Ne2_Energy",drawcut);
    profNe2 = (TProfile*)gPad->GetPrimitive("h_Ne2_Energy");
    profNe2->SetTitle("Ne Longitudinal profile first HALF vs/ Energy");
    profNe2->SetXTitle("log10(Energy) (MeV)");
    profNe2->SetYTitle("log10(Ne Longitudinal profile first HALF) (g/cm^{2})");
    profNe2->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<profNe2->GetNbinsX(); m++) {
        if(profNe2->GetBinContent(m+1) && fabs(min) == 0) min = profNe2->GetBinLowEdge(m+1);
	if(profNe2->GetBinContent(profNe2->GetNbinsX() - m) && fabs(max) == 0) max = profNe2->GetBinLowEdge(profNe2->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Ne2_Energy",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10);
    mypol->SetParLimits(1,1.,1.);

    textpos = min + 0.1*(max-min);
    profNe2->Fit("mypol_Ne2_Energy","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"\"Ne_{1/2}\" = %.3e  *  (E/MeV)  g/cm^{2}",pow(10,par[0]));
    t->DrawLatex(textpos,profNe2->GetMaximum()*0.54,txt);
    
    
    // *************** PLOTS FOR SIGMA  ****************************************

    c2->Clear();
    c2->cd();
    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();
    TProfile* prof = 0;
    
    pad11->cd();
    prof = profXmax;
    nbin = profXmax->GetNbinsX();
    min = profXmax->GetBinLowEdge(1);
    max =  profXmax->GetBinLowEdge(nbin) + profXmax->GetBinWidth(nbin);
    resolXmax = new TProfile("resolXmax","Normalized sigma of Xmax VS Energy",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(profXmax->GetBinContent(i+1)) resolXmax->Fill(profXmax->GetBinCenter(i+1),profXmax->GetBinError(i+1)/profXmax->GetBinContent(i+1));  // sigma/mean
    }
    resolXmax->SetMarkerStyle(27);
    resolXmax->SetXTitle("log10(Energy) (MeV)");
    resolXmax->Draw();
    
    pad21->cd();
    treepainter->DrawHistos("Nemax_Energy_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nemax_Energy_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNemax = new TProfile("resolNemax","Normalized sigma of Nemax VS Energy",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNemax->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNemax->SetXTitle("log10(Energy) (MeV)");
    resolNemax->SetMarkerStyle(27);
    resolNemax->Draw();
  
    pad31->cd();
    treepainter->DrawHistos("Ne2_Energy_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Ne2_Energy_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNe2 = new TProfile("resolNe2","#splitline{Normalized sigma of \"Ne longitudinal}{profile first HALF\" VS Energy}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNe2->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    gPad->SetTopMargin(0.2);
    resolNe2->SetXTitle("log10(Energy) (MeV)");
    resolNe2->SetMarkerStyle(27);
    resolNe2->SetTitleOffset(2);
    resolNe2->Draw();
}

//__________________________________________________________________________________________________________________________-
void shower_theta_correl() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    TF1* mypol = 0;    

    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treeatmoEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *cc = pad_shower->FindCanvas("c_treeshower_correltheta");
    if(!cc ) cc = pad_shower->AddTab("c_treeshower_correltheta","c_treeshower_correltheta");


    // *************** PLOTS FOR BEHAVIOR ********************

    cc->Clear();
    cc->cd();

    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Ne_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Ne_Theta");
    histo->SetTitle("Electrons profile integration vs/ theta -- energy normalized");
    histo->SetYTitle("Electron profile integration    g/(MeV.cm^{2})");
    histo->SetXTitle("#theta (deg)");
    histo->SetStats(0);
        
    pad21->cd();
    treepainter->DrawHistos("Ne2_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Ne2_Theta");
    histo->SetTitle("HALF electrons profile integration vs/ theta -- energy normalized");
    histo->SetYTitle("HALF lectron profile integration    g/(MeV.cm^{2})");
    histo->SetXTitle("#theta (deg)");
    histo->SetStats(0);
    
    pad31->cd();
    treepainter->DrawHistos("Nemax_Theta",drawcut);
    prof_widthTheta = (TProfile*)gPad->GetPrimitive("h_Nemax_Theta");
    prof_widthTheta->SetTitle("Nemax vs/ #theta -- energy normalized");
    prof_widthTheta->SetXTitle("#theta (deg)");
    prof_widthTheta->SetYTitle("Nemax  (Mev^{-1})");
    prof_widthTheta->SetStats(0);
    
    pad41->cd();
    treepainter->DrawHistos("Widthshow_Hmax",drawcut);
    prof_widthTheta = (TProfile*)gPad->GetPrimitive("h_Widthshow_Hmax");
    prof_widthTheta->SetTitle("Shower width vs/ Hmax");
    prof_widthTheta->SetXTitle("Hmax  (km)");
    prof_widthTheta->SetYTitle("Width  (km)");
    prof_widthTheta->SetStats(0);
}

//__________________________________________________________________________________________________________________________-
void phot_elec_correl() {
    //
    //
    //
    
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;

    TCanvas *c = pad_atmo_photons->FindCanvas("c_treeshowphot_correl");
    if(!c ) c = pad_atmo_photons->AddTab("c_treeshowphot_correl","c_treeshowphot_correl");
    TCanvas *c2 = pad_atmo_photons->FindCanvas("c_treeshowphot_correl_F");
    if(!c2 ) c2 = pad_atmo_photons->AddTab("c_treeshowphot_correl_F","c_treeshowphot_correl_F");

    
    // *************** PLOTS FOR BEHAVIOR ********************

    c->Clear();
    c->cd();
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
        delete c; c= 0;
	Warning("treeshowphotcorrel()","No Tree found");
	return;
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    treepainter->DrawHistos("Nph2fluo_Ne2",drawcut);
    prof_Nph2fluoNe = (TProfile*)gPad->GetPrimitive("h_Nph2fluo_Ne2");
    prof_Nph2fluoNe->SetTitle("HALF Fluo vs/ integration of Ne profile");
    prof_Nph2fluoNe->SetXTitle("(log10) HALF integration of Ne profile (g/cm^{2})");
    prof_Nph2fluoNe->SetYTitle("(log10) HALF fluo photons generated");
    prof_Nph2fluoNe->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nph2fluoNe->GetNbinsX(); m++) {
        if(prof_Nph2fluoNe->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nph2fluoNe->GetBinLowEdge(m+1);
	if(prof_Nph2fluoNe->GetBinContent(prof_Nph2fluoNe->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nph2fluoNe->GetBinLowEdge(prof_Nph2fluoNe->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2fluo_Ne2",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nph2fluoNe->Fit("mypol_Nph2fluo_Ne2","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{1/2} = %.1f  *  Ne_{1/2}",pow(10,par[0]));
    t->DrawLatex(textpos,prof_Nph2fluoNe->GetMaximum()*0.54,txt);

    pad2->cd();
    treepainter->DrawHistos("Nph2ckov_Ne2",drawcut);
    prof_Nph2ckovNe = (TProfile*)gPad->GetPrimitive("h_Nph2ckov_Ne2");
    prof_Nph2ckovNe->SetTitle("HALF Cerenkov vs/ HALF integration of Ne profile");
    prof_Nph2ckovNe->SetXTitle("(log10) HALF integration Ne profile (g/cm^{2})");
    prof_Nph2ckovNe->SetYTitle("(log10) HALF cerenkov photons generated");
    prof_Nph2ckovNe->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nph2ckovNe->GetNbinsX(); m++) {
        if(prof_Nph2ckovNe->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nph2ckovNe->GetBinLowEdge(m+1);
	if(prof_Nph2ckovNe->GetBinContent(prof_Nph2ckovNe->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nph2ckovNe->GetBinLowEdge(prof_Nph2ckovNe->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2ckov_Ne2",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nph2ckovNe->Fit("mypol_Nph2ckov_Ne2","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nckov_{1/2} = %.1f  *  Ne_{1/2}",pow(10,par[0]));
    t->DrawLatex(textpos,prof_Nph2ckovNe->GetMaximum()*0.54,txt);

    pad3->cd();
    treepainter->DrawHistos("Nphmaxfluo_Nemax",drawcut);
    prof_Nphmax = (TProfile*)gPad->GetPrimitive("h_Nphmaxfluo_Nemax");
    prof_Nphmax->SetTitle("Maximum of fluo profile vs/ Maximum of Ne profile");
    prof_Nphmax->SetXTitle("log10(Nemax)");
    prof_Nphmax->SetYTitle("(log10) Maximum of fluo profile");
    prof_Nphmax->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nphmax->GetNbinsX(); m++) {
        if(prof_Nphmax->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nphmax->GetBinLowEdge(m+1);
	if(prof_Nphmax->GetBinContent(prof_Nphmax->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nphmax->GetBinLowEdge(prof_Nphmax->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nphmaxfluo_Nemax",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nphmax->Fit("mypol_Nphmaxfluo_Nemax","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{max} = %.1f  *  Ne_{max}  cm^{2}/g",pow(10,par[0]));
    t->DrawLatex(textpos,prof_Nphmax->GetMaximum()*0.54,txt);

    pad4->cd();
    treepainter->DrawHistos("Nph2fluo_Nemax",drawcut);
    prof_NphfluoNemax = (TProfile*)gPad->GetPrimitive("h_Nph2fluo_Nemax");
    prof_NphfluoNemax->SetTitle("HALF Fluo vs/ Maximum of Ne profile");
    prof_NphfluoNemax->SetXTitle("log10(Nemax)");
    prof_NphfluoNemax->SetYTitle("(log10) HALF fluo generated");
    prof_NphfluoNemax->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NphfluoNemax->GetNbinsX(); m++) {
        if(prof_NphfluoNemax->GetBinContent(m+1) && fabs(min) == 0) min = prof_NphfluoNemax->GetBinLowEdge(m+1);
	if(prof_NphfluoNemax->GetBinContent(prof_NphfluoNemax->GetNbinsX() - m) && fabs(max) == 0) max = prof_NphfluoNemax->GetBinLowEdge(prof_NphfluoNemax->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2fluo_Nemax",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_NphfluoNemax->Fit("mypol_Nph2fluo_Nemax","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{1/2} = %.1f  *  Ne_{max}",pow(10,par[0]));
    t->DrawLatex(textpos,prof_NphfluoNemax->GetMaximum()*0.54,txt);

    
    // *************** PLOTS FOR SIGMA  ***********************************
    
    c2->Clear();
    c2->cd();
    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();
    TProfile* prof = 0;
    
    pad11->cd();
    treepainter->DrawHistos("Nph2fluo_Ne2_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph2fluo_Ne2_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNph2FluoNe = new TProfile("resolNph2FluoNe","#splitline{Normalized sigma of Fluo first HALF VS/}{Ne first HALF profile}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++)
        if(prof->GetBinContent(i+1)) resolNph2FluoNe->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    resolNph2FluoNe->SetMarkerStyle(27);
    resolNph2FluoNe->SetXTitle("(log10) HALF integration of Ne profile (g/cm^{2})");
    resolNph2FluoNe->Draw();
    gPad->SetTopMargin(0.18);
    
    pad21->cd();
    treepainter->DrawHistos("Nph2ckov_Ne2_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph2ckov_Ne2_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNph2CkovNe = new TProfile("resolNph2CkovNe","#splitline{Normalized sigma of Cerenkov first HALF VS/}{Ne first HALF profile}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNph2CkovNe->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNph2CkovNe->SetMarkerStyle(27);
    resolNph2CkovNe->SetXTitle("(log10) HALF integration of Ne profile (g/cm^{2})");
    resolNph2CkovNe->Draw();
    gPad->SetTopMargin(0.18);
  
    pad31->cd();
    treepainter->DrawHistos("Nphmaxfluo_Nemax_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nphmaxfluo_Nemax_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNmaxFluoNemax = new TProfile("resolNmaxFluoNemax","#splitline{Normalized sigma of max Fluo VS/}{Max of electrons}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNmaxFluoNemax->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNmaxFluoNemax->SetMarkerStyle(27);
    resolNmaxFluoNemax->SetXTitle("log10(Nemax)");
    resolNmaxFluoNemax->Draw();
    gPad->SetTopMargin(0.18);
      
    pad41->cd();
    treepainter->DrawHistos("Nph2fluo_Nemax_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph2fluo_Nemax_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNphFluoNemax = new TProfile("resolNphFluoNemax","#splitline{Normalized sigma of Fluo first HALF  VS/}{Max of electrons}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNphFluoNemax->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNphFluoNemax->SetMarkerStyle(27);
    resolNphFluoNemax->SetXTitle("log10(Nemax)");
    resolNphFluoNemax->Draw();
    gPad->SetTopMargin(0.18);
}

//__________________________________________________________________________________________________________________________-
void crash_ground() {
   
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    
    TCanvas *cc = pad_atmo_photons->FindCanvas("c_treeshower_crashGround");
    if(!cc ) cc = pad_atmo_photons->AddTab("c_treeshower_crashGround","c_treeshower_crashGround");

    cc->Clear();
    cc->cd();

    TPad *pad10 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad20 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad30 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad40 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad10->Draw();
    pad20->Draw();
    pad30->Draw();
    pad40->Draw();

    pad10->cd();
    treepainter->DrawHistos("Ne2_Over_Ne_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Ne2_Over_Ne_Hmax");
    histo->SetTitle("Shower crash effect on electrons profile");
    histo->SetXTitle("Local altitude of shower maximum (km)");
    histo->SetYTitle("Ratio ( first HALF / FULL profile )");
    histo->GetYaxis()->SetTitleOffset(1.5);
    gPad->SetLeftMargin(0.13);
    histo->SetStats(0);
    
    pad20->cd();
    treepainter->DrawHistos("Nph2_Over_Nph_Hmax_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Over_Nph_Hmax_fluo");
    histo->SetTitle("Shower crash effect on fluo photons nb");
    histo->SetXTitle("Local altitude of shower maximum (km)");
    histo->SetYTitle("Ratio ( first HALF / FULL profiles )");
    histo->SetStats(0);
    
    pad30->cd();
    treepainter->DrawHistos("Width2_Over_Width_Hmax_shower",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Width2_Over_Width_Hmax_shower");
    histo->SetTitle("Crash effect on shower Width");
    histo->SetXTitle("Local altitude of shower maximum (km)");
    histo->SetYTitle("Ratio ( first HALF width / FULL width )");
    histo->SetStats(0);
    
    pad40->cd();
    treepainter->DrawHistos("Width2_Over_Width_Hmax_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Width2_Over_Width_Hmax_fluo");
    histo->SetTitle("Shower crash effect on fluo profile Width");
    histo->SetXTitle("Local altitude of shower maximum (km)");
    histo->SetYTitle("Ratio ( first HALF width / FULL width )");
    histo->SetStats(0);
    
    /* for CKOV : not really used so far
    treepainter->DrawHistos("Nph2_Over_Nph_Hmax_cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Over_Nph_Hmax_cer");
    histo->SetTitle("Shower crash effect on cerenkov photons nb");
    histo->SetXTitle("Local altitude of shower maximum (km)");
    histo->SetYTitle("Ratio ( first HALF / FULL profiles )");
    histo->SetStats(0);
    */
    
//////////////////////////////////////////
    
    TCanvas *c = pad_atmo_photons->FindCanvas("c_treeshower_crashGround2");
    if(!c ) c = pad_atmo_photons->AddTab("c_treeshower_crashGround2","c_treeshower_crashGround2");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    
    pad1->cd();
    treepainter->DrawHistos("Nph2_Over_Nph_Theta_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Over_Nph_Theta_fluo");
    histo->SetTitle("Shower crash effect on fluo photons nb");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio ( first HALF / FULL profiles )");
    histo->SetStats(0);
    
    pad2->cd();
    treepainter->DrawHistos("Nph2_Over_Nph_Theta_cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Over_Nph_Theta_cer");
    histo->SetTitle("Shower crash effect on cerenkov photons nb");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio ( first HALF / FULL profiles )");
    histo->SetStats(0);
    
    pad3->cd();
    treepainter->DrawHistos("Ne2_Over_Ne_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Ne2_Over_Ne_Theta");
    histo->SetTitle("Shower crash effect on electrons profile");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio ( first HALF / FULL profile )");
    histo->GetYaxis()->SetTitleOffset(1.5);
    gPad->SetLeftMargin(0.13);
    histo->SetStats(0);
    
    pad4->cd();
    treepainter->DrawHistos("Width2_Over_Width_Theta_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Width2_Over_Width_Theta_fluo");
    histo->SetTitle("Shower crash effect on fluo profile Width");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio ( first HALF width / FULL width )");
    histo->SetStats(0);
    

}

//__________________________________________________________________________________________________________________________-
void Xmax_shift() {
   
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    
    TCanvas *cc2 = pad_atmo_photons->FindCanvas("c_treeshower_Xmaxshift");
    if(!cc2 ) cc2 = pad_atmo_photons->AddTab("c_treeshower_Xmaxshift","c_treeshower_Xmaxshift");

    cc2->Clear();
    cc2->cd();

    TPad *pad102 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad202 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad302 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad402 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad102->Draw();
    pad202->Draw();
    pad302->Draw();
    pad402->Draw();

    pad102->cd();
    treepainter->DrawHistos("Xmaxfluo_Xmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxfluo_Xmax_Theta");
    histo->SetTitle("Xmax shift between fluo and electrons VS/ #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("X_{max}Fluo - X_{max}Shower (g/cm^{2})");
    
    pad202->cd();
    treepainter->DrawHistos("Xmaxfluo_Xmax_Energy",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxfluo_Xmax_Energy");
    histo->SetTitle("Xmax shift between fluo and electrons VS/ Energy");
    histo->SetXTitle("log10(Energy) (MeV)");
    histo->SetYTitle("Xmax_fluo - Xmax_shower (g/cm^{2})");
    
    pad302->cd();
    treepainter->DrawHistos("Xmaxfluo_Xmax_X1",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxfluo_Xmax_X1");
    histo->SetTitle("Xmax shift between fluo and electrons VS/ X1");
    histo->SetXTitle("X1 (g/cm^{2})");
    histo->SetYTitle("Xmax_fluo - Xmax_shower (g/cm^{2})");
    
    pad402->cd();
    treepainter->DrawHistos("Xmaxfluo_Xmax_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxfluo_Xmax_Hmax");
    histo->SetTitle("Xmax shift between fluo and electrons VS/ Hmax");
    histo->SetXTitle("Hmax  (km)");
    histo->SetYTitle("X_{max}Fluo - X_{max}Shower (g/cm^{2})");


    TCanvas * c = pad_atmo_photons->FindCanvas("c_treeshower_Xmaxshift2");
    if(!c )   c = pad_atmo_photons->AddTab("c_treeshower_Xmaxshift2","c_treeshower_Xmaxshift2");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    treepainter->DrawHistos("Hmaxfluo_Hmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmaxfluo_Hmax_Theta");
    histo->SetTitle("Hmax shift between fluo and electrons VS #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Hmax_fluo - Hmax_shower (km)");
    
    pad2->cd();
    treepainter->DrawHistos("Lmaxfluo_Lmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Lmaxfluo_Lmax_Theta");
    histo->SetTitle("Lmax shift between fluo and electrons VS #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Lmax_fluo - Lmax_shower (km)");
    
    pad3->cd();
    treepainter->DrawHistos("xmax_ymax_shift",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_xmax_ymax_shift");
    histo->SetTitle("Position shift between fluo and electrons");
    histo->SetXTitle("Shift in x-coordinate");
    histo->SetYTitle("Shift in y-coordinate");
    
    pad4->cd();
    treepainter->DrawHistos("Rmaxfluo_Rmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Rmaxfluo_Rmax_Theta");
    histo->SetTitle("Rmax shift between fluo and electrons VS #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Rmax_fluo - Rmax_shower (km)");
    
    TCanvas * c1 = pad_atmo_photons->FindCanvas("c_treeshower_Xmaxshift3");
    if(!c1 )   c1 = pad_atmo_photons->AddTab("c_treeshower_Xmaxshift3","c_treeshower_Xmaxshift3");

    c1->Clear();
    c1->cd();
    
    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Hmaxcer_Hmax_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmaxcer_Hmax_Theta");
    histo->SetTitle("Hmax shift between ckov and electrons VS #theta");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Hmax_ckov - Hmax_shower (km)");
    
    pad21->cd();
    treepainter->DrawHistos("Yield_mean_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Yield_mean_Hmax");
    histo->SetTitle("Mean Fluo yield");
    histo->SetXTitle("Hmax_show  (km)");
    histo->SetYTitle("Yield  (m_{1})");
    
    pad31->cd();
    treepainter->DrawHistos("Yield_max_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Yield_max_Hmax");
    histo->SetTitle("Fluo yield at shower maximum");
    histo->SetXTitle("Hmax_show  (km)");
    histo->SetYTitle("Yield  (m_{1})");
    
    pad41->cd();
    treepainter->DrawHistos("Yield_max_cer_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Yield_max_cer_Hmax");
    histo->SetTitle("Ckov yield at shower maximum");
    histo->SetXTitle("Hmax_show  (km)");
    histo->SetYTitle("Yield  (m_{1})");

}

//__________________________________________________________________________________________________________________________-
void treeatmohistos() {
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treeatmohistos()","No Tree found");
	return;
    }
    
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    
    if(!pad_atmo_photons) pad_atmo_photons = new EGViewer();
    TCanvas *c = pad_atmo_photons->FindCanvas("c_treeatmo_histo");
    if(!c ) c = pad_atmo_photons->AddTab("c_treeatmo_histo","c_treeatmo_histo");
    TCanvas *c2 = pad_atmo_photons->FindCanvas("c_treeatmo_histo2");
    if(!c2 ) c2 = pad_atmo_photons->AddTab("c_treeatmo_histo2","c_treeatmo_histo2");
    //TCanvas *c3 = pad_atmo_photons->FindCanvas("c_treeatmo_histo3");
    //if(!c3 ) c3 = pad_atmo_photons->AddTab("c_treeatmo_histo3","c_treeatmo_histo3");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    pad1->SetLogy(1);
    treepainter->DrawHistos("Nph_Fluo_L",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Fluo_L");
    histo->SetTitle("Fluorescence photons at creation");
    histo->SetXTitle("Nph_fluo");
   
    pad2->cd();
    pad2->SetLogy(1);
    treepainter->DrawHistos("Nph_Cer_L",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Cer_L");
    histo->SetTitle("Cerenkov photons at creation");
    histo->SetXTitle("Nph_cer");
   
    pad3->cd();
    pad3->SetLogy(1);
    treepainter->DrawHistos("Nmax_Fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nmax_Fluo");
    histo->SetTitle("Fluo photons maximum");
    histo->SetXTitle("Nphmax_fluo (cm^{2}/g -- per depth unit)");
   
    /*
    pad4->cd();
    pad4->SetLogy(1);
    treepainter->DrawHistos("Nmax_Cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nmax_Cer");
    histo->SetTitle("Cerenkov photons maximum");
    histo->SetXTitle("Nphmax_cer (cm^{2}/g -- per depth unit)");
    */
    
    
////////////////////////////////
    c2->Clear();
    c2->cd();

    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    pad11->SetLogy(1);
    treepainter->DrawHistos("Xmaxph_Fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxph_Fluo");
    histo->SetTitle("Depth of Fluorescence maximum");
    histo->SetXTitle("Xmaxph_fluo (g/cm^{2})");
    min = histo->GetBinCenter(histo->GetMaximumBin()) + 10;
    max = histo->GetBinLowEdge(histo->GetXaxis()->GetLast()) + histo->GetBinWidth(histo->GetXaxis()->GetLast());
    mypol = new TF1("mypol_Xmax_fluo","expo",min,max);   // fit the Xmax_fluo distribution tail (10 g/cm2 after maximum)
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    textpos = histo->GetBinLowEdge(1) + 0.05*(histo->GetBinLowEdge(histo->GetXaxis()->GetLast()) - histo->GetBinLowEdge(1));
    histo->Fit("mypol_Xmax_fluo","WOQR");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.2g  exp(%.2g Xfluo_{max})",exp(par[0]),par[1]);
    t->DrawLatex(textpos,pow(10,log10(histo->GetMaximum())*0.15),txt);

   /* pad21->cd();
    pad21->SetLogy(1);
    treepainter->DrawHistos("Xmaxph_Cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmaxph_Cer");
    histo->SetTitle("Depth of Cerenkov maximum");
    histo->SetXTitle("Xmaxph_cer (g/cm^{2})");
    */
    pad31->cd();
    treepainter->DrawHistos("Hmax_Fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmax_Fluo");
    histo->SetTitle("Height of Fluorescence maximum");
    histo->SetXTitle("Hmax_fluo (km)");
    pad41->cd();
    treepainter->DrawHistos("Hmax_Cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Hmax_Cer");
    histo->SetTitle("Height of Cerenkov maximum");
    histo->SetXTitle("Hmax_cer (km)");
    
    
/////////////////////////
/* c3 disabled too
    c3->Clear();
    c3->cd();
    TPad *pad12 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad22 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad32 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad42 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad12->Draw();
    pad22->Draw();
    pad32->Draw();
    pad42->Draw();

    pad12->cd();
    treepainter->DrawHistos("Wlmean_f",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_f");
    histo->SetTitle("Fluorescence mean wavelength at creation");
    histo->SetXTitle("Wlmean_f (nm)");
    pad22->cd();
    treepainter->DrawHistos("Wlmean_c",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_c");
    histo->SetTitle("Cerenkov mean wavelength at creation");
    histo->SetXTitle("Wlmean_c (nm)");
    pad32->cd();
    //treepainter->DrawHistos("Wlmean_tot",drawcut);
    //histo = (TH1*)gPad->GetPrimitive("h_Wlmean_tot");
    //histo->SetTitle("Photons mean wavelength at creation");
    //histo->SetXTitle("Wlmean_tot (nm)");
*/
    
}

//__________________________________________________________________________________________________________________________-
void atmo_theta_correl() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    TF1* mypol = 0;    

    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treeatmoEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *cc = pad_atmo_photons->FindCanvas("c_treeatmo_correltheta");
    if(!cc ) cc = pad_atmo_photons->AddTab("c_treeatmo_correltheta","c_treeatmo_correltheta");
    TCanvas *cc2 = pad_atmo_photons->FindCanvas("c_treeatmo_correltheta_ckov");
    if(!cc2 ) cc2 = pad_atmo_photons->AddTab("c_treeatmo_correltheta_ckov","c_treeatmo_correltheta_ckov");


    // *************** PLOTS FOR BEHAVIOR ********************

    cc->Clear();
    cc->cd();

    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Nph2_Theta_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Theta_fluo");
    histo->SetTitle("HALF Fluo photons vs/ theta -- energy normalized");
    histo->SetYTitle("HALF fluo photons generated");
    histo->SetXTitle("#theta (deg)");
    histo->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_Nph2_Theta_fluo",my_overcos60,min,max,1);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameters(1.,0);
    mypol->SetParLimits(0,0.1,100.);
    textpos = min + 0.1*(max-min);
    histo->Fit("mypol_Nph2_Theta_fluo","WROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.3g MeV^{-1}/ cos(#theta)",par[0]);
    t->DrawLatex(textpos,histo->GetMaximum()*0.74,txt);
    */
        
    pad21->cd();
    treepainter->DrawHistos("Nmax_Nph2_fluo_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nmax_Nph2_fluo_Hmax");
    histo->SetTitle("ratio (fluo max #times1000 / Fluo HALF) vs/ Hmax_fluo");
    histo->SetYTitle("Fluo_{max} / Fluo_{HALF}  (#times 10^{-3})");
    histo->SetXTitle("Hmax fluo (km)");
    histo->SetStats(0);
    
    pad31->cd();
    treepainter->DrawHistos("Width2_fluo_Theta",drawcut);
    prof_widthTheta = (TProfile*)gPad->GetPrimitive("h_Width2_fluo_Theta");
    prof_widthTheta->SetTitle("Half width of Fluo profile vs/ #theta");
    prof_widthTheta->SetXTitle("#theta (deg)");
    prof_widthTheta->SetYTitle("HALF width of fluo profile (km)");
    prof_widthTheta->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_widthTheta->GetNbinsX(); m++) {
        if(prof_widthTheta->GetBinContent(m+1) && fabs(min) == 0) min = prof_widthTheta->GetBinLowEdge(m+1);
	if(prof_widthTheta->GetBinContent(prof_widthTheta->GetNbinsX() - m) && fabs(max) == 0) max = prof_widthTheta->GetBinLowEdge(prof_widthTheta->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_Width2_fluo_Theta",my_overcos60,min,max,1);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameters(0.01.,0);
    mypol->SetParLimits(0,1.,100.);
    textpos = min + 0.08*(max-min);
    prof_widthTheta->Fit("mypol_Width2_fluo_Theta","WROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"W_{1/2} = %.3g km /cos(#theta)",par[0]);
    t->DrawLatex(textpos,prof_widthTheta->GetMaximum()*0.84,txt);
    */
    
    
    pad41->cd();
    treepainter->DrawHistos("Width2_fluo_Hmax_fluo",drawcut);
    prof_widthHmax = (TProfile*)gPad->GetPrimitive("h_Width2_fluo_Hmax_fluo");
    prof_widthHmax->SetTitle("Half width of Fluo profile vs/ fluo maximum height");
    prof_widthHmax->SetXTitle("Height of fluo max (km)");
    prof_widthHmax->SetYTitle("HALF width of fluo profile (km)");
    prof_widthHmax->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_widthHmax->GetNbinsX(); m++) {
        if(prof_widthHmax->GetBinContent(m+1) && fabs(min) == 0) min = prof_widthHmax->GetBinLowEdge(m+1);
	if(prof_widthHmax->GetBinContent(prof_widthHmax->GetNbinsX() - m) && fabs(max) == 0) max = prof_widthHmax->GetBinLowEdge(prof_widthHmax->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_Width2_fluo_Hmax_fluo",my_expon15,2,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameters(1.,0.01);
    mypol->SetParLimits(0,0.1,10.);
    mypol->SetParLimits(1,0.01,10.);
    textpos = min + 0.08*(max-min);
    prof_widthHmax->Fit("mypol_Width2_fluo_Hmax_fluo","ROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"W_{1/2} = %.3g km * exp(H_{max}/%.3g)",exp(par[0]),1/par[1]);
    t->DrawLatex(textpos,prof_widthHmax->GetMaximum()*0.84,txt);
    */


////////////////////////// ckov
    cc2->Clear();
    cc2->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    treepainter->DrawHistos("Nmax_Hmax_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nmax_Hmax_fluo");
    histo->SetTitle("Fluo max vs/ Hmax -- energy normalized");
    histo->SetYTitle("Fluo max generated km^{-1}");
    histo->SetXTitle("Hmax fluo (km)");
    histo->SetStats(0);
    
    pad2->cd();
    treepainter->DrawHistos("Nph2_Theta_cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Theta_cer");
    histo->SetTitle("HALF Cerenkov photons vs/ #theta -- energy normalized");
    histo->SetYTitle("HALF cerenkov photons generated");
    histo->SetXTitle("#theta (deg)");
    histo->SetStats(0);
   
    pad3->cd();
    treepainter->DrawHistos("Nph2_Hmax_fluo",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Hmax_fluo");
    histo->SetTitle("Fluo HALF vs/ Hmax -- energy normalized");
    histo->SetYTitle("Fluo HALF generated");
    histo->SetXTitle("Hmax (km)");
    histo->SetStats(0);
    
    pad4->cd();
    treepainter->DrawHistos("Nph2_Hmax_cer",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph2_Hmax_cer");
    histo->SetTitle("HALF Cerenkov photons vs/ Hmax -- energy normalized");
    histo->SetYTitle("HALF cerenkov photons generated");
    histo->SetXTitle("Hmax (km)");
    histo->SetStats(0);

}

//__________________________________________________________________________________________________________________________-
void atmo_energy_correl() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    TF1* mypol = 0;
    if(!pad_atmo_photons) pad_atmo_photons = new EGViewer();
    


    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treeatmoEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *c = pad_atmo_photons->FindCanvas("c_treeatmo_correlenergy");
    if(!c ) c = pad_atmo_photons->AddTab("c_treeatmo_correlenergy","c_treeatmo_correlenergy");
    TCanvas *c2 = pad_atmo_photons->FindCanvas("c_treeatmo_correlenergy_F");
    if(!c2 ) c2 = pad_atmo_photons->AddTab("c_treeatmo_correlenergy_F","c_treeatmo_correlenergy_F");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();


    pad1->cd();
    treepainter->DrawHistos("Nph2_Energy_fluo",drawcut);
    prof_Nphfluo_Energy = (TProfile*)gPad->GetPrimitive("h_Nph2_Energy_fluo");
    prof_Nphfluo_Energy->SetTitle("HALF Fluo photons vs/ Energy -- cos#theta normalized");
    prof_Nphfluo_Energy->SetYTitle("(log10) HALF fluo photons generated");
    prof_Nphfluo_Energy->SetXTitle("log10(Energy) (MeV)");
    prof_Nphfluo_Energy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nphfluo_Energy->GetNbinsX(); m++) {
        if(prof_Nphfluo_Energy->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nphfluo_Energy->GetBinLowEdge(m+1);
	if(prof_Nphfluo_Energy->GetBinContent(prof_Nphfluo_Energy->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nphfluo_Energy->GetBinLowEdge(prof_Nphfluo_Energy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2_Energy_fluo",my_linear,min,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nphfluo_Energy->Fit("mypol_Nph2_Energy_fluo","RCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{1/2} = %.3g  *  (E/MeV)",pow(10,par[0]));
    t->DrawLatex(textpos,prof_Nphfluo_Energy->GetMaximum()*0.54,txt);
    
    
    pad2->cd();
    treepainter->DrawHistos("Nph2_Energy_cer",drawcut);
    prof_NphcerEnergy = (TProfile*)gPad->GetPrimitive("h_Nph2_Energy_cer");
    prof_NphcerEnergy->SetTitle("HALF Cerenkov photons vs/ Energy");
    prof_NphcerEnergy->SetYTitle("(log10) HALF cerenkov photons generated");
    prof_NphcerEnergy->SetXTitle("log10(Energy) (MeV)");
    prof_NphcerEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NphcerEnergy->GetNbinsX(); m++) {
        if(prof_NphcerEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_NphcerEnergy->GetBinLowEdge(m+1);
	if(prof_NphcerEnergy->GetBinContent(prof_NphcerEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_NphcerEnergy->GetBinLowEdge(prof_NphcerEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2_Energy_cer",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_NphcerEnergy->Fit("mypol_Nph2_Energy_cer","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nckov_{1/2} = %.3g  *  (E/MeV)",pow(10,par[0]));
    t->DrawLatex(textpos,prof_NphcerEnergy->GetMaximum()*0.54,txt);


    pad3->cd();
    treepainter->DrawHistos("Nmax_Energy_fluo",drawcut);
    prof_NmaxfluoEnergy = (TProfile*)gPad->GetPrimitive("h_Nmax_Energy_fluo");
    prof_NmaxfluoEnergy->SetTitle("Fluorescence maximum vs/ Energy -- cos#theta normalized");
    prof_NmaxfluoEnergy->SetYTitle("(log10) Fluo max");
    prof_NmaxfluoEnergy->SetXTitle("log10(Energy) (MeV)");
    prof_NmaxfluoEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NmaxfluoEnergy->GetNbinsX(); m++) {
        if(prof_NmaxfluoEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_NmaxfluoEnergy->GetBinLowEdge(m+1);
	if(prof_NmaxfluoEnergy->GetBinContent(prof_NmaxfluoEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_NmaxfluoEnergy->GetBinLowEdge(prof_NmaxfluoEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nmax_Energy_fluo",my_linear,min,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_NmaxfluoEnergy->Fit("mypol_Nmax_Energy_fluo","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{max} = %.3g  *  (E/MeV) cm^{2}/g",pow(10,par[0]));
    t->DrawLatex(textpos,prof_NmaxfluoEnergy->GetMaximum()*0.54,txt);

    /*
    pad4->cd();
    treepainter->DrawHistos("Nmax_Energy_cer",drawcut);
    prof_NmaxcerEnergy = (TProfile*)gPad->GetPrimitive("h_Nmax_Energy_cer");
    prof_NmaxcerEnergy->SetTitle("Cerenkov maximum vs/ Energy");
    prof_NmaxcerEnergy->SetYTitle("(log10) Cerenkov max");
    prof_NmaxcerEnergy->SetXTitle("log10(Energy) (MeV)");
    prof_NmaxcerEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NmaxcerEnergy->GetNbinsX(); m++) {
        if(prof_NmaxcerEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_NmaxcerEnergy->GetBinLowEdge(m+1);
	if(prof_NmaxcerEnergy->GetBinContent(prof_NmaxcerEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_NmaxcerEnergy->GetBinLowEdge(prof_NmaxcerEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nmax_Energy_cer",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-10.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_NmaxcerEnergy->Fit("mypol_Nmax_Energy_cer","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    //sprintf(txt,"Nckov_{max} = %.3g  *  (E/MeV)^{%.3g} cm^{2}/g",pow(10,par[0]),par[1]);
    sprintf(txt,"Nckov_{max} = %.3g  *  (E/MeV) cm^{2}/g",pow(10,par[0]));
    t->DrawLatex(textpos,prof_NmaxcerEnergy->GetMaximum()*0.54,txt);
    */

    

    // *************** PLOTS FOR SIGMA  *********************************************
    
    c2->Clear();
    c2->cd();
    TPad *pad12 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad22 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad32 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad42 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad12->Draw();
    pad22->Draw();
    pad32->Draw();
    pad42->Draw();
    TProfile* prof = 0;

    pad12->cd();
    treepainter->DrawHistos("Nph2_Energy_fluo_linear",drawcut);
    prof = (TProfile*) gDirectory->Get("h_Nph2_Energy_fluo_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNphfluoEnergy = new TProfile("resolNphfluoEnergy","#splitline{Normalized sigma of Fluo first HALF}{VS/ Energy}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++)
        if(prof->GetBinContent(i+1)) resolNphfluoEnergy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    resolNphfluoEnergy->SetMarkerStyle(27);
    resolNphfluoEnergy->SetXTitle("log10(Energy) (MeV)");
    resolNphfluoEnergy->Draw();
    gPad->SetTopMargin(0.18);
    
    pad22->cd();
    treepainter->DrawHistos("Nph2_Energy_cer_linear",drawcut);
    prof = (TProfile*) gDirectory->Get("h_Nph2_Energy_cer_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNphcerEnergy = new TProfile("resolNphcerEnergy","#splitline{Normalized sigma of Cerenkov first HALF}{VS/ Energy}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNphcerEnergy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNphcerEnergy->SetMarkerStyle(27);
    resolNphcerEnergy->SetXTitle("log10(Energy) (MeV)");
    resolNphcerEnergy->Draw();
    gPad->SetTopMargin(0.18);

    pad32->cd();
    treepainter->DrawHistos("Nmax_Energy_fluo_linear",drawcut);
    prof = (TProfile*) gDirectory->Get("h_Nmax_Energy_fluo_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNmaxfluoEnergy = new TProfile("resolNmaxfluoEnergy","Normalized sigma of Fluo max VS Energy",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNmaxfluoEnergy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNmaxfluoEnergy->SetMarkerStyle(27);
    resolNmaxfluoEnergy->SetXTitle("log10(Energy) (MeV)");
    resolNmaxfluoEnergy->Draw();
      
    /*
    pad42->cd();
    treepainter->DrawHistos("Nmax_Energy_cer_linear",drawcut);
    prof = (TProfile*) gDirectory->Get("h_Nmax_Energy_cer_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNmaxcerEnergy = new TProfile("resolNmaxcerEnergy","Normalized sigma of Cerenkov max VS Energy",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNmaxcerEnergy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNmaxcerEnergy->SetMarkerStyle(27);
    resolNmaxcerEnergy->SetXTitle("log10(Energy) (MeV)");
    resolNmaxcerEnergy->Draw();
    */
}

//__________________________________________________________________________________________________________________________-
void treepupilhistos() {
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
        delete c; c= 0;
	Warning("treepupilhistos()","No Tree found");
	return;
    }
    if(!pad_p_on_pupil) pad_p_on_pupil = new EGViewer();
    
    TCanvas *c = pad_p_on_pupil->FindCanvas("c_treepupil_histo");
    if(!c ) c = pad_p_on_pupil->AddTab("c_treepupil_histo","c_treepupil_histo");
    TCanvas *c2 = pad_p_on_pupil->FindCanvas("c_treepupil_histo2");
    if(!c2 ) c2 = pad_p_on_pupil->AddTab("c_treepupil_histo2","c_treepupil_histo2");

    c->Clear();
    c->cd();
    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.67,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.67,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.34,0.49,0.66);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.34,0.99,0.66);
    TPad *pad5 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.33);
    TPad *pad6 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.33);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    pad5->Draw();
    pad6->Draw();


    pad1->cd();
    pad1->SetLogy(1);
    treepainter->DrawHistos("Nph_CB_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_CB_P");
    histo->SetTitle("Backscattered cerenkov on pupil");
    histo->SetXTitle("Nph_CB_P");
    pad2->cd();
    pad2->SetLogy(1);
    treepainter->DrawHistos("Nph_CR_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_CR_P");
    histo->SetTitle("Reflected cerenkov on pupil");
    histo->SetXTitle("Nph_CR_P");
    pad3->cd();
    pad3->SetLogy(1);
    treepainter->DrawHistos("Nph_Fluo_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Fluo_P");
    histo->SetTitle("Fluorescence on pupil");
    histo->SetXTitle("Nph_Fluo_P");
    pad4->cd();
    pad4->SetLogy(1);
    treepainter->DrawHistos("Nph_tot_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_tot_P");
    histo->SetTitle("Total nb of photons on pupil");
    histo->SetXTitle("Nph_tot_P");
    pad5->cd();
    pad5->SetLogy(1);
    treepainter->DrawHistos("Nmax_Fluo_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nmax_Fluo_P");
    histo->SetTitle("Fluorescence maximum on pupil");
    histo->SetXTitle("Nmax_Fluo_P");
    pad6->cd();
    pad6->SetLogy(1);
    treepainter->DrawHistos("Sig_Fluo_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Sig_Fluo_P");
    histo->SetTitle("Half Width of fluo time profile on pupil");
    histo->SetXTitle("Half width (GTU)");


//////////////////////////
    c2->Clear();
    c2->cd();
    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Wlmean_f_p",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_f_p");
    histo->SetTitle("Mean wavelength for fluo on pupil");
    histo->SetXTitle("Wlmean_f_p");
    pad21->cd();
    treepainter->DrawHistos("Wlmean_cb_p",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_cb_p");
    histo->SetTitle("Mean wavelength for backscattered cerenkov on pupil");
    histo->SetXTitle("Wlmean_cb_p");
    pad31->cd();
    treepainter->DrawHistos("Wlmean_cr_p",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_cr_p");
    histo->SetTitle("Mean wavelength for reflected cerenkov on pupil");
    histo->SetXTitle("Wlmean_cr_p");
    pad41->cd();
    treepainter->DrawHistos("Wlmean_tot_p",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Wlmean_tot_p");
    histo->SetTitle("Mean wavelength of photons on pupil");
    histo->SetXTitle("Wlmean_tot_p");
}

//__________________________________________________________________________________________________________________________-
void pupil_theta_correl() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treepupilEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *c2 = pad_p_on_pupil->FindCanvas("c_treepupil_correltheta");
    if(!c2 ) c2 = pad_p_on_pupil->AddTab("c_treepupil_correltheta","c_treepupil_correltheta");    
    
    c2->Clear();
    c2->cd();

    TPad *pad11 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad21 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad31 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad41 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad11->Draw();
    pad21->Draw();
    pad31->Draw();
    pad41->Draw();

    pad11->cd();
    treepainter->DrawHistos("Nph_Theta_CB_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Theta_CB_P");
    histo->SetTitle("#splitline{Backscattered Cerenkov on pupil}{VS/ #theta -- energy normalized}");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Backscattered Cerenkov on pupil");
    histo->SetStats(0);
    gPad->SetTopMargin(0.2);
    
    pad21->cd();
    treepainter->DrawHistos("Nph_Theta_CR_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Theta_CR_P");
    histo->SetTitle("#splitline{Reflected Cerenkov on pupil}{VS/ #theta -- energy normalized}");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Reflected Cerenkov on pupil");
    histo->SetStats(0);
    gPad->SetTopMargin(0.2);
    
    pad31->cd();
    treepainter->DrawHistos("Nph_Theta_Fluo2_P",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Theta_Fluo2_P");
    histo->SetTitle("HALF Fluo on pupil vs/ #theta -- energy normalized");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("HALF Fluorescence on pupil");
    histo->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_Nph_Theta_Fluo2_P",my_overcos20,min,max,1);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameters(1.e-11,0);
    mypol->SetParLimits(0,0.1e-11,100.e-11);
    textpos = min + 0.1*(max-min);
    histo->Fit("mypol_Nph_Theta_Fluo2_P","WROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.3g EeV^{-1}/ cos(#theta)",par[0]*1e12);
    t->DrawLatex(textpos,histo->GetMaximum()*0.74,txt);
    */
    gPad->SetTopMargin(0.15);
    
    pad41->cd();
    treepainter->DrawHistos("Nph_airscat_effect",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_airscat_effect");
    histo->SetTitle("Increase of signal due to air scattered ckov");
    histo->SetYTitle("total / (fluo+reflected ckov)");
    histo->SetXTitle("#theta (deg)");
    
/////////////////////////////////////    
    TCanvas *c = pad_p_on_pupil->FindCanvas("c_treepupil_correltheta2");
    if(!c ) c = pad_p_on_pupil->AddTab("c_treepupil_correltheta2","c_treepupil_correltheta2");    
    
    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    
    pad1->cd();
    treepainter->DrawHistos("Nph_Theta_Fluo_P_crash",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Nph_Theta_Fluo_P_crash");
    histo->SetTitle("Crash effect on fluo on pupil");
    histo->SetYTitle("Half/full FLuo on pupil");
    histo->SetXTitle("#theta (deg)");
    
    pad2->cd();
    treepainter->DrawHistos("TimeWidth2_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_TimeWidth2_Theta");
    histo->SetTitle("HALF width of fluo time profile on pupil");
    histo->SetYTitle("Half time width FLuo on pupil (nb of GTU)");
    histo->SetXTitle("#theta (deg)");
    min = 0;
    max = 0;
    for(Int_t m=0; m<histo->GetNbinsX(); m++) {
        if(histo->GetBinContent(m+1) && fabs(min) == 0) min = histo->GetBinLowEdge(m+1);
	if(histo->GetBinContent(histo->GetNbinsX() - m) && fabs(max) == 0) max = histo->GetBinLowEdge(histo->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    /*
    mypol = new TF1("mypol_TimeWidth2_Theta",my_overcos20,min,max,1);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameters(1.,0);
    mypol->SetParLimits(0,0.1,100.);
    textpos = min + 0.1*(max-min);
    histo->Fit("mypol_TimeWidth2_Theta","WROQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"%.3g / cos(#theta)  GTU",par[0]);
    t->DrawLatex(textpos,histo->GetMaximum()*0.74,txt);
    */
    gPad->SetTopMargin(0.15);
}

//__________________________________________________________________________________________________________________________-
void pupil_energy_correl() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treepupilEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *c = pad_p_on_pupil->FindCanvas("c_treepupil_correlenergy");
    if(!c ) c = pad_p_on_pupil->AddTab("c_treepupil_correlenergy","c_treepupil_correlenergy");
    TCanvas *cc1 = pad_p_on_pupil->FindCanvas("c_treepupil_correlenergy_F");
    if(!cc1 ) cc1 = pad_p_on_pupil->AddTab("c_treepupil_correlenergy_F","c_treepupil_correlenergy_F");

    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
       
    pad1->cd();
    treepainter->DrawHistos("Nph2_Energy_Fluo_P",drawcut);
    prof_Nph2fluoEnergy = (TProfile*)gPad->GetPrimitive("h_Nph2_Energy_Fluo_P");
    prof_Nph2fluoEnergy->SetTitle("(log10) HALF fluo on pupil vs/ Energy -- cos(#theta) normalization");
    prof_Nph2fluoEnergy->SetXTitle("Energy (MeV)");
    prof_Nph2fluoEnergy->SetYTitle("HALF fluorescence on pupil");
    prof_Nph2fluoEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nph2fluoEnergy->GetNbinsX(); m++) {
        if(prof_Nph2fluoEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nph2fluoEnergy->GetBinLowEdge(m+1);
	if(prof_Nph2fluoEnergy->GetBinContent(prof_Nph2fluoEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nph2fluoEnergy->GetBinLowEdge(prof_Nph2fluoEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph2_Energy_Fluo_P",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-20.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nph2fluoEnergy->Fit("mypol_Nph2_Energy_Fluo_P","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{1/2} = %.2g  *  (E/EeV)",pow(10,par[0])*1e12);
    t->DrawLatex(textpos,prof_Nph2fluoEnergy->GetMaximum()*0.44,txt);
    
    pad2->cd();
    treepainter->DrawHistos("Nph_Energy_CB_P",drawcut);
    prof_NphbackEnergy = (TProfile*)gPad->GetPrimitive("h_Nph_Energy_CB_P");
    prof_NphbackEnergy->SetTitle("(log10) Backscattered Cerenkov on pupil vs/ Energy -- cos(#theta) normalized");
    prof_NphbackEnergy->SetXTitle("Energy (MeV)");
    prof_NphbackEnergy->SetYTitle("Backscattered cerenkov on pupil");
    prof_NphbackEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NphbackEnergy->GetNbinsX(); m++) {
        if(prof_NphbackEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_NphbackEnergy->GetBinLowEdge(m+1);
	if(prof_NphbackEnergy->GetBinContent(prof_NphbackEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_NphbackEnergy->GetBinLowEdge(prof_NphbackEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph_Energy_CB_P",my_linear,min,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-20.,10);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_NphbackEnergy->Fit("mypol_Nph_Energy_CB_P","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nckov_back = %.2g  *  (E/EeV)",pow(10,par[0])*1e12);
    t->DrawLatex(textpos,prof_NphbackEnergy->GetMaximum()*0.44,txt);

    pad3->cd();
    treepainter->DrawHistos("Nmax_Energy_Fluo_P",drawcut);
    prof_Nmaxfluo_pupil_Energy = (TProfile*)gPad->GetPrimitive("h_Nmax_Energy_Fluo_P");
    prof_Nmaxfluo_pupil_Energy->SetTitle("(log10) Fluo max on pupil vs/ Energy -- cos(#theta) normalized");
    prof_Nmaxfluo_pupil_Energy->SetXTitle("Energy (MeV)");
    prof_Nmaxfluo_pupil_Energy->SetYTitle("Fluo max on pupil");
    prof_Nmaxfluo_pupil_Energy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_Nmaxfluo_pupil_Energy->GetNbinsX(); m++) {
        if(prof_Nmaxfluo_pupil_Energy->GetBinContent(m+1) && fabs(min) == 0) min = prof_Nmaxfluo_pupil_Energy->GetBinLowEdge(m+1);
	if(prof_Nmaxfluo_pupil_Energy->GetBinContent(prof_Nmaxfluo_pupil_Energy->GetNbinsX() - m) && fabs(max) == 0) max = prof_Nmaxfluo_pupil_Energy->GetBinLowEdge(prof_Nmaxfluo_pupil_Energy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nmax_Energy_Fluo_P",my_linear,min,max,2);
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-20.,10.);
    mypol->SetParLimits(1,1.,1.);
    textpos = min + 0.1*(max-min);
    prof_Nmaxfluo_pupil_Energy->Fit("mypol_Nmax_Energy_Fluo_P","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nfluo_{max} = %.2g  *  (E/EeV) GTU^{-1}",pow(10,par[0])*1e12);
    t->DrawLatex(textpos,prof_Nmaxfluo_pupil_Energy->GetMaximum()*0.30,txt);
    
    pad4->cd();
    treepainter->DrawHistos("Nph_Energy_CR_P",drawcut);
    prof_NphrefEnergy = (TProfile*)gPad->GetPrimitive("h_Nph_Energy_CR_P");
    prof_NphrefEnergy->SetYTitle("(log10) Reflected cerenkov on pupil");
    prof_NphrefEnergy->SetXTitle("Energy (MeV)");
    prof_NphrefEnergy->SetTitle("Reflected Cerenkov on pupil vs/ Energy");
    prof_NphrefEnergy->SetStats(0);
    min = 0;
    max = 0;
    for(Int_t m=0; m<prof_NphrefEnergy->GetNbinsX(); m++) {
        if(prof_NphrefEnergy->GetBinContent(m+1) && fabs(min) == 0) min = prof_NphrefEnergy->GetBinLowEdge(m+1);
	if(prof_NphrefEnergy->GetBinContent(prof_NphrefEnergy->GetNbinsX() - m) && fabs(max) == 0) max = prof_NphrefEnergy->GetBinLowEdge(prof_NphrefEnergy->GetNbinsX() - m);
	if(fabs(min*max) > 0) break;
    }
    mypol = new TF1("mypol_Nph_Energy_CR_P",my_linear,min,max,2);  // pente fixee a 1
    mypol->SetLineWidth(1);
    mypol->SetLineColor(2);
    mypol->SetParameter(1.,1.);
    mypol->SetParLimits(0,-20.,10.);
    mypol->SetParLimits(1,1.,1.);

    textpos = min + 0.07*(max-min);
    prof_NphrefEnergy->Fit("mypol_Nph_Energy_CR_P","WRCOQ");
    mypol->GetParameters(par);
    t = new TLatex();
    char txt[200];
    t->SetTextFont(62);  
    t->SetTextSize(0.06);
    t->SetTextColor(2);
    t->SetTextColor(1);
    sprintf(txt,"Nckov_reflected = %.2g  *  (E/EeV)",pow(10,par[0])*1e12);
    t->DrawLatex(textpos,prof_NphrefEnergy->GetMaximum()*0.34,txt);
    
    
    // *************** PLOTS FOR SIGMA  ********************************************************
    
    cc1->Clear();
    cc1->cd();
    TPad *pad111 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad112 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad113 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad114 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad111->Draw();
    pad112->Draw();
    pad113->Draw();
    pad114->Draw();
    TProfile* prof = 0;
    

    pad111->cd();
    treepainter->DrawHistos("Nph2_Energy_Fluo_P_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph2_Energy_Fluo_P_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNph2_pupil_fluoEnergy = new TProfile("resolNph2_pupil_fluoEnergy","#splitline{Normalized #sigma of Fluo first HALF}{VS/ Energy  (on pupil) -- cos(#theta) normalized}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++)
        if(prof->GetBinContent(i+1)) resolNph2_pupil_fluoEnergy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    resolNph2_pupil_fluoEnergy->SetMarkerStyle(27);
    resolNph2_pupil_fluoEnergy->SetXTitle("log10(Energy) (MeV)");
    resolNph2_pupil_fluoEnergy->Draw();
    gPad->SetTopMargin(0.18);
    
    pad112->cd();
    treepainter->DrawHistos("Nph_Energy_CB_P_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph_Energy_CB_P_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resol_Nphcerback_Energy = new TProfile("resol_Nphcerback_Energy","#splitline{Normalized #sigma of backscattered ckov}{VS/ Energy   (on pupil) -- cos(#theta) normalized}",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resol_Nphcerback_Energy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resol_Nphcerback_Energy->SetMarkerStyle(27);
    resol_Nphcerback_Energy->SetXTitle("log10(Energy) (MeV)");
    resol_Nphcerback_Energy->Draw();
    gPad->SetTopMargin(0.18);

    pad113->cd();
    treepainter->DrawHistos("Nmax_Energy_Fluo_P_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nmax_Energy_Fluo_P_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resolNmaxfluo_pupil_Energy = new TProfile("resolNmaxfluo_pupil_Energy","Normalized #sigma of Fluo max on pupil VS Energy",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resolNmaxfluo_pupil_Energy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resolNmaxfluo_pupil_Energy->SetMarkerStyle(27);
    resolNmaxfluo_pupil_Energy->SetXTitle("log10(Energy) (MeV)");
    resolNmaxfluo_pupil_Energy->Draw();
      
    pad114->cd();
    treepainter->DrawHistos("Nph_Energy_CR_P_linear",drawcut);
    prof = (TProfile*)gDirectory->Get("h_Nph_Energy_CR_P_linear");
    nbin = prof->GetNbinsX();
    min = prof->GetBinLowEdge(1);
    max =  prof->GetBinLowEdge(nbin) + prof->GetBinWidth(nbin);
    resol_Nohcerreflec_Energy = new TProfile("resol_Nohcerreflec_Energy","Normalized #sigma of reflected Ckov VS Energy  (on pupil)",nbin,min,max,1e-6,1e6);
    for(Int_t i=0; i<nbin; i++) {
        if(prof->GetBinContent(i+1)) resol_Nohcerreflec_Energy->Fill(prof->GetBinCenter(i+1),prof->GetBinError(i+1)/prof->GetBinContent(i+1));
    }
    resol_Nohcerreflec_Energy->SetMarkerStyle(27);
    resol_Nohcerreflec_Energy->SetXTitle("log10(Energy) (MeV)");
    resol_Nohcerreflec_Energy->Draw();
}

//__________________________________________________________________________________________________________________________-
void transmission() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treepupilEnergycorrel()","No Tree found");
	return;
    }
    
    TCanvas *c3 = pad_p_on_pupil->FindCanvas("c_treepupil_correl_trans");
    if(!c3 ) c3 = pad_p_on_pupil->AddTab("c_treepupil_correl_trans","c_treepupil_correl_trans");

    c3->Clear();
    c3->cd();
    TPad *pad12 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad22 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad32 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad42 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad12->Draw();
    pad22->Draw();
    pad32->Draw();
    pad42->Draw();
    
    pad12->cd();
    treepainter->DrawHistos("Trans_max_mean",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Trans_max_mean");
    histo->SetTitle("Relation between T_{mean} and T_{fluomax} for FLUO");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("(T_{fluomax} - T_{mean})/T_{mean}");
    histo->SetStats(0);
   
    pad22->cd();
    treepainter->DrawHistos("Trans_GTUmax_max",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Trans_GTUmax_max");
    histo->SetTitle("Relative difference between T_{maxfluo} and T_{GTUmax} for FLUO");
    histo->SetXTitle("(T_{GTUmax} - T_{fluomax})/T_{fluomax}");
    
    pad32->cd();
    treepainter->DrawHistos("Trans_ckov_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Trans_ckov_Theta");
    histo->SetTitle("Cerenkov transmission");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio   on_pupil / created");
    histo->SetStats(0);
    
    pad42->cd();
    treepainter->DrawHistos("Trans_fluo_Theta",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Trans_fluo_Theta");
    histo->SetTitle("Fluo transmission");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("Ratio   on_pupil / created");
    histo->SetStats(0);
}

//__________________________________________________________________________________________________________________________-
void Miscellaneous() {
    Double_t min = 0;
    Double_t max = 0;
    Double_t textpos = 0;
    Double_t par[3];
    TLatex *t = 0;
    Int_t nbin = 0;
    TF1* fitfunc = 0;
    Bool_t flag = true;
    if(!chain&&!etree&&!stree) flag = checktree();	
    if(!flag) {
	Warning("treepupilEnergycorrel()","No Tree found");
	return;
    }

    if(!pad_p_on_pupil) pad_p_on_pupil = new EGViewer();
    
    TCanvas *c = pad_p_on_pupil->FindCanvas("c_treepupil_miscel");
    if(!c ) c = pad_p_on_pupil->AddTab("c_treepupil_miscel","c_treepupil_miscel");


/////////////////////////
    c->Clear();
    c->cd();

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    
    pad1->cd();
    treepainter->DrawHistos("TimeWidth2_Hmax",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_TimeWidth2_Hmax");
    histo->SetTitle("Time HALF width on pupil vs/ height of shower maximum");
    histo->SetXTitle("Height of fluo max (km)");
    histo->SetYTitle("Fluo time half width on pupil (GTU)");
    histo->SetStats(0);
   
    pad2->cd();
    treepainter->DrawHistos("Yield_max_mean",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Yield_max_mean");
    histo->SetTitle("Difference between FLUO Yield_{mean} and Yield_{atmax}");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("(Y_{fluomax} - Y_{mean}) / Y_{mean}");
    histo->SetStats(0);
   
    pad3->cd();
    treepainter->DrawHistos("Yield_GTUmax_max",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Yield_GTUmax_max");
    histo->SetTitle("Difference between FLUO Yield_{GTUmax} and Yield_{atmax}");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("(Y_{GTUmax} - Y_{atmax}) / Y_{atmax}");
    histo->SetStats(0);
   
    pad4->cd();
    treepainter->DrawHistos("Xmax_shift",drawcut);
    histo = (TH1*)gPad->GetPrimitive("h_Xmax_shift");
    histo->SetTitle("Xmax shift due to scattered cerenkov");
    histo->SetXTitle("#theta (deg)");
    histo->SetYTitle("(Xscat_{GTUmax} - X_{GTUmax}) / X_{GTUmax}");
}

//__________________________________________________________________________________________________________________________-
void simpletreeviewer() {    
    stree=(TTree*)gDirectory->Get("stree"); 
    if(!stree) {
	Warning("simpletreeviewer()","No stree found");
        Printf("I do not find simple results tree.");
        return;
    }
    stree->StartViewer();
}

//__________________________________________________________________________________________________________________________-
void savesimpletree() {
    if(!stree) {
      Printf("Cannot save stree, it has not been generated");
      return;
    }

    TFile *f = 0;
    delete f;
    Bool_t wantosave = true;
    
    TGFileInfo* info = new TGFileInfo();
    while ( info->fFilename == 0 || !f || f->IsZombie() ) {
        TGFileDialog* filedialog = new TGFileDialog(gClient->GetRoot(), 0, kFDSave, info);
        if( info->fFilename && !SAVED ) f = new TFile( info->fFilename,"create");
	else {wantosave = false; break;}
    }
    if(wantosave && !SAVED) stree->Write();
    else if(SAVED) Info("savesimpletree()","stree is already saved");
    SAVED = true;
    
    stree->Write();
    f->Close();
}

//__________________________________________________________________________________________________________________________-
Double_t my_linear(Double_t *x, Double_t *par) {
    return par[0] + par[1]*x[0];
}

//__________________________________________________________________________________________________________________________-
Double_t my_expon12(Double_t *x, Double_t *par) {
    if(x[0] > 12. || x[0]<0.2) {
       TF1::RejectPoint();
    }
    return exp(par[0] + par[1]*x[0]);
}

//__________________________________________________________________________________________________________________________-
Double_t my_expon15(Double_t *x, Double_t *par) {
    if(x[0] > 15. || x[0]<0.2) {
       TF1::RejectPoint();
    }
    return exp(par[0] + par[1]*x[0]);
}

//__________________________________________________________________________________________________________________________-
Double_t my_logcos(Double_t *x, Double_t *par) {
    if(cos(x[0]*TMath::Pi()/180.) <= 0) {
       TF1::RejectPoint();
       return 0;
    }
    if(x[0] > 60 || x[0]<5) {
       TF1::RejectPoint();
    }
    return par[0] + par[1]*log(cos(x[0]*TMath::Pi()/180.));
}

//__________________________________________________________________________________________________________________________-
Double_t my_overcos60(Double_t *x, Double_t *par) {
    if(cos(x[0]*TMath::Pi()/180.) <= 0.) {
       TF1::RejectPoint();
       return 0;
    }
    if(x[0] > 60) {
       TF1::RejectPoint();
    }
    return par[0] / cos(x[0]*TMath::Pi()/180.);
}

//__________________________________________________________________________________________________________________________-
Double_t my_overcos20(Double_t *x, Double_t *par) {
    if(cos(x[0]*TMath::Pi()/180.) <= 0.) {
       TF1::RejectPoint();
       return 0;
    }
    if(x[0] > 20) {
       TF1::RejectPoint();
    }
    return par[0] / cos(x[0]*TMath::Pi()/180.);
}

