#include "TStyle.h"
void setwhitestyle() {
    gStyle->SetCanvasColor(0);
    gStyle->SetHistLineWidth(2);
    gStyle->SetPalette(1);
//    gStyle->SetFillColor(kWhite);
    gStyle->SetFrameFillColor(kWhite);
    gStyle->SetPadColor(kWhite);
    gStyle->SetStatColor(kWhite);
    gStyle->SetTitleFillColor(kWhite);

//    gStyle->SetPaperSize(26,20);
    gStyle->SetOptStat("uo");

    gStyle->SetStatX(gStyle->GetStatW()+0.12);
    gStyle->SetStatY(gStyle->GetStatH()+0.25);

    gStyle->SetPalette(51);
}

const int n_angles=7;
const int n_lambdas=3;

const char* esaf="esaf";
const char* geant="g4";

const char* fileformat="output/simu.%s.a%i.l%i.n50000.root";

int angles[n_angles]={0,5,10,15,20,25,30};
int lambdas[n_lambdas]={323,357,391};

int theta_y_limits[n_angles][2]= {  {-10,10},  //0
                                    {190,210},   //5
                                    {395,396},   //10
                                    {580,595},   //15
                                    //{815,830},   //20
                                    {765,880},   //20
                                    {940,970},   //25
                                    {1115,1135}  //30
   	              		        };

int theta_x_limits[n_angles][2]=  {  {-5,5},  //0
                                     {-5,5},  //5
                                     {-1,1},  //10
                                     {-5,5},  //15
                                     {-5,5},  //20
                                     {-5,5},  //25
                                     {-5,5}   //30
   	              	 	          };

TTree* GetTree(int angle, int lambda, const char* str){
    TFile* f  = new TFile(Form(fileformat,str,angle,lambda));
    if ( !f || f->IsZombie() ) {
        cerr<<"error 1"<<endl;
        exit(0);
    }

    TTree* tr=(TTree*)f->Get("etree");
    if ( !tr ){
        cerr<<"error 2"<<endl;
        exit(0);
    }
    return tr;
}

void draw_on_pad(TTree* tree, TVirtualPad* pad, TH2D* hist, TString options="") {

    hist->SetMarkerSize(2);
    hist->SetMarkerStyle(7);

    tree->Draw(Form("detector.fPhotons.fIdealFocalPosY:detector.fPhotons.fIdealFocalPosX>>%s",hist->GetName()),"detector.fPhotons.fStatusBits&0x1","goff");
    hist->GetXaxis()->SetTitle("x, mm");
    hist->GetYaxis()->SetTitle("y, mm");

	hist->Draw(options+"COLZ");
}

TH2D* make_histo(int angle, int lambda,const char* str){
    int nx=200;
    int ny=200;
    double y1=(double)theta_y_limits[angle/5][0];
    double y2=(double)theta_y_limits[angle/5][1];
    double x1=(double)theta_x_limits[angle/5][0];
    double x2=(double)theta_x_limits[angle/5][1];
    TH2D* h1=new TH2D(Form("h1_%s_%i_%i",str,angle,lambda),Form("%s, #theta=%i, #lambda=%i nm",str,angle,lambda),nx,x1,x2,ny,y1,y2);
    TAxis* ax=h1->GetZaxis();
    ax->SetLabelOffset(-0.1);

    return h1;
}

void DrawFSPointsForAllAngles(const char* str){
    setwhitestyle();
    for ( int i(0); i<n_angles;i++) DrawFSPointsForAngle(angles[i],str);
}

void DrawFSPointsForAll(){
    setwhitestyle();
    for ( int i(0); i<n_angles;i++) {
        DrawFSPointsForAngleForAll(angles[i]);
    }
}

void DrawFSPointsForAngleForAll(int angle){
	setwhitestyle();
    TCanvas* c  = new TCanvas();
    c->Divide(n_lambdas,2);
    TTree* tr=0;
    TH2D* h=0;

    for( int i(0); i<n_lambdas; i++ ){
        tr=GetTree(angle,lambdas[i],geant);
        h=make_histo(angle,lambdas[i],geant);
        draw_on_pad(tr,c->cd(i+1),h);
    }
    for( int i(0); i<n_lambdas; i++ ){
        tr=GetTree(angle,lambdas[i],esaf);
        h=make_histo(angle,lambdas[i],esaf);
        draw_on_pad(tr,c->cd(i+4),h);
    }
	c->SetName(Form("output/fs.all.a%i.lall",angle));
    c->Print(Form("output/fs.all.a%i.lall.pdf",angle));
}

void DrawFSPointsForAngle(int angle,const char* str){
    TCanvas* c  = new TCanvas();
    c->Divide(n_lambdas,1);
    TTree* tr=0;
    TH2D* h=0;
    for( int i(0); i<n_lambdas; i++ ){
        tr=GetTree(angle,lambdas[i],str);
        h=make_histo(angle,lambdas[i],str);
        draw_on_pad(tr,c->cd(i+1),h);
    }
    c->SetName(Form("output/fs.%s.a%i.lall.pdf",str,angle));
    c->Print(Form("output/fs.%s.a%i.lall.pdf",str,angle));
}

void DrawFSpointsOne(const char* filename1, double x1=-5,double x2=5,int n=200,double y1=-5, double y2=5, TVirtualPad* pad=0) {
    setwhitestyle();
    TFile* f1=new TFile(filename1);

    if ( !f1 || f1->IsZombie() ) {
        cerr<<"error 1"<<endl;
        return;
    }

    TTree* tr1=(TTree*)f1->Get("etree");
    if ( !tr1 ) {
        cerr<<"error 2"<<endl;
        return;
    }

    TH2D* h1=new TH2D("h1","",n,x1,x2,n,y1,y2);

    if ( !pad ) {
        TCanvas* c = new TCanvas();
        draw_on_pad(tr1,c,h1);
        c->cd(0);
        c->Print("test.pdf");
    }
    else draw_on_pad(tr1,pad,h1,"same");
}


void DrawFSpoints(const char* filename1, const char* filename2, int angle, int l) {
    setwhitestyle();
    TFile* f1=new TFile(filename1);
    TFile* f2=new TFile(filename2);

    if ( !f1 || !f2 || f1->IsZombie() || f2->IsZombie() ) {
        cerr<<"error 1"<<endl;
        return;
    }

    TTree* tr1=(TTree*)f1->Get("etree");
    TTree* tr2=(TTree*)f2->Get("etree");
    if ( !tr1 || !tr2 ) {
        cerr<<"error 2"<<endl;
        return;
    }

    int nx=200;
    int ny=200;
    double y1=(double)theta_y_limits[angle/5][0];
    double y2=(double)theta_y_limits[angle/5][1];
    double x1=(double)theta_x_limits[angle/5][0];
    double x2=(double)theta_x_limits[angle/5][1];
    TH2D* h1=new TH2D("h1","ESAF+Geant4",nx,x1,x2,ny,y1,y2);
    TH2D* h2=new TH2D("h2","ESAF (standard)",nx,x1,x2,ny,y1,y2);

    TCanvas* c = new TCanvas();
    c->Divide(2,1);
    draw_on_pad(tr1,new TCanvas(),h1);
    draw_on_pad(tr2,new TCanvas(),h2);
    draw_on_pad(tr1,c->cd(1),h1);
    draw_on_pad(tr2,c->cd(2),h2);
    c->cd(0);

    c->Print("test.pdf");
}

void DrawFSpoints3(const char* filename1, const char* filename2, const char* filename3, int angle, int l) {
    setwhitestyle();
    TFile* f1=new TFile(filename1);
    TFile* f2=new TFile(filename2);
    TFile* f3=new TFile(filename3);

    if ( !f1 || !f2 || !f3 || f1->IsZombie() || f2->IsZombie() || f3->IsZombie() ) {
        cerr<<"error 1"<<endl;
        return;
    }

    TTree* tr1=(TTree*)f1->Get("etree");
    TTree* tr2=(TTree*)f2->Get("etree");
    TTree* tr3=(TTree*)f3->Get("etree");
    if ( !tr1 || !tr2 || !tr3 ) {
        cerr<<"error 2"<<endl;
        return;
    }

    int nx=200;
    int ny=200;
    double y1=(double)theta_y_limits[angle/5][0];
    double y2=(double)theta_y_limits[angle/5][1];
    double x1=(double)theta_x_limits[angle/5][0];
    double x2=(double)theta_x_limits[angle/5][1];
    TH2D* h1=new TH2D("G4_CSG","ESAF+Geant4 (CSG solids)",nx,x1,x2,ny,y1,y2);
    TH2D* h2=new TH2D("G4_FS","ESAF+Geant4 (new approach)",nx,x1,x2,ny,y1,y2);
    TH2D* h3=new TH2D("ESAF","ESAF (standard)",nx,x1,x2,ny,y1,y2);

    TCanvas* c = new TCanvas();
    c->Divide(3,1);
    draw_on_pad(tr1,c->cd(1),h1);
    draw_on_pad(tr2,c->cd(2),h2);
    draw_on_pad(tr2,c->cd(3),h3);
    c->cd(0);

    draw_on_pad(tr1,new TCanvas(),h1);
    draw_on_pad(tr2,new TCanvas(),h2);
    draw_on_pad(tr3,new TCanvas(),h3);

    c->Print("test.pdf");
}

void DrawFSpoints1(const char* filename1, const char* filename2,const char* filename3, const char* filename4, int angle, int l) {
    TFile* f1=new TFile(filename1);
    TFile* f2=new TFile(filename2);
    TFile* f3=new TFile(filename3);
    TFile* f4=new TFile(filename4);

    if ( !f1 || !f2 || f1->IsZombie() || f2->IsZombie() ) {
        cerr<<"error 1"<<endl;
        return;
    }

    TTree* tr1=(TTree*)f1->Get("etree");
    TTree* tr2=(TTree*)f2->Get("etree");
    TTree* tr3=(TTree*)f3->Get("etree");
    TTree* tr4=(TTree*)f4->Get("etree");
    if ( !tr1 || !tr2 ) {
        cerr<<"error 2"<<endl;
        return;
    }

    int nx=200;
    int ny=200;
    double y1=(double)theta_y_limits[angle/5][0];
    double y2=(double)theta_y_limits[angle/5][1];
    double x1=(double)theta_x_limits[angle/5][0];
    double x2=(double)theta_x_limits[angle/5][1];
    TH2D* h1=new TH2D("h1","g4 (diffraction)",nx,x1,x2,ny,y1,y2);
    TH2D* h2=new TH2D("h2","g4 (no diffraction)",nx,x1,x2,ny,y1,y2);
    TH2D* h3=new TH2D("h3","esaf (diffraction)",nx,x1,x2,ny,y1,y2);
    TH2D* h4=new TH2D("h4","esaf (no diffraction)",nx,x1,x2,ny,y1,y2);


    TCanvas* c = new TCanvas();
    c->Divide(2,2);
    draw_on_pad(tr1,c->cd(1),h1);
    draw_on_pad(tr2,c->cd(2),h2);
    draw_on_pad(tr3,c->cd(3),h3);
    draw_on_pad(tr4,c->cd(4),h4);

    c->Print("test.pdf");

}
