
int angles[3]={323,357,391};

TList l_graphs;
TList th_graphs;

void MakeAll(const char* fname){
	l_graphs.SetOwner();
	th_graphs.SetOwner();
	l_graphs.Clear();
	th_graphs.Clear();

    for ( int i(0); i<3; i++ ){
		Efficiency(fname,0,32,angles[i],50000);
    }

    TFile f(Form("%s_eff.root",fname),"RECREATE");
    l_graphs.Write("l_graphs",TObject::kSingleKey);
    th_graphs.Write("th_graphs",TObject::kSingleKey);
    f.Close();

}

void Efficiency(const char* fname,int theta1,int theta2,int l,int nphot=50000){
    int th=theta1;
    double x_max,y_max;

    while(th<theta2){
        TFile* f=new TFile(Form("%sa%d.l%d.n%i.root",fname,th,l,nphot),"READ");
        if (  !f || f->IsZombie() ) {
            cerr<<"Error opening file."<<endl;
            return;
        }

        TTree* tr=(TTree*)f->Get("etree");
        if ( !tr ) {
            cerr<<"Error getting tree."<<endl;
            return;
        }

        double xn=600.;
        if(th>10) xn=1200.;
        TH2F* h1=new TH2F(Form("eff%d",th),Form("eff%d",th),1000,-200.,200.,1000,-200.,xn);
        tr->Draw(Form("detector.fPhotons.fIdealFocalPosX:detector.fPhotons.fIdealFocalPosY>>eff%d",th),"detector.fPhotons.fStatusBits&0x1","goff");

        int bmax_x, bmax_y, bmax_z;
        h1->GetMaximumBin(bmax_x, bmax_y, bmax_z);

        x_max = h1->GetXaxis()->GetBinCenter(bmax_x);
        y_max = h1->GetYaxis()->GetBinCenter(bmax_y);

        h1->GetXaxis()->SetRangeUser(x_max-7.,x_max+7.);
        h1->GetYaxis()->SetRangeUser(y_max-7.,y_max+7.);

//        TCanvas *c = new TCanvas();
//        c->Print(Form("%s_Analysis_%d_%d.eps[",fname,th,l));
//        h1->SetMarkerStyle(8);
//        h1->SetMarkerSize(0.6);
//        h1->Draw();
//        c->Print(Form("%s_Analysis_%d_%d.eps",fname,th,l));

        double box=0.;
        while( box<=5. ){
            box+=2.5;
            double max=0.;
            float x[5]={max-(box*0.5),max-(box*0.5),max+(box*0.5),max+(box*0.5),max-(box*0.5)};
            float y[5]={y_max-(box*0.5),y_max+(box*0.5),y_max+(box*0.5),y_max-(box*0.5),y_max-(box*0.5)};
            TCutG *cut = new TCutG(Form("cut_%fmm",box),5,x,y);
            cut->SetLineWidth(1);
            cut->SetLineColor(2);

            double e_total = cut->Integral(h1);
            double n_fp = h1 ->GetEntries();
//            cout<<"e_total"<<"      "<<e_total<<endl;
//            cout<<"e_total"<<"      "<<n_fp<<endl;
//            cut->Draw("same");
//            c->Print(Form("%s_Analysis_%d_%d.eps",fname,th,l));
            //cut->Print();

            TGraph* th_fpeff =    GetGraph(&th_graphs, box, Form("#lambda=%i. FP efficiency",l));
            TGraph* th_eff =      GetGraph(&th_graphs, box, Form("#lambda=%i. Total efficiency",l));
            TGraph* th_efficacy = GetGraph(&th_graphs, box, Form("#lambda=%i. Efficacy",l));
            TGraph* l_fpeff =     GetGraph(&l_graphs , box, Form("#theta=%i. FP efficiency",th));
            TGraph* l_eff =       GetGraph(&l_graphs , box, Form("#theta=%i. Total efficiency",th));
            TGraph* l_efficacy =  GetGraph(&l_graphs , box, Form("#theta=%i. Efficacy",th));

            double ttotalefficiency=e_total/nphot;
            double tefficiency=e_total/n_fp;
            double tefficacy=ttotalefficiency*TMath::Cos(th*TMath::DegToRad())*1.250*1.250*TMath::Pi();

            th_fpeff->SetPoint(th_fpeff->GetN(), th, tefficiency);
            th_eff->SetPoint(th_eff->GetN(), th, ttotalefficiency);
            th_efficacy->SetPoint(th_efficacy->GetN(), th, tefficacy );
            l_fpeff->SetPoint(l_fpeff->GetN(), l, tefficiency);
            l_eff->SetPoint(l_eff->GetN(), l, ttotalefficiency);
            l_efficacy->SetPoint(l_efficacy->GetN(), l, tefficacy);
        }

//        c->Print(Form("%s_Analysis_%d_%d.eps]",fname,th,l));
        th+=5;

//		delete c;
		delete h1;
    }
}

TGraph* GetGraph(TList* list, Double_t box, const char* title){
    TString name=Form("%s. Box %.2f X %.2f cm S = %.2fcm^{2}",title,box/10.,box/10.,(box)*(box)/100.);
    TGraph* gr=(TGraph*)list->FindObject(name.Data());
    if( gr ) return gr;
    else {
        TGraph *gr = new TGraph();
        gr->SetName(name.Data());
        gr->SetTitle(name.Data());
        list->Add(gr);
    }
    return gr;
}
