#include "EusoDetector.hh"
#include "EusoElectronics.hh"
#include "DetectorTransportManager.hh"
#include "ElementaryCell.hh"
#include "EsafMsgDispatcher.hh"
#include "PmtGeometry.hh"
#include "OpticalAdaptor.hh"
#include <TGraph.h>
#include <TGraph2D.h>
#include <TAxis.h>
#include <TStyle.h>
#include <TCanvas.h>

EusoDetector* e = 0;
enum { kCenter = 0,
       kUpLeft = 1,
       kDownLeft = 2
};

void DrawShift(Int_t refPoint = kCenter);
void DrawProfile();

//______________________________________________________________________________
void CheckFocalSurfaceShift() {

    Config* c = Config::Get();
    c->Reset();
    EsafMsgDispatcher::Get()->SetScreenSeverity(EsafMsg::Debug );

//    Bool_t useLOptics = kFALSE;
//    if ( useLOptics ) {
//
//        c->AddUserDefinition("EusoElectronics","EusoElectronics.fFocalSurfaceFile = config/Electronics/loptics.fsr");
//        c->AddUserDefinition("PolarFocalPlane","PolarFocalPlane.fPos.Z = 3283.28");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.rho = -2496.29");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.k = 0.138660");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.a =0.161736E-11");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.b = 0.521060E-16");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.c =-0.649507E-22");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.d = 0.197400E-28");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.precision = 0.01");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.fR = 1130");
//        c->AddUserDefinition("KIdealFocalSurface","KIdealFocalSurface.fPos.Z = 3283.38");
//    }

    gStyle->SetPalette(1);

    TCanvas* c_fsshift = new TCanvas("c_fsshift","Focal surface shift");
    c_fsshift->Divide(2,2);
    c_fsshift->cd(1);
    DrawShift( kCenter);
    c_fsshift->cd(2);
    DrawShift( kUpLeft);
    c_fsshift->cd(3);
    DrawShift( kDownLeft);
    c_fsshift->cd(4);
    DrawProfile();
    
}


//______________________________________________________________________________
void DrawProfile() {
    
    DetectorTransportManager *dm = e->GetDetectorTransportManager();
    IdealFocalSurface* ifs = dm->GetIdealFocalSurface();

    
    Int_t nPoints = 100;
    TGraph* profile = new TGraph(nPoints);
    profile->SetBit(kCanDelete);
    
    for ( Int_t i(0); i<nPoints ; i++ ) {
        Double_t r = i*ifs->GetR()/nPoints;
        profile->SetPoint(i, r, ifs->Profile(r) );
    }
    
    profile->SetNameTitle("fsprofile","Ideal Focal Surface Profile");
    profile->SetLineColor(kRed);
    profile->SetLineWidth(2);
    profile->GetXaxis()->SetTitle("r");
    profile->GetYaxis()->SetTitle("z");
    profile->Draw("AL");
    
}

//______________________________________________________________________________
void DrawShift(Int_t refPoint ) {

    if ( !e ) e = new EusoDetector();
    
    EusoElectronics* ee = e->GetEusoElectronics();
    DetectorTransportManager *dm = e->GetDetectorTransportManager();
    IdealFocalSurface* ifs = dm->GetIdealFocalSurface();
    Double_t hOA = ee->Pmt(0)->Geometry()->GetOA()->GetHeight();

    TVector3 fsPos = dm->GetFocalPlane()->Position();

    // One sector is enough
    Int_t n = ee->NumElemCell()/6;

    TGraph2D *gShift= new TGraph2D(n);
    gShift->SetBit(kCanDelete);

    for (Int_t i(0); i< n; i++) {
        ElementaryCell* ec = ee->ElemCell(i);
        TVector3 pos;
        pos = ec->GetCenter();
        switch (refPoint) {
            case 1:
                pos = ec->GetUpLeft();
                break;
            case 2:
                pos = ec->GetDownLeft();
                break;
            case kCenter:
            default:
                pos = ec->GetCenter();
        }

        TVector3 normal = ec->GetNormal();

        // cancel the displacement due to the adapters
        pos += normal*hOA;
        Double_t ecZ = pos.Z()+fsPos.Z();
        Double_t ifsZ = ifs->Profile(pos.Perp());
        Double_t diff = ecZ-ifsZ;

        // some output
        cout << "ec(" << i << ").Z(): " << ecZ << "\tec.Perp(): " << pos.Perp() << "\tifs(ec.Perp()): " << ifsZ
             << "\tdiff: " << diff << endl;

        gShift->SetPoint(i, pos.X(), pos.Y(), diff);
    }

    switch (refPoint) {
        case 1:
            gShift->SetNameTitle("G2DUpLeft", "EC upleft corner - IFS distance");
            break;
        case 2:
            gShift->SetNameTitle("G2DDownLeft", "EC downleft corner - IFS distance");
            break;
        case kCenter:
        default:
            gShift->SetNameTitle("G2DCenter", "EC center corner - IFS distance");
    }
    gShift->GetXaxis()->SetTitle("x");
    gShift->GetYaxis()->SetTitle("y");
    gShift->GetZaxis()->SetTitle("Shift");


    gShift->Draw("TRI1");
//    delete e;

}
    

//______________________________________________________________________________
void Clear() {
    Config* c = Config::Get();
    c->Reset();
    if ( e ) {  delete e; e = 0; }
}
