// $Id: Aerosol.hh 2676 2006-04-28 15:22:49Z moreggia $
// Author: Sylvain Moreggia   2006/04/24

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: Aerosol                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __AEROSOL_HH__
#define __AEROSOL_HH__

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "EsafMsgSource.hh"

class EarthVector;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Aerosol                                                                    //
//                                                                            //  
// Boundary layers aerosol, between altitude [0-2km]                          //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class Aerosol : public EsafConfigurable, public EsafMsgSource {
public:
    // ctor
    Aerosol(string);
    // dtor
    virtual ~Aerosol();

    // Return impact for given position and direction
    // Convention :    - if already within aerosol, returns pos
    //                 - no impact, returns (0,0,HUGE)
    virtual EarthVector GetImpact(const EarthVector& pos, const EarthVector& dir,string opt = "default") const = 0;

    // to know if a position is within aerosol
    virtual Bool_t IsInAerosol(const EarthVector&) const = 0;

    // Returns the transmission at given wl, along given track
    // set "scat" to the scattering rate along a direction
    virtual Double_t Trans(const EarthVector&,const EarthVector&,Double_t, Double_t& scat) const = 0;
    
    // returns a scattering position along a given track
    // depends on  wavelength
    virtual Bool_t RandomScatPos(const EarthVector&,const EarthVector&,Double_t,EarthVector&) const = 0;
    
    // set incoming direction to the outgoing direction after scattering
    // depends on  wavelength
    virtual void RandomDir(EarthVector&,Double_t) const = 0;
    
    // aerosol scattering phase function - Normalized when integrated over full solid angle
    // depends on  wavelength
    virtual Double_t PhaseFunction(const EarthVector&,const EarthVector&,Double_t) const = 0;
    
    // returns maximum value of currently used scattering phase function
    // depends on  wavelength range
    virtual Double_t GetMaxPhaseFunction(Double_t wlmin,Double_t wlmax) const = 0;
    
    // reset the instance
    virtual void Reset() = 0;


protected:
    
    // read config files
    virtual void Configure() = 0;

    // build the configured object
    virtual void Build() = 0;
    
    string fName;
    
    ClassDef(Aerosol,0)
};

#endif  /* __AEROSOL_HH__ */

