// $Id: AtmosphereFactory.hh 2676 2006-04-28 15:22:49Z moreggia $
// Sylvain Moreggia created Dec,  1 2003

#ifndef __ATMOSPHEREFACTORY_HH_
#define __ATMOSPHEREFACTORY_HH_

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: AtmosphereFactory                                                    *
 *  Package: atmosphere                                                      *
 *  Coordinator: S. Moreggia                                                 *
 *                                                                           *
 *****************************************************************************/

///////////////////////////////////////////////////////////////////////////////////
// AtmosphereFactory                                                             //
//                                                                               //
// Builds atmosphere related objects according to config files                   //
// Does not build Atmosphere object itself, even if it appears in config files   //
//                                                                               //
///////////////////////////////////////////////////////////////////////////////////

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "Atmosphere.hh"
#include "EsafMsgSource.hh"


class Clouds;
class Aerosol;

class AtmosphereFactory : public EsafConfigurable, public EsafMsgSource {
public:
    
    // dtor
    virtual ~AtmosphereFactory();

    // instance 
    static AtmosphereFactory *Get();
    
    // Build clouds object
    Clouds* GetClouds();
        
    // Build aerosol object
    Aerosol* GetAerosol();
        
    EsafConfigClass(Atmosphere,AtmosphereFactory)
 
private:
    // ctor
    AtmosphereFactory();

    // unique instance of AtmosphereFactory
    static AtmosphereFactory *fMe;
    

    ClassDef(AtmosphereFactory,0)
};

#endif  /* __ATMOSPHEREFACTORY_HH_ */

