// ESAF : Euso Simulation and Analysis Framework
// $Id: LinsleyAtmosphereData.hh 2695 2006-05-27 19:09:42Z moreggia $
// Sylvain Moreggia created May, 17 2004

#ifndef __LINSLEYATMOSPHEREDATA_HH_
#define __LINSLEYATMOSPHEREDATA_HH_

#include "euso.hh"
#include "AtmosphereData.hh"
#include "EsafMsgSource.hh"


/*******************************************************************************
 *
 * Atmosphere : class description
 *
 * Handles layers of the linsley parametrization
 *
 ******************************************************************************/


class LinsleyAtmosphereData : public AtmosphereData {
public:
    // ctors
    LinsleyAtmosphereData();
    LinsleyAtmosphereData(const Int_t&, const vector<Double_t>&, const Int_t&);

    // dtor
    virtual ~LinsleyAtmosphereData();
    
    // GetDM methods
    inline Int_t GetNumber() const {return fNb;}
    inline string GetName() const {return fName;}
    inline Double_t Bottom(Int_t i) const {return fBottom[i];}
    inline Double_t A(Int_t i) const {return fA[i];}
    inline Double_t B(Int_t i) const {return fB[i];}
    inline Double_t C(Int_t i) const {return fC[i];}
    
    // Linsley parametrization
    Double_t DensityParametrization(Double_t) const;   

    // dummy
    virtual Double_t GetLatitude() const {return 104;}
    virtual Double_t GetLongitude() const {return -1000;}
    virtual Int_t GetDoy() const {return -1000;}
    virtual Double_t GetDate() const {return -1000;}
    
    EsafConfigClass(Atmosphere,LinsleyAtmosphereData)
        
private:

    // Set tables
    virtual void ParamTables();
    virtual void SetTables();
    
    string fName;                // type of linsley atmosphere
    Int_t fNb;                   // total number of layers
     
    Double_t* fBottom;           // bottom altitude of the layer
    Double_t* fA;                //    
    Double_t* fB;                // Linsley parameters tables (ith element matches ith layer)
    Double_t* fC;               
    
    ClassDef(LinsleyAtmosphereData,0)
};

#endif  /* __LINSLEYATMOSPHEREDATA_HH_ */

