// ESAF : Euso Simulation and Analysis Framework
// $Id: LowtranAtmosphereData.hh 2695 2006-05-27 19:09:42Z moreggia $
// Corinne Berat created Mar, 30 2004

#ifndef __LOWTRANATMOSPHEREDATA_HH_
#define __LOWTRANATMOSPHEREDATA_HH_

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "AtmosphereData.hh"
#include "EsafMsgSource.hh"

/*******************************************************************************
 *
 * LowtranAtmosphereData : class description
 *
 * Used by LowtranAtmosphere for data storage
 * Handle LOWTRAN atmospheric parameter tables from standard models
 *
 ******************************************************************************/


class LowtranAtmosphereData : public AtmosphereData {
public:
    // ctor
    LowtranAtmosphereData();

    // dtor
    virtual ~LowtranAtmosphereData();

    // Simple get methods
    inline Int_t NumberOfElements() const {return fNb;}
    
    // used to identify type of lowtran atmosphere in the output
    virtual Double_t GetLatitude() const;
    // dummy
    virtual Double_t GetLongitude() const {return -1000;}
    virtual Int_t GetDoy() const {return -1000;}
    virtual Double_t GetDate() const {return -1000;}
    
    
private:
    // Set tables using read data
    virtual void SetTables();
    // to achieve constant steps when SetTables is called
    Double_t LinearInterpol(Int_t h, const vector<Double_t>&, const vector<Double_t>&);
    
    // Initialize  parameters to work out tables calculations
    void Init();
    
    string fModelName;
    Int_t fNb; 
    
    struct lowtran_input {
    Int_t model;
    Int_t M1;
    Int_t M2;
    Int_t M3;
    Int_t M4;
    Int_t M5;
    Int_t M6;
    Int_t Mdef;
    };
    struct lowtran_input fInput;  // "input of Lowtran"

    ClassDef(LowtranAtmosphereData,0)
};

#endif  /* __LOWTRANATMOSPHEREDATA_HH_ */












