// $Id: MSISE_00Atmosphere.hh 2695 2006-05-27 19:09:42Z moreggia $
// S. Moreggia created 18 November 2003

// Mass Spectrometer, Incoherent Scatter Radar Extended Model : NRLMSISE_00
// For atmospheric description

#ifndef __MSISE_00ATMOSPHERE_HH__
#define __MSISE_00ATMOSPHERE_HH__

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: MSISE_00Atmosphere                                                   *
 *  Package: atmosphere                                                      *
 *  Coordinator: S. Moreggia                                                 *
 *                                                                           *
 *****************************************************************************/

////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                        //
// MSISE_00Atmosphere                                                                     //
//                                                                                        //                
// Mass Spectrometer, Incoherent Scatter Radar Extended Model : NRLMSISE_00               //
// Atmospheric description using msise model                                              //
// Source code (see MSISETool) has been downloaded from Nasa Website                      //
//                                                                                        //
////////////////////////////////////////////////////////////////////////////////////////////

#include "Atmosphere.hh"
#include "MSISEtool.hh"
#include "EsafMsgSource.hh"
#include "MSISE_00AtmosphereData.hh"

class MSISE_00Atmosphere : public Atmosphere {

public:

    // create the single atmosphere instance as a MSISE_00Atmosphere object
    static void CreateInstance();
    
    // in random mode, randomly chose a new atmosphere
    void ResetInstance();
    
    // To get specific parameters of MSISE atmosphere from the interface
    virtual Double_t GetValue(const string& parname) const {return 0;}

    // get date and geographic atmosphere parameters
    virtual Double_t GetLatitude() const;              // from -90 (south) to 90 (north)
    virtual Double_t GetLongitude() const;             // from 0 to 360 (east)
    virtual Double_t GetDate() const;	               // in sec UT, 1sec <==> 1st Jan 00:00:01 (year not taken into account)
    inline Double_t GetDoy() const {return fData->GetDoy();}
    
    // Following methods give relevant densities, temperature, pressure
    virtual Double_t Pressure(Double_t h) const;    
    virtual Double_t Temperature(Double_t h) const;
    virtual Double_t AbsoluteHumidity(Double_t h) const;
    virtual Double_t Air_Density(Double_t h) const;
    virtual Double_t O_Density(Double_t h) const;
    virtual Double_t O2_Density(Double_t h) const;
    virtual Double_t O3_Density(Double_t h) const;
    virtual Double_t O3_DensityPPMV(Double_t h) const;
    virtual Double_t N2_Density(Double_t h) const;
    virtual Double_t Aerosols_Density(string& type,Double_t h) const;

    EsafConfigClass(Atmosphere,MSISE_00Atmosphere)
    
    //virtual void Test(); to test MSISE code interface
    //void Build(Int_t doy,Double_t lst, Double_t lat, Double_t longi);
    

private:
    // default ctor (not used)
    MSISE_00Atmosphere() {}
    
    // ctor
    MSISE_00Atmosphere(string);   
    
    // dtor
    virtual ~MSISE_00Atmosphere();
        
    // Build atmosphere according to config parameters
    virtual void Build();
  
    // Calculate interpolations (if working with tables)
    Double_t Interpolate(string&,string&,Double_t) const;
        
    MSISE_00AtmosphereData* fData;           // tables storage

    ClassDef(MSISE_00Atmosphere,0)
};

//________________________________________________________________________________________
inline Double_t MSISE_00Atmosphere::GetLatitude() const {
    if(fData) return fData->GetLatitude();
    else Msg(EsafMsg::Info) << "No MSISE data --> command ignored"<< MsgDispatch;
    return -1000.;
}

//________________________________________________________________________________________
inline Double_t MSISE_00Atmosphere::GetLongitude() const {
    if(fData) return fData->GetLongitude();
    else Msg(EsafMsg::Info) << "No MSISE data --> command ignored"<< MsgDispatch;
    return -1000.;
}

//________________________________________________________________________________________
inline Double_t MSISE_00Atmosphere::GetDate() const {
    if(fData) return fData->GetDate();
    else Msg(EsafMsg::Info) << "No MSISE data --> command ignored"<< MsgDispatch;
    return -1000.;
}


#endif /*__MSISE_00ATMOSPHERE_HH__*/
