// ESAF : Euso Simulation and Analysis Framework
// $Id: MSISE_00AtmosphereData.hh 2695 2006-05-27 19:09:42Z moreggia $
// Sylvain Moreggia created Dec, 12 2003

#ifndef __MSISE_00ATMOSPHEREDATA_HH_
#define __MSISE_00ATMOSPHEREDATA_HH_

#include "euso.hh"
#include "AtmosphereData.hh"
#include "MSISEtool.hh"
#include "EsafMsgSource.hh"

/*******************************************************************************
 *
 * MSISE_00AtmosphereData : class description
 *
 * Handle data storage for MSISE_00Atmosphere
 *
 ******************************************************************************/


class MSISE_00AtmosphereData : public AtmosphereData {
public:
    // ctor
    MSISE_00AtmosphereData(Bool_t);

    // dtor
    virtual ~MSISE_00AtmosphereData();

    // Set tables using MSISEtool
    virtual void SetTables();
    
    // Get Members
    inline int NumberOfElements() const {return fNb;}
    inline Double_t LayerSize() const {return fStep;}
    
    // within [-90,90] (positive <==> north)
    virtual Double_t GetLatitude() const {return fInput.g_lat;}
    
    // within [0,360[ (toward east)
    virtual Double_t GetLongitude() const {return fInput.g_long;}
    
    // get day of year
    virtual Int_t GetDoy() const {return fInput.doy;}
    
    // NB : day starts from 1 and can reach 365
    // in sec UT, 1sec <==> 1st Jan 00:00:01 (year not taken into account)
    virtual Double_t GetDate() const {return Double_t((fInput.doy - 1)*3600*24) + fInput.sec;}
    
    EsafConfigClass(Atmosphere,MSISE_00AtmosphereData)

    // only for MSISEscan macro
    //void InitForMacro(Int_t doy,Double_t lst, Double_t lat, Double_t longi);
    

private:    
    
    // Initialize msise parameters to work out tables calculations
    void Init();
    
    Bool_t fRandomMode;                 // if true, build the atmosphere randomly
    MSISEtool* fTool;                   // holds a set of methods using msise C-code
    struct nrlmsise_input fInput;       // input of MSISE (date,location..)
    struct nrlmsise_flags fTuning;      // set particular requirements for msise

    Int_t fNb;                          // number of table elements
    Double_t fStep;                     // size of a layer
    
    ClassDef(MSISE_00AtmosphereData,0)
};

#endif  /* __MSISE_00ATMOSPHEREDATA_HH_ */

