// $Id: NoAerosol.hh 2794 2008-02-11 07:56:34Z naumov $
// Author: Sylvain Moreggia   2006/04/24

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: NoAerosol                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __NOAEROSOL_HH__
#define __NOAEROSOL_HH__

#include "euso.hh"
#include "Aerosol.hh"
#include "EarthVector.hh"
#include <TMath.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// NoAerosol                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class NoAerosol : public Aerosol {
public:
    // ctor
    NoAerosol(string name) : Aerosol(name) {}
    // dtor
    virtual ~NoAerosol();

    // Return impact for given position and direction
    // Convention :    - if already within aerosol, returns pos
    //                 - no impact, returns (0,0,HUGE)
    virtual EarthVector GetImpact(const EarthVector& pos, const EarthVector& dir,string opt = "default") const  {return EarthVector(0,0,HUGE);}

    // to know if a position is within aerosol
    virtual Bool_t IsInAerosol(const EarthVector&) const {return kFALSE;}

    // Returns the transmission at given wl, along given track
    // set "scat" to the scattering rate along a direction
    virtual Double_t Trans(const EarthVector&,const EarthVector&,Double_t, Double_t& scat) const {return 1.;}
    
    
    // returns a scattering position along a given track
    // depends on  wavelength
    virtual Bool_t RandomScatPos(const EarthVector&,const EarthVector&,Double_t,EarthVector& rtn) const {rtn.SetXYZ(0,0,HUGE); return kTRUE;}
    
    // set incoming direction to the outgoing direction after scattering
    // depends on  wavelength
    virtual void RandomDir(EarthVector&,Double_t) const {}
    
    // aerosol scattering phase function - Normalized when integrated over full solid angle
    // depends on  wavelength
    virtual Double_t PhaseFunction(const EarthVector&,const EarthVector&,Double_t) const {return 0.;}
    
    // returns maximum value of currently used scattering phase function
    // depends on  wavelength range
    virtual Double_t GetMaxPhaseFunction(Double_t wlmin,Double_t wlmax) const {return 0;}
    
    // reset the instance
    virtual void Reset() {}

    EsafConfigClass(Atmosphere,NoAerosol)

protected:
    
    // read config files
    virtual void Configure() {}

    // build the configured object
    virtual void Build() {}
    

    ClassDef(NoAerosol,0)
};

#endif  /* __NOAEROSOL_HH__ */

