// $Id: AtmosphereFactory.cc 2676 2006-04-28 15:22:49Z moreggia $
// Sylvain Moreggia created Dec,  1 2003

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: AtmosphereFactory                                                    *
 *  Package: atmosphere                                                      *
 *  Coordinator: S. Moreggia                                                 *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// AtmosphereFactory
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "AtmosphereFactory.hh"
#include "TestClouds.hh"
#include "NoClouds.hh"
#include "LowtranAerosol.hh"
#include "NoAerosol.hh"
#include "Config.hh"

AtmosphereFactory *AtmosphereFactory::fMe = NULL;

ClassImp(AtmosphereFactory)

//_________________________________________________________________________________________
AtmosphereFactory::AtmosphereFactory() {
    //
    // ctor
    //
}

//_________________________________________________________________________________________
AtmosphereFactory::~AtmosphereFactory() {
    //
    // dtor
    //
}

//_________________________________________________________________________________________
AtmosphereFactory* AtmosphereFactory::Get() {
    //
    // instance
    //
    if ( fMe == NULL )
        fMe = new AtmosphereFactory();
	
    return fMe;
}

//_________________________________________________________________________________________
Clouds* AtmosphereFactory::GetClouds() {
    //
    // Build clouds object
    //
    string type = Conf()->GetStr("AtmosphereFactory.Clouds.type");
    
    if(type == "test")
        return new TestClouds();     
    
    else if(type == "none")
        return new NoClouds();
       
    else  Msg(EsafMsg::Panic)<< " Wrong clouds type"<< MsgDispatch;

    return 0;

}

//_________________________________________________________________________________________
Aerosol* AtmosphereFactory::GetAerosol() {

    //
    // Build aerosol object
    //
    string type = Conf()->GetStr("AtmosphereFactory.Aerosol.name");
    
    if(type == "lowtran")
        return new LowtranAerosol("lowtran");     
    
    else if(type == "none")
        return new NoAerosol("none");
       
    else  Msg(EsafMsg::Panic)<< " Wrong aerosol type"<< MsgDispatch;

    return 0;

}

