// $Id: EsafSys.hh 2590 2006-03-17 14:21:38Z thea $
// Author: Alessandro Thea   2005/05/19

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EsafSys                                                              *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __ESAFSYS_HH__
#define __ESAFSYS_HH__

#include "euso.hh"
#include <TString.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// EsafSys                                                                    //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class EsafSys {
public:
    EsafSys( Bool_t absolute = kFALSE );
    EsafSys( const char* path );
    virtual ~EsafSys();

    const char* GetArch() const;
    const char* GetAuxPath() const;
    const char* GetBinPath() const;
    const char* GetCxxVers() const;
    const char* GetCfgPath() const;
    const char* GetLibPath() const;
    const char* GetOutPath() const;
    const char* GetTmpPath() const;
    const char* GetPath() const;
    const char* GetRootVers() const;

    void SetAuxPath( const char* );
    void SetBinPath( const char* );
    void SetCfgPath( const char* );
    void SetLibPath( const char* );
    void SetOutPath( const char* );
    void SetTmpPath( const char* );
    void SetPath( const char* );
    
    const char* AbsolutePath( const char* ) const; 
    TString& AbsolutePath( TString& ) const; 

private:
    TString& CheckDirSyntax( TString& path ) const;

    TString fArch;
    TString fAuxPath;
    TString fBinPath;
    TString fCxxVers;
    TString fCfgPath;
    TString fLibPath;
    TString fOutPath;
    TString fTmpPath;
    TString fPath;
    TString fRootVers;
    

    ClassDef(EsafSys,0)
};

extern EsafSys* gEsafSys;

inline const char* EsafSys::GetArch() const {
    return fArch;
}

inline const char* EsafSys::GetAuxPath() const {
    return fAuxPath;
}

inline void EsafSys::SetAuxPath( const char* path){
    fAuxPath = path;
    CheckDirSyntax( fAuxPath );
}

inline const char* EsafSys::GetBinPath() const {
    return fBinPath;
}

inline void EsafSys::SetBinPath( const char* path){
    fBinPath = path;
    CheckDirSyntax( fBinPath );
}

inline const char* EsafSys::GetCfgPath() const {
    return fCfgPath;
}

inline void EsafSys::SetCfgPath( const char* path){
    fCfgPath = path;
    CheckDirSyntax( fCfgPath );
}

inline const char* EsafSys::GetCxxVers() const {
    return fCxxVers;
}

inline const char* EsafSys::GetLibPath() const {
    return fLibPath;
}

inline void EsafSys::SetLibPath( const char* path){
    fLibPath = path;
    CheckDirSyntax( fLibPath );
}

inline const char* EsafSys::GetOutPath() const {
    return fOutPath;
}

inline void EsafSys::SetOutPath( const char* path){
    fOutPath = path;
    CheckDirSyntax( fOutPath );
}

inline const char* EsafSys::GetTmpPath() const {
    return fTmpPath;
}

inline void EsafSys::SetTmpPath( const char* path){
    fTmpPath = path;
    CheckDirSyntax( fTmpPath );
}

inline const char* EsafSys::GetPath() const {
    return fPath;
}

inline void EsafSys::SetPath( const char* path){
    fPath = path;
    CheckDirSyntax( fPath );
}

inline const char* EsafSys::GetRootVers() const {
    return fRootVers;
}
#endif  /* __ESAFSYS_HH__ */

