// $Id: EFocalSurfacePainterEditor.hh 2782 2007-07-16 09:25:45Z thea $
// Author: Alessandro Thea   2005/02/20

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EFocalSurfacePainterEditor                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __EFOCAlSURFACEPAINTEREDITOR_HH__
#define __EFOCALSURFACEPAINTEREDITOR_HH__

#include "TGedFrame.h"
#include "EFocalSurfacePainter.hh"

class TList;
class TGComboBox;
class TGColorSelect;
class TGedPatternSelect;
class TGHSlider;
class TGCheckButton;
class TGButtonGroup;
class TGRadioButton;
class TGTab;
class TGLabel;
class TGNumberEntry;

enum Threshold_t {
    kNONE,
    kABSOLUTE,
    kRELATIVE
};

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// EFocalSurfacePainterEditor                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class EFocalSurfacePainterEditor : public TGedFrame {
public:
    EFocalSurfacePainterEditor(const TGWindow* p = 0 , Int_t width = 140, Int_t height = 30, UInt_t options = kChildFrame, Pixel_t back = GetDefaultFrameBackground());
    virtual ~EFocalSurfacePainterEditor();

//    virtual void   SetModel(TVirtualPad *pad, TObject *obj, Int_t event);
    virtual void   SetModel(TObject *obj);

    void ConnectSignals2Slots();
    
    virtual void DoMapID( Int_t );
    virtual void DoIntegral( Int_t );
    virtual void DoTrigger( Int_t );
    virtual void DoViewRangeSliderMoved( Int_t );
    virtual void DoGtuSliderMoved( Int_t );
    virtual void DoGtuEntrySet();
    virtual void DoCountsSliderMoved( Int_t );
    virtual void DoCountsEntrySet();
    virtual void DoThresholdSliderMoved( Int_t );
    virtual void DoAlphaEntrySet();
    virtual void DoAddGtuCount(Bool_t);
    virtual void DoAddEmptyPixels(Bool_t);
    virtual void DoDrawNightRate(Bool_t);
    virtual void DoDrawNightCounts(Bool_t);
    virtual void DoAddOffPixels(Bool_t on);
    virtual void DoThreshold(Threshold_t);
    virtual void DoPmtFillColor(Pixel_t);
    virtual void DoEmptyPmtFillColor(Pixel_t);
    
private:

    void CreateThresholdTab();
    void CreateColorTab();

    EFocalSurfacePainter* fPainter;
    Int_t  fCurrentMapID;
    Int_t  fCurrentIntegral;
    Int_t  fCurrentTrigger;

//    TGComboBox* BuildNspreadComboBox(TGFrame* parent, Int_t id);
    TGComboBox* BuildMapIDCombo(TGFrame* parent, Int_t id);
    TGComboBox* BuildIntegralCombo(TGFrame* parent, Int_t id);
    TGComboBox* BuildChipTriggerCombo(TGFrame* parent, Int_t id);

//    TGTab* fTab;
    TGCompositeFrame* fColor; 
    TGCompositeFrame* fColorContainer;
    TGCompositeFrame* fThreshold; 
    TGCompositeFrame* fThresholdContainer;
    TGComboBox* fMapIDCombo; //     font size combo box
    TGComboBox* fIntegralCombo; //     font size combo box
    TGComboBox* fTriggerCombo; //     font size combo box
    TGHSlider* fGtuSlider;
    TGNumberEntry* fGtuEntry;
    TGHSlider* fViewRangeSlider;
    TGHSlider* fCountsSlider;
    TGHSlider* fThresholdSlider;
    TGNumberEntry* fAlphaEntry;
    TGNumberEntry* fCountsEntry;
    TGLabel *fViewRMinLabel;
    TGLabel *fViewRMaxLabel;
    TGLabel *fGtuMinLabel;
    TGLabel *fGtuMaxLabel;
    TGLabel *fAlphaMinLabel;
    TGLabel *fAlphaMaxLabel;
//    TGCheckButton* fMakeTimeInt;
    TGCheckButton* fAddGtuCount;
    TGCheckButton* fAddEmptyPixels;
    TGCheckButton* fAddOffPixels;
    TGCheckButton* fDrawNightRate;
    TGCheckButton* fDrawNightCounts;
    TGButtonGroup* fThresholdType;
    TGRadioButton* fThNone;
    TGRadioButton* fThAbs;
    TGRadioButton* fThRel;
    TGColorSelect* fPmtColorSelect;  //  fill color widget
    TGColorSelect* fEmptyPmtColorSelect;  //  fill color widget

    TList* fGarbage;

    ClassDef(EFocalSurfacePainterEditor,0)
};

#endif  /* __EFOCALSURFACEPAINTEREDITOR_HH__ */

