// $Id: ESimpleDetectorPainter.hh 2845 2010-02-25 05:43:43Z fenu $
// Author: Sylvain Moreggia   2006/03/29

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ESimpleDetectorPainter                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __ESIMPLEDETECTORPAINTER_HH__
#define __ESIMPLEDETECTORPAINTER_HH__

#include "ESystemOfUnits.hh"
#include "EAnimator.hh"
#include "TSeqCollection.h"
#include <TVector3.h>
#include <TVector2.h>
#include <TArrayI.h>
#include <TArrayF.h>

class ESimpleDetector;
class EGeometry;
class ETruth;
class TH1F;
class TH2F;
class TH2I;
class TTimer;


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// ESimpleDetectorPainter                                                     //
//                                                                            //  
// this class plots a set of standard histograms describing the photoelectron //
// signal generated by SimpleDetector simulation                              //
// generation and transmission in atmosphere                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class ESimpleDetectorPainter : public EAnimator {
public:
    
    ESimpleDetectorPainter( ESimpleDetector* det, const EGeometry* geom, ETruth* truth,Int_t maxscatorder);
    
    virtual ~ESimpleDetectorPainter();
    
    
    virtual void Draw(Option_t *opt="");

    // preprocess : build histos NBpixels_hit vs/ GTU (as function of SNR)
    void Preprocess();

    // generate all histos
    void BuildAll();

    // generate histos of pe- time spectrum
    void BuildTimeHistos();

    // generate wavelength spectra of photons that gave pe-
    void BuildWlHistos();

    // generate FoV histo : pixel_i vs/ pixel_j
    void BuildFoVHistos();

    inline TVector3 EUSO() const {return fEUSO;}
    inline Float_t DetectorRadius() const {return fRadius;}


    TH1F* GetTimeHisto(const char*, const char* =0);
    TH1F* GetWlHisto(const char*, const char* =0);
    TH2I* GetFoVHisto(const char*, const char* =0,Int_t nbsigmas=-1000);

    TSeqCollection* GetListOfHistos() {return fHistos;}

    virtual void NextFrame() {}

private:

    TVector3 fEUSO;
    Float_t fRadius;
    Float_t fGTUlength;
    UInt_t fNbPixelOnSide;           // nb of pixels on a side of the square FS
    Float_t fFirstGtuTime;          // first GTU up-edge time
    UInt_t fNbGTUs;                  // nb of GTUs containing signal
    Int_t fNumPe;                   // nb of photoelectrons generated by detector simu 
    Int_t fMaxScatOrder;             // mutliple scattering max order
    Float_t fMeanBgnd;              // mean bgnd in pe-/pixel/GTU
    Int_t fFoV_xmin;                 // FoV bounds for all pe- -->
    Int_t fFoV_xmax;                 //
    Int_t fFoV_ymin;                 //
    Int_t fFoV_ymax;                 // <--
    Int_t fFoV_xmin2;             // FoV bounds only signal > 2sigmas -->
    Int_t fFoV_xmax2;             //
    Int_t fFoV_ymin2;             //
    Int_t fFoV_ymax2;             // <--
    Int_t fFoV_xmin_sig;             // FoV bounds only signal > 3sigmas -->
    Int_t fFoV_xmax_sig;             //
    Int_t fFoV_ymin_sig;             //
    Int_t fFoV_ymax_sig;             // <--

    TSeqCollection *fHistos;

    ESimpleDetector *fDetector;            // contains simu detector data
    ETruth          *fTruth;               // contains the truth (used to get shower impact)
        

    ClassDef(ESimpleDetectorPainter,1)
};

#endif  /* __ESIMPLEDETECTORPAINTER_HH__ */

