// $Id: ESimuEventViewer.hh 2745 2006-06-29 07:55:55Z thea $
// Author: Alessandro Thea   2005/05/31

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ESimuEventViewer                                                     *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __ESIMUEVENTVIEWER_HH__
#define __ESIMUEVENTVIEWER_HH__

#include "EGViewer.hh"
#include <TStyle.h>

class EAtmosphereHistoPainter;
class EDetectorHistoPainter;
class EDetectorPhotonsPainter;
class EEvent;
class EFocalSurfacePainter;
class EFocalSurface3DPainter;
class EGViewe;
class EOpticsPhotonsPainter;
class ERunParameters;
class EShowerHistoPainter;
class EShowerPainter;
class ESinglePhotonsPainter;
class ETriggerPainter;
class ETruth;
class TCut;
class TEventList;
class TGDockableFrame;
class TGTextEntry;
class TH1F;
class TTree;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// ESimuEventViewer                                                           //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class ESimuEventViewer : public EGViewer {
protected:
//public:
    TGDockableFrame* fEventBar;
         TGComboBox* fEventCombo;
        TGTextEntry* fEventCut;
              TStyle fSmallHistSty;
              TStyle fMainSty;

    void AddUserFrame();
    Bool_t Open( const char* name );
    Bool_t OpenDialog();
    Bool_t LoadEvent( Int_t i=0 );
    void BuildPainters(); 
    void ClearPainters();

    void UpdateTabs();
    void DrawTruth();

    virtual Bool_t ProcessMessageCommand(Long_t submsg, Long_t mp1, Long_t mp2);
protected:
    virtual void DoError(int level, const char *location, const char *fmt, va_list va) const; 
public:
    ESimuEventViewer( const char* name );
    virtual ~ESimuEventViewer();

    virtual Bool_t HandleKey(Event_t* event); 

    void NextEvent();
    void PrevEvent();
    void ShowEvent( Int_t i=0 ) { LoadEvent(i); }

    EShowerPainter*            GetShowerPainter() { return fShowerPainter; }
    EShowerHistoPainter*       GetShowerHistoPainter() { return fShowerHistoPainter; }
    EAtmosphereHistoPainter*   GetAtmosphereHistoPainter() { return fAtmosphereHistoPainter; }
    ESinglePhotonsPainter*     GetSinglePhotonsPainter() { return fSinglePhotonsPainter; }
    EFocalSurfacePainter*      GetFocalSurfacePainter() { return fFocalSurfacePainter; }
    EFocalSurface3DPainter*    GetFocalSurface3DPainter() { return fFocalSurface3DPainter; }
    ETriggerPainter*           GetTriggerPainter () { return fTriggerPainter ; }
    EDetectorHistoPainter*     GetFocalSurfaceHistoPainter() { return fFocalSurfaceHistoPainter; }
    EDetectorPhotonsPainter*   GetDetectorPhotonsPainter() { return fDetectorPhotonsPainter; }
    EOpticsPhotonsPainter*     GetOpticsPhotonsPainter() { return fOpticsPhotonsPainter; }

    EEvent         *GetEvent() { return fEvent; }
    ERunParameters *GetRunPars() { return fRunPars; }

    void ApplyBarCut();
    void ApplyCut( const char* selection );
    void ApplyCut( const TCut& selection );


    const char* GetCut() const;
private:
    void CreateViewer();
    void UpdateEventList(const TCut& );
    void UpdateEventList(const char* = "" );
public:
    EShowerPainter*            fShowerPainter;
    EShowerHistoPainter*       fShowerHistoPainter;
    EAtmosphereHistoPainter*   fAtmosphereHistoPainter;
    ESinglePhotonsPainter*     fSinglePhotonsPainter;
    EFocalSurfacePainter*      fFocalSurfacePainter;
    EFocalSurface3DPainter*    fFocalSurface3DPainter;
    ETriggerPainter*           fTriggerPainter ;
    EDetectorHistoPainter*     fFocalSurfaceHistoPainter;
    EDetectorPhotonsPainter*   fDetectorPhotonsPainter;
    EOpticsPhotonsPainter*     fOpticsPhotonsPainter;
private:
        TEventList* fEventList;
             TTree* fEvTree;
            EEvent* fEvent;
    ERunParameters* fRunPars;
            ETruth* fTruth;

              Int_t fCurrentEvent;
              Int_t fCurrentEventId;

              Bool_t fForceLoading;

    ClassDef(ESimuEventViewer,0)
};

#endif  /* __ESIMUEVENTVIEWER_HH__ */

