// $Id: ESimpleDetectorPainter.cc 2845 2010-02-25 05:43:43Z fenu $
// Author: Sylvain Moreggia   2006/03/29

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ESimpleDetectorPainter                                               *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// ESimpleDetectorPainter
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "ESimpleDetectorPainter.hh"
#include "ESimpleDetector.hh"
#include "EPhotoElectron.hh"
#include "EGeometry.hh"
#include "ESystemOfUnits.hh"
#include "ETruth.hh"
#include "Etypes.hh"

#include <TH1F.h>
#include <TH2F.h>
#include <TH2I.h>
#include <THStack.h>
#include <TF1.h>
#include <TTimer.h>
#include <TPad.h>
#include <TPaveText.h>
#include <TGaxis.h>
#include <TVector3.h>
#include <TMath.h>
#include <TLegend.h>
#include <TCanvas.h>
#include <TFile.h>

#include <map>
#include <vector>
using std::vector;
using std::map;


ClassImp(ESimpleDetectorPainter)

extern Double_t Zv(const TVector3&);
using namespace TMath;
using namespace sou;
using std::vector;
using std::map;



//_____________________________________________________________________________
ESimpleDetectorPainter::ESimpleDetectorPainter( ESimpleDetector* det, const EGeometry* geometry, ETruth* truth,Int_t maxscatorder) {
    //
    // Constructor
    //

    fDetector = 0;

    if ( !det || !geometry) {
        MakeZombie();
        return;
    }
    fDetector = new ESimpleDetector(*det);
    fTruth = new ETruth(*truth);
    fHistos = new TList();
    
    fGTUlength = fDetector->GetGTUlength();
    fEUSO = geometry->GetPos();
    fRadius = geometry->GetRadius();
    fNumPe = fDetector->GetNumPhotoElectrons();
    fNbGTUs = fDetector->GetNbGTUs();
    fFirstGtuTime = fDetector->GetFirstGtuTime();
    fMaxScatOrder = maxscatorder;  // if < 0 (no atmosphere in the output, fixed in preprocess method)
    fMeanBgnd = fDetector->GetMeanBgnd();
    fNbPixelOnSide = fDetector->GetNbPixelOnSide();
    
    Info("ESimpleDetectorPainter::Build()","USE pupil radius = %.2f mm, euso altitude = %.2f km, GTUlength = %.1f nanosec",fRadius/mm,fEUSO.Z()/km,fGTUlength);
    
    Preprocess();
    Info("ESimpleDetectorPainter::Build()","Max scattering order = %d",fMaxScatOrder);
    BuildAll();
}

//_____________________________________________________________________________
ESimpleDetectorPainter::~ESimpleDetectorPainter() {
    //
    // Destructor
    //
    if (IsZombie()) return;
    if ( fHistos ) fHistos->Delete();
    TFile* temp = (TFile*)gROOT->FindObject("temp_file_viewer");
    temp->Close();
}

//_____________________________________________________________________________
void ESimpleDetectorPainter::Preprocess() {
    //
    // preprocess : build histos NBpixelshit vs/ GTU (as function of SNR)
    // 
    
    // list of photoelectrons, stored per GTU (index=0 means GTU nb=0, GTU id within [0,fNbGTUs-1])
    // NB : EPhotoElectron elements belong to ROOT tree, MUST not be destroyed
    map<UInt_t, vector<EPhotoElectron*>* > PEMap;
    for(UInt_t i=0; i < fNbGTUs; i++) PEMap[i] = NULL;
    
    // array of pe- time
    TArrayD tofarray(fNumPe);
    if(fMaxScatOrder < 0) {
        // search for maximum scattering order
	Int_t maxSCAT = 0;
        for (Int_t i=0; i<fNumPe; i++) {
	    EPhotoElectron* peee = fDetector->GetPhotoElectron(i);
	    tofarray[i] = peee->GetTime();
	    if(peee->GetAtmoNbinter() > maxSCAT) maxSCAT = peee->GetAtmoNbinter();
	}
	fMaxScatOrder = maxSCAT;
    }
    else
        for (Int_t i=0; i<fNumPe; i++) tofarray[i] = fDetector->GetPhotoElectron(i)->GetTime();
    
    
    // 1. fill the map + find FoV bounds where signal exists
        // 1.1 sort tofarray elements using time info
    Int_t sortedindex[fNumPe];
    Sort(fNumPe,tofarray.GetArray(),sortedindex,false); // false means in increasing order
        // 1.2 fill PEMap using GTU info + find FoV bounds
    Float_t tof = 0.;
    EPhotoElectron* pe = 0;
    UInt_t GTUnb(0), oldGTUnb(100000);
    fFoV_xmin = 100000;
    fFoV_ymin = 0;
    fFoV_xmax = 0;
    fFoV_ymax = -100000;
    fFoV_xmin2 = 100000;
    fFoV_ymin2 = 0;
    fFoV_xmax2 = 0;
    fFoV_ymax2 = -100000;
    fFoV_xmin_sig = 100000;
    fFoV_ymin_sig = 0;
    fFoV_xmax_sig = 0;
    fFoV_ymax_sig = -100000;
    for(Int_t i=0; i < fNumPe; i++) {
        //
	pe = fDetector->GetPhotoElectron(sortedindex[i]);
        tof = pe->GetTime();
	GTUnb = UInt_t(floor( (tof - fFirstGtuTime) / fGTUlength ));
	if(GTUnb == 0 && oldGTUnb == 100000)   PEMap[0] = new vector<EPhotoElectron*>;
	else if(oldGTUnb < GTUnb)              PEMap[GTUnb] = new vector<EPhotoElectron*>;
	else if(GTUnb > 0 && oldGTUnb > GTUnb) Printf("<Preprocess> It seems pe- are not chronologically sorted");
	PEMap[GTUnb]->push_back(pe);
	oldGTUnb = GTUnb;
	//
	Int_t pix_i = - fDetector->GetPixelRowID(pe->GetPixelID());
	Int_t pix_j = fDetector->GetPixelColumnID(pe->GetPixelID());
	if(pix_i < fFoV_ymin) fFoV_ymin = pix_i;
	if(pix_i > fFoV_ymax) fFoV_ymax = pix_i;
	if(pix_j < fFoV_xmin) fFoV_xmin = pix_j;
	if(pix_j > fFoV_xmax) fFoV_xmax = pix_j;
	if(pe->GetSNR() > 2) {
	    if(pix_i < fFoV_ymin2) fFoV_ymin2 = pix_i;
	    if(pix_i > fFoV_ymax2) fFoV_ymax2 = pix_i;
	    if(pix_j < fFoV_xmin2) fFoV_xmin2 = pix_j;
	    if(pix_j > fFoV_xmax2) fFoV_xmax2 = pix_j;
	}
	if(pe->GetSNR() > 3) {
	    if(pix_i < fFoV_ymin_sig) fFoV_ymin_sig = pix_i;
	    if(pix_i > fFoV_ymax_sig) fFoV_ymax_sig = pix_i;
	    if(pix_j < fFoV_xmin_sig) fFoV_xmin_sig = pix_j;
	    if(pix_j > fFoV_xmax_sig) fFoV_xmax_sig = pix_j;
	}
    }
    
    
    // 2. build "Nbpixels_GTU" histo
    TH1F* h  = GetTimeHisto("Nbpixels_GTU_all","Mean bgnd to be substracted");
    TH1F* h0 = GetTimeHisto("Nbpixels_GTU_0sigma");
    TH1F* h1 = GetTimeHisto("Nbpixels_GTU_1sigma");
    TH1F* h2 = GetTimeHisto("Nbpixels_GTU_2sigma");
    TH1F* h3 = GetTimeHisto("Nbpixels_GTU_3sigma");
    TH1F* h4 = GetTimeHisto("Nbpixels_GTU_trigg");
    TH1F* Signal = GetTimeHisto("Nbpixels_GTU_Signal"); // just to know nb of pixels hit by signal pe-
    
        // 2.1 loop on GTUs
    map<UInt_t, vector<EPhotoElectron*>* >::const_iterator it;
    for(it = PEMap.begin(); it != PEMap.end(); it++) {    
	vector<EPhotoElectron*>* pelist = it->second;

	// if pe- exist in this GTU
	if(pelist) {
	    // init
	    Int_t prev_pixID = -1;
	    Int_t pixID = 0;
            TArrayI pixelIDs(pelist->size());
	    
	    // sort pe- according to pixelID
	    for(size_t n=0; n < pelist->size(); n++) pixelIDs[n] = Int_t((*pelist)[n]->GetPixelID());
            Int_t sortedindex[pelist->size()];
            Sort((Int_t)pelist->size(),pixelIDs.GetArray(),sortedindex,false); // false means in increasing order
	    
	    // then fill Nbpixels_GTU histo once per pixel
	    for(size_t m=0; m < pelist->size(); m++) {
	        EPhotoElectron * pe = (*pelist)[sortedindex[m]];
		pixID = Int_t(pe->GetPixelID());
		if(pixID > prev_pixID) {
		    UInt_t GTUid = it->first;
		    h->Fill(GTUid,fMeanBgnd);
		    if(pe->GetSNR() > 0)  h0->Fill(GTUid,fMeanBgnd);
		    if(pe->GetSNR() > 1)  h1->Fill(GTUid,fMeanBgnd);
		    if(pe->GetSNR() > 2)  h2->Fill(GTUid,fMeanBgnd);
		    if(pe->GetSNR() > 3)  h3->Fill(GTUid,fMeanBgnd);
		    if(pe->IsTriggered()) h4->Fill(GTUid,fMeanBgnd);
		    if(pe->GetType() == 0 && pe->GetAtmoHistory()==0) Signal->Fill(GTUid,1.);
		    if(pe->GetType() == 1 && pe->GetAtmoHistory()==1) Signal->Fill(GTUid,1.);
		}
		else if(prev_pixID > pixID) Printf("<Preprocess> It seems pe- are not pixeliD sorted");
		prev_pixID = pixID;
	    }
	}
    }
    
    // delete map
    map<UInt_t, vector<EPhotoElectron*>* >::const_iterator it2;
    for(it2 = PEMap.begin(); it2 != PEMap.end(); it2++) {    
        vector<EPhotoElectron*>* pelist = it2->second;
        if(pelist) {
            pelist->clear();
            SafeDelete(pelist);
        }
    }
    PEMap.clear();
} 

//_____________________________________________________________________________
void ESimpleDetectorPainter::BuildAll() {
    //
    // Build
    // 
    
    BuildTimeHistos();
    BuildWlHistos();
    BuildFoVHistos();
} 

//_____________________________________________________________________________
void ESimpleDetectorPainter::BuildTimeHistos() {
    //
    // generate histos of photons on pupil as function of time since PRIMARY cosmic ray first interaction
    //
    
    // if no data available
    if ( !fDetector ) {
        Info("BuildTimeHistos()","ESimpleDetector object is NULL. Painter made zombie.");
        MakeZombie();
        return;
    }
        
    // if no pe- exist
    if ( fNumPe <= 0 ) {
        Info("BuildWlHistos()","No PhotoElectrons. Painter made zombie.");
        MakeZombie();
        return;
    }
    
		
    // all pe-
    TH1F *th_tot0  = GetTimeHisto("Pe_total_all","All PhotoElectrons vs time (<B> substracted)");
    TH1F *th_tot  = GetTimeHisto("Pe_fluo_all","Fluo PhotoElectrons vs time (<B> substracted)");
    TH1F *th_tot1  = GetTimeHisto("Pe_ckov_all","Ckov PhotoElectrons vs time (<B> substracted)");
    TH1F *th_tot2  = GetTimeHisto("Pe_bgnd_all","Bgnd PhotoElectrons vs time (<B> substracted)");
    
    // all w.r.t type (0sigma)
    TH1F *th030  = GetTimeHisto("Pe_total_0sigma","PhotoElectrons > <B> vs time (<B> substracted)");
    TH1F *th03  = GetTimeHisto("Pe_fluo_0sigma","Fluo PhotoElectrons vs time (<B> substracted)");
    TH1F *th031 = GetTimeHisto("Pe_ckov_0sigma","Ckov PhotoElectrons vs time (<B> substracted)");
    TH1F *th032 = GetTimeHisto("Pe_bgnd_0sigma","Bgnd PhotoElectrons vs time (<B> substracted)");
    // only triggered (nb sigma set in detector simu)
    TH1F *th130  = GetTimeHisto("Pe_total_trigg","Triggered PhotoElectrons vs time");
    TH1F *th13  = GetTimeHisto("Pe_fluo_trigg","Fluo Triggered PhotoElectrons vs time");
    TH1F *th131 = GetTimeHisto("Pe_ckov_trigg","Ckov Triggered PhotoElectrons vs time");
    TH1F *th132 = GetTimeHisto("Pe_bgnd_trigg","Bgnd Triggered PhotoElectrons vs time");
    // above 1sigma
    TH1F *th230  = GetTimeHisto("Pe_total_1sigma","PhotoElectrons > 1sigma vs time (<B> substracted)");
    TH1F *th23  = GetTimeHisto("Pe_fluo_1sigma","Fluo PhotoElectrons > 1sigma vs time");
    TH1F *th231 = GetTimeHisto("Pe_ckov_1sigma","Ckov PhotoElectrons > 1sigma vs time");
    TH1F *th232 = GetTimeHisto("Pe_bgnd_1sigma","Bgnd PhotoElectrons > 1sigma vs time");
    // above 2sigma
    TH1F *th330  = GetTimeHisto("Pe_total_2sigma","PhotoElectrons > 2sigma vs time (<B> substracted)");
    TH1F *th33  = GetTimeHisto("Pe_fluo_2sigma","Fluo PhotoElectrons > 2sigma vs time");
    TH1F *th331 = GetTimeHisto("Pe_ckov_2sigma","Ckov PhotoElectrons > 2sigma vs time");
    TH1F *th332 = GetTimeHisto("Pe_bgnd_2sigma","Bgnd PhotoElectrons > 2sigma vs time");
    // above 3sigma
    TH1F *th430  = GetTimeHisto("Pe_total_3sigma","PhotoElectrons > 3sigma vs time (<B> substracted)");
    TH1F *th43  = GetTimeHisto("Pe_fluo_3sigma","Fluo PhotoElectrons > 3sigma vs time");
    TH1F *th431 = GetTimeHisto("Pe_ckov_3sigma","Ckov PhotoElectrons > 3sigma vs time");
    TH1F *th432 = GetTimeHisto("Pe_bgnd_3sigma","Bgnd PhotoElectrons > 3sigma vs time");
    
    // all w.r.t atmosphere last history
/*
    TH1F *th04  = GetTimeHisto("Pe_direct","Direct PhotoElectrons vs time");
    TH1F *th041 = GetTimeHisto("Pe_refl","Reflected PhotoElectrons vs time");
    TH1F *th042 = GetTimeHisto("Pe_rayl","Rayleigh scat PhotoElectrons vs time");
    TH1F *th043 = GetTimeHisto("Pe_cloud","Cloud scat PhotoElectrons vs time");
    TH1F *th044 = GetTimeHisto("Pe_aerosol","Aerosol scat PhotoElectrons vs time");
    // only triggered
    TH1F *th14  = GetTimeHisto("Pe_direct_trigg","Direct PhotoElectrons vs time");
    TH1F *th141 = GetTimeHisto("Pe_refl_trigg","Reflected PhotoElectrons vs time");
    TH1F *th142 = GetTimeHisto("Pe_rayl_trigg","Rayleigh scat PhotoElectrons vs time");
    TH1F *th143 = GetTimeHisto("Pe_cloud_trigg","Cloud scat PhotoElectrons vs time");
    TH1F *th144 = GetTimeHisto("Pe_aerosol_trigg","Aerosol scat PhotoElectrons vs time");
*/
    
    // interaction order contribution
    TString prefix("Pe_nbinter");
    TString suffix_tot("_all");
    TString suffix("_trigg");
    TString suffix0("_0sigma");
    TString suffix1("_1sigma");
    TString suffix2("_2sigma");
    TString suffix3("_3sigma");
    TString name(prefix);
    TH1F* temp_all = 0;
    TH1F* temp_sigma0 = 0;
    TH1F* temp_trigg = 0;
    TH1F* temp_sigma1 = 0;
    TH1F* temp_sigma2 = 0;
    TH1F* temp_sigma3 = 0;
    for(Int_t i=0; i<=fMaxScatOrder; i++) {
	// 0sigma
        name = prefix;
	name += i;
	name += suffix_tot;
	temp_all = GetTimeHisto(name.Data(),"ALl PhotoElectrons v.s. atmo scattering order");
	// 0sigma
        name = prefix;
	name += i;
	name += suffix0;
	temp_sigma0 = GetTimeHisto(name.Data(),">0sigma PhotoElectrons v.s. atmo scattering order");
	// only triggered
        name = prefix;
	name += i;
	name += suffix;
	temp_trigg = GetTimeHisto(name.Data(),"Triggered PhotoElectrons v.s. atmo scattering order");
	// 1sigma
        name = prefix;
	name += i;
	name += suffix1;
	temp_sigma1 = GetTimeHisto(name.Data(),">1sigma PhotoElectrons v.s. atmo scattering order");
	// 2sigma
        name = prefix;
	name += i;
	name += suffix2;
	temp_sigma2 = GetTimeHisto(name.Data(),">2sigma PhotoElectrons v.s. atmo scattering order");
	// 3sigma
        name = prefix;
	name += i;
	name += suffix3;
	temp_sigma3 = GetTimeHisto(name.Data(),">3sigma PhotoElectrons v.s. atmo scattering order");
    }
              
    
  
    // fill histograms
    TString histonb0(prefix);
    TString histonb_tot(prefix);
    TString histonb_trigg(prefix);
    TString histonb1(prefix);
    TString histonb2(prefix);
    TString histonb3(prefix);
    for (Int_t i=0; i<fNumPe; i++) { 
        EPhotoElectron* pe = fDetector->GetPhotoElectron(i);
	// look for histo matching the pe- AtmoNbinter
	Int_t nbinter  = pe->GetAtmoNbinter();
	
	// all
	histonb_tot = prefix;
	histonb_tot += nbinter;
	histonb_tot += suffix_tot;
	temp_all = (TH1F*)fHistos->FindObject(histonb_tot.Data());
	// 0sigma
	histonb0 = prefix;
	histonb0 += nbinter;
	histonb0 += suffix0;
	temp_sigma0 = (TH1F*)fHistos->FindObject(histonb0.Data());
	// trigg only
	histonb_trigg = prefix;
	histonb_trigg += nbinter;
	histonb_trigg += suffix;
	temp_trigg = (TH1F*)fHistos->FindObject(histonb_trigg.Data());
	// 1sigma
	histonb1 = prefix;
	histonb1 += nbinter;
	histonb1 += suffix1;
	temp_sigma1 = (TH1F*)fHistos->FindObject(histonb1.Data());
	// 2sigma
	histonb2 = prefix;
	histonb2 += nbinter;
	histonb2 += suffix2;
	temp_sigma2 = (TH1F*)fHistos->FindObject(histonb2.Data());
	// 3sigma
	histonb3 = prefix;
	histonb3 += nbinter;
	histonb3 += suffix3;
	temp_sigma3 = (TH1F*)fHistos->FindObject(histonb3.Data());
        
	
	Float_t time = (pe->GetTime() - fFirstGtuTime) / fGTUlength;
	
	
	
	
	
	// 1. if pe- triggered
	if(pe->IsTriggered())  {
	    if(temp_trigg && pe->GetType() < 2) temp_trigg->Fill(time);
	    else if(pe->GetType() < 2) Printf("<BuildTimeHistos> Wrong histo identification  %s",histonb_trigg.Data());
	    
	    th130->Fill(time,1);
            if ( pe->GetType() == 0 ) { // fluo
		th13->Fill(time,1);
		//if(pe->GetAtmoHistory()==0) th14->Fill(time,1);
		//if(pe->GetAtmoHistory()==1) th141->Fill(time,1);
		//if(pe->GetAtmoHistory()==2) th142->Fill(time,1);
        	//if(pe->GetAtmoHistory()==3) th143->Fill(time,1);
        	//if(pe->GetAtmoHistory()==4) th144->Fill(time,1);
	    }
            else if ( pe->GetType() == 1 ) { // ckov
		th131->Fill(time,1);
		//if(pe->GetAtmoHistory()==0) th14->Fill(time,1);
		//if(pe->GetAtmoHistory()==1) th141->Fill(time,1);
		//if(pe->GetAtmoHistory()==2) th142->Fill(time,1);
        	//if(pe->GetAtmoHistory()==3) th143->Fill(time,1);
        	//if(pe->GetAtmoHistory()==4) th144->Fill(time,1);
            }
            else if ( pe->GetType() == 2 ) { // bgnd
		th132->Fill(time,1);
		//if(pe->GetAtmoHistory()==0) th14->Fill(time,1);
		//if(pe->GetAtmoHistory()==1) th141->Fill(time,1);
		//if(pe->GetAtmoHistory()==2) th142->Fill(time,1);
        	//if(pe->GetAtmoHistory()==3) th143->Fill(time,1);
        	//if(pe->GetAtmoHistory()==4) th144->Fill(time,1);
            }
	}
	
	
	
	// 2. pe- as function of SNR
	th_tot0->Fill(time);
	if(temp_all && pe->GetType() < 2) temp_all->Fill(time);
	else if(pe->GetType() < 2) Printf("<BuildTimeHistos> ALL. Wrong histo identification  %s",histonb_tot.Data());
        // fluo
	if ( pe->GetType() == 0 ) {
	    th_tot->Fill(time,1);
	    if(pe->GetSNR() > 0) {
	        th03->Fill(time,1);
	        th030->Fill(time,1);
	        if(temp_sigma0) temp_sigma0->Fill(time);
	        else Printf("<BuildTimeHistos> 0. Wrong histo identification  %s",histonb0.Data());
	    }
	    if(pe->GetSNR() > 1) {
	        if(temp_sigma1) temp_sigma1->Fill(time);
	        else Printf("<BuildTimeHistos> 1. Wrong histo identification  %s",histonb1.Data());
		th23->Fill(time);
	        th230->Fill(time,1);
	    }
	    if(pe->GetSNR() > 2) {
	        if(temp_sigma2) temp_sigma2->Fill(time);
	        else Printf("<BuildTimeHistos> 2. Wrong histo identification  %s",histonb2.Data());
		th33->Fill(time);
	        th330->Fill(time,1);
	    }
	    if(pe->GetSNR() > 3){
	        if(temp_sigma3) temp_sigma3->Fill(time);
	        else Printf("<BuildTimeHistos> 3. Wrong histo identification  %s",histonb3.Data());
		th43->Fill(time);
	        th430->Fill(time,1);
	    }
	    //if(pe->GetAtmoHistory()==0) th04->Fill(time,1);
	    //if(pe->GetAtmoHistory()==1) th041->Fill(time,1);
	    //if(pe->GetAtmoHistory()==2) th042->Fill(time,1);
            //if(pe->GetAtmoHistory()==3) th043->Fill(time,1);
            //if(pe->GetAtmoHistory()==4) th044->Fill(time,1);
	}
        // ckov
        else if ( pe->GetType() == 1 ) {
	    th_tot1->Fill(time,1);
	    if(pe->GetSNR() > 0) {
	        th030->Fill(time,1);
     	        th031->Fill(time,1);
	        if(temp_sigma0) temp_sigma0->Fill(time);
	        else Printf("<BuildTimeHistos> 0. Wrong histo identification  %s",histonb0.Data());
	    }
	    if(pe->GetSNR() > 1) {
	        if(temp_sigma1) temp_sigma1->Fill(time);
	        else Printf("<BuildTimeHistos> 1. Wrong histo identification  %s",histonb1.Data());
		th231->Fill(time);
	        th230->Fill(time,1);
	    }
	    if(pe->GetSNR() > 2) {
	        if(temp_sigma2) temp_sigma2->Fill(time);
	        else Printf("<BuildTimeHistos> 2. Wrong histo identification  %s",histonb2.Data());
		th331->Fill(time);
	        th330->Fill(time,1);
	    }
	    if(pe->GetSNR() > 3){
	        if(temp_sigma3) temp_sigma3->Fill(time);
	        else Printf("<BuildTimeHistos> 3. Wrong histo identification  %s",histonb3.Data());
		th431->Fill(time);
	        th430->Fill(time,1);
	    }
	    //if(pe->GetAtmoHistory()==0) th04->Fill(time,1);
	    //if(pe->GetAtmoHistory()==1) th041->Fill(time,1);
	    //if(pe->GetAtmoHistory()==2) th042->Fill(time,1);
            //if(pe->GetAtmoHistory()==3) th043->Fill(time,1);
            //if(pe->GetAtmoHistory()==4) th044->Fill(time,1);
        }
        // bgnd
        else if ( pe->GetType() == 2 ) {
	    th_tot2->Fill(time,1);
	    if(pe->GetSNR() > 0) {
	        th030->Fill(time,1);
     	        th032->Fill(time,1);
	    }
	    if(pe->GetSNR() > 1) {
		th232->Fill(time);
	        th230->Fill(time,1);
	    }
	    if(pe->GetSNR() > 2) {
		th332->Fill(time);
	        th330->Fill(time,1);
	    }
	    if(pe->GetSNR() > 3){
		th432->Fill(time);
	        th430->Fill(time,1);
	    }
	    //if(pe->GetAtmoHistory()==0) th04->Fill(time,1);
	    //if(pe->GetAtmoHistory()==1) th041->Fill(time,1);
	    //if(pe->GetAtmoHistory()==2) th042->Fill(time,1);
            //if(pe->GetAtmoHistory()==3) th043->Fill(time,1);
            //if(pe->GetAtmoHistory()==4) th044->Fill(time,1);
        }
    }
}

//_____________________________________________________________________________
void ESimpleDetectorPainter::BuildWlHistos() {
    //
    // generate also wavelength spectra of photons that gave pe-
    //

    // if no data available
    if ( !fDetector ) {
        Info("BuildWlHistos()","EAtmosphere object is NULL. Painter made zombie.");
        MakeZombie();
        return;
    }
        
    // if no pe- exist
    if ( fNumPe <= 0 ) {
        Info("BuildWlHistos()","No PhotoElectrons. Painter made zombie.");
        MakeZombie();
        return;
    }
    
    
    
    // type contribution
    TH1F *th03  = GetWlHisto("Pe_wl_fluo","Fluo signal PhotoElectrons vs wl");
    TH1F *th031 = GetWlHisto("Pe_wl_ckov","Ckov signal PhotoElectrons vs wl");
    // only triggered
    TH1F *th13  = GetWlHisto("Pe_wl_fluo_trigg","Fluo signal Triggered PhotoElectrons vs wl");
    TH1F *th131 = GetWlHisto("Pe_wl_ckov_trigg","Ckov signal Triggered PhotoElectrons vs wl");
    
  
  
    // fill histograms
    for (Int_t i=0; i<fNumPe; i++) { 
        EPhotoElectron* pe = fDetector->GetPhotoElectron(i);
        Float_t wl = pe->GetWl();
	if(wl == kHuge) continue;  // means bgnd pe-
	wl /= nm;
	
	// if pe- triggered
	if(pe->IsTriggered())  {
            if ( pe->GetType() == 0 ) { // fluo
		th13->Fill(wl,1);
	    }
            else if ( pe->GetType() == 1 ) { // ckov
		th131->Fill(wl,1);
            }
	}
	
	// all pe- even those not triggered
        if ( pe->GetType() == 0 ) { // fluo
	    th03->Fill(wl,1);
	}
        else if ( pe->GetType() == 1 ) { // ckov
	    th031->Fill(wl,1);
        }
    }
    
}

//_____________________________________________________________________________
void ESimpleDetectorPainter::BuildFoVHistos() {
    //
    // generate FoV histo : pixel_i vs/ pixel_j
    // ONLY SIGNAL PHOTO ELECTRONS (excepted one plot)
    //
    
    // if no data available
    if ( !fDetector ) {
        Info("BuildTimeHistos()","ESimpleDetector object is NULL. Painter made zombie.");
        MakeZombie();
        return;
    }
        
    // if no pe- exist
    if ( fNumPe <= 0 ) {
        Info("BuildWlHistos()","No PhotoElectrons. Painter made zombie.");
        MakeZombie();
        return;
    }
    
		
    // all (<B> substracted)
    TH2I *th  = GetFoVHisto("Pe_Fov_all","All PhotoElectrons");
    // only triggered (nb sigma set in detector simu)
    TH2I *th_trigg  = GetFoVHisto("Pe_Fov_trigg","Triggered PhotoElectrons",3);
    // above 0sigma
    TH2I *th0  = GetFoVHisto("Pe_Fov_0sigma","PhotoElectrons > <B>");
    // above 1sigma
    TH2I *th1  = GetFoVHisto("Pe_Fov_1sigma","PhotoElectrons > 1sigma");
    // above 2sigma
    TH2I *th2  = GetFoVHisto("Pe_Fov_2sigma","PhotoElectrons > 2sigma",2);
    // above 3sigma
    TH2I *th3  = GetFoVHisto("Pe_Fov_3sigma","PhotoElectrons > 3sigma",3);
    


    // fill histograms
    for (Int_t k=0; k<fNumPe; k++) { 
        EPhotoElectron* pe = fDetector->GetPhotoElectron(k);
	Int_t i = - fDetector->GetPixelRowID(pe->GetPixelID());
	Int_t j = fDetector->GetPixelColumnID(pe->GetPixelID());
	
	th->Fill(j,i);
	if(pe->IsTriggered()) th_trigg->Fill(j,i);
	if(pe->GetSNR() > 0) th0->Fill(j,i);
	if(pe->GetSNR() > 1) th1->Fill(j,i);
	if(pe->GetSNR() > 2) th2->Fill(j,i);
	if(pe->GetSNR() > 3) th3->Fill(j,i);
    }
}

//_____________________________________________________________________________
TH1F *ESimpleDetectorPainter::GetTimeHisto(const char *name, const char *title) {
    //
    // Get wavelength histograms 
    //

    if ( title == 0 ) title = name;
    TH1F* th = (TH1F*)fHistos->FindObject(name);

    //Int_t nbins = fNbGTUs + 20;    

    if (!th) {
      th = new TH1F( name, title, 150, 0, 150);
        fHistos->Add(th);
        th->SetStats(0);
        th->SetXTitle("Time  (GTU)");
        th->SetFillColor(0);
        //th->SetDirectory(0);
    }

    return th;
}

//_____________________________________________________________________________
TH1F *ESimpleDetectorPainter::GetWlHisto(const char *name, const char *title) {
    //
    // Get wavelength histograms 
    //

    if ( title == 0 ) title = name;
    TH1F* th = (TH1F*)fHistos->FindObject(name);

    if (!th) {
      th = new TH1F( name, title, 200, 200., 600.);
        fHistos->Add(th);
        th->SetStats(0);
        th->SetXTitle("Wavelength, nm");
        th->SetFillColor(9);
        //th->SetDirectory(0);
    }

    return th;
}

//_____________________________________________________________________________
TH2I *ESimpleDetectorPainter::GetFoVHisto(const char *name, const char *title,Int_t nbsigmas) {
    //
    // Get FoV histograms 
    //

    if ( title == 0 ) title = name;
    TH2I* th = (TH2I*)fHistos->FindObject(name);
    Int_t xmin, xmax, ymin, ymax;
    if(nbsigmas<0) {
        xmin = fFoV_xmin;
	xmax = fFoV_xmax;
	ymin = fFoV_ymin;
	ymax = fFoV_ymax;
    }
    else if(nbsigmas == 2) {
        xmin = fFoV_xmin2;
	xmax = fFoV_xmax2;
	ymin = fFoV_ymin2;
	ymax = fFoV_ymax2;
    }
    else {
        xmin = fFoV_xmin_sig;
	xmax = fFoV_xmax_sig;
	ymin = fFoV_ymin_sig;
	ymax = fFoV_ymax_sig;
    }

    if (!th) {
      th = new TH2I( name, title,(xmax - xmin) + 2,xmin - 1,xmax + 1,(ymax - ymin) + 2,ymin - 1,ymax + 1);
        fHistos->Add(th);
        th->SetXTitle("pixel X");
        th->SetYTitle("pixel Y");
        th->SetDirectory(0);
    }

    return th;
}

//______________________________________________________________________________
void ESimpleDetectorPainter::Draw(Option_t *opt) {
    //
    //
    //
    
    if ( IsZombie() ) return;
    TString option(opt);
    TH1F* th(0);
    TH1F* th1(0);
    TH1F* th2(0);
    TH1F* th3(0);
    TH1F* th4(0);
    TH1F* th5(0);
    TH1F* th6(0);
    TH1* hcopy(0);
    TH2I* h2Dint(0);
    if ( option == "" ) option = "Nph";


// ************************************ PhotoElectrons TIME in DETECTOR ***************************************//
    
    // <B> per GTU (summed over hit pixels)
    else if ( option == "Nbpixels_GTU" ){
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th = GetTimeHisto("Nbpixels_GTU_all");
	th->SetLineColor(2);
        sprintf(txt," Total    %.1f",th->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th,name.Data(),"f");
	th->Draw();
        
	th1 = GetTimeHisto("Nbpixels_GTU_0sigma");
	th1->SetFillColor(7);
	th1->SetLineColor(7);
        sprintf(txt," 0 sigma    %.1f",th1->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th1,name.Data(),"f");
	th1->Draw("same");
        
	th2 = GetTimeHisto("Nbpixels_GTU_1sigma");
	th2->SetFillColor(6);
	th2->SetLineColor(6);
        sprintf(txt," 1 sigma    %.1f",th2->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th2,name.Data(),"f");
	th2->Draw("same");
	
        th3 = GetTimeHisto("Nbpixels_GTU_2sigma");
	th3->SetFillColor(3);
	th3->SetLineColor(3);
        sprintf(txt," 2 sigma    %.1f",th3->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th3,name.Data(),"f");
	th3->Draw("same");

        th4 = GetTimeHisto("Nbpixels_GTU_3sigma");
	th4->SetFillColor(9);
	th4->SetLineColor(9);
        sprintf(txt," 3 sigma    %.1f",th4->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th4,name.Data(),"f");
	th4->Draw("same");

        th5 = GetTimeHisto("Nbpixels_GTU_trigg");
	th5->SetFillColor(2);
	th5->SetLineColor(2);
        sprintf(txt," 4 sigma    %.1f",th5->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th5,name.Data(),"f");
	th5->Draw("same");
	
        th6 = GetTimeHisto("Nbpixels_GTU_Signal");
	th6->SetFillColor(1);
        sprintf(txt," Signal    %.1f",th6->Integral());
	name = TString(txt);
	name += TString("  ");
        leg->AddEntry(th6,name.Data(),"f");
	th6->Draw("same");
	
	leg->Draw();
    }
    
    // type contribution : all pe- (no SNR treatment)
    else if ( option == "PEtime_type_all" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th  = GetTimeHisto("Pe_total_all");
        th1 = GetTimeHisto("Pe_fluo_all");
        th2 = GetTimeHisto("Pe_ckov_all");
        th3 = GetTimeHisto("Pe_bgnd_all");
	th4 = GetTimeHisto("Nbpixels_GTU_all");
	
	THStack* STACK_PEtype = new THStack("STACK_PEtypeall","Type contribution - All pe^{-}");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone();
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone();
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype->Draw("");
        if ( th ) {
            hcopy = (TH1F*)th->Clone();
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
        }    
	leg->Draw();  
    }
    
    // type contribution : above <B>
    else if ( option == "PEtime_type_0sigma" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th  = GetTimeHisto("Pe_total_0sigma");
        th1 = GetTimeHisto("Pe_fluo_0sigma");
        th2 = GetTimeHisto("Pe_ckov_0sigma");
        th3 = GetTimeHisto("Pe_bgnd_0sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_0sigma");
	
	THStack* STACK_PEtype = new THStack("STACK_PEtype0","Type contribution - pe^{-}  >  <B>");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone();
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone();
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype->Draw("");
        if ( th ) {
            hcopy = (TH1F*)th->Clone();
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
        }    
	leg->Draw();  
    }

    // type contribution : above <B> + 1sigma
    else if ( option == "PEtime_type_1sigma" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th = GetTimeHisto("Pe_total_1sigma");
        th1 = GetTimeHisto("Pe_fluo_1sigma");
        th2 = GetTimeHisto("Pe_ckov_1sigma");
        th3 = GetTimeHisto("Pe_bgnd_1sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_1sigma");
	
	THStack* STACK_PEtype = new THStack("STACK_PEtype1","Type contribution - pe^{-}  >  1#sigma");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone();
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone();
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 	    hcopy->Add(th4,-1.);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype->Draw();
        if ( th ) {
            hcopy = (TH1F*)th->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
        }    
	leg->Draw();  
    }

    // type contribution : above <B> + 2sigma
    else if ( option == "PEtime_type_2sigma" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th = GetTimeHisto("Pe_total_2sigma");
        th1 = GetTimeHisto("Pe_fluo_2sigma");
        th2 = GetTimeHisto("Pe_ckov_2sigma");
        th3 = GetTimeHisto("Pe_bgnd_2sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_2sigma");
	
	THStack* STACK_PEtype = new THStack("STACK_PEtype2","Type contribution - pe^{-}  >  2#sigma");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone();
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone();
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype->Draw("");
        if ( th ) {
            hcopy = (TH1F*)th->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
        }    
	leg->Draw();  
    }

    // type contribution : above <B> + 3sigma
    else if ( option == "PEtime_type_3sigma" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th = GetTimeHisto("Pe_total_3sigma");
        th1 = GetTimeHisto("Pe_fluo_3sigma");
        th2 = GetTimeHisto("Pe_ckov_3sigma");
        th3 = GetTimeHisto("Pe_bgnd_3sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_3sigma");
	
	THStack* STACK_PEtype = new THStack("STACK_PEtype3","Type contribution - pe^{-}  >  3#sigma");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone();
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone();
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype->Draw("");
        if ( th ) {
            hcopy = (TH1F*)th->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %.2f",hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
        }    
	leg->Draw();  
    }

    // type contribution - triggered only
    else if ( option == "PEtime_type_trigg" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        th = GetTimeHisto("Pe_total_trigg");
        th1 = GetTimeHisto("Pe_fluo_trigg");
        th2 = GetTimeHisto("Pe_ckov_trigg");
        th3 = GetTimeHisto("Pe_bgnd_trigg");
	THStack* STACK_PEtype_trigg = new THStack("STACK_PEtype_trigg","Type contribution");
        if ( th ) {
            hcopy = (TH1F*)th->DrawCopy(""); 
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone(); 
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone(); 
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->DrawCopy("same"); 
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype_trigg->Draw("same");
	leg->Draw();  
    }
    
    // type contribution - triggered only -- Bgnd substracted
    else if ( option == "PEtime_type_trigg_bgndsub" ) { 
	TString name("name");
        char txt[200];
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
	TLegend* leg_warning = new TLegend(0.3,0.8,0.6,0.9);
        th = GetTimeHisto("Pe_total_trigg");
        th1 = GetTimeHisto("Pe_fluo_trigg");
        th2 = GetTimeHisto("Pe_ckov_trigg");
        th3 = GetTimeHisto("Pe_bgnd_trigg");
	th4 = GetTimeHisto("Nbpixels_GTU_trigg");
	THStack* STACK_PEtype_trigg = new THStack("STACK_PEtype_trigg_bgndsub","Type contribution -- triggered pe^{-} -- <B> substracted");
        if ( th1 ) {
            hcopy = (TH1F*)th1->Clone(); 
            hcopy->SetFillColor(2);
            sprintf(txt," Fluo   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th2 ) {
            hcopy = (TH1F*)th2->Clone(); 
            hcopy->SetFillColor(4);
            sprintf(txt," Ckov   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
        if ( th3 ) {
            hcopy = (TH1F*)th3->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetFillColor(17);
            sprintf(txt," Bgnd   %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    STACK_PEtype_trigg->Add(hcopy);
            leg->AddEntry(hcopy,name.Data(),"f");
        }    
	STACK_PEtype_trigg->Draw("");
        if ( th ) {
            hcopy = (TH1F*)th->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
            hcopy->SetLineColor(1);
            hcopy->SetFillColor(0);
            sprintf(txt," Total    %d",(Int_t)hcopy->Integral());
	    name = TString(txt);
	    name += TString("  ");
	    hcopy->Draw("same");
            leg->AddEntry(hcopy,name.Data(),"l");
	    leg_warning->AddEntry(hcopy,"OK only if trigg = 4#sigma","");
        }    
	leg->Draw();  
	leg_warning->Draw();  
    }
    
    // nb of scattering in atmo contribution : above <B>
    else if ( option == "PEtime_atmonbinter_all" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_all");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter = new THStack("STACK_PEnbinterall","Scattering order contribution - All pe-");
        th2 = GetTimeHisto("Pe_total_all");
        th1 = GetTimeHisto("Pe_bgnd_all");
	th4 = GetTimeHisto("Nbpixels_GTU_all");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	   hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter->Draw("");
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contribution - All pe-");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
        } 
	leg->Draw();
    }
   
    
    // nb of scattering in atmo contribution : above <B>
    else if ( option == "PEtime_atmonbinter_0sigma" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_0sigma");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter = new THStack("STACK_PEnbinter0","Scattering order contribution > <B>");
        th2 = GetTimeHisto("Pe_total_0sigma");
        th1 = GetTimeHisto("Pe_bgnd_0sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_0sigma");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	   hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter->Draw();
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contribution > <B>");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
        } 
	leg->Draw();
    }

    // nb of scattering in atmo contribution : above <B>
    else if ( option == "PEtime_atmonbinter_1sigma" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_1sigma");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter = new THStack("STACK_PEnbinter1","Scattering order contribution > 1 sigma");
        th2 = GetTimeHisto("Pe_total_1sigma");
        th1 = GetTimeHisto("Pe_bgnd_1sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_1sigma");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	   hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter->Draw();
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("cattering order contribution > 1 sigma");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
        } 
	leg->Draw();
    }

    // nb of scattering in atmo contribution : above <B>
    else if ( option == "PEtime_atmonbinter_2sigma" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_2sigma");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter = new THStack("STACK_PEnbinter2","Scattering order contribution > 2 sigma");
        th2 = GetTimeHisto("Pe_total_2sigma");
        th1 = GetTimeHisto("Pe_bgnd_2sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_2sigma");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	   hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter->Draw();
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contribution > 2 sigma");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
        } 
	leg->Draw();
    }

    // nb of scattering in atmo contribution : above <B>
    else if ( option == "PEtime_atmonbinter_3sigma" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_3sigma");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter = new THStack("STACK_PEnbinter3","Scattering order contribution > 3 sigma");
        th2 = GetTimeHisto("Pe_total_3sigma");
        th1 = GetTimeHisto("Pe_bgnd_3sigma");
	th4 = GetTimeHisto("Nbpixels_GTU_3sigma");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	   hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter->Draw();
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contribution > 3 sigma");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
        } 
	leg->Draw();
    }

    // nb of scattering in atmo contribution -- triggered only
    else if ( option == "PEtime_atmonbinter_trigg" ) { 
        TString prefix("Pe_nbinter");
        TString suffix("_trigg");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter_trigg = new THStack("STACK_PEnbinter_trigg","Scattering order contribution -- triggered pe^{-}");
        th = GetTimeHisto("Pe_total_trigg");
        th1 = GetTimeHisto("Pe_bgnd_trigg");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
        if( th ) {
           hcopy = (TH1F*)th->Clone(); 
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contributions - Triggered pe^{-}");
	   histonb = TString(" Total  ");
	   histonb += hcopy->Integral();
	   histonb += TString("  ");
	   hcopy->Draw();
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter_trigg->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
	   histonb += hcopy->Integral();
	   histonb += TString("  ");
	   STACK_PEnbinter_trigg->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter_trigg->Draw("same");
	leg->Draw();
    }
    
    // nb of scattering in atmo contribution -- triggered only -- Bgnd substracted
    else if ( option == "PEtime_atmonbinter_trigg_bgndsub" ) { 
        char txt[200];
        TString prefix("Pe_nbinter");
        TString suffix("_trigg");
        TString prefix2("order ");
        TString histonb(prefix);
	THStack* STACK_PEnbinter_trigg = new THStack("STACK_PEnbinter_trigg_bgndsub","Scattering order contribution -- triggered pe^{-} -- <B> substracted");
        th2 = GetTimeHisto("Pe_total_trigg");
        th1 = GetTimeHisto("Pe_bgnd_trigg");
	th4 = GetTimeHisto("Nbpixels_GTU_trigg");
	TLegend* leg = new TLegend(0.6,0.2,0.9,0.9);
	TLegend* leg_warning = new TLegend(0.3,0.8,0.6,0.9);
        for (Int_t i=0; i<=fMaxScatOrder; i++) { 
	    histonb = prefix;
	    histonb += i;
	    histonb += suffix;
	    th = (TH1F*)fHistos->FindObject(histonb.Data());
            if( th ) {
                hcopy = (TH1F*)th->Clone(); 
		histonb = prefix2;
		histonb += i;
		histonb += TString("  ");
		histonb += hcopy->Integral();
		leg->AddEntry(hcopy,histonb.Data(),"f");
		hcopy->SetFillColor(i+2); 
	        STACK_PEnbinter_trigg->Add(hcopy);
	    }
	    else Printf("<Draw TimeHistos> Wrong histo identification  %s",histonb.Data());
	}
        if( th1 ) {
           hcopy = (TH1F*)th1->Clone(); 
	    hcopy->Add(th4,-1);
           hcopy->SetFillColor(17);
	   histonb = TString(" Bgnd  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   STACK_PEnbinter_trigg->Add(hcopy);
	   leg->AddEntry(hcopy,histonb.Data(),"f");
        } 
	STACK_PEnbinter_trigg->Draw("");
        if( th2 ) {
           hcopy = (TH1F*)th2->Clone(); 
	    hcopy->Add(th4,-1);
            hcopy->SetLineWidth(2);
           hcopy->SetLineColor(1);
           hcopy->SetFillColor(0);
	   hcopy->SetTitle("Scattering order contributions - Triggered pe^{-}");
	   histonb = TString(" Total  ");
           sprintf(txt,"%.2f",hcopy->Integral());
	   histonb += TString("  ");
	   histonb += TString(txt);
	   hcopy->Draw("same");
	   leg->AddEntry(hcopy,histonb.Data(),"l");
	   leg_warning->AddEntry(hcopy,"OK only if trigg = 4#sigma","");
        } 
	leg->Draw();
    }

// ************************************ FoV ***************************************//
    else if ( option == "Pe_Fov_all" ) {
        h2Dint = GetFoVHisto("Pe_Fov_all");
	h2Dint->Draw("box");
    }

    else if ( option == "Pe_Fov_trigg" ) {
        h2Dint = GetFoVHisto("Pe_Fov_trigg");
	h2Dint->Draw("box");
    }

    else if ( option == "Pe_Fov_0sigma" ) {
        h2Dint = GetFoVHisto("Pe_Fov_0sigma");
	h2Dint->Draw("box");
    }

    else if ( option == "Pe_Fov_1sigma" ) {
        h2Dint = GetFoVHisto("Pe_Fov_1sigma");
	h2Dint->Draw("box");
    }

    else if ( option == "Pe_Fov_2sigma" ) {
        h2Dint = GetFoVHisto("Pe_Fov_2sigma");
	h2Dint->Draw("box");
    }
    
    else if ( option == "Pe_Fov_3sigma" ) {
        h2Dint = GetFoVHisto("Pe_Fov_3sigma");
	h2Dint->Draw("box");
    }


// ************************************ WAVELENGTH ***************************************//
/*  // all w.r.t type
    TH1F *th03  = GetWlHisto("Pe_wl_fluo","Fluo signal PhotoElectrons vs wl");
    TH1F *th031 = GetWlHisto("Pe_wl_ckov","Ckov signal PhotoElectrons vs wl");
    // only triggered
    TH1F *th13  = GetWlHisto("Pe_wl_fluo_trigg","Fluo signal Triggered PhotoElectrons vs wl");
    TH1F *th131 = GetWlHisto("Pe_wl_ckov_trigg","Ckov signal Triggered PhotoElectrons vs wl");
*/
    
    // FLUO spectrum at pe- level
    else if ( option == "PEwl_fluo" ) {
        th1 = GetWlHisto("Pe_wl_fluo");        
        th2 = GetWlHisto("Pe_wl_fluo_trigg");        
        if ( th1 ) {
           hcopy = (TH1F*)th1->DrawCopy(""); 
           hcopy->SetFillColor(2);
        }  
        if ( th2 ) {
           hcopy = (TH1F*)th2->DrawCopy("same"); 
           hcopy->SetFillColor(4);
        }  
    }

    // CKOV spectrum at pe- level
    else if ( option == "PEwl_ckov" ) {
        th1 = GetWlHisto("Pe_wl_ckov");        
        th2 = GetWlHisto("Pe_wl_ckov_trigg");        
        if ( th1 ) {
           hcopy = (TH1F*)th1->DrawCopy(""); 
           hcopy->SetFillColor(2);
        }  
        if ( th2 ) {
           hcopy = (TH1F*)th2->DrawCopy("same"); 
           hcopy->SetFillColor(4);
        }  
    }

}


