// $Id: ESimuEventViewer.cc 2819 2009-03-13 11:48:16Z naumov $
// Author: Alessandro Thea   2005/05/31

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ESimuEventViewer                                                     *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// ESimuEventViewer
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include <iostream>

#include "ESimuEventViewer.hh"
#include "EShowerPainter.hh"
#include "EShowerHistoPainter.hh"
#include "EAtmosphereHistoPainter.hh"
#include "ESinglePhotonsPainter.hh"
#include "EFocalSurfacePainter.hh"
#include "EFocalSurface3DPainter.hh"
#include "ETriggerPainter.hh"
#include "EDetectorHistoPainter.hh"
#include "EDetectorPhotonsPainter.hh"
#include "EOpticsPhotonsPainter.hh"
#include "Etypes.hh"
#include "EEvent.hh"
#include "ERunParameters.hh"
#include "ETruth.hh"
#include "EDetStatus.hh"
#include "EGeometry.hh"
#include "ESystemOfUnits.hh"
#include "EShower.hh"
#include "EAtmosphere.hh"
#include <TCanvas.h>
#include <TCut.h>
#include <TEventList.h>
#include <TFile.h>
#include <TG3DLine.h>
#include <TGButton.h>
#include <TGComboBox.h>
#include <TGDockableFrame.h>
#include <TGFileDialog.h>
#include <TGLabel.h>
#include <TGString.h>
#include <TGTextBuffer.h>
#include <TGTextEntry.h>
#include <TH1.h>
#include <TTree.h>
#include <TEllipse.h>
#include <TMarker.h>
#include <TLegend.h>
#include "KeySymbols.h"

using namespace std;

ClassImp(ESimuEventViewer)

enum ESimyEventViewerCommands{
        
        kCheckPainter1,
        kCheckPainter2,
        kCheckPainter3,
        kCheckPainter4,
        kCheckPainter5,
        kCheckPainter6,
        
        kButtonNextEvent,
        kButtonPrevEvent,
        kButtonApplyCut,

        kComboNevents=999999
            
};

//using namespace sou;
using namespace TMath;
using namespace sou;

const Float_t kHdetector = 420*km;
const Float_t kFoV = 30*deg;

//_____________________________________________________________________________
ESimuEventViewer::ESimuEventViewer( const char* name=0 ) : EGViewer(kTRUE, "ESimuViewer") {
    //
    // Constructor
    //
    CreateViewer();
    MapWindow();

    fShowerPainter            = 0;
    fShowerHistoPainter       = 0;
    fAtmosphereHistoPainter   = 0;
    fSinglePhotonsPainter     = 0;
    fFocalSurfacePainter      = 0;
    fFocalSurface3DPainter    = 0;
    fTriggerPainter           = 0;
    fFocalSurfaceHistoPainter = 0;
    fDetectorPhotonsPainter   = 0;
    fOpticsPhotonsPainter     = 0;
    
    fEventList = 0;
    fEvTree  = 0;
    fEvent   = 0;
    fTruth   = 0;
    fRunPars = 0;

    fCurrentEventId = -1;
    fCurrentEvent = -1;
    fForceLoading = kTRUE;

    //FIXME
    //Custom style would be better
    //
    gStyle->SetOptStat(0);
//    gStyle->SetOptTitle(0);
    gStyle->SetTitleFont(72,"");
    gStyle->SetTitleFontSize(0.045);
    gStyle->SetPadTopMargin(0.09);
    gStyle->SetPadBottomMargin(0.09);
    gStyle->SetPadLeftMargin(0.09);
    gStyle->SetPadRightMargin(0.09);
    gStyle->SetPadBorderMode(0);


    if ( !(name && !Open(name)) || !OpenDialog() )
        return;
}

//_____________________________________________________________________________
ESimuEventViewer::~ESimuEventViewer() {
    //
    // Destructor
    //

    ClearPainters();
    SafeDelete(fEvent);
    SafeDelete(fEvTree);
}

//______________________________________________________________________________
void ESimuEventViewer::DoError(int level, const char *location, const char *fmt, va_list va) const
{
    // Interface to ErrorHandler (protected).

    SetStatusText(Form(fmt,va),0);
    ::ErrorHandler(level, Form("%s::%s", ClassName(), location), fmt, va);
}


//______________________________________________________________________________
void ESimuEventViewer::CreateViewer() {
    //
    // Update the user frame and remap the window
    //

    // save current width and height
    UInt_t w = GetWidth();
    UInt_t h = GetHeight();

    gVirtualX->GrabKey(fId, gVirtualX->KeysymToKeycode(kKey_n),
            kKeyControlMask, kTRUE);
    gVirtualX->GrabKey(fId, gVirtualX->KeysymToKeycode(kKey_p),
            kKeyControlMask, kTRUE);

    AddUserFrame();
//    Layout();
    MapSubwindows();

    ShowTabList(kFALSE);
    ShowEditor(kFALSE);

    // reset the windows size to the correct value.
    Resize(w,h);
}

//______________________________________________________________________________
void ESimuEventViewer::ApplyBarCut() {
    //
    //
    //

    ApplyCut(fEventCut->GetText());
}

//______________________________________________________________________________
void ESimuEventViewer::ApplyCut( const char* selection ) {
    //
    //
    //

    UpdateEventList(selection);
    fEventCut->SetText(selection);

    if ( LoadEvent() )
        UpdateTabs();

}
//______________________________________________________________________________
void ESimuEventViewer::ApplyCut( const TCut& selection ) {
    //
    //
    //

    ApplyCut(selection.GetTitle());
}

//______________________________________________________________________________
const char* ESimuEventViewer::GetCut() const {
    //
    //
    //
    return fEventCut->GetText(); 
}

//______________________________________________________________________________
void ESimuEventViewer::UpdateEventList( const TCut& selection ) {
    //
    // Update the eventlist according the cut 
    //
    
    UpdateEventList( selection.GetTitle() );
}

//______________________________________________________________________________
void ESimuEventViewer::UpdateEventList( const char* selection ) {
    //
    // Update the eventlist according the cut 
    //

    if ( !fEvTree ) return;
    
    if ( !fEventList ) fEventList = new TEventList("simuEventList");

    fEvTree->Draw(">>simuEventList",selection);
    
    fEventCombo->RemoveEntries(0,fEventCombo->GetNumberOfEntries()-1);
    for (Int_t i(0); i<fEventList->GetN(); i++) 
        fEventCombo->AddEntry(Form("%d",fEventList->GetEntry(i)),i);
    
    
    fForceLoading = kTRUE;
}

//______________________________________________________________________________
void ESimuEventViewer::AddUserFrame() {
    //
    // Add the Event toolbar
    //
    TGLabel* label;

    fEventBar = new TGDockableFrame( fUserFrame, -1, kHorizontalFrame );
    fEventBar->Connect("Docked()","ESimuEventViewer",this,"Layout()");
    fEventBar->Connect("Undocked()","ESimuEventViewer",this,"Layout()");
    fEventBar->EnableHide(kFALSE);
    //    fList->Add(fEventBar); FIXME: WHY??!?!?
    TGLayoutHints* lay = new TGLayoutHints(kLHintsExpandX ,0,0,1,0);
    fList->Add(lay);

    //
    // Radio Buttons
    //
//    TGLayoutHints* laygh = new TGLayoutHints(kLHintsNormal | kLHintsExpandX,0,0,0,0);
//    fList->Add(laygh);
//    
//    TGGroupFrame* gh = new TGGroupFrame(fEventBar,"Active Painters",kHorizontalFrame);
//    fList->Add(gh);
//    
//    gh->SetLayoutManager(new TGMatrixLayout(gh, 0, 2, 10));
//
//    TGCheckButton* radio = new TGCheckButton(gh,"FocalSurface",kCheckPainter1);
//    radio->SetState(kButtonDown);
//    gh->AddFrame(radio);
//    fList->Add(radio);
//    
//    radio = new TGCheckButton(gh,"FocalSurface3D",kCheckPainter2);
//    radio->SetState(kButtonDown);
//    gh->AddFrame(radio);
//    fList->Add(radio);
//
//    radio = new TGCheckButton(gh,"SignalNoiseProfile",kCheckPainter4);
//    radio->SetState(kButtonDown);
//    gh->AddFrame(radio);
//    fList->Add(radio);
//    
//    radio = new TGCheckButton(gh,"Histos",kCheckPainter4);
//    radio->SetState(kButtonDown);
//    gh->AddFrame(radio);
//    fList->Add(radio);
//    
//    gh->Resize();
//
//    fEventBar->AddFrame(gh,laygh);
    //
    // First horizontal frame
    //

    TGLayoutHints* layh = new TGLayoutHints(kLHintsNormal | kLHintsExpandX );
    fList->Add(layh);
    
    TGHorizontalFrame* h = new TGHorizontalFrame(fEventBar);
    fList->Add(h);

    // 
    // buttons
    // 

    TGTextButton* b;
    TGLayoutHints* layb;

    layb = new TGLayoutHints(kLHintsNormal ,2,2,3,2);
    fList->Add(layb);

    b = new TGTextButton(h, " Prev ", kButtonPrevEvent);
    b->Associate(this);
    b->SetToolTipText("Previous Event");
    fList->Add(b);

    h->AddFrame(b,layb);

    b = new TGTextButton(h, " Next ", kButtonNextEvent);
    b->Associate(this);
    b->SetToolTipText("Next Event");
    fList->Add(b);

    layb = new TGLayoutHints(kLHintsNormal ,2,4,3,2);
    fList->Add(layb);

    h->AddFrame(b,layb);

    label = new TGLabel(h,"Event");
    fList->Add(label);

    TGLayoutHints* laylbl = new TGLayoutHints(kLHintsLeft | kLHintsCenterY, 10, 4, 2, 2);
    fList->Add(laylbl);

    h->AddFrame(label, laylbl);

    // 
    // Event combo
    //
    TGLayoutHints* laycombo = new TGLayoutHints(kLHintsLeft | kLHintsCenterY, 4, 4, 2, 2);
    fList->Add(laycombo);
    fEventCombo = new TGComboBox(h, kComboNevents);
    fEventCombo->Associate(this);

    h->AddFrame(fEventCombo, laycombo);

    fList->Add(fEventCombo); 

    fEventCombo->Resize(50,20);

    fEventBar->AddFrame(h,layh);
    
    //
    // second horizontal frame
    //
    h = new TGHorizontalFrame(fEventBar);
    fList->Add(h);

    //
    // Cuts
    //
    label = new TGLabel(h,"Cut");
    fList->Add(label);

    h->AddFrame(label, laylbl);

    fEventCut = new TGTextEntry(h, new TGTextBuffer(15),-1);
    h->AddFrame(fEventCut, new TGLayoutHints(kLHintsLeft | kLHintsTop | kLHintsExpandX,5,5,5,5));
    fList->Add(fEventCut);

    fEventCut->SetMaxLength(255);
    fEventCut->SetAlignment(kTextLeft);
    fEventCut->SetText("");
    fEventCut->Resize(220,fEventCut->GetDefaultHeight());
    fEventCut->Resize(300,20);
    fEventCut->Connect("ReturnPressed()","ESimuEventViewer",this,"ApplyBarCut()");

    b = new TGTextButton(h, " Apply ", kButtonApplyCut);
    b->Associate(this);
    b->SetToolTipText("Apply Cut");
    fList->Add(b);

    layb = new TGLayoutHints(kLHintsNormal ,2,4,3,2);

    fList->Add(layb);

    h->AddFrame(b,layb);
    fEventBar->AddFrame(h,layh);

    //
    // Line3D
    //
    fUserFrame->AddFrame(fEventBar,lay);

    TGLayoutHints *lay3D = new TGLayoutHints(kLHintsExpandX);
    fList->Add(lay3D);

    TGHorizontal3DLine *line = new TGHorizontal3DLine(fUserFrame);
    fList->Add(line);

    fUserFrame->AddFrame(line, lay3D);

    fUserFrame->Layout();
    fUserFrame->MapSubwindows();

    //    fEventBar->EnableUndock(kFALSE);

}

//______________________________________________________________________________
Bool_t ESimuEventViewer::Open( const char* name ) {

    if (!name)
        return kFALSE;

    TFile *f = new TFile(name);
    if ( f->IsZombie() ) {
        delete f;
        Printf("Unable to open %s",name);

        return kFALSE;
    }

    Printf("%s successfully opened.", name);

    TTree *etree = (TTree*)gDirectory->Get("etree");
    if ( !etree ) {
        Printf("etree not found");
        return kFALSE;
    }

    fEvTree = etree;

    Printf("etree found with %d events stored.", (Int_t)fEvTree->GetEntries());

    Int_t n = fEventCombo->GetNumberOfEntries();
    if ( n != 0 ) 
        fEventCombo->RemoveEntries(0,n-1);

    cout << "UpdateEventList" << endl;
    UpdateEventList();

    cout << "LoadEvent" << endl;
    if ( LoadEvent() ) {
        UpdateTabs();
    }

    return kTRUE;
}

//______________________________________________________________________________
Bool_t ESimuEventViewer::OpenDialog() {
    //
    //
    //

    if ( fEvent ) {
        delete fEvent;
        fEvent = 0;
    }

    TGFileInfo* info = new TGFileInfo();

    new TGFileDialog(gClient->GetRoot(), 0, kFDOpen, info);
    if ( info->fFilename ) 
        return Open( info->fFilename );

    return kFALSE;
}

//______________________________________________________________________________
Bool_t ESimuEventViewer::LoadEvent( Int_t i ) {

    if (!fEvTree) {
        Info("LoadEvent","fEvTree not defined.");
        return kFALSE;
    }

    if ( i >= fEventList->GetN() ) {
        Info("LoadEvent","Entry id %d is greater than the maximum number of selected events (%d)",
             i,fEventList->GetN());
        return kFALSE;
    }
    
    if ( !fEvent ) {
        fEvent = new EEvent;
        fEvent->SetBranches(fEvTree, kTRUE);
    }

    Long64_t entry = fEventList->GetEntry(i);

    if ( fCurrentEvent == entry && !fForceLoading )
        return kFALSE;
    
    fRunPars = fEvent->GetRunPars();
    
    ClearPainters();
        
    Printf("Entry %ld, %ld  bytes read",entry,(Long_t)fEvTree->GetEntry(entry));

    BuildPainters();

    fCurrentEventId = i;

    fEventCombo->Select(fCurrentEventId);
    fCurrentEvent = entry;

    fForceLoading = kFALSE;
    return kTRUE;
}

//______________________________________________________________________________
void ESimuEventViewer::BuildPainters() {

    //EAtmosphere *atmosphere = fEvent->GetAtmosphere();
    EShower *shower = fEvent->GetShower();
    EDetector *detector = fEvent->GetDetector();
//    EChipTrackTrigger *trigger = fEvent->GetChipTrackTrigger();   
    ERunParameters *runpars = fEvent->GetRunPars();
    fTruth = fEvent->GetTruth();
    EGeometry *det_geo;
    
//    Double_t gtulength = runpars->GetPmtData().GetGtuLength();
    if (!fEvent->GetGeometry()) {
        EGeometry *det_geo1 = new EGeometry();
        det_geo1->SetPos(TVector3(0.,0.,430.*km));
        det_geo1->SetRadius(1250.*mm);
        det_geo = (EGeometry*)det_geo1;
    } else {
        det_geo = (EGeometry*)fEvent->GetGeometry();
    }


    Info("BuildPainters","Building painters");
    if(shower && shower->GetNumSteps() > 0 && det_geo && detector) {
        fShowerPainter = new EShowerPainter( shower );
        fShowerHistoPainter = new EShowerHistoPainter( shower, det_geo, detector );
    }
//    if(atmosphere && atmosphere->GetNumBunches() > 0 )
//        fAtmosphereHistoPainter = new EAtmosphereHistoPainter( atmosphere, det_geo, gtulength, fTruth, shower );
//    if(atmosphere && atmosphere->GetNumSingles() > 1)
//        fSinglePhotonsPainter = new ESinglePhotonsPainter( atmosphere );
    if(detector) { 
        fFocalSurfacePainter = new EFocalSurfacePainter( detector, runpars );
        fFocalSurface3DPainter = new EFocalSurface3DPainter( detector, runpars );
    }
    if(detector && detector->GetNumPhotons() ) {
//        fDetectorPhotonsPainter = new EDetectorPhotonsPainter( detector );
//        fOpticsPhotonsPainter = new EOpticsPhotonsPainter( detector );
        fFocalSurfaceHistoPainter = new EDetectorHistoPainter( detector, runpars );
    }
//    if(detector && trigger)
//        fTriggerPainter = new ETriggerPainter( detector, runpars, trigger);
}

//______________________________________________________________________________
void ESimuEventViewer::ClearPainters() {

    Printf("Clearing painters");
    SafeDelete(fShowerPainter);
    SafeDelete(fShowerHistoPainter);
    SafeDelete(fAtmosphereHistoPainter);
    SafeDelete(fSinglePhotonsPainter);
    SafeDelete(fDetectorPhotonsPainter);
    SafeDelete(fOpticsPhotonsPainter);
    SafeDelete(fFocalSurfacePainter);
    SafeDelete(fFocalSurface3DPainter);
    SafeDelete(fFocalSurfaceHistoPainter);
    SafeDelete(fTriggerPainter);
    fTruth = 0; // NOT delete, it belongs to the tree.

    //RemoveTabs();
}


//______________________________________________________________________________
void ESimuEventViewer::UpdateTabs() {
    //
    // Refresh canvases
    //
    TCanvas* c;
    if ( fTruth && !(fTruth->IsZombie()) ) {
        c = FindAddTab("Summary");
        c->Clear();
        c->cd();
        DrawTruth();
        c->Modified();
        c->Update(); 
    } else {
        RemoveTab("Summary");
    }
    
    if ( kFALSE && fShowerPainter && !(fShowerPainter->IsZombie()) ) {
        c = FindAddTab("Shower");
        c->Clear();
        c->cd();
        c->Divide(3);
        TVirtualPad* c1 = c->cd(1);
        c1->SetPad(0.,0.,0.75,1);
        fShowerPainter->Draw();
        gPad->Modified();
        gPad->Update();

        TVirtualPad* c2 = c->cd(2);
        c2->SetPad(0.75,0.,1.,0.5);
        fShowerPainter->DrawXY();
        gPad->Modified();
        gPad->Update();

        TVirtualPad* c3 = c->cd(3);
        c3->SetPad(0.75,0.5,1.,1.);
        fShowerPainter->DrawXYZ();
        gPad->Modified();
        gPad->Update(); 

    } else { 
        RemoveTab("Shower");
    }
    
    if ( fFocalSurfacePainter && !(fFocalSurfacePainter->IsZombie()) ){

        c = FindAddTab("FocalSurface");
        c->Clear();
        c->Divide(6);
        
        TVirtualPad* c1 = c->cd(1);
        c1->SetPad(0.,0.,0.75,1);
        fFocalSurfacePainter->Draw();

        gPad->Modified();
        gPad->Update();

        gPad->Modified();
        gPad->Update();

        TVirtualPad* c2 = c->cd(2);
       
        
        c2->SetPad(0.75,0.75,1,1);
        c2->SetTopMargin(0.05);
        c2->SetBottomMargin(0.15);
        c2->SetLeftMargin(0.15);
        c2->SetRightMargin(0.05);


        TH1F* hits = fFocalSurfacePainter->GetHitsHist();
        TStyle* sty = new TStyle;
        sty->SetHistLineWidth(2);
        sty->SetHistLineColor(213);
        sty->SetHistFillColor(214);
        sty->SetLabelSize(0.12,"X");
        sty->SetLabelSize(0.12,"Y");
        sty->SetNdivisions(7,"X");
        sty->SetNdivisions(7,"Y");
        fFocalSurfacePainter->SetHitsHistStyle(sty);
        fFocalSurfacePainter->UpdateHist(kTRUE);
        
        hits->SetBit(TH1::kNoTitle);
        hits->Draw();
        c1->Connect("Modified()","TPad",c2,"Modified()");

        gPad->Modified();
        gPad->Update();

        TVirtualPad* c3 = c->cd(3);
        c3->SetPad(0.75,0.5,1.,0.75);
        c3->SetTopMargin(0.05);
        c3->SetBottomMargin(0.15);
        c3->SetLeftMargin(0.15);
        c3->SetRightMargin(0.05);
        
        TH1F* prof = (TH1F*)fFocalSurfacePainter->GetSigProfile()->DrawCopy();
        if (prof) {
            TStyle* savestyle = gStyle;
            gStyle = sty;
            prof->UseCurrentStyle();
            gStyle = savestyle;
            prof->SetBit(TH1::kNoTitle);
        }

        c3->Modified();
        c3->Update();

        TVirtualPad* c4 = c->cd(4);
        c4->SetPad(0.75,0.25,1.,0.5);
        c4->SetTopMargin(0.05);
        c4->SetBottomMargin(0.15);
        c4->SetLeftMargin(0.15);
        c4->SetRightMargin(0.05);
        
        prof = (TH1F*)fFocalSurfacePainter->GetSigMaxProfile()->DrawCopy();
        if (prof) {
            TStyle* savestyle = gStyle;
            gStyle = sty;
            prof->UseCurrentStyle();
            gStyle = savestyle;
            prof->SetBit(TH1::kNoTitle);
        
        }

        c4->Modified();
        c4->Update();

        TVirtualPad* c5 = c->cd(5);
        c5->SetPad(0.75,0.,0.875,0.25);
        Double_t rFS = 1250;
        Double_t rView = rFS*1.1;
        c5->DrawFrame( -rView, -rView, rView, rView );

        TEllipse *el = new TEllipse(0., 0., rFS, rFS, 0, 360);
        el->SetLineWidth(1);
        el->SetBit(kCanDelete);
        el->Draw();

        TVector3 vc = fFocalSurfacePainter->GetViewCenter();

        TMarker* p = new TMarker(vc.X(),vc.Y(),20);
        p->SetMarkerColor(kRed);
        p->SetBit(kCanDelete);
        p->Draw();

        TText* txt = new TText(-rView,rView,"Focal Surface");
        txt->SetTextAlign(13);
        txt->SetTextSize(0.15);
        txt->Draw();

        c5->Modified();
        c5->Update();

        TVirtualPad* c6 = c->cd(6);
        c6->SetPad(0.875,0.,1.,0.25);
        Double_t rFoV = kHdetector*Tan(kFoV);
        rView = rFoV*1.1;
        c6->DrawFrame( -rView, -rView, rView, rView );

        el = new TEllipse(0., 0., rFoV, rFoV, 0, 360);
        el->SetLineWidth(1);
        el->SetBit(kCanDelete);
        el->Draw();

        vc = fTruth->GetTrueShowerMaxPos();

        p = new TMarker(vc.X(),vc.Y(),20);
        p->SetMarkerColor(kRed);
        p->SetBit(kCanDelete);
        p->Draw();

        txt = new TText(-rView,rView,"Field of View");
        txt->SetTextAlign(13);
        txt->SetTextSize(0.15);
        txt->Draw();
        
        c6->Modified();
        c6->Update(); 
    } else {
        RemoveTab("FocalSurface");
    }

    if ( fFocalSurface3DPainter && !(fFocalSurface3DPainter->IsZombie()) ){

        c = FindAddTab("FocalSurface3D");
        c->Clear();

        fFocalSurface3DPainter->SetDrawCode(1);
        fFocalSurface3DPainter->Update();
        fFocalSurface3DPainter->Draw();
        c->Modified();
        c->Update();

        c->Modified();
        c->Update(); 
    } else {
        RemoveTab("FocalSurface3D");
    }

    if ( fFocalSurfacePainter && !(fFocalSurfacePainter->IsZombie()) ) {
        c = FindAddTab("SignalNoiseProfile");
        c->Clear();
        c->cd();
        c->Divide(3,2);

        Style_t fill = 3001;

        TVirtualPad* c1 = c->cd(1);
        c1->SetPad(0., 0.5, 0.25, 0.75);
        TH1F* hHmaxProf = (TH1F*)fFocalSurfacePainter->GetHitsMaxProfile()->DrawCopy();
        Float_t max = hHmaxProf->GetMaximum();
        hHmaxProf->UseCurrentStyle();
        hHmaxProf->SetFillStyle(fill);
        hHmaxProf->SetLineColor(kBlue);
        hHmaxProf->SetFillColor(kBlue);

        TVirtualPad* c2 = c->cd(2);
        c2->SetPad(0., 0.25, 0.25, 0.5);
        TH1F* hSmaxProf = (TH1F*)fFocalSurfacePainter->GetSigMaxProfile()->DrawCopy();
        hSmaxProf->UseCurrentStyle();
        hSmaxProf->SetFillStyle(fill);
        hSmaxProf->SetLineColor(kBlue);
        hSmaxProf->SetFillColor(kBlue);
        hSmaxProf->SetMaximum(max);


        TVirtualPad* c3 = c->cd(3);
        c3->SetPad(0.25, 0.75, 0.5, 1.00);
        TH1F* hNminProf = (TH1F*)fFocalSurfacePainter->GetNoiseMinProfile()->DrawCopy();
        hNminProf->UseCurrentStyle();
        hNminProf->SetFillStyle(fill);
        hNminProf->SetLineColor(kBlue);
        hNminProf->SetFillColor(kBlue);
        hNminProf->SetMaximum(max);

        TVirtualPad* c4 = c->cd(4);
        c4->SetPad(0.50, 0.75, 0.75, 1.00);
        TH1F* hNmaxProf = (TH1F*)fFocalSurfacePainter->GetNoiseMaxProfile()->DrawCopy();
        hNmaxProf->UseCurrentStyle();
        hNmaxProf->SetFillStyle(fill);
        hNmaxProf->SetLineColor(kBlue);
        hNmaxProf->SetFillColor(kBlue);
        hNmaxProf->SetMaximum(max);

        TVirtualPad* c5 = c->cd(5);
        c5->SetPad(0.75, 0.75, 1.00, 1.00);
        TH1F* hNmeanProf = (TH1F*)fFocalSurfacePainter->GetNoiseMeanProfile()->DrawCopy();
        hNmeanProf->UseCurrentStyle();
        hNmeanProf->SetFillStyle(fill);
        hNmeanProf->SetLineColor(kBlue);
        hNmeanProf->SetFillColor(kBlue);
        hNmeanProf->SetMaximum(max);

        TVirtualPad* c6 = c->cd(6);
        c6->SetPad(0.25, 0.00, 1.00, 0.75);

        TH1F* hitsH = (TH1F*)hHmaxProf->DrawCopy();
        hitsH->SetFillColor(2);
        hitsH->SetFillStyle(0);
        hitsH->SetLineWidth(2);
        hitsH->SetLineWidth(2);

        TH1F* meanH = (TH1F*)hNmeanProf->DrawCopy("E2same");
        meanH->SetFillColor(43);
        meanH->SetFillStyle(3002);
        meanH->SetLineColor(4);
        meanH->SetLineWidth(2);
        
        TH1F* minH = (TH1F*)hNminProf->DrawCopy("same");
        minH->SetFillColor(8);
        minH->SetFillStyle(0);
        minH->SetLineColor(8);
        minH->SetLineWidth(2);

        TH1F* maxH = (TH1F*)hNmaxProf->DrawCopy("same");
        maxH->SetFillColor(2);
        maxH->SetFillStyle(0);
        maxH->SetLineColor(2);
        maxH->SetLineWidth(2);

        //Float_t max = Max(hSmaxProf->GetMaximum(), hNmaxProf->GetMaximum());
        //hSmaxProf->SetMaximum(max);
        //hNmaxProf->SetMaximum(max);
        //hNmaxProf->SetTitle("Max noise per GTU");
        
        c->Modified();
        c->Update();


/*        TLegend* leg = new TLegend(0.8, 0.8, 0.9, 0.9);
        leg->AddEntry(hHmaxProf, "Total", "f");
        leg->AddEntry(hNmaxProf, "Background", "f");
        leg->AddEntry(hSmaxProf, "Signal", "f");
        leg->Draw(); */

    } else {
        RemoveTab("Shower");
    }

    if ( fFocalSurfaceHistoPainter && !(fFocalSurfaceHistoPainter->IsZombie()) ) {
        c = FindAddTab("Histos");
        c->Clear();
        c->cd();
        fFocalSurfaceHistoPainter->DoGtuPhotonsHisto(c); 
        c->Modified();
        c->Update();
    } else {
        RemoveTab("Histos");
    }
}

//______________________________________________________________________________
void ESimuEventViewer::DrawTruth() {

    TPaveText* tpave;

    // first column
    tpave = new TPaveText(0.025,0.025,0.475,1.,"");
    tpave->SetTextAlign(13);
    tpave->SetTextFont(52);
    tpave->SetTextColor(kBlack);
    tpave->SetFillStyle(0);
    tpave->SetBorderSize(0);
    tpave->AddText("");
    tpave->AddText("This event was simulated with:");
    tpave->AddText("");
    tpave->AddLine();
    // Energy, Theta, Phi
    tpave->AddText(Form("E :      %.4e MeV",fTruth->GetTrueEnergy()/1.));
    tpave->AddText(Form("#theta :      %.2f   deg",fTruth->GetTrueTheta()*RadToDeg()));
    tpave->AddText(Form("#varphi :      %.2f   deg",fTruth->GetTruePhi()*RadToDeg()));
    // line of sight-incident direction angle
    TVector3 dir = (fTruth->GetTrueEarthImpact()-fTruth->GetTrueInitPos()).Unit();
    TVector3 pos = fTruth->GetTrueShowerMaxPos()-TVector3(0,0,420*sou::km);
    Float_t fovdir = RadToDeg()*ACos((pos.Unit()).Dot(dir));
    tpave->AddText(Form("#alpha :      %.2f   deg",fovdir));
    
    tpave->AddText("");
    tpave->AddText(Form("X_{1} :       %.2f   g/cm^{2}",fTruth->GetTrueX1()*cm2/g));
    tpave->AddText(Form("#vec{P}_{init} :      (%.2f,%.2f,%.2f)  km",
                        fTruth->GetTrueInitPos().X()/km, fTruth->GetTrueInitPos().Y()/km,
                        fTruth->GetTrueInitPos().Z()/km));
    tpave->AddText("");
    tpave->AddText(Form("X_{max} :      %.2f   g/cm^{2}",fTruth->GetTrueShowerXMax()*cm2/g));
    tpave->AddText(Form("#vec{P}_{max} :       (%.2f,%.2f,%.2f)   km",
                         fTruth->GetTrueShowerMaxPos().X()/km, fTruth->GetTrueShowerMaxPos().Y()/km,
                         fTruth->GetTrueShowerMaxPos().Z()/km));

    tpave->AddText("");
    tpave->AddText(Form("Age_{earth} :   %.2f",fTruth->GetTrueEarthAge()));
    tpave->AddText(Form("#vec{P}_{earth} :      (%.2f,%.2f,%.2f)   km",
                        fTruth->GetTrueEarthImpact().X()/km,fTruth->GetTrueEarthImpact().Y()/km,
                        fTruth->GetTrueEarthImpact().Z()/km));
    tpave->AddText("");
    tpave->AddText(Form("L_{Shower}  : %.2f km",
                (fTruth->GetTrueEarthImpact()-fTruth->GetTrueInitPos()).Mag()/km));
    if ( fTruth->GetTrueHclouds() > 0 ) 
        tpave->AddText(Form("H_{clouds} :      %.2f    km", fTruth->GetTrueHclouds()/km));
    else 
        tpave->AddText(Form("H_{clouds} :      None"));
    tpave->AddText("");
    tpave->AddText("");
    tpave->AddText("");
    tpave->AddText("");
    tpave->Draw();
    
    // second column
    const EDetStatus *fStatus = fEvent->GetDetector()->GetStatus();
    if ( !fStatus || !fStatus->GetNumPhotons() ) return;
    tpave = new TPaveText(.525,0.025,.975,1.,"");
    tpave->SetTextAlign(13);
    tpave->SetTextFont(52);
    tpave->SetTextColor(kBlack);
    tpave->SetFillStyle(0);
    tpave->SetBorderSize(0);
    tpave->AddText("");
    tpave->AddText("Detector summary");
    tpave->AddText("");
    tpave->AddLine();
    tpave->AddText(Form("Number of photons: %d", fStatus->GetNumPhotons()));
    tpave->AddText(Form(" #diamond Lost: %d", fStatus->GetPhotonHistory(kUndefined)));
    tpave->AddText(Form(" #diamond Not inside detector: %d", fStatus->GetPhotonHistory(kNotInside)));
    tpave->AddText(Form(" #diamond Baffle: %d", fStatus->GetPhotonHistory(kBaffle)));
    tpave->AddText(Form(" #diamond Optics: %d", fStatus->GetPhotonHistory(kOptics)));
    tpave->AddText(Form(" #diamond Walls: %d", fStatus->GetPhotonHistory(kWalls)));
    tpave->AddText(Form(" #diamond Focal Plane: %d", fStatus->GetPhotonHistory(kFocalPlane)));
    tpave->AddText(Form(" #diamond Reflected: %d", fStatus->GetPhotonHistory(kEarth)));
    tpave->AddText("");
    tpave->AddText(Form("Number of Fee: %d", fEvent->GetDetector()->GetNumFee()));
    tpave->AddText(Form("#diamond with signals: %d", fStatus->GetNumSignals()));
    tpave->AddText(Form("#diamond with hits (in pixels with signal): %d", fStatus->GetNumHits()));
    tpave->AddText(Form("#diamond only background hits: %d", fStatus->GetNumBkg()));
    tpave->AddText("");
    tpave->AddText(Form("GTUs with signal: %d", fStatus->GetNumGtu()));
    tpave->AddText("");
    tpave->AddText(Form("#nu^{bkg}_{min}: %.2f p.e.#upoint pixel^{-1}#upoint #mus^{-1}", fRunPars->GetMinNightGlowRate()));
    tpave->AddText(Form("#nu^{bkg}_{max}: %.2f p.e.#upoint pixel^{-1}#upoint #mus^{-1}", fRunPars->GetMaxNightGlowRate()));
    tpave->Draw();

}

//______________________________________________________________________________
void ESimuEventViewer::NextEvent() {
    //
    // Jump to the previous event
    //
    if ( fCurrentEventId >= fEventList->GetN() )
        return;
    
    if ( LoadEvent( fCurrentEventId+1 ) )
        UpdateTabs();
}


//______________________________________________________________________________
void ESimuEventViewer::PrevEvent() {
    //
    // Jump to the next event
    //
    if ( fCurrentEventId <= 0 ) 
        return;

    if ( LoadEvent( fCurrentEventId-1 ) )
    UpdateTabs();

}

//______________________________________________________________________________
Bool_t ESimuEventViewer::ProcessMessageCommand(Long_t submsg, Long_t mp1, Long_t mp2) {
    //
    //
    //
    
    switch (submsg) {
        case kCM_BUTTON:
            switch (mp1) {
                case kButtonNextEvent:
                    NextEvent();
                    return kTRUE;
                case kButtonPrevEvent:
                    PrevEvent();
                    return kTRUE;
                case kButtonApplyCut:
                    ApplyBarCut();
                    return kTRUE;
            } 
        case kCM_COMBOBOX: // 7
            if (mp1==kComboNevents) {
                LoadEvent(mp2);
                UpdateTabs();
            }
            return kTRUE;
    }
                    
    return EGViewer::ProcessMessageCommand(submsg, mp1, mp2);
}

//______________________________________________________________________________
Bool_t ESimuEventViewer::HandleKey(Event_t* event) {
    //
    // Handle keybord commands
    //
    //

    if ((event->fType == kGKeyPress) && (event->fState & kKeyControlMask)) {
        UInt_t keysym;
        char str[2];

        gVirtualX->LookupString(event, str, sizeof(str), keysym);

        switch ( keysym ) {
            case kKey_n:
                NextEvent();
                return kTRUE;
            case kKey_p:
                PrevEvent();
                return kTRUE;
        }
    }

    return EGViewer::HandleKey(event);
}


