// ESAF : Euso Simulation and Analysis Framework
// $Id: RecoPhotoElectronData.hh 1012 2004-09-14 12:40:23Z naumov $
// R. Pesce created Jan, 16 2004
// this class manages data about photons counted by front end electronics

#ifndef __RECOPHOTOELECTRONDATA_HH_
#define __RECOPHOTOELECTRONDATA_HH_

#include "euso.hh"


class RecoPhotoElectronData {
public:
    // ctor
    RecoPhotoElectronData(Int_t fe, Int_t fech, Int_t gtu, Int_t cell, Bool_t fastor, 
        Int_t xc, Int_t yc, Float_t xpos, Float_t ypos);

    // dtor
    virtual ~RecoPhotoElectronData();

    // Getters
    inline Int_t GetFe() const { return fFe; }
    inline Int_t GetFeCh() const { return fFeCh; }
    inline Int_t GetGtu() const { return fGtu; }
    inline Int_t GetMacroCell() const { return fMacroCell; }
    inline Bool_t GetMadeFastOR() const { return fMadeFastOR; }
    inline Int_t GetXCell() const { return fXCell; }
    inline Int_t GetYCell() const { return fYCell; }
    inline Float_t GetXPos() const { return fXPos; }
    inline Float_t GetYPos() const { return fYPos; }

private:

    Int_t fFe;              // front end chip
    Int_t fFeCh;            // front end channel
    Int_t fGtu;             // GTU photon belong to
    Int_t fMacroCell;       // macrocell number it
    Bool_t fMadeFastOR;     // true if photon in counted in macrocell
    Int_t fXCell;           // X coordinate in macrocell
    Int_t fYCell;           // Y coordinate in macrocell
    Float_t fXPos;          // X pos in focal surface
    Float_t fYPos;          // Y pos in focal surface

    friend class RootInputModule;
    ClassDef(RecoPhotoElectronData,0)
};

#endif  /* __RECOPHOTOELECTRONDATA_HH_ */

