// $Id: RecoShowerStepData.hh 2387 2005-11-23 16:02:40Z moreggia $
// Author: Sylvain Moreggia   2005/11/17

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoShowerStepData                                                   *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOSHOWERSTEPDATA_HH__
#define __RECOSHOWERSTEPDATA_HH__

#include "euso.hh"
#include "EarthVector.hh"

class RecoShowerTrackData;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoShowerStepData                                                         //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoShowerStepData {
public:
    // ctor
    RecoShowerStepData(Float_t,Float_t,Float_t,EarthVector,EarthVector);
    
    // dtor
    virtual ~RecoShowerStepData();
    
    //getters
    inline Float_t GetXi()              const {return fXi;}
    inline Float_t GetXf()              const {return fXf;}
    inline const EarthVector& GetPosi() const {return fPosi;}
    inline const EarthVector& GetPosf() const {return fPosf;}
    inline Float_t GetNe()              const {return fNe;}
    
    //setters
    inline void SetXi( Float_t xi ) { fXi = xi; }
    inline void SetXf( Float_t xf ) { fXf = xf; }
    inline void SetPosi(const EarthVector& posi) {fPosi = posi;}
    inline void SetPosf(const EarthVector& posf) {fPosf = posf;}
    inline void SetNe( Float_t ne ) { fNe = ne; }


private:

    Float_t      fXi;          // Slant Depth of the first point of the step
    Float_t      fXf;          // Slant Depth of the last point of the step
    EarthVector  fPosi;        // 3D coordinates in MES of the first point of the step
    EarthVector  fPosf;        // 3D coordinates in MES of the last point of the step  
    Float_t      fNe;          // Number of electrons at (fXf+fXi)/2
    
    ClassDef(RecoShowerStepData,0)
};

#endif  /* __RECOSHOWERSTEPDATA_HH__ */

