// $Id: RecoShowerTrackData.hh 2796 2008-02-11 13:08:54Z naumov $
// Author: Sylvain Moreggia   2005/11/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoShowerTrackData                                                  *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOSHOWERTRACKDATA_HH__
#define __RECOSHOWERTRACKDATA_HH__

#include "euso.hh"
#include "EarthVector.hh"
#include <vector>
class RecoShowerStepData;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoShowerTrackData                                                        //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoShowerTrackData {
public:
    
    // ctor
    RecoShowerTrackData();
    
    // dtor
    virtual ~RecoShowerTrackData();
    
    // getters
    inline Float_t                           GetNemax()          const {return fNemax;}
    inline Float_t                           GetXmax()           const {return fXmax;}
    inline Float_t                           GetX1()             const {return fX1;}
    inline const EarthVector&                GetDir()            const {return fDirVers;}
    inline const EarthVector&                GetInitPos()        const {return fInitPos;}
    inline const EarthVector&                GetEarthImpact()    const {return fEarthImpact;}
    inline const EarthVector&                GetTOAImpact()    const {return fTOAImpact;}
    inline const EarthVector&                GetShowerMaxPos()   const {return fShowerMaxPos;}
    inline Int_t                             GetNumSteps()       const {return fNumSteps;}
    inline const vector<RecoShowerStepData*> GetSteps()          const {return fSteps;};
    inline const RecoShowerStepData*         GetStep(Int_t)      const;

    // setters
    inline void SetNemax(Float_t nemax)                    {fNemax = nemax;}
    inline void SetXmax(Float_t xmax)                      {fXmax = xmax;}
    inline void SetX1(Float_t t)                           {fX1 = t;}
    inline void SetDir(const EarthVector& dir)             {fDirVers = dir;}
    inline void SetInitPos(const EarthVector& pos)         {fInitPos = pos;}
    inline void SetEarthImpact(const EarthVector& impact)  {fEarthImpact = impact;}
    inline void SetShowerMaxPos(const EarthVector& maxpos) {fShowerMaxPos = maxpos;}
    inline void SetTOAImpact(const EarthVector& impact)    {fTOAImpact = impact;}
    inline void AddStep(RecoShowerStepData* step)          {fSteps.push_back(step);}
    inline void SetNumSteps(Int_t nb);

private:

    // global features
    Float_t     fNemax;
    Float_t     fXmax;
    Float_t     fX1;
    EarthVector fDirVers;         // track direction in the MES
    EarthVector fInitPos;         // first interaction point (3D coord, meters) in MES
    EarthVector fEarthImpact;     // Impact point on the Earth if any, else (0,0,HUGE)
    EarthVector fTOAImpact;       // Impact point on Top of Atmosphere
    EarthVector fShowerMaxPos;    // True shower max position

    // shower profile details
    vector<RecoShowerStepData*> fSteps;
    Int_t fNumSteps;

    ClassDef(RecoShowerTrackData,0)
};

//_____________________________________________________________________________
inline const RecoShowerStepData* RecoShowerTrackData::GetStep(Int_t i) const {
    //
    // returns NULL when reaches end of the list
    //
    
    if(size_t(i) < fSteps.size()) return fSteps[i];
    else return (RecoShowerStepData*)0;
}

//_____________________________________________________________________________
inline void RecoShowerTrackData::SetNumSteps(Int_t nb) {
    //
    //
    //
    
    fNumSteps = nb;
    if(!fSteps.size()) fSteps.reserve(nb+1);
}


#endif  /* __RECOSHOWERTRACKDATA_HH__ */

