// $Id: RecoData.hh 2713 2006-06-06 12:28:46Z thea $
// Author:    2006/06/05

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoData                                                             *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECODATA_HH__
#define __RECODATA_HH__

#include <string>
#include <map>

#include "euso.hh"
#include "EsafMsgSource.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoData                                                                   //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoData : public EsafMsgSource {
public:
    virtual ~RecoData();

    virtual const string& GetName() const = 0;

    // add an integer value to the map
    // returns 0 if OK, 1 if value already existing
    virtual Int_t Add(const string&, const Int_t&);

    // add an integer value to the map
    // returns 0 if OK, 1 if value already existing
    virtual Int_t Add(const string&, const Double_t&);

    // add an integer value to the map
    // returns 0 if OK, 1 if value already existing
    virtual Int_t Add(const string&, void*);

    // returns an Int_t
    virtual Int_t GetInt(const string&) const;

    // returns a Double_t
    virtual Double_t GetDouble(const string&) const;

    // returns any other object type
    // the user must know the object type and cast it
    virtual void* GetObj(const string&) const;

    // remove an object 
    // the object is NOT deleted; user should do it by hand
    virtual Bool_t RemoveObj(const string&);
    

protected:
    RecoData();

    // check name
    Bool_t CheckName(const string&);

    // map of integers
    map<string,Int_t> fIntMap;

    // map of Double_t
    map<string,Double_t> fDoubleMap;

    // map of pointers to any object
    // the user must know which type the object is
    map<string,void*> fObjMap;

    // clean integer and Double_t maps
    virtual void CleanMaps();

    friend class RecoModule;
    ClassDef(RecoData,0)
};

#endif  /* __RECODATA_HH__ */

