// ESAF : Euso Simulation and Analysis Framework
// $Id: RootInputModule.hh 2637 2006-03-29 14:29:36Z thea $
// Marco Pallavicini created Oct, 16 2003

#ifndef __ROOTINPUTMODULE_HH_
#define __ROOTINPUTMODULE_HH_

#include "euso.hh"
#include "InputModule.hh"

class TFile;
class TTree;
class TBranch;
class EEvent;
class ERunParameters;
class RecoRootEvent;

class RootInputModule : public InputModule {
public:
    // ctor
    RootInputModule();

    // dtor
    virtual ~RootInputModule();

    // return on event reading from root file
    virtual RecoEvent *GetEvent( const char* opt = 0);

    // destroy last event
    virtual void DestroyEvent();
    
    // destroy last event
    virtual Bool_t Init();
    
    // destroy last event
    virtual Bool_t Done();
   
    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );
     
    EsafConfigClass(Reco,RootInputModule)

private:

    // pointer to root file
    TFile* fRootFile;

    // pointer to TTree
    TTree* fTree;

    // pointer to runtree
    TTree* fRunTree;
    
    // number of events in TTree
    Int_t fNumEvents;

    // first and last event id
    Int_t fFirstEvent;
    Int_t fLastEvent;

    // pointer to current event in the TTree
    Int_t fEventCounter;

    // pointer to euso event
    EEvent *fEv;

    // pointer to simu run parameters
    ERunParameters *fRunPars;

    // pointer to branch of entire simu event
    TBranch *fBranch;

    // pointer to branch of entire run parameters
    TBranch *fRunBranch;
    ClassDef(RootInputModule,0)
};

#endif  /* __ROOTINPUTMODULE_HH_ */

