// ESAF : Euso Simulation and Analysis Framework
// $Id: EusoCluster.hh 2717 2006-06-07 07:53:42Z pesce $
// R. Pesce created Jan, 23 2004

#ifndef __EUSOCLUSTER_HH_
#define __EUSOCLUSTER_HH_

#include <vector>
#include <map>
#include "euso.hh"

class EusoCluster {
public:
    // ctor
    EusoCluster();

    // dtor
    virtual ~EusoCluster();

    // add a point in cluster
    void AddPoint( Int_t id );

    // getters
    inline Int_t GetNumPoints() const { return fNumPoints; }
    inline Double_t GetThetaMin() const { return fThetaMin; }
    inline Double_t GetThetaMax() const { return fThetaMax; }
    inline Double_t GetPhiMin() const { return fPhiMin; }
    inline Double_t GetPhiMax() const { return fPhiMax; }
    inline Int_t GetGtuMin() const { return fGtuMin; }
    inline Int_t GetGtuMax() const { return fGtuMax; }
    inline Int_t GetThreshold() const { return fThreshold; }
    inline vector<Int_t> GetPixelVector() { return fPixelId; }
    const Int_t GetPixelId( Int_t i );
    inline Bool_t GetFitted() const { return fIsFitted; }
    inline Double_t GetSlopeTheta() const { return fSlopeTheta; }
    inline Double_t GetOffsetTheta() const { return fOffsetTheta; }
    inline Double_t GetAbsoluteDeviationTheta() const { return fAbsoluteDeviationTheta; }
    inline Double_t GetSlopePhi() const { return fSlopePhi; }
    inline Double_t GetOffsetPhi() const { return fOffsetPhi; }
    inline Double_t GetAbsoluteDeviationPhi() const { return fAbsoluteDeviationPhi; }
    
    // setters
    inline void SetThetaMin( Double_t th ) { fThetaMin = th; }
    inline void SetThetaMax( Double_t th ) { fThetaMax = th; }
    inline void SetPhiMin( Double_t phi ) { fPhiMin = phi; }
    inline void SetPhiMax( Double_t phi ) { fPhiMax = phi; }
    inline void SetGtuMin( Int_t gtu ) { fGtuMin = gtu; }
    inline void SetGtuMax( Int_t gtu ) { fGtuMax = gtu; }
    inline void SetThreshold( Int_t t ) { fThreshold = t; }
    inline void SetFitted( Bool_t isfit ) { fIsFitted = isfit; }
    inline void SetSlopeTheta( Double_t slope ) { fSlopeTheta = slope; }
    inline void SetOffsetTheta( Double_t offset ) { fOffsetTheta = offset; }
    inline void SetAbsoluteDeviationTheta( Double_t abdev ) { fAbsoluteDeviationTheta = abdev; }
    inline void SetSlopePhi( Double_t slope ) { fSlopePhi = slope; }
    inline void SetOffsetPhi( Double_t offset ) { fOffsetPhi = offset; }
    inline void SetAbsoluteDeviationPhi( Double_t abdev ) { fAbsoluteDeviationPhi = abdev; }


private:

    Int_t fNumPoints;   // number of points in cluster
    Double_t fThetaMin;
    Double_t fThetaMax;
    Double_t fPhiMin;
    Double_t fPhiMax;
    Int_t fGtuMin;
    Int_t fGtuMax;
    
    Int_t fThreshold; // threshold in pixel hits per gtu
    
    Bool_t fIsFitted; // true if cluster if fitted
    // parameters of straight line that fit the points
    // in theta-time and phi-time
    Double_t fSlopeTheta;
    Double_t fOffsetTheta;
    Double_t fAbsoluteDeviationTheta;
    Double_t fSlopePhi;
    Double_t fOffsetPhi;
    Double_t fAbsoluteDeviationPhi;
    
    // points of cluster
    vector<Int_t> fPixelId; // not is unique id but reference to fPixelData vector in RecoEvent
    ClassDef(EusoCluster,0)
};

#endif  /* __EUSOCLUSTER_HH_ */

