// Author: E. Taddei

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: HoughModule                                                          *
 *  Package: Clustering                                                      *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/
#ifndef __HOUGHMODULE_HH_
#define __HOUGHMODULE_HH_

#include <vector>
#include "euso.hh"
#include "RecoModule.hh"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//  HoughModule                                                             //
//                                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

class EusoCluster;
class RecoRootEvent;
class TList;

class HoughModule : public RecoModule {
public:
    // ctor
    HoughModule();

    // dtor
    virtual ~HoughModule();

    // Init method: called at the beginning of a run
    virtual Bool_t Init();

    // called before each event process
    virtual Bool_t PreProcess();

    // event processing
    virtual Bool_t Process( RecoEvent* );
	
    // called after processing an event
    virtual Bool_t PostProcess();

    // called at the end of each run
    virtual Bool_t Done();

    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );

    // memory clean
    virtual void UserMemoryClean();

    // tmp memory clean
    virtual void TmpMemoryClean();
    // get a specified cluster
    EusoCluster* GetCluster( Int_t num );
	
    EsafConfigClass(Reco,HoughModule)

private:

    Double_t fGtuLength;  //length of gtu
        
    Int_t fIdEv; // id of the event
    RecoEvent *fEv;
    RecoRootEvent *fRecoRootEvent; 
    Int_t    fMainMacroCellId;     // id of the most populous macrocell
    Int_t    fTotalNumPoints;      // total number of points
    Int_t    fNumPointsMinimum;    // minimum number of points in a significant cluster
    Int_t    fNumHitsMinimum;      // minimum number of points in a significant pixel (from config file)
    Int_t    fFinalNumHitsMinimum; // minimum number of points in a significant pixel ( from ProcessHoughAll() )
    Double_t fSignalToNoise;       // take pixels with signal/sqrt(noise) >= fSignalToNoise number [valid for ProcessOnlySignal option]

    // number of pixels with minimum number of hits
    Int_t fNumPointsAll; //=fHittedAll.size();
    
    //
    // RecoModuleData objects 
    //
    //finally selected in ProcessHoughAll() - written in final cluster
    vector<Int_t> fHittedFinal;
    // vector of significant clusters
    vector<EusoCluster*> fClusters;
    
    //
    // Temporary objects
    //
    vector<Int_t> fPreviousPattReco; // pixels selected by previous pattern reco modules
    // references to pixels with minimum number of hits
    vector<Int_t> fHittedAll; //initially selected (con fNumHitsMinimum) in ProcessHoughAll()
    vector<Int_t> fHittedSelHough;
    vector<Int_t> fHittedSel2Hough; 
    vector<Int_t> fHittedFinalMacrocell; //selected by ProcessHoughSingleMacrocell() or ProcessVelocity() and passed to ProcessHoughAll()
    vector<Int_t> fIdMacrocellsSelected;//id of macrocell selected by ProcessHoughSingleMacrocell() or ProcessVelocity()
    vector<Int_t> fHittedHoughMerged;

    Bool_t fIsAlreadyPattReco;
    Bool_t fDebugPlots;    
    Bool_t ProcessHoughSingleMacrocell();
    Bool_t ProcessHoughAll();
    Bool_t ProcessVelocity();
	
    void WriteSingleCluster();
    Int_t fClustersWritten;   //number of written clusters, for debug only
	
    Bool_t HoughTransform(vector<Double_t> x,vector<Double_t> y,vector<Double_t> t,vector<Int_t> c,vector<Int_t> id = vector<Int_t>() );
    Float_t fAbsDevX,fAbsDevY;
    Double_t fTmin,fTmax,fXmin,fXmax,fYmin,fYmax;
    Float_t fCmaxToCmeanX,fCmaxToCmeanY;
    Float_t fCmaxToCmeanMinX,fCmaxToCmeanMinY;

    TList *fListObj;  //list of objects to be deleted

    void ClearTmpMemory();
    
    string fProcedure;
    string fKindHough;
    ClassDef(HoughModule,0)
};

#endif  /* __HOUGHMODULE_HH_ */

