// ESAF : Euso Simulation and Analysis Framework
// $Id: HmaxForProtonModule.hh 2633 2006-03-28 09:13:43Z thea $
// Naumov Dmitry created Jul,  4 2004

#ifndef __HMAXFORPROTONMODULE_HH_
#define __HMAXFORPROTONMODULE_HH_

#include "euso.hh"
#include "RecoModule.hh"

class RecoRootEvent;
class RecoEvent;

class HmaxForProtonModule : public RecoModule {
public:
    // ctor
    HmaxForProtonModule();

    // dtor
    virtual ~HmaxForProtonModule();

    // Init method: called at the beginning of a run
    virtual Bool_t Init();

    // called before each event process
    virtual Bool_t PreProcess();

    // event processing
    virtual Bool_t Process( RecoEvent* );

    // called after processing an event
    virtual Bool_t PostProcess();

    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );

    // called at the end of each run
    virtual Bool_t Done();

    // memory clean
    virtual void UserMemoryClean();

    // tmp clean
    virtual void TmpMemoryClean();

    // Get reconstructed Theta
    Double_t GetTheta() {return fTheta;}
    
    // Depth in the Atmosphere corresponding to the Shower Maximum
    Double_t XMaxEstimate();
    EsafConfigClass(Reco, HmaxForProtonModule)
    
private:
    Double_t fEnergy;
    Double_t fTheta;
    Double_t fPhi;
    Double_t fAltitude;
    Int_t                         fUseTrueAngles; //  1 - use true Theta and Phi angles, 0 - use reconstructed angles 
    						  // (see HmaxForProtonModule.cfg for explanations)
    
    ClassDef(HmaxForProtonModule,0)
};

#endif  /* __HMAXFORPROTONMODULE_HH_ */

