// ESAF : Euso Simulation and Analysis Framework
// $Id: ProfileModule.hh 2759 2006-09-07 08:10:49Z moreggia $
// Author: Anne Stutz   Jun,  6 2005

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ProfileModule                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __PROFILEMODULE_HH_
#define __PROFILEMODULE_HH_

#include "euso.hh"
#include "RecoModule.hh"
#include "EarthVector.hh"
#include "TArrayD.h"
#include "TAxis.h"
#include <TH1F.h>

class RecoEvent;
class RecoRootEvent;

enum ProfileAnalysis {Xmaxshift_airdensity,EShift_airDensity};

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// ProfileModule                                                              //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class ProfileModule : public RecoModule  {
public:
    // ctor
    ProfileModule();
    // dtor
    virtual ~ProfileModule();

    // Init method: called at the beginning of a run
    Bool_t Init();

    // called before each event process
    Bool_t PreProcess();

    // event processing
    Bool_t Process( RecoEvent* );

    // called after processing an event
    Bool_t PostProcess();

    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );

    // called at the end of each run
    Bool_t Done();

    // memory clean
    void UserMemoryClean();

    // load previous modules
    void LoadModules();

    // study Xmax shift due to realistic air density  profiles, or unprecise reco of shower geometry
    // Profile is NOT entirely reconstructed
    void XmaxShift_AirDensity();
    
    // study Energy shift due to realistic air density  profiles, or unprecise reco of shower geometry
    // Track is used, NO FITS !!
    void EShift_AirDensity();
    
    // introduce an error in Z-cordinate, then applied to XmaxShift_AirDensity() reco part
    // gaussian distribution (fMeanZerror,fSigmaZerror)
    Double_t IntroduceZerror();
    
    // introduce an error in Theta-cordinate, then applied to XmaxShift_AirDensity() reco part
    // gaussian distribution (fMeanThetaerror,fSigmaThetaerror)
    Double_t IntroduceThetaerror();

    EsafConfigClass(Reco,ProfileModule)


private:
   
    RecoEvent *fEvent;                    // event being analysed
    ProfileAnalysis fAnalysisType;        // type of analysis to carry out
    Double_t fSigmaZerror;                // config used in IntroduceZerror() method
    Double_t fMeanZerror;                 // config used in IntroduceZerror() method
    Double_t fSigmaThetaerror;            // config used in IntroduceThetaerror() method
    Double_t fMeanThetaerror;             // config used in IntroduceThetaerror() method
    string fThetaMode;                    // tell if refce pos is ShowerMax or EarthImpact
    
    // data to describe simu and reco electron profile
    void ResetArrays(Int_t n =0);
    Int_t fNbsteps_simu;
    Int_t fNbsteps_crash;   // handle crash of simu and reco shower : the quicker crash is stored here
    TArrayD fx_simu;
    TArrayD fy_simu;
    TArrayD fz_simu;
    TArrayD fx_reco;
    TArrayD fy_reco;
    TArrayD fz_reco;
    TArrayD fX_simu;
    TArrayD fX_reco;
    TArrayD fNe_simu;
    TH1F* fElecProfile_simu;
    TH1F* fElecProfile_reco;
   
   
   
////// results
    Double_t fXmax;                       // reco depth of maximum
    Double_t fRecoProfIntegral;           // reco integrate profile
    Double_t fSimuProfIntegral;           // simu integrate profile
    Double_t fZshift;
    Double_t fThetashift;
    Double_t fQuality;                    // tell if reco is ok

////// simu inputs + profile truth
    Double_t fTheta;                      // local zenith angle (not in MES)
    Double_t fPhi;                        // azimuth zenith angle (not in MES)
    EarthVector fTrackDir;                // shower track 3D-direction (in MES)
    EarthVector fEarthImpact;             // 
    EarthVector fTOAImpact;               // 
    EarthVector fShowerMaxPos;            // true shower max position
    Double_t fTrueXmax;                   // true depth of maximum


    ClassDef(ProfileModule,0)
};



#endif  /* __PROFILEMODULE_HH_ */





