// $Id: RecoTreePainter.hh 2703 2006-06-01 09:42:14Z pesce $
// Author: Dmitry.Naumov   2006/02/03

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoTreePainter                                                      *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOTREEPAINTER_HH__
#define __RECOTREEPAINTER_HH__

#include "EsafMsgSource.hh"

#include <TObject.h>
#include <TCut.h>
#include <TString.h>
#include <TSystem.h>
#include <Riostream.h>
#include <map>

using std::map;
class TTree;
class RecoRootEvent;
class TH1;
class TList;
class TF1;
class TCanvas;
class TEventList;
class EGViewer;
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  RecoTreePainter                                                           //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoTreePainter : public TObject, public EsafMsgSource {
public:
    RecoTreePainter(TTree*);
    virtual ~RecoTreePainter();
    void   Loop();
    void   DrawAll();
    //void   DrawDirection();
    void   DrawTheta();
    void   DrawPhi();
    void   DrawEnergy();
    void   SaveAsPs( Option_t* ="A", const char* prefix = "");
    TH1*   GetHisto(const char *name, const char *title, Option_t *opt, Int_t, Float_t, Float_t, Int_t, Float_t, Float_t);
    void   SetFileName(TString f)           { fFilename = f; }
    inline void SetEnergyIndependentFromDir(Bool_t b=kTRUE) { fEnergyIndependentFromDir = b;}
    inline void SetUseAA1(Bool_t b=kTRUE) { fUseAA1 = b; }
    inline void   SaveEpsFiles(TString t);

    const char* GetCut() const { return fCut.GetTitle(); }
    void SetCut( const char* cut ) { fCut = cut; }
    void SetCut( const TCut& cut ) { fCut = cut; } 
    Long64_t UpdateEventList();
    Long64_t UpdateEventList( const char* selection );
    Long64_t GetEntry( Int_t i=0 );
    TTree* GetTree() { return fTree; }
private:
    void   FillHistos();
    void   SaveHistos();
    void   FillSimHistos();
    void   FillThetaHistos();
    void   FillPhiHistos();
    void   FillDirectionHistos();
    void   FillEnergyHistos();
    void   FillAngularResolutions();
    void   MakeBaseDataMap();
    Bool_t AcceptDirection();
    Bool_t AcceptEnergy();

    // if true, in the energy plots accept also
    // the events rejected from the direction module
    Bool_t fEnergyIndependentFromDir;
    
    TTree              *fTree;
    RecoRootEvent       *fEvent;
    TEventList          *fEventList;
    TCut                 fCut;
    Int_t                fNevents;
    TList               *fHistoList;
    EGViewer            *fEnergyViewer;
    EGViewer            *fThetaViewer;
    EGViewer            *fPhiViewer;
    map <string,Float_t> fRec;
    map <string,Float_t> fSim;
    map <string,Float_t> left_edge;
    map <string,Float_t> right_edge;
    map <string,Bool_t>  fCuts;
    map <string,Bool_t>  fEnergyCuts;
    TString              fFilename;
    Bool_t               fSaveEpsFiles;
    Bool_t               fUseAA1; // use the AA1 method track direction data
    TString              fEpsDir;

    ClassDef(RecoTreePainter,0)
};
inline void RecoTreePainter::SaveEpsFiles(TString t) {
    //
    //
    //
    fSaveEpsFiles = kTRUE;
    fEpsDir = t;
    gSystem->mkdir(t,kTRUE);
}

#endif  /* __RECOTREEPAINTER_HH__ */

