// $Id: RecoClustering.hh 2736 2006-06-11 16:04:19Z pesce $
// Author: R. Pesce   2006/06/05

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoClustering                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOCLUSTERING_HH__
#define __RECOCLUSTERING_HH__

#include <TObject.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoClustering                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoClustering : public TObject {
public:
    RecoClustering();
    // ctor

    virtual ~RecoClustering();
    // dtor

    RecoClustering( const RecoClustering& );
    //copy ctor
    
    virtual void Copy( TObject& ) const;
    //copy to a new obj

    void Clear();
    //clear method

    //setters
    inline void SetNumPoints( Int_t n ) { fNumPoints = n; }
    inline void SetSignalFraction( Double_t f ) { fSignalFraction = f; }
    inline void SetSigFracOverThresh( Double_t f ) { fSigFracOverThresh = f; }
    inline void SetContamination( Double_t c ) { fContamination = c; }
    inline void SetContaminationInCluster( Double_t c ) { fContaminationInCluster = c; }
    inline void SetAbsDevMainTheta( Double_t ad ) { fAbsDevMainTheta = ad; }
    inline void SetAbsDevMainPhi( Double_t ad ) { fAbsDevMainPhi = ad; }

    //getters
    inline Int_t GetNumPoints() { return fNumPoints; }
    inline Double_t GetSignalFraction() { return fSignalFraction; }
    inline Double_t GetSigFracOverThresh() { return fSigFracOverThresh; }
    inline Double_t GetContamination() { return fContamination; }
    inline Double_t GetContaminationInCluster() { return fContaminationInCluster; }
    inline Double_t GetAbsDevMainTheta() { return fAbsDevMainTheta;}
    inline Double_t GetAbsDevMainPhi() { return fAbsDevMainPhi;}

private:
    Int_t fNumPoints;
    Double_t fSignalFraction;         // fraction of all signal in the selected pixels
    Double_t fSigFracOverThresh;      // fraction of signal in the pixels over threshold
                                      // in the selected points
    Double_t fContamination;          // percentual of pure noise pixels in the selected points
    Double_t fContaminationInCluster; // percetual of pure noise pixels in the main cluster
    
    // absolute deviations of the fit in theta-t and phi-t
    // of the main cluster
    Double_t fAbsDevMainTheta;
    Double_t fAbsDevMainPhi;
    
    ClassDef(RecoClustering,1)
};

#endif  /* __RECOCLUSTERING_HH__ */

