// $Id: RecoEnergy.hh 2714 2006-06-06 13:21:07Z naumov $
// Author: R.Pesce   2005/01/04

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoEnergy                                                           *
 *  Package: RecoRootEvent                                                   *
 *  Coordinator: Dmitry.Naumov                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOENERGY_HH__
#define __RECOENERGY_HH__

#include <TObject.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoEnergy                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoEnergy : public TObject {
public:
    RecoEnergy();
    //ctor

    RecoEnergy( const RecoEnergy& );
    //copy ctor
    
    virtual ~RecoEnergy();
    //dtor

    virtual void Copy( TObject& ) const;
    //copy to a new object

    void Clear();
    //clear method

    //setters
    void SetQuality(Double_t t)                         { fQuality = t;         }
    void SetEnergy(Double_t t)                          { fEnergy = t;          }
    void SetEnergyMCError(Double_t t)                   { fEnergyMCError = t;   }
    void SetThetaFoV(Double_t t)                        { fThetaFoVMax = t;     }
    void SetPhiFoV(Double_t t)                          { fPhiFoVMax = t;       }
    void SetAltitude(Double_t t)                        { fAltitudeMax = t;     }
    void SetDepthMax(Double_t t)                        { fDepthMax = t;        }
    void SetAirDensityMax(Double_t t)                   { fAirDensityMax = t;   }
    void SetShapeMethod(Bool_t t)                       { fShapeMethod = t;     }
    void SetFluorescence(Double_t tfl,Double_t ts)      { fTimeFluo = tfl; fTimeSigma = ts;        }
    void SetCherenkov(Bool_t k, Double_t t)             { fCherenkovFound = k; fTimeCherenkov = t; }
    void SetNoiseCluster(Double_t k)                    { fNoiseCluster = k;    }
    void SetSignalCluster(Double_t k)                   { fSignalCluster = k;   }
    void SetNoisePixel(Double_t k)                      { fNoisePerPixel = k;   }
    void SetSignalPixel(Double_t k)                     { fSignalPerPixel = k;  }

    void SetTimeDelay();

    //getters
    Double_t GetQuality()            const { return fQuality;       }
    Double_t GetEnergy()             const { return fEnergy;        }
    Double_t GetEnergyMCError()      const { return fEnergyMCError; }
    Double_t GetTimeSigma()          const { return fTimeSigma;     }
    Double_t GetAltitude()           const { return fAltitudeMax;   }
    Double_t GetDepthMax()           const { return fDepthMax;      }
    Double_t GetAirDensityMax()      const { return fAirDensityMax; }
    Bool_t   IsCherenkovFound()      const { return fCherenkovFound;}
    Bool_t   IsShapeMethod()         const { return fShapeMethod;   }
    Double_t GetTimeDelay()          const { return fTimeDelay;     }
    Double_t GetNoiseCluster()       const { return fNoiseCluster;  }
    Double_t GetSignalCluster()      const { return fSignalCluster; }
    Double_t GetNoisePerPixel()      const { return fNoisePerPixel; }
    Double_t GetSignalPerPixel()     const { return fSignalPerPixel;}
    
private:
    Double_t    fQuality;               // reconstruction quality variable defined by the the module responsible
    Double_t    fEnergy;                // reconstructed energy
    Double_t    fTimeSigma;             // gaussian width of the fluorescence time distribution
    Double_t    fTimeFluo;              // time of the maximum of fluorescence time distribution
    Double_t    fEnergyMCError;         // Absolute rrror in energy
    Double_t    fThetaFoVMax;           // Theta FoV for shower maximum
    Double_t    fPhiFoVMax;             // Phi FoV for shower maximum
    Double_t    fAltitudeMax;           // Altitude of shower maximum 
    Double_t    fDepthMax;              // atmosphere depth at the shower maximum
    Double_t    fAirDensityMax;         // atmosphere density at the shower maximum
    Bool_t      fCherenkovFound;        // 1- found, 0 - not found cherenkov peak
    Bool_t      fShapeMethod;           // 1 - shape method activated, 0 - otherwise (when shape method gives negative altitude)
    Double_t    fTimeCherenkov;         // time of the Cherenkov
    Double_t    fTimeDelay;             // difference between time of cherenkov and time of fluo maximum. Valid if cherenkov is found.
    Double_t    fNoiseCluster;          // estimated number of noise p.e. in the cluster
    Double_t    fSignalCluster;         // estimated number of signal p.e. in the cluster
    Double_t    fNoisePerPixel;         // estimated number of noise p.e. per pixel
    Double_t    fSignalPerPixel;        // estimated _maximum_ number of signal p.e. per pixel
    ClassDef(RecoEnergy,1)
};

#endif  /* __RECOENERGY_HH__ */

