// $Id: RecoProfile.hh 2759 2006-09-07 08:10:49Z moreggia $
// Author: Sylvain Moreggia   2005/11/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoProfile                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOPROFILE_HH__
#define __RECOPROFILE_HH__

#include <TObject.h>

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoProfile                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoProfile : public TObject {
public:
    
    // ctor
    RecoProfile();
    
    // cpy ctor
    RecoProfile( const RecoProfile& );
    
    // dtor
    virtual ~RecoProfile();

    //copy to a new object
    virtual void Copy( TObject& ) const;

    //clear method
    void Clear();

    //setters
    inline void SetQuality(Double_t t)      {fQuality = t;}
    inline void SetXmax(Double_t t)         {fXmax = t;}
    inline void SetTrueXmax(Double_t t)     {fTrueXmax = t;}
    inline void SetZshift(Double_t t)       {fZshift = t;}
    inline void SetThetashift(Double_t t)   {fThetashift = t;}
    inline void SetTOAImpact(Double_t x, Double_t y, Double_t z)
    {
        fRecoTOAImpactX = x;
        fRecoTOAImpactY = y;
        fRecoTOAImpactZ = z;
    }
    inline void SetRecoProfIntegral(Double_t I) {fRecoProfIntegral = I;}
    inline void SetSimuProfIntegral(Double_t I) {fSimuProfIntegral = I;}

    //getters
    inline Double_t GetQuality() const {return fQuality;}
    inline Double_t GetXmax()    const {return fXmax;}
    inline Double_t GetTrueXmax()    const {return fTrueXmax;}
    
private:
    
    Double_t fQuality;               // reconstruction quality variable
    Double_t fXmax;                  // reconstructed Xmax
    Double_t fTrueXmax;              // simulated Xmax
    Double_t fRecoProfIntegral;           // reco integrate profile
    Double_t fSimuProfIntegral;           // simu integrate profile
    Double_t fZshift;                // applied in reco to assess Xmax reco sensitivity to Zmax resolution
    Double_t fThetashift;            // applied in reco to assess Xmax reco sensitivity to Theta resolution
    
    Double_t fTrueEarthImpactX;      // impact of shower on earth (clouds ignored)
    Double_t fTrueEarthImpactY;      // impact of shower on earth (clouds ignored)
    Double_t fTrueEarthImpactZ;      // impact of shower on earth (clouds ignored)
    Double_t fRecoTOAImpactX;        // impact of shower on top of the atmosphere
    Double_t fRecoTOAImpactY;        // impact of shower on top of the atmosphere
    Double_t fRecoTOAImpactZ;        // impact of shower on top of the atmosphere
    

    ClassDef(RecoProfile,1)
};

#endif  /* __RECOPROFILE_HH__ */

