// $Id: RecoTruth.hh 2757 2006-08-24 08:52:32Z moreggia $
// Author: R.Pesce   2005/01/04

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoTruth                                                            *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __RECOTRUTH_HH__
#define __RECOTRUTH_HH__

#include "TObject.h"
#include "TVector3.h"
#include "ESystemOfUnits.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// RecoTruth                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class RecoTruth : public TObject {
public:
    RecoTruth();
    //ctor

    RecoTruth( const RecoTruth& );
    //copy ctor
    
    virtual ~RecoTruth();
    //dtor

    virtual void Copy( TObject& ) const;
    //copy to a new obj

    void Clear();
    //clear method

    //setters
    inline void SetParticleCode(Int_t t)      {fTrueParticleCode = t;}
    inline void SetEnergy(Double_t t)      {fEnergy = t;}
    inline void SetTheta(Double_t t)       {fTheta = t;}
    inline void SetPhi(Double_t t)         {fPhi = t;}
    inline void SetX1(Double_t t)          {fX1 = t;}
    inline void SetInitPos(TVector3 t)     {
        fInitPosX = t.X()/sou::km; 
        fInitPosY = t.Y()/sou::km; 
        fInitPosZ = t.Z()/sou::km;
    }
    inline void SetDir(TVector3 t)     {
        fDirX = t.X()/sou::km; 
        fDirY = t.Y()/sou::km; 
        fDirZ = t.Z()/sou::km;
    }
    inline void SetTOAImpact(TVector3 t) {
        fTOAImpactX = t.X()/sou::km; 
        fTOAImpactY = t.Y()/sou::km; 
        fTOAImpactZ = t.Z()/sou::km;
    }
    inline void SetEarthImpact(TVector3 t) {
        fEarthImpactX = t.X()/sou::km; 
        fEarthImpactY = t.Y()/sou::km; 
        fEarthImpactZ = t.Z()/sou::km;
    }
    inline void SetEarthAge(Double_t t)    {fEarthAge = t;}
    inline void SetXMax(Double_t t)        {fXMax = t;}
    inline void SetNemax(Double_t t)       {fNemax = t;}
    inline void SetMaxPos(TVector3 t)      {
        fMaxPosX = t.X()/sou::km; 
        fMaxPosY = t.Y()/sou::km; 
        fMaxPosZ = t.Z()/sou::km;
    }
    inline void SetHclouds( Double_t h ) { fHclouds = h;    }
    inline void SetcloudsThick( Double_t h )  { fTrueCloudsthick=h; }
    inline void SetcloudsOD( Double_t h )  { fTrueCloudsOD=h; }
    inline void SetLatitude( Double_t h )  { fTrueLatitude=h; }
    inline void SetLongitude( Double_t h )  { fTrueLongitude=h; }
    inline void SetDate( Double_t h )   { fTrueDate=h; }
    inline void SetHground( Double_t h )   { fTrueHGround=h; }
    inline void SetGroundAlbedo( Double_t h )   { fTrueGroundAlbedo=h; }
    inline void SetGroundSpecular( Double_t h )   { fTrueGroundSpecular=h; }
    
    
    //getters
    inline Int_t    GetParticleCode()   const { return fTrueParticleCode;  }
    inline Double_t GetEnergy()   const { return fEnergy;  }
    inline Double_t GetTheta()    const { return fTheta;   }
    inline Double_t GetPhi()      const { return fPhi;     }
    inline Double_t GetX1()       const { return fX1;      }
    inline Double_t GetXmax()     const { return fXMax;    }
    inline Double_t GetNemax()    const { return fNemax;    }
    inline Double_t GetHmax()     const { return fMaxPosZ; }
    inline Double_t GetEarthAge() const { return fEarthAge;}
    inline TVector3 GetInitPos()     const {return TVector3(fInitPosX,fInitPosY,fInitPosZ);}
    inline TVector3 GetDir()         const {return TVector3(fDirX,fDirY,fDirZ);}
    inline TVector3 GetMaxPos()      const {return TVector3(fMaxPosX,fMaxPosY,fMaxPosZ);}
    inline TVector3 GetTOAImpact()   const {return TVector3(fTOAImpactX,fTOAImpactY,fTOAImpactZ);}
    inline TVector3 GetEarthImpact() const {return TVector3(fEarthImpactX,fEarthImpactY,fEarthImpactZ);}
    Double_t GetInitPos( Int_t )     const;
    Double_t GetMaxPos( Int_t )      const;
    Double_t GetEarthImpact( Int_t ) const;
    inline Double_t GetHclouds() const  { return fHclouds; }
    inline Double_t GetcloudsThick() const  { return fTrueCloudsthick; }
    inline Double_t GetcloudsOD() const  { return fTrueCloudsOD; }
    inline Double_t GetLatitude() const  { return fTrueLatitude; }
    inline Double_t GetLongitude() const  { return fTrueLongitude; }
    inline Double_t GetDate() const  { return fTrueDate; }
    inline Double_t GetHground() const  { return fTrueHGround; }
    inline Double_t GetGroundAlbedo() const  { return fTrueGroundAlbedo; }
    inline Double_t GetGroundSpecular() const  { return fTrueGroundSpecular; }

private:
    Int_t    fTrueParticleCode;
    Double_t fEnergy;
    Double_t fTheta;
    Double_t fPhi;
    Double_t fX1;
    Double_t fInitPosX;
    Double_t fInitPosY;
    Double_t fInitPosZ;
    Double_t fDirX; // in MES
    Double_t fDirY; // in MES
    Double_t fDirZ; // in MES
    Double_t fMaxPosX;
    Double_t fMaxPosY;
    Double_t fMaxPosZ;
    Double_t fXMax;
    Double_t fNemax;
    Double_t fTOAImpactX;
    Double_t fTOAImpactY;
    Double_t fTOAImpactZ;
    Double_t fEarthImpactX;
    Double_t fEarthImpactY;
    Double_t fEarthImpactZ;
    Double_t fEarthAge;
    Double_t fHclouds;
    Double_t fTrueCloudsthick;       // clouds thickness
    Double_t fTrueCloudsOD;          // clouds OD 
    Double_t fTrueLatitude;          // geodetic latitude (also used to identify std atmospheres)
                                     // USStd         : 100
				     // Tropical      : 101
				     // MidLat Summer : 102
				     // MidLat Winter : 103
				     // LinsleyAtmo   : 104
    Double_t fTrueLongitude;         // geodetic longitude
    Double_t fTrueDate;              // event date
    Double_t fTrueHGround;           // ground altitude
    Double_t fTrueGroundAlbedo;      // ground albedo
    Double_t fTrueGroundSpecular;    // ground specular component intensity

    ClassDef(RecoTruth,2)
};

#endif  /* __RECOTRUTH_HH__ */

