#ifndef __ARRAYSURFACE__
#define __ARRAYSURFACE__

#include "SSurface.h"
#include "LensUtils.h"
#include "FSIntersectedSegment.h"

#include <globals.hh>
#include <geomdefs.hh>
#include <G4ThreeVector.hh>
#include <float.h>

namespace G4FresnelLens{

class ArraySurface;
typedef int (ArraySurface::*ASGetIdxFun)(double);
typedef double (ArraySurface::*ASDistToSegFun)(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment*);

class SecondOrderSurface;
class ArraySurface : public SSurface {
public:
    ArraySurface(int n, double* rho, double* z);
    virtual ~ArraySurface();

    void Configure();
    double PointOnSurf(double x);
    double PointOnSurf(int idx, double x);
    double PointOnSurf2(double x);
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v);
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* i);
    double DistanceToSurfSafe(const G4ThreeVector& p);
    double TestEdgeTolerance(int dir, const G4ThreeVector& p, const G4ThreeVector& v);
    bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm);
    bool Point(int i, double step, double& rho, double& z);

    // plugs
    EInside Inside(int idx, double rho, double z, FSIntersectedSegment *intS=0);
    EInside Inside(double rho, double z, FSIntersectedSegment *intS=0);
    double FindNearestIntersection(const G4ThreeVector& p1,
                                           const G4ThreeVector& v,
                                           FSIntersectedSegment* intS=0);
    double Derivative(double rho);
    double Derivative(int idx, double rho);

    void FillSegment(double rho, FSIntersectedSegment *intS);
    void FillSegment(int idx, double rho, FSIntersectedSegment *intS);

    bool operator==(SSurface&);

    int GetCurIndex(double perp);
    /*Nonfixed step*/// ASGetIdxFun GetCurIndexPointer;
    /*Nonfixed step*/// int GetCurIndexConstStep(double perp);
    /*Nonfixed step*/// int GetCurIndexVarStep(double perp);

    int GetN()   { return fN; }
    double* GetRho() { return fRho; }
    double* GetZ()   { return fZ; }
    void print();

    static double ConeIntersection ( const G4ThreeVector& p, const G4ThreeVector& v,
                                     double r1, double z1, double r2, double z2);
    static double ConeIntersection1( const G4ThreeVector& p, const G4ThreeVector& v,
                                     double r1, double z1, double r2, double z2,
                                     bool& inlimits, double mindist=0.);
    EInside InsideCone(double rho, double z, double rho1, double z1, double rho2, double z2);
protected:
    ASDistToSegFun DetermineDistToSeg;
    double DetermineDistToSegDirect(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* intseg=0);
    double DetermineDistToSegSmart(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* intseg=0);
    double FindCone(const G4ThreeVector& p, const G4ThreeVector&  v, int point1, int point2, int dir, int& idx);
    double FindConeConcave(double perp, const G4ThreeVector& p, const G4ThreeVector&  v, int idx, FSIntersectedSegment* intseg=0);
    double FindConeConvex(double perp, const G4ThreeVector& p, const G4ThreeVector&  v, int idx, FSIntersectedSegment* intseg=0);

    int     fN;
    double* fRho;
    double* fZ;
    double  fStep;
    int     fConvexity;
    //SecondOrderSurface* fFit;

    static int fStepsLimit;
};

//__________________________________________________________________________________
inline double ArraySurface::PointOnSurf2(double x2){
    return PointOnSurf(sqrt(x2));
}

//__________________________________________________________________________________
inline double ArraySurface::FindNearestIntersection(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* intSeg){
    return (this->*DetermineDistToSeg)(p,v,intSeg);
}

//______________________________________________________________________________________________________________________
inline double ArraySurface::TestEdgeTolerance(int dir, const G4ThreeVector& p, const G4ThreeVector& v){
    switch (dir){
        case -1: return ConeIntersection(p, v, fRho[fN-2], fZ[fN-2], fRho[fN-1], fZ[fN-1]); break;
        case  1: return ConeIntersection(p, v, fRho[0], fZ[0], fRho[1], fZ[1]); break;
        default: printf("ArraySurface::TestEdgeTolerance: bad value"); exit(1);
    }

    return 0.;
}

//__________________________________________________________________________________
/*Nonfixed step*///inline int ArraySurface::GetCurIndexConstStep(double perp){
inline int ArraySurface::GetCurIndex(double perp){
    return int((perp-fRho[0])/fStep);
}

/*Nonfixed step*/////__________________________________________________________________________________
/*Nonfixed step*///inline int ArraySurface::GetCurIndexVarStep(double perp){
    /*Nonfixed step*///return BinarySearch(fRho, perp, 0, fN);
/*Nonfixed step*///}

/*Nonfixed step*/////__________________________________________________________________________________
/*Nonfixed step*///inline int ArraySurface::GetCurIndex(double perp){
    /*Nonfixed step*///return (this->*GetCurIndexPointer)(perp);
/*Nonfixed step*///}

//__________________________________________________________________________________
inline EInside ArraySurface::Inside(double rho, double z, FSIntersectedSegment *intS){
    if ( z>fZmax+kHalfTolerance ) return kOutside;
    if ( z<fZmin-kHalfTolerance ) return kInside;
    int idx=this->GetCurIndex(rho);
    if ( idx<0. || idx>=fN ) return kOutside;
    return Inside(idx, rho, z, intS);
}

//______________________________________________________________________________
inline double ArraySurface::PointOnSurf(double x){
    int idx=this->GetCurIndex(x);
    return PointOnSurf(idx, x);
};

//______________________________________________________________________________
inline double ArraySurface::PointOnSurf(int idx, double x){
    return fZ[idx]+(x-fRho[idx])*Derivative(idx,x);
};

//______________________________________________________________________________
inline double ArraySurface::Derivative(double rho){
    int idx=this->GetCurIndex(rho);
    return Derivative(idx, rho);
}

//______________________________________________________________________________
inline double ArraySurface::Derivative(int idx, double rho){
    return (fZ[idx+1]-fZ[idx])/(fRho[idx+1]-fRho[idx]);
}

//______________________________________________________________________________
inline double ArraySurface::DetermineDistToSegSmart(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* intseg){
    double perp=p.perp();
    int idx=this->GetCurIndex(perp);
    if( idx>=fN-1 ) idx=fN-2;
    double dz=p.z()-PointOnSurf(idx, perp);
    return dz*fConvexity<0.?FindConeConcave(perp, p, v, idx, intseg):FindConeConvex(perp, p, v, idx, intseg);
}

//______________________________________________________________________________
inline double ArraySurface::DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v, FSIntersectedSegment* intSeg){
   return (this->*DetermineDistToSeg)(p,v,intSeg);
}

//______________________________________________________________________________
inline double ArraySurface::DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v){
   return (this->*DetermineDistToSeg)(p,v,0);
}

//______________________________________________________________________________
inline void ArraySurface::FillSegment(double rho, FSIntersectedSegment *intS){
    int idx=this->GetCurIndex(rho);
    return FillSegment(idx, rho, intS);
}

//______________________________________________________________________________
inline void ArraySurface::FillSegment(int idx, double rho, FSIntersectedSegment *intSeg){
    intSeg->surf=this;
    intSeg->perp=rho;
    intSeg->idx=idx;
}

//______________________________________________________________________________
inline void ArraySurface::print(){
    printf("array surface\tr(%g,%g)\tz(%g,%g)\tn=%i\n", fR1, fR, fZ1, fZ2, fN);
}

//______________________________________________________________________________
inline bool ArraySurface::Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double out, G4ThreeVector* norm){
    int idx=ints->idx;
    double dz = fZ[idx+1]-fZ[idx];
    double dr = fRho[idx+1]-fRho[idx];
    double hyp=sqrt(dz*dz+dr*dr);
    double cos_theta = fabs(dr)/hyp;
    double psin_theta = fabs(dz)/hyp/p.perp();
    double left=(dz<0?+1:-1)*out;
    norm->set(left*p.x()*psin_theta,left*p.y()*psin_theta,cos_theta*out);
    return true;
}

//______________________________________________________________________________
inline bool ArraySurface::Point(int i, double step, double& rho, double& z){
    if ( i>=fN ) return false;
    rho=fRho[i];
    z=fZ[i];
    return true;
}

};
#endif
