#ifndef __CYLINDRICSURFACE__
#define __CYLINDRICSURFACE__

#include "ParametricSurface.h"

namespace G4FresnelLens{

class CylindricSurface : public ParametricSurface
{
public:
    CylindricSurface(double z1=0., double z2=0., double rmax=0.);
   ~CylindricSurface();

    void Configure();
    double PointOnSurf(double x);
    double PointOnSurf2(double x);
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v);
    double DistanceToSurfSafe(const G4ThreeVector& p);
    double Derivative(double x);
    EInside Inside(double rho, double z, FSIntersectedSegment *intSeg=0);
    virtual void FillSegment(double rho, FSIntersectedSegment *intS);
    bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm);
    bool operator==(SSurface&);

    void SetZ(double zmin, double zmax); //keeps direction
    void print();
protected:
    void GuessParameters(int n, int tn, int* idx, double* rho, double* z, bool minimum){};

private:
    int fDirection; // +1 up of -1 for down
};

//__________________________________________________________________________________
inline double CylindricSurface::PointOnSurf(double x){
    printf("should not be called\n");
    return fZmin;
}

//__________________________________________________________________________________
inline double CylindricSurface::PointOnSurf2(double x2){
    printf("should not be called\n");
    return fZmin;
}

//__________________________________________________________________________________
inline double CylindricSurface::Derivative(double x){
    return fDirection*DBL_MAX;
}

//______________________________________________________________________________________________________________________
inline void CylindricSurface::FillSegment(double rho, FSIntersectedSegment *intSeg){
    intSeg->surf=this;
    intSeg->perp=rho;
}

//______________________________________________________________________________________________________________________
inline EInside CylindricSurface::Inside(double rho, double z, FSIntersectedSegment *intSeg){
    if ( z>(fZmax+kHalfTolerance) || rho>fR+kHalfTolerance ) return kOutside;
    if ( rho<fR-kHalfTolerance || z<(fZmin-kHalfTolerance) ) return kInside;
    if (intSeg) FillSegment(rho, intSeg);
    return kSurface;
}

//______________________________________________________________________________________________________________________
inline void CylindricSurface::SetZ(double zmin, double zmax){
    fZmin=zmin;
    fZmax=zmax;
    if (fDirection>0){
        fZ1=fZmin;
        fZ2=fZmax;
    }
    else {
        fZ1=fZmax;
        fZ2=fZmin;
    }
}

//______________________________________________________________________________________________________________________
inline void CylindricSurface::print(){
    printf("cyl surface\tr=%g    \tz(%g, %g)\tup=%i\tmin/max=%g/%g\n", fR, fZ1, fZ2, fDirection, fZmin, fZmax);
}

//______________________________________________________________________________
inline bool CylindricSurface::Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double out, G4ThreeVector* norm){
    double invperp=-fDirection*out/p.perp();
    norm->set(p.x()*invperp,p.y()*invperp,0.);
    return true;
}

};

#endif
