#ifndef __FIXEDSTACK__
#define __FIXEDSTACK__
#include <stack>

namespace G4FresnelLens{
//__________________________________________________________________________________
struct fcIterationStep{
    //fcIterationStep ( int i1, int i2, double lr, double lz, double rr, double rz ) :
    //                 leftI(i1), rightI(i2), leftRho(lr), rightRho(rr), leftZ(lz), rightZ(rz) {};
    fcIterationStep(){};
    fcIterationStep( int i1, int i2 ) : leftI(i1), rightI(i2) {};
    int leftI, rightI;
    //double leftRho, rightRho, leftZ, rightZ;
};

//__________________________________________________________________________________
#define LIFOStackSize 20
//#define LIFOStackDebug
class fixedISStack {
    public:
        fixedISStack() : size(LIFOStackSize), last(-1) {};

        inline void push(int i1, int i2/*, double lr, double lz, double rr, double rz*/);
        inline fcIterationStep* pop();
        bool empty() { return last<0; }
        inline void clear();

    private:
        fcIterationStep steps[LIFOStackSize];
        int size, last;
        std::stack<fcIterationStep*> overSteps;
};

//__________________________________________________________________________________
void fixedISStack::push(int i1, int i2/*, double lr, double lz, double rr, double rz*/){
    last++;
    fcIterationStep* cstep;
    if ( last>=LIFOStackSize ) {
        //Warning_("fixedISStack overfill. Recompile!");
        //return;
        cstep=new fcIterationStep(i1,i2);
        overSteps.push(cstep);
        #ifdef LIFOStackDebug
          printf("stack push (dynamic)  %i: %9i, %9i\n", last, cstep->leftI, cstep->rightI);
        #endif
    }
    else {
        cstep=(steps+last);
        cstep->leftI=i1;
        cstep->rightI=i2;
        #ifdef LIFOStackDebug
          printf("stack push (static)  %i: %9i, %9i\n", last, steps[last].leftI, steps[last].rightI);
        #endif
    }
}

//__________________________________________________________________________________
fcIterationStep* fixedISStack::pop(){
    if ( last<0 ) {
        #ifdef LIFOStackDebug
          printf("stack pop: empty\n");
        #endif
        return 0;
    }
    if ( last>=LIFOStackSize ) {
        fcIterationStep* out=overSteps.top();
        overSteps.pop();
        last--;
        #ifdef LIFOStackDebug
          printf("stack pop (dynamic)  %i: %9i, %9i\n", last, out->leftI, out->rightI);
        #endif
        return out;
    }

    #ifdef LIFOStackDebug
      printf("stack pop (static)  %i: %9i, %9i\n", last, steps[last].leftI, steps[last].rightI);
    #endif
    return steps+last--;
}

//__________________________________________________________________________________
void fixedISStack::clear(){
    #ifdef LIFOStackDebug
      printf("stack pop: clear\n");
    #endif
    if ( last>=LIFOStackSize ) {
        while ( !overSteps.empty() ){
            delete overSteps.top();
            overSteps.pop();
        }
    }
    last=-1;
}

};
#endif
