#ifndef FlatSurface_HH
#define FlatSurface_HH
#include "ParametricSurface.h"

namespace G4FresnelLens{

class FlatSurface : public ParametricSurface {
public:
    FlatSurface(double zpos=0., double frhomax=0.);
    virtual ~FlatSurface(){};

    void Configure();
    double PointOnSurf(double x) { return fZpos; }
    double PointOnSurf2(double x2) { return fZpos; }
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v);
    double DistanceToSurfSafe(const G4ThreeVector& p);
    void FillSegment(double rho, FSIntersectedSegment *intSeg);
    bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm);

    EInside Inside(double rho, double z, FSIntersectedSegment* intSeg=0);
    double Derivative(double rho) { return 0.; }

protected:
    void GuessParameters(int n, int tn, int* idx, double* rho, double* z, bool minimum){};
    void GuessParameters(const std::vector<SSurface*>& surfaces, bool minimum){};

private:

};

//__________________________________________________________________________________
inline void FlatSurface::FillSegment(double rho, FSIntersectedSegment* intSeg){
    intSeg->surf=this;
    intSeg->perp=rho;
}

//______________________________________________________________________________
inline bool FlatSurface::Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double out, G4ThreeVector* norm){
    norm->set(0.,0.,out);
    return true;
}

};
#endif
