#ifndef __FresnelOutputDef__
#define __FresnelOutputDef__

#include "FresnelOutput_def.h"

#ifdef __FOD_FRESNEL__
  #include "Verbosity.h"
  #define __INHERITSTRING           ,public VirtualVerbosity
  #define __INHERITSTRING1          :public VirtualVerbosity
  #define __INHERITCONSTRUCTOR(str) VirtualVerbosity(str),
  #define __REG_ACTION(str)         //str

  #define debug_                debug()
  #define info_                 info()
  #define warning_              warning()
  #define error_                error()
  #define endline_              endline
  #define Info_(msg)            this->InfoMessage(msg);
  #define Warning_(msg)         this->WarningMessage(msg);
  #define Error_(msg)           this->ErrorMessage(msg);
  #define Abort_(msg)           this->AbortMessage(msg);
#endif

#ifdef __FOD_ESAF__
  #include "EsafMsgSource.hh"
  #define __INHERITSTRING           ,public EsafMsgSource
  #define __INHERITSTRING1          :public EsafMsgSource
  #define __INHERITCONSTRUCTOR(str) /*str*/
  #define __REG_ACTION(str)         SetSender(str);

  #define debug_                Msg(EsafMsg::Debug)
  #define info_                 Msg(EsafMsg::Info)
  #define warning_              Msg(EsafMsg::Warning)
  #define error_                Msg(EsafMsg::Error)
  #define endline_              MsgDispatch
  #define Info_(msg)            MsgForm(EsafMsg::Info,msg);
  #define Warning_(msg)         MsgForm(EsafMsg::Warning,msg);
  #define Error_(msg)           MsgForm(EsafMsg::Error,msg);
  #define Abort_(msg)           {MsgForm(EsafMsg::Panic,msg);exit(1);}
#endif

#ifdef __FOD_GEANT4__
  #include <G4ios.hh>
  #define __INHERITSTRING
  #define __INHERITSTRING1
  #define __INHERITCONSTRUCTOR(str) /*str*/
  #define __REG_ACTION(str)     //str

  #define debug_                G4cout<<"Debug: "
  #define info_                 G4cout<<"Info: "
  #define warning_              G4cout<<"Warning: "
  #define error_                G4cerr<<"Error: "
  #define endline_              G4endl
  #define Info_(msg)            G4cout<<"Info: "<<msg<<G4endl;
  #define Warning_(msg)         G4cout<<"Warning: "<<msg<<G4endl;
  #define Error_(msg)           G4cerr<<"Error: "<<msg<<G4endl;
  #define Abort_(msg)           {G4cerr<<"Abort: "<<msg<<G4endl;exit(1);}
#endif

#ifdef __FOD_STANDALONE__
  #include <iostream>

  #define __INHERITSTRING
  #define __INHERITSTRING1
  #define __INHERITCONSTRUCTOR(str) /*str*/
  #define __REG_ACTION(str)     //str

  #define debug_                std::cout
  #define info_                 std::cout
  #define warning_              std::cout
  #define error_                std::cerr
  #define endline_              std::endl
  #define Info_(msg)            std::cout<<msg<<std::endl;
  #define Warning_(msg)         std::cout<<msg<<std::endl;
  #define Error_(msg)           std::cerr<<msg<<std::endl;
  #define Abort_(msg)         { std::cerr<<msg<<std::endl; exit(1); }
#endif

#endif
