#ifndef __FRESNELSURFACEHELPER__
#define __FRESNELSURFACEHELPER__

#include "SSurface.h"
#include <vector>

namespace G4FresnelLens {
class FresnelSurface;
class ParametricSurface;
class FresnelSurfaceHelper {
public:
    FresnelSurfaceHelper(const char* filename=0);
    FresnelSurfaceHelper(int n, double *rho, double *z);
   ~FresnelSurfaceHelper();

    void Read(const char* file);
    void Read(int n, double *rho, double *z);

    void SetOptions(double shift, double scale, double zfilter)
        { fShift=shift; fScale=scale; fZfilter=zfilter; }
    void SetFitAccuracy(double a) { fFitAccuracy=a; }

    std::vector<SSurface*> fSurfaces;
    FresnelSurface* CreateFresnelLens(ParametricSurface* downsurf, ParametricSurface* upsurf);
    bool fOwnSurfaces;
private:
    void AddPoint(double rho, double z);
    void Init();
    void Convert();
    void Clear();

    bool fOwnArrays;
    int fN;
    double *fRho;
    double *fZ;

    double fFitAccuracy;
    double fZfilter;
    double fShift;
    double fScale;
};

//__________________________________________________________________________________
inline void FresnelSurfaceHelper::AddPoint(double rho, double z){
    if (z==fZfilter) {
        //printf("filter %i %g %g\n", fN, rho, z);
        return;
    }
    //printf("%i %g %g\n", fN, rho, z);
    fRho[fN]=rho;
    fZ[fN]=z*fScale+fShift;
    fN++;
}

};
#endif
