#ifndef G4FresnelLens_HH
#define G4FresnelLens_HH
#include "FSIntersectedSegment.h"

#include <geomdefs.hh>
#include <G4ThreeVector.hh>

namespace G4FresnelLens{
    bool InLimits(double perp, double z, double r1, double z1, double r2, double z2);
    bool InLimits2(double perp2, double z, double r1, double z1, double r2, double z2);
    int  SolveQuadratic(double a, double b, double c, double& x1, double& x2);
    int  BinarySearch(double*, double, int, int);
    bool CompareVectors(const G4ThreeVector& v1, const G4ThreeVector& v2, double tolerance);
    bool VectorIsZero(const G4ThreeVector& v1);
    double Hyp(double x1, double y1, double x2, double y2);
    bool VectorGoesOut(const G4ThreeVector& p, const G4ThreeVector& v);
};

//__________________________________________________________________________________
inline double G4FresnelLens::Hyp(double x1, double y1, double x2, double y2){
    double dx=x2-x1;
    double dy=y2-y1;
    return sqrt(dx*dx+dy*dy);
}

//__________________________________________________________________________________
inline bool G4FresnelLens::VectorGoesOut(const G4ThreeVector& p, const G4ThreeVector& v){
    return (p.x()*v.x()+p.y()*v.y())>=0.;
}

//__________________________________________________________________________________
inline bool G4FresnelLens::InLimits(double perp, double z, double r1, double z1, double r2, double z2){
    if ( z<z1 || z>z2 ) return false;
    if ( perp<r1 || perp>r2 ) return false;
    return true;
}

//__________________________________________________________________________________
inline bool G4FresnelLens::InLimits2(double perp2, double z, double r1, double z1, double r2, double z2){
    if ( z<z1 || z>z2 ) return false;
    if ( perp2<r1*r1 || perp2>r2*r2 ) return false;
    return true;
}

//__________________________________________________________________________________
inline bool G4FresnelLens::CompareVectors(const G4ThreeVector& v1, const G4ThreeVector& v2, double tolerance) {
    if ( fabs(v1.x()-v2.x())<tolerance &&
         fabs(v1.y()-v2.y())<tolerance &&
         fabs(v1.z()-v2.z())<tolerance
       ) return true;

   return false;
}

//__________________________________________________________________________________
inline bool G4FresnelLens::VectorIsZero(const G4ThreeVector& v1){
    if ( v1.x()==0 &&
         v1.y()==0 &&
         v1.z()==0
       ) return true;

   return false;
}

#endif
