#ifndef __PARABOLICSURFACE__
#define __PARABOLICSURFACE__

#include "ParametricSurface.h"

namespace G4FresnelLens{

class ParabolicSurface : public ParametricSurface
{
public:
    ParabolicSurface(double k=DBL_MAX, double z=0., double rhomax=0.);
   ~ParabolicSurface();

    void Configure();
    double PointOnSurf(double x);
    double PointOnSurf2(double x);
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v);
    double DistanceToSurfSafe(const G4ThreeVector& p);
    void FillSegment(double rho, FSIntersectedSegment *intSeg);
    double Derivative(double rho);
    bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm);

    void print();
protected:
    void GuessParameters(int n, int tn, int* idx, double* rho, double* z, bool minimum);
    void GuessParameters(const std::vector<SSurface*>& surfaces, bool minimum);

private:
    double fK;
};

//__________________________________________________________________________________
inline void ParabolicSurface::FillSegment(double rho, FSIntersectedSegment *intSeg){
    intSeg->surf=this;
    intSeg->perp=rho;
}

//__________________________________________________________________________________
inline double ParabolicSurface::PointOnSurf(double x){
    return fZpos+fK*x*x;
}

//__________________________________________________________________________________
inline double ParabolicSurface::PointOnSurf2(double x2){
    return fZpos+fK*x2;
}

//__________________________________________________________________________________
inline double ParabolicSurface::Derivative(double x){
    return 2.*fK*x;
}

//______________________________________________________________________________
inline bool ParabolicSurface::Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double out, G4ThreeVector* norm){
    double perp=p.perp();
    double tgth=Derivative(perp);
    double c=out/sqrt(1.+tgth*tgth);
    double s=tgth*c;
    double iperp=-1./perp;
    norm->set(p.x()*iperp*s,p.y()*iperp*s,c);
    return true;
}

};

#endif
