#ifndef __SPHERICSURFACE__
#define __SPHERICSURFACE__

#include "ParametricSurface.h"

namespace G4FresnelLens{

class SphericSurface : public ParametricSurface
{
public:
    SphericSurface(double rcurv, double z=0., double rmax=0.);
   ~SphericSurface();

    void Configure();
    double PointOnSurf(double x);
    double PointOnSurf2(double x);
    double DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v);
    double DistanceToSurfSafe(const G4ThreeVector& p);
    double Derivative(double x);
    bool Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double up, G4ThreeVector* norm);
    void FillSegment(double rho, FSIntersectedSegment *intSeg);

    void print(){ printf("z=%g r=%g curv=%i min/max=%g/%g\n", fZpos, fRcurv, fCurv, fZmin, fZmax); }

protected:
    void GuessParameters(int n, int tn, int* idx, double* rho, double* z, bool minimum);
    void GuessParameters(const std::vector<SSurface*>& surfaces, bool minimum);
private:
    double fRcurv;
    int    fCurv;
};

//__________________________________________________________________________________
inline void SphericSurface::FillSegment(double rho, FSIntersectedSegment *intSeg){
    intSeg->surf=this;
    intSeg->perp=rho;
}

//__________________________________________________________________________________
inline double SphericSurface::PointOnSurf(double x){
    return fZpos-fCurv*sqrt(fRcurv*fRcurv-x*x);
}

//__________________________________________________________________________________
inline double SphericSurface::PointOnSurf2(double x2){
    return fZpos-fCurv*sqrt(fRcurv*fRcurv-x2);
}

//__________________________________________________________________________________
inline double SphericSurface::Derivative(double x){
    return fCurv*x/sqrt(fRcurv*fRcurv-x*x);
}

//__________________________________________________________________________________
inline bool SphericSurface::Normal(FSIntersectedSegment* ints, const G4ThreeVector& p, double out, G4ThreeVector* norm){
    norm->set(p.x(), p.y(), p.z()-fZpos);
    *norm=norm->unit()*(-out*fCurv);
    return true;
}

};
#endif
