#include "CylindricSurface.h"
#include "LensUtils.h"

using std::swap;
using namespace G4FresnelLens;

//______________________________________________________________________________
CylindricSurface::CylindricSurface(double z1, double z2, double r):ParametricSurface(z1, r)
{
    fZ1=z1;
    fZ2=z2;
    if ( z1>z2 ){
        fZmin=z2;
        fZmax=z1;
        fDirection=-1;
    }
    else {
        fZmin=z1;
        fZmax=z2;
        fDirection=1;
    }
}

//______________________________________________________________________________
CylindricSurface::~CylindricSurface(){


}

//______________________________________________________________________________
void CylindricSurface::Configure(){
}

//__________________________________________________________________________________
double CylindricSurface::DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v){
    double a = v.perp2();
    double b = 2.0*(p.x()*v.x() + p.y()*v.y());
    double c = p.perp2()-fR*fR;

    double d1, d2;
    int nroot=SolveQuadratic(a, b, c, d1, d2);
    if(nroot<2 || d2<0) return kInfinity;

    if( d1>=0.0 ){
        double z=p.z()+v.z()*d1;
        if( (z-fZmin)*(z-fZmax)<0 ) return d1;
    }
    double z=p.z()+v.z()*d2;
    if( (z-fZmin)*(z-fZmax)<0 ) return d2;
    return kInfinity;
}

//__________________________________________________________________________________
double CylindricSurface::DistanceToSurfSafe(const G4ThreeVector& p){
    if ( p.z()>fZmax ) return p.z()-fZmax;
    if ( p.z()<fZmin ) return fZmin-p.z();
    return fabs(p.perp()-fR);
}

//__________________________________________________________________________________
bool CylindricSurface::operator==(SSurface& s){
    CylindricSurface* to1=static_cast<CylindricSurface*>(&s);
    if (!to1) {
        return false;
    }
    CylindricSurface& to=*to1;
    if( fR!=to.fR ||
        fabs(fZ1-to.fZ1)>1.e-13 ||
        fabs(fZ2-to.fZ2)>1.e-13 ||
        fDirection!=to.fDirection ){
        print();
        to.print();
        printf("%g %g %g\n",fR-to.fR, fZ1-to.fZ1, fZ2-to.fZ2);
        return false;
    }
    return true;
}
