#include "FlatSurface.h"

#include <float.h>
using namespace G4FresnelLens;

//__________________________________________________________________________________
FlatSurface::FlatSurface(double zpos, double rhomax) : ParametricSurface(zpos, rhomax) {
    Configure();
}

//__________________________________________________________________________________
void FlatSurface::Configure(){
    fZ1=fZ2=fZmin=fZmax=fZpos;
}

//__________________________________________________________________________________
double FlatSurface::DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& v){
    if(v.z()==0) return kInfinity;

    double dz=fZpos-p.z();
    double cos_theta=v.cosTheta();
    double dist = dz/cos_theta;
    if( dist<0 ) return kInfinity;
    if ( (p+v*dist).perp2()<=fR*fR) return dist;

    return kInfinity;
}

//__________________________________________________________________________________
double FlatSurface::DistanceToSurfSafe(const G4ThreeVector& p){
    double perp  = p.perp();
    if(perp<=fR) return fabs(p.z()-fZpos);

    double drho1 = p.perp()-fR;
    double dz1 = fZpos - p.z();
    return sqrt(drho1*drho1 + dz1*dz1);
}

//__________________________________________________________________________________
EInside FlatSurface::Inside(double rho, double z, FSIntersectedSegment* intseg){
    if ( intseg ) FillSegment(rho, intseg);

    if ( rho>(fR+kTolerance) || z>(fZpos+kHalfTolerance) ) return kOutside;
    if ( z<(fZpos-kHalfTolerance) ) return kInside;
    return kSurface;
}
