#include "SphericSurface.h"
#include "LensUtils.h"

using std::swap;
using namespace G4FresnelLens;

//______________________________________________________________________________
SphericSurface::SphericSurface(double rcurv, double z, double rmax):
ParametricSurface(z, rmax),
fRcurv(fabs(rcurv))
{
    fCurv=rcurv>0?+1:-1;
    fZmin=fZmax=0.;
    Configure();
}

//______________________________________________________________________________
SphericSurface::~SphericSurface(){


}

//__________________________________________________________________________________
void SphericSurface::GuessParameters(int n, int tn, int* idx, double* rho, double* z, bool minimum){
    //AbstractMethod();
    return;
    //if ( fabs(fRcurv)==DBL_MAX) return;
    //int i1a=idx[1];
    //int i1b=i2a+1;
    //int i1=(z[i1a]>z[i1b])^minimum?i1a:i1b;
    //int i2a=idx[n-1];
    //int i2b=i2a+1;
    //int i2=(z[i2a]>z[i2b])^minimum?i2a:i2b;
    //fCurv=z[i2]>z[i1]?+1:-1;

    //
    // TODO: not implemented
    //
}

//______________________________________________________________________________
void SphericSurface::GuessParameters(const std::vector<SSurface*>& surfaces, bool minimum){
    //AbstractMethod();
    return;
}

//______________________________________________________________________________
void SphericSurface::Configure(){
    fZ1=fZmin=PointOnSurf(0.);
    fZ2=fZmax=PointOnSurf(fR);
    if ( fZmin>fZmax ) swap(fZmin,fZmax);
}

//__________________________________________________________________________________
double SphericSurface::DistanceToSurf(const G4ThreeVector& p, const G4ThreeVector& L){
    G4ThreeVector C(p.x(), p.y(), p.z()-fZpos);
    double b=2.*L*C;
    double c=C.mag2()-fRcurv*fRcurv;
    double d1, d2;
    int nroots=SolveQuadratic(1., b, c, d1, d2);
    if(!nroots) return kInfinity;

    G4ThreeVector pos;
    double zpos;
    if ( d1>=0. ) {
        pos = p+L*d1;
        zpos=pos.z();
        if ( pos.perp2()<=fR*fR && zpos>=fZmin && zpos<=fZmax ) return d1;
    }
    if ( nroots==1 || d2<0. ) return kInfinity;
    pos=p+L*d2;
    zpos=pos.z();
    if ( pos.perp2()<=fR*fR && zpos>=fZmin && zpos<=fZmax ) return d2;
    return kInfinity;
}

//__________________________________________________________________________________
double SphericSurface::DistanceToSurfSafe(const G4ThreeVector& p){
    double center=fZpos;
    double dz=(center-p.z())*fCurv;
    double rad=sqrt(dz*dz+p.perp2());
    if( rad==0. ) return fRcurv;

    double cos_theta=dz/rad;
    double z_edge=fCurv>0?fZmax:fZmin;
    double cos_theta0=( center - z_edge )*fCurv/fRcurv;
    if(cos_theta>=cos_theta0) return fabs(fRcurv-rad);

    double drho1 = fR-p.perp();
    double dz1 = z_edge - p.z();
    return sqrt(drho1*drho1 + dz1*dz1);
}
