#ifndef MaterialParameters_h
#define MaterialParameters_h

#include "EsafMsgSource.hh"
#include "EsafConfigurable.hh"
#include <globals.hh>
class TGraph;
class TF1;
class G4Material;
class G4MaterialPropertiesTable;
class G4OpticalSurface;
class MaterialParameters : public EsafMsgSource {
public:
    MaterialParameters(const char* re, const char* im);
    MaterialParameters(const char* re=0);
    ~MaterialParameters();

    // real refraction index
    void setRealRI(const char* re);

    void setRealRI(const char* name, double (*function)(double));
    void setRealRI(TF1* f,double emin,double emax);
    // imaginary refraction index
    void setImaRI(const char* im);
    //both
    void setRI(const char* ri);
    // or set absorption length
    void setAL(const char* abs);

    void SetData(G4Material*);
    //void SetData(G4MaterialPropertiesTable*);
    void SetData(G4OpticalSurface*);

    // set transmittance for plate with width 3.*mm
    void setTrans(const char* trans, double unitlength=3.*mm);

    G4MaterialPropertiesTable* GetTable(bool own=false){return theTable; isTableOwn=own;}
private:
    void init();
    void loadRealRI(const char* re);
    void loadImaRI(const char* im);
    void loadRI(const char* ri);
    void setRealRI();
    void setImaRI();
    void getLimits(TGraph* gr, double& min, double& max);

    void ConvertNmToEv(TGraph*);
    double theConvConst;

    G4MaterialPropertiesTable* theTable;

    TGraph* theReIndex;
    TGraph* theImIndex;
    TGraph* theAbs;
    TGraph* theTransmittance;

    int    theEnergyN;
    double theEnergyMin;
    double theEnergyMax;
    double theEnergyStep;

    bool isTableOwn;
};

#endif
