#ifndef OpEUSODiffraction_h
#define OpEUSODiffraction_h 1

#include <globals.hh>
#include <templates.hh>
#include <geomdefs.hh>
#include <G4VDiscreteProcess.hh>

#include "EsafMsgSource.hh"
#include "EsafConfigurable.hh"
#include "VirtualTelParm.hh"
enum OpEUSODiffractionStatus { EUSODiffraction, edNotAtBoundary, edSameMaterial, edStepTooSmall, edNoINDEX };

class G4Step;
class G4Material;
class TGraph;
class OpEUSODiffraction : public G4VDiscreteProcess, public VirtualTelParm, public EsafMsgSource, public EsafConfigurable
{
public:
    OpEUSODiffraction(const G4String& processName = "OpESAFDiffraction",
                            G4ProcessType type = fOptical);
    ~OpEUSODiffraction();

public:
    G4bool IsApplicable(const G4ParticleDefinition& aParticleType);

	G4double GetMeanFreePath(const G4Track&, G4double, G4ForceCondition* condition);
    G4VParticleChange* PostStepDoIt(const G4Track& aTrack, const G4Step&  aStep);
    OpEUSODiffractionStatus GetStatus() { return theStatus; }

private:
    G4Material* Material1;
    double      Rindex1;
    double      Rindex2;
    TGraph*     theDindex;
    double      Dindex1;
    double      theMaximumPerp;

    double      theLambda0;       // in nm

    G4Material* Material2;
    G4double thePhotonMomentum;
    G4ThreeVector OldMomentum;
    G4ThreeVector OldPolarization;
    G4ThreeVector NewMomentum;
    G4ThreeVector NewPolarization;

    G4ThreeVector theGlobalNormal;

    OpEUSODiffractionStatus theStatus;

    double kCarTolerance;
    EsafConfigClass(G4,G4EusoGeometryShape)
};

#endif /* OpEUSODiffraction_h */
