#ifndef OptPhysicsList_h
#define OptPhysicsList_h 1

#include <globals.hh>
#include <G4VUserPhysicsList.hh>

#include "EsafConfigurable.hh"
#include "EsafMsgSource.hh"

class G4Cerenkov;
class G4Scintillation;
class G4OpAbsorption;
class G4OpRayleigh;
class G4OpBoundaryProcess;
class OptPhysicsListMessenger;
class OpEUSODiffraction;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
class OptPhysicsList : public G4VUserPhysicsList, public EsafConfigurable, public EsafMsgSource
{
  public:
    OptPhysicsList();
    virtual ~OptPhysicsList();

  public:
    void ConstructParticle();
    void ConstructProcess();

    void SetCuts();

    //these methods Construct particles
    void ConstructBosons();
    void ConstructLeptons();
    void ConstructMesons();
    void ConstructBaryons();

    //these methods Construct physics processes and register them
    void ConstructGeneral();
    void ConstructEM();
    void ConstructOp();

    //for the Messenger
    void SetVerbose(G4int);
    void SetNbOfPhotonsCerenkov(G4int);

  private:
    G4Cerenkov*          theCerenkovProcess;
    G4Scintillation*     theScintillationProcess;
    G4OpAbsorption*      theAbsorptionProcess;
    G4OpRayleigh*        theRayleighScatteringProcess;
    G4OpBoundaryProcess* theBoundaryProcess;
    OpEUSODiffraction*   theEUSODiffractionProcess;

    OptPhysicsListMessenger* pMessenger;

    EsafConfigClass(G4,G4TransportManager)
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif /* OptPhysicsList_h */
