#include "DetectorFactory.hh"
#include "G4TusGeometry.hh"
#include "G4EusoGeometryShape.hh"

#include <TString.h>
#include <globals.hh>
#include <stdexcept>

DetectorFactory* DetectorFactory::fMe = 0;
//_____________________________________________________________________________
DetectorFactory::DetectorFactory() {
    fMe = this;
    fDetector = NULL;
    SetSender("DetectorFactory");
}

//_____________________________________________________________________________
DetectorFactory::~DetectorFactory() {
     if(fDetector)delete fDetector;
}

//_____________________________________________________________________________
G4VUserDetectorConstruction* DetectorFactory::GetDetector(){
    if (fDetector) return fDetector;

    TString det=Conf()->GetStr("G4Detector.DetectorType");
    if(det=="") det="G4Euso";

    if (det == "G4EusoShape") {
        fDetector = new G4EusoGeometryShape();
        Msg(EsafMsg::Info)<<"DetectorFactory : G4EusoGeometryShape (EUSO)"<<MsgDispatch;
    }
    else if (det == "G4Tus") {
        fDetector = new G4TusGeometry();
        Msg(EsafMsg::Info)<<"DetectorFactory : G4TusGeometry (TUS)"<<MsgDispatch;
    }
    else throw runtime_error(Form("DetectorFactory does not support detector %s",det.Data()));

    return fDetector;
}
