// ESAF : Euso Simulation and Analysis Framework
// $Id: BoolAlgebra.hh 1799 2005-05-02 10:11:11Z pesce $
// M. Pallavicini - created Mar,  3 2004
// Boolean algebra classes used by Chip Tracking

#ifndef __BOOLALGEBRA_HH_
#define __BOOLALGEBRA_HH_

#include <exception>
#include "euso.hh"

class BoolMatrix;

class BoolVector {
public:

    // empty ctor
    BoolVector();
  
    // ctor
    BoolVector(Int_t);

    // dtor
    virtual ~BoolVector();

    // get / set
    inline Int_t Size() const { return fSize;}
    
    inline Bool_t operator()(Int_t i) const { return fVector[i]; }

    inline Bool_t& operator()(Int_t i) {return fVector[i]; }

    inline void Zero() {for(Int_t i=0; i<fSize; i++) fVector[i]=kFALSE;}

    // return the logic OR of all elements
    Bool_t OR() const;

    // return the logic AND of all elements
    Bool_t AND() const;

    // assignement operator
    BoolVector& operator = (const BoolVector& );

    // left multiplication with a matrix
    BoolVector& operator *= (const BoolMatrix&);

    // bitwise AND
    BoolVector& operator &= (const BoolVector&);

private:
    Int_t fSize;
    Bool_t fVector[256];
    
    ClassDef(BoolVector,0)

    friend Bool_t operator % (const BoolVector& v1, const BoolVector &v2);
    friend class BoolMatrix;
};

// boolean scalar product
inline Bool_t operator%(const BoolVector& v1, const BoolVector &v2) {
    Bool_t res = kFALSE;
    for(Int_t i=0; i < v1.fSize; i++)
        res |= v1(i)&v2(i);
    return res;
}

// very simple matrix with boolean elements
class BoolMatrix {
public:
    // ctor
    BoolMatrix(Int_t);

    // dtor
    virtual ~BoolMatrix();

    // get / set operator
    inline Bool_t operator()(Int_t i,Int_t j) const { return fMatrix[i][j]; }

	// access to elements
    Bool_t& operator()(Int_t i,Int_t j) {return fMatrix[i][j]; }
   
	// assignment operator
	BoolMatrix & operator = (const BoolMatrix&);

	// left multiplication with another matrix
	BoolMatrix & operator *= (const BoolMatrix&);

    // set to a neighbors matrix
    void SetNeighborsMatrix();

private:
	
    void Zero();
    Int_t fSize;
    Bool_t fMatrix[256][256];
	friend class BoolVector;

    ClassDef(BoolMatrix,0)
};


#endif  /* __BOOLALGEBRA_HH_ */

