// $Id: EusoDetector.hh 2908 2011-05-12 14:33:08Z biktem $
// Author: M. Pallavicini   08-02-2002

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: EusoDetector                                                         *
 *  Package: Electronics                                                     *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/

#ifndef __EUSODETECTOR_H_
#define __EUSODETECTOR_H_

#include "Detector.hh"
#include <map>

#include "EusoElectronics.hh"

class VirtualDetectorTransportManager;
class EEvent;
class Photomultiplier;
class Photon;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// EusoDetector                                                               //
//                                                                            //
// The whole EusoDetector is divided in two main pieces:                      //
// DetectorTransportManager, which brings photons from pupil to focal surface //
// EusoElectronics, which simulates the whole chain from Pmts to Trigger      //
// EusoDetector is just a wrapping to these objects which are the one that do //
// the job                                                                    //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class EusoDetector : public Detector, public EsafConfigurable, public EsafMsgSource {
public:
    EusoDetector();
    // ctor

    virtual ~EusoDetector();
    // dtor

    virtual Telemetry* Get( PhotonsOnPupil* );
    // do the complete simulation of one event

    void Build();
    // build or rebuild the detector

    virtual void Reset();
    // get ready for next event

    //inline EusoElectronics* GetEusoElectronics() { return fElectronics; }
    // access to Electronics

    //inline VirtualDetectorTransportManager* GetDetectorTransportManager() { return fDetectorTransporter;}
    // access to DetectorTrasnportManager

    virtual Bool_t ClearMemory();
    // physically release the memory allocated by the arrays of this object

    EsafConfigClass(Electronics,EusoDetector)

private:

    EusoElectronics *fElectronics;
//     VirtualDetectorTransportManager *fDetectorTransporter;

    ClassDef(EusoDetector,0)
};

// global pointer set by SimuApplication
extern EusoDetector* gEusoDetector;

#define GetEusoDetector() gEusoDetector
#endif
