// $Id: PhotoElectron.hh 2766 2006-11-15 15:49:03Z moreggia $
// Author: Sylvain Moreggia   2006/03/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: PhotoElectron                                                           *
 *  Package: electronics                                                   *
 *  Coordinator: Marco.Pallavicini                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __PHOTOELECTRON_HH__
#define __PHOTOELECTRON_HH__

#include "euso.hh"
#include "EsafMsgSource.hh"
#include "EarthVector.hh"

class Photon;


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// PhotoElectron                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

enum PEType {FluoPE=0, CkovPE=1, BckgndPE=2};



class PhotoElectron : public EsafMsgSource {
public:
    
    // ctor
    PhotoElectron();
    
    // ctor
    PhotoElectron(const Photon&);
    
    // dtor
    virtual ~PhotoElectron();
    
    // getters
    inline Double_t Time() const {return fTime;}
    inline PEType Type() const {return fType;}
    inline EarthVector AtmoPos() const {return fAtmoPos;}
    inline UInt_t PixelID() const {return fPixelID;}
    inline UInt_t Dist2ShiftToShowPt() const {return fDist2ShiftToShowPt;}
    inline Double_t SNR() const {return fSNR;}
    inline Bool_t IsTriggered() const {return fTriggered;}
    inline Double_t Wl() const {return fWl;}
    inline Double_t ShowerAge() const {return fShowerAge;}
    inline Int_t AtmoHistory() const {return fAtmoHistory;}
    inline Int_t AtmoNbinter() const {return fAtmoNbinter;}
    
    // setters
    inline void SetPixelID(UInt_t id) {fPixelID = id;}
    inline void SeDist2ShiftToShowPt(UInt_t dist2shift) {fDist2ShiftToShowPt = dist2shift;}
    inline void SetTime(Double_t t) {fTime = t;}
    inline void SetSNR(Double_t snr) {fSNR = snr;}
    inline void Triggered(Bool_t trig = kTRUE) {fTriggered = trig;}

private:

    PEType fType;
    UInt_t fPixelID;
    UInt_t fDist2ShiftToShowPt;
    Double_t fTime;
    Double_t fSNR;
    Bool_t fTriggered;
    
    // parent photon related infos
    Double_t fWl;
    Double_t fShowerAge;
    EarthVector fAtmoPos;
    Int_t fAtmoHistory;         // last interaction in atmosphere (0: direct, 1: reflected, 2: rayleigh, 3: cloud, 4: aerosol)
    Int_t fAtmoNbinter;         // nb of interaction in atmosphere
    
    

    ClassDef(PhotoElectron,0)
};

#endif  /* __PHOTOELECTRON_HH__ */

