// $Id$
// Author: fenu   2007/12/05

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: CCB_LTTTrigger                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// CCB_LTTTrigger
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>

// WORKS JUST IF THE ENTIRE MC IS SIMULATED FOR INCOMING SIGNAL
//WORKS JUST IF PLUGGED TO A PREVIOUS TRIGGER (LTT or PTT)
/*
Works exaclty like the LTT but:  
*on much longer tracks
*on more directions 
*ALWAYS starting from the previous triggers

How the initial condition is chosen: 
*select the pixel triggering in the previous trigger
*select the box 3X3 around this pixel 
*select all the GTUs from -7 to +7
*start the integration from every pixel inside the box for every GTU which we consider.
(we have  3X3X15=135 starting conditions)


CCB_LTT inherits all the methods and attributes from LTT trigger exept Simulate function.
Look to LTTTrigger.cc to see how all the other methods are.

*/



//
#include "CCB_LTTTrigger.hh"
#include "MacroCellData.hh"
#include "MacroCell.hh"
#include "ERunParameters.hh"
#include "NumbersFileParser.hh"
#include "ECCB_LTTTriggerDataAdder.hh"
#include "CCB_LTTTriggerSegment.hh"
#include "EEvent.hh"
#include "EHeader.hh"
#include "LTTTrigger.hh"

using namespace sou;
ClassImp(CCB_LTTTrigger)

//_____________________________________________________________________________
CCB_LTTTrigger::CCB_LTTTrigger() : LTTTrigger(){

    //
    // Constructor
    //
    
  fYellowThreshold = (Int_t)Conf()->GetNum("CCB_LTTTrigger.fYellowThreshold");
  fIntegrateSum= (Int_t)Conf()->GetNum("CCB_LTTTrigger.fIntegrateSum");
  flag_EC= (Int_t)Conf()->GetNum("CCB_LTTTrigger.EC_PDM");


  //read and calculate path of the box on the chip
  // starting from the directions given as input
  fDataDirectory=Conf()->GetStr("CCB_LTTTrigger.fDataDirectory");
  fDirectionFileName=Conf()->GetStr("CCB_LTTTrigger.fDirectionFileName");
  fDirectionFileName=fDataDirectory+"/"+fDirectionFileName;
  fDirectionFileName=Conf()->GetCfgDir()+"/"+Conf()->GetType()+"/"+fDirectionFileName;
  Flag_debug_table =(Int_t)Conf()->GetNum("CCB_LTTTrigger.fDebug");

  fPixel_size = (Float_t) Conf ()->GetNum ("CCB_LTTTrigger.fPixel_size");
  fGtuLight = (Float_t) Conf ()->GetNum ("CCB_LTTTrigger.fGtu_light");

  fNum_Dir =  (Int_t) Conf ()->GetNum ("CCB_LTTTrigger.fNum_Dir");

  fNum_Steps =  (Int_t) Conf ()->GetNum ("CCB_LTTTrigger.fNum_Steps");

  fInputTrigger = (Float_t) Conf ()->GetNum ("CCB_LTTTrigger.fInputTrigger");

  ConfigFileParser* pConf = Config::Get()->GetCF("Electronics","MacroCell");
  fGTU = pConf->GetNum("MacroCell.fGtuTimeLength");
//   fGTU/=1000.;
  fGTU/=microsecond;
  
  pConf = Config::Get()->GetCF("Electronics","EusoElectronics");
  fBack = pConf->GetNum("EusoElectronics.fNightGlowRateOnAxis");

  NumbersFileParser *dummy2 = new NumbersFileParser(fDirectionFileName,2);


  for(Int_t j(0);j<fNum_Dir;j++){
                
    vector<Double_t> dummy2_line=dummy2->GetRow(j);
                
    for(size_t k(0);k<dummy2_line.size();k++){
                  
      angle.push_back(dummy2_line[k]);
    }
    dummy2_line.clear();
  }
   Calculate_Directions();

    angle.clear();
}

//_____________________________________________________________________________
CCB_LTTTrigger::~CCB_LTTTrigger() {
    //
    // Destructor
    //

   cout<<"Destroy "<<endl;
   Clear();
}

//______________________________________________________________________________
void CCB_LTTTrigger::Simulate( MacroCellData* pData) {    
    //
    // Simulate CCB_LTT algorythm
    //
  if((pData->GetLTTTrigger()>-1 && fInputTrigger==2) || (pData->GetPTTTrigger()>-1 && fInputTrigger==1)){
  Msg (EsafMsg::Info) << "CCB LTT algorythm (Third level) starting..." << MsgDispatch;
 Int_t posit_row,posit_col,flag_inside,chip_id_local,row_second,col_second,chip_ID,max_gtu,max_counts;
  //Hits(Int_t gtu);
  Int_t matsize = pData->Cell()->GetEC(0)->FrontEnd()->Channels();
     Int_t   side_EC=sqrt(matsize);
     Int_t   side_PMT=side_EC/2;
	if(fInputTrigger==2){
           Msg (EsafMsg::Info) << "Output from LTT trigger is used" << MsgDispatch;
           row_second=pData->GetLTTTriggerMaxRow();
           col_second=pData->GetLTTTriggerMaxCol();
           chip_ID=pData->GetLTTTriggerChipID();
	   max_gtu=pData->GetLTTTriggerMaxGtu();
	   max_counts=pData->GetLTTTriggerMaxCounts();
	}
	if(fInputTrigger==1){
 Msg (EsafMsg::Info) << "Output from PTT trigger is used" << MsgDispatch;
           row_second=pData->GetPTTTriggerMaxRow();
           col_second=pData->GetPTTTriggerMaxCol();
           chip_ID=pData->GetPTTTriggerChipID();
	   max_gtu=pData->GetPTTTriggerMaxGtu();
	   max_counts=pData->GetPTTTriggerMaxCounts();
	}
  

  // get map of ChipGtuData elements
  map<Int_t, map<Int_t,ChipGtuData*>* > &m1 = pData->fChipData2;
  map<Int_t,ChipGtuData*> &m2 = *m1[chip_ID];
  map<Int_t,ChipGtuData*> &m3 = *m1[chip_ID+3];
  map<Int_t,ChipGtuData*> &m4 =  *m1[chip_ID-3];
  map<Int_t,ChipGtuData*> &m5 = *m1[chip_ID-1];
  map<Int_t,ChipGtuData*> &m6 = *m1[chip_ID+1];
  map<Int_t,ChipGtuData*> m_aux;


      Double_t divide = (chip_ID - 1) / 9.;
      Double_t num_PDM_DOUB;
      Double_t rest = modf (divide, &num_PDM_DOUB);
      rest *= 9;
      Double_t integ;

      Double_t funz = round (1.e10 * rest / 3.) * 1.e-10;
      //round at the 10th digit. Otherwise rounding problems arise
      // for example: modf (1.9999999999999998,&int) is different from 
      //  modf (1.9999999999999999,&int) 
      // why? 

      rest = funz;
      Double_t rest2 = modf (rest, &integ);


      Int_t row_PDM = integ;
      Int_t col_PDM = rest2 * 3.;
      Int_t ec_num=num_PDM_DOUB;


//function for initial table
for(Int_t off_row=-1;off_row<2;off_row++){
  for(Int_t off_col=-1;off_col<2;off_col++){
    for(Int_t off_gtu=-7;off_gtu<8;off_gtu++){
	flag_inside=0;
	CorrectTransformation(row_second,col_second,off_row,off_col,&posit_row,&posit_col,&flag_inside,side_PMT,side_EC);

	if(flag_inside==1){
	    chip_id_local=chip_ID;
	    m_aux=m2;
	}
	else if(flag_inside==2 && row_PDM<2){
		chip_id_local=chip_ID+3;
		m_aux=m3;
	}
	else if(flag_inside==3 && row_PDM>0){
		chip_id_local=chip_ID-3;
		m_aux=m4;
	}
	else if(flag_inside==4 && col_PDM>0){
		chip_id_local=chip_ID-1;
		m_aux=m5;
	}
	else if(flag_inside==5 && row_PDM<2){
		chip_id_local=chip_ID+1;
		m_aux=m6;
	}

	if(flag_inside>0 && flag_inside<6){
	  if(m_aux.begin()->first+7<off_gtu+max_gtu && m_aux.end()->first-7>off_gtu+max_gtu){
	      pos_PDM_map.push_back (PDM_Map.size ());
	      PDM_Map.push_back(chip_ID);
	      PDM_Map.push_back(posit_row);
	      PDM_Map.push_back(posit_col);
	      PDM_Map.push_back(off_gtu+max_gtu); // In wich gtu this event is
	  }
	  else {
	      Msg (EsafMsg::Warning) << "Starting condition out of bound" << MsgDispatch;cout<<m_aux.begin()->first<<
				  " "<<m_aux.end()->first<<" "<<off_gtu<<" "<<max_gtu<<endl;
	  }
	}
    }
  }
}
  // call integration method//////////////////////////////////////////////////////////////////////////

 integrate(PDM_Map,pos_PDM_map,pData,side_PMT,side_EC);
m_aux.clear();
  //Clear vectors. 
  Clear();
  } else {
    if(fInputTrigger==1)  Msg (EsafMsg::Info) << "PTT trigger at the previous stage was not triggering" << MsgDispatch;
    if(fInputTrigger==2)  Msg (EsafMsg::Info) << "LTT trigger at the previous stage  was not triggering" << MsgDispatch;
  }
  
}

//______________________________________________________________________________
void CCB_LTTTrigger::Clear() {
//clear/////////////////////////////////////////////////////////////////////////////////////
  PDM_Map.clear();
  pos_PDM_map.clear();

}

void CCB_LTTTrigger::FillInfoVector(Int_t up_counts,Int_t  GTU_up,Int_t row_up,Int_t col_up,Int_t chip_ID_up,Int_t dir_up,Int_t Triggering){
 
    fTracks.push_back( CCB_LTTTriggerSegment() );
    CCB_LTTTriggerSegment &trkseg = fTracks.back();
    trkseg.SetNumEvt(EEvent::GetCurrent()->GetHeader()->GetNum());
    trkseg.SetMaxCount(up_counts);
    trkseg.SetMaxGtu(GTU_up);
    trkseg.SetMaxRow(row_up);
    trkseg.SetMaxCol(col_up);
    trkseg.SetMaxChipID(chip_ID_up);
    trkseg.SetMaxDir(dir_up);

  vector<CCB_LTTTriggerSegment>::iterator it;  

  for( it=fTracks.begin(); it!=fTracks.end(); it++) {

    if ( EEvent::GetCurrent() ){

      ECCB_LTTTriggerDataAdder a(&(*it),Triggering,1);
      EEvent::GetCurrent()->Fill(a);
    }
    
  }
   fTracks.clear();


}

void CCB_LTTTrigger::FillChipData(Int_t evtNum,Int_t up_counts,Int_t GTU_up,Int_t row_up,Int_t col_up,Int_t chip_ID_up,Int_t dir_up,MacroCellData* pData){
        pData->SetCCB_LTTTrigger(evtNum);
	pData->SetCCB_LTTTriggerMaxCounts(up_counts);
	pData->SetCCB_LTTTriggerMaxGtu(GTU_up);
	pData->SetCCB_LTTTriggerMaxRow(row_up);
	pData->SetCCB_LTTTriggerMaxCol(col_up);
	pData->SetCCB_LTTTriggerChipID(chip_ID_up);
	pData->SetCCB_LTTTriggerMaxDir(dir_up);
}

