// ESAF : Euso Simulation and Analysis Framework
// $Id: ChipTrackingTrgEngine3.cc 2385 2005-11-18 14:00:07Z pesce $
// R. Pesce created

//_____________________________________________________________________________
//  
//  Chip Tracking Trigger Engine 3
//  ==============================
//
//  Tracking trigger built in front end chip.
//
//  This class is equivalent to ChipTrackingTrgEngine1
//  for studying multiple trigger configurations using different thresholds
//


#include "ChipTrackingTrgEngine3.hh"
#include "MacroCellData.hh"
#include "MacroCell.hh"
#include "ElementaryCell.hh"
#include "FrontEndChip.hh"
#include "BoolAlgebra.hh"
#include "ChipTrackSegment.hh"
#include "EusoElectronics.hh"
#include "Photomultiplier.hh"
#include "EusoDetector.hh"
#include "TMath.h"
#include "EChipTrackTriggerDataAdder.hh"
#include "Config.hh"

ClassImp(ChipTrackingTrgEngine3)

//______________________________________________________________________________
ChipTrackingTrgEngine3::ChipTrackingTrgEngine3() : 
ChipTrackingTrgEngine1(string("ChipTrackingTrgEngine3"), kChipTrackingTrigger3 ) {
    //
    // Constructor
    //

    fMinTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fMinTrackLength");
    fMaxTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fMaxTrackLength");
    fMinTriggerTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fMinTriggerTrackLength");
    fMinTriggerTwoLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fMinTriggerTwoLength");
    fMaxTwoLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fMaxTwoLength");
 
    if ( Conf()->GetStr("ChipTrackingTrgEngine3.fThresholdType")=="absolute" ) fRelativeThreshold = kFALSE;
    else if ( Conf()->GetStr("ChipTrackingTrgEngine3.fThresholdType")=="relative" ) fRelativeThreshold = kTRUE;
    else Msg(EsafMsg::Panic) << "Wrong cfg value fThresholdType" << MsgDispatch;
    fAcceptHole = Conf()->GetBool("ChipTrackingTrgEngine3.fAcceptHole");
    fOnlyWithSignal = Conf()->GetBool("ChipTrackingTrgEngine3.fOnlyWithSignal");
    fThreshold = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine3.fThreshold");

    // panic if we want to simulate >32 trigger patterns
    if ( ((fMaxTrackLength-fMinTriggerTrackLength+1)+(fMaxTwoLength-fMinTriggerTwoLength+1)) > 32  )
        Msg(EsafMsg::Panic) << "Request for > 32 patterns of trigger." << MsgDispatch;

    ConfigFileParser *pConfig = Config::Get()->GetCF("Electronics","MacroCell");
    if ( !pConfig->GetBool("MacroCell.fSaveAllChipGtuData") )
        Msg(EsafMsg::Warning) << "MacroCell.fSaveAllChipGtuData is not enabled. Some infos are missing" << MsgDispatch;

    FillRunPars();
    Clear();
}


//______________________________________________________________________________
ChipTrackingTrgEngine3::~ChipTrackingTrgEngine3() {
    //
    // Destructor
    //
}

