// ESAF : Euso Simulation and Analysis Framework
// $Id: ChipTrackingTrgEngine4.cc 2385 2005-11-18 14:00:07Z pesce $
// R. Pesce created

//_____________________________________________________________________________
//  
//  Chip Tracking Trigger Engine 4
//  ==============================
//
//  Tracking trigger built in front end chip.
//
//  This class is equivalent to ChipTrackingTrgEngine1
//  for studying multiple trigger configurations using different thresholds
//


#include "ChipTrackingTrgEngine4.hh"
#include "MacroCellData.hh"
#include "MacroCell.hh"
#include "ElementaryCell.hh"
#include "FrontEndChip.hh"
#include "BoolAlgebra.hh"
#include "ChipTrackSegment.hh"
#include "EusoElectronics.hh"
#include "Photomultiplier.hh"
#include "EusoDetector.hh"
#include "TMath.h"
#include "EChipTrackTriggerDataAdder.hh"
#include "Config.hh"

ClassImp(ChipTrackingTrgEngine4)

//______________________________________________________________________________
ChipTrackingTrgEngine4::ChipTrackingTrgEngine4() : 
ChipTrackingTrgEngine1(string("ChipTrackingTrgEngine4"), kChipTrackingTrigger4 ) {
    //
    // Constructor
    //

    fMinTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fMinTrackLength");
    fMaxTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fMaxTrackLength");
    fMinTriggerTrackLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fMinTriggerTrackLength");
    fMinTriggerTwoLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fMinTriggerTwoLength");
    fMaxTwoLength = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fMaxTwoLength");

    if ( Conf()->GetStr("ChipTrackingTrgEngine4.fThresholdType")=="absolute" ) fRelativeThreshold = kFALSE;
    else if ( Conf()->GetStr("ChipTrackingTrgEngine4.fThresholdType")=="relative" ) fRelativeThreshold = kTRUE;
    else Msg(EsafMsg::Panic) << "Wrong cfg value fThresholdType" << MsgDispatch;
    fAcceptHole = Conf()->GetBool("ChipTrackingTrgEngine4.fAcceptHole");
    fOnlyWithSignal = Conf()->GetBool("ChipTrackingTrgEngine4.fOnlyWithSignal");
    fThreshold = (Int_t)Conf()->GetNum("ChipTrackingTrgEngine4.fThreshold");

    // panic if we want to simulate >32 trigger patterns
    if ( ((fMaxTrackLength-fMinTriggerTrackLength+1)+(fMaxTwoLength-fMinTriggerTwoLength+1)) > 32  )
        Msg(EsafMsg::Panic) << "Request for > 32 patterns of trigger." << MsgDispatch;

    ConfigFileParser *pConfig = Config::Get()->GetCF("Electronics","MacroCell");
    if ( !pConfig->GetBool("MacroCell.fSaveAllChipGtuData") )
        Msg(EsafMsg::Warning) << "MacroCell.fSaveAllChipGtuData is not enabled. Some infos are missing" << MsgDispatch;

    FillRunPars();
    Clear();
}


//______________________________________________________________________________
ChipTrackingTrgEngine4::~ChipTrackingTrgEngine4() {
    //
    // Destructor
    //
}

