// $Id: ESimpleDetectorPhotonAdder.cc 2619 2006-03-24 16:23:27Z moreggia $
// Author: Sylvain Moreggia   2006/03/21

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ESimpleDetectorPhotonAdder                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// ESimpleDetectorPhotonAdder
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "ESimpleDetectorPhotonAdder.hh"
#include "EEvent.hh"
#include "Etypes.hh"
#include "Photon.hh"
#include "ParentPhoton.hh"

#include "ESimpleDetector.hh"
#include "EDetector.hh"
#include "ESimpDetPhoton.hh"

ClassImp(ESimpleDetectorPhotonAdder)

//_____________________________________________________________________________
ESimpleDetectorPhotonAdder::ESimpleDetectorPhotonAdder(Photon* ph, Bool_t newph) 
                           : EFiller("ESimpleDetector"), EsafMsgSource() {
    //
    // Constructor
    //
    fPhoton = ph;
    fNewPh = newph;
}

//_____________________________________________________________________________
ESimpleDetectorPhotonAdder::~ESimpleDetectorPhotonAdder() {
    //
    // Destructor
    //
}

//______________________________________________________________________________
void ESimpleDetectorPhotonAdder::Fill( EFillable* f ) {
    //
    // filler
    //

    ESimpleDetector *det;
    if ( !(det = dynamic_cast<ESimpleDetector*>(f)) ) {
        MsgForm(EsafMsg::Warning,"Fill(): f is not an ESimpleDetector* ");
        return; 
    }


    if ( !fPhoton ) return;


    // if photons are disabled return
    UInt_t mode = det->GetPhotonFillingMode();
    Bool_t addPhoton;
    switch(mode) {
        case EDetector::kAllPhotons:
            addPhoton = fNewPh; 
            break;
        case EDetector::kMadeSignalOnly:
            addPhoton = fNewPh; 
            break;
        default:
            addPhoton = kFALSE;
            return;
            break;
    }


    // if photons filling is enabled add a new ph
    if(addPhoton) new ( (*(det->fPhotons))[det->fNumPhotons++] ) ESimpDetPhoton();
    ESimpDetPhoton& p = *(ESimpDetPhoton*)(*(det->fPhotons))[det->fNumPhotons-1];


    // photon params at the pupil
    if ( addPhoton ) {
        p.fType = fPhoton->parent->Type();                // type of photon 
        
	p.fShowerPosX = fPhoton->parent->ShowerPos()[X];  // X position of origin in shower
        p.fShowerPosY = fPhoton->parent->ShowerPos()[Y];  // Y position of origin in shower
        p.fShowerPosZ = fPhoton->parent->ShowerPos()[Z];  // Z position of origin in shower
       
        p.fTheta = fPhoton->dir.Theta();    // photon theta direction in Euso detector
        p.fPhi = fPhoton->dir.Phi();        //    "   phi       "      "   "     "
        p.fLambda = fPhoton->wl;            // photon wavelength
        p.fTime = fPhoton->time;            // photon time (detection)

        p.fPosX           = kHuge;      // X position on pupil
        p.fPosY           = kHuge;      // Y position on pupil
        p.fPosZ           = kHuge;      // Z position on pupil

        p.fAtmoHistory    = -1000;      // code of the last atmospheric process undergone
        p.fAtmoNbinter    = -1000;      // code of the last atmospheric process undergone
        p.fFate           = 0;          // 0 as long as photon is alive
    }

    // impact point on pupil 
    p.fPosX = fPhoton->pos[X];
    p.fPosY = fPhoton->pos[Y];
    p.fPosZ = fPhoton->pos[Z];


    p.fAtmoHistory = fPhoton->history;
    p.fAtmoNbinter = fPhoton->iterations;
    p.fFate = fPhoton->fate;


}
