// $Id: PhotoElectron.cc 2789 2008-02-11 07:26:05Z naumov $
// Author: Sylvain Moreggia   2006/03/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: PhotoElectron                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// PhotoElectron
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "PhotoElectron.hh"
#include "ParentPhoton.hh"
#include <TMath.h>
ClassImp(PhotoElectron)

//_____________________________________________________________________________
PhotoElectron::PhotoElectron() : EsafMsgSource() {
    //
    // Constructor for background
    //
    
    fType        = BckgndPE;
    fPixelID     = 0;
    fTime        = 0.;
    fWl          = 0.;
    fShowerAge   = -1.;
    fAtmoHistory = -1;
    fAtmoNbinter = -1;
    fSNR         = -1.;
    fTriggered   = kFALSE;
    fAtmoPos.SetXYZ(0,0,HUGE);
}

//_____________________________________________________________________________
PhotoElectron::PhotoElectron(const Photon& ph) : EsafMsgSource() {
    //
    // Constructor from a photon
    //

    fPixelID     = 0;
    fTime        = ph.time;
    fWl          = ph.wl;
    fAtmoHistory = ph.history;
    fAtmoNbinter = ph.iterations;
    fSNR         = -1.;
    fTriggered   = kFALSE;
    
    const ParentPhoton* parentph = ph.parent;
    fShowerAge = parentph->GetShowerAge();
    fAtmoPos = EarthVector(parentph->ShowerPos());
    if(parentph->Type() == 1)       fType = FluoPE;
    else if(parentph->Type() == 2)  fType = CkovPE;
    else Msg(EsafMsg::Panic)<< "<PhotoElectron ctor> ParentPhoton type not allowed : "<< parentph->Type()<< MsgDispatch;
}

//_____________________________________________________________________________
PhotoElectron::~PhotoElectron() {
    //
    // Destructor
    //
}

