// DetectorTransportManager
// $Id: DetectorTransportManager.hh 2855 2010-11-25 15:47:14Z biktem $
//
// This is the kernel of the optics simulation, it trasnports photons
// between the detector elements.
//
#ifndef DETECTORTRANSPORTMANAGER_HH
#define DETECTORTRANSPORTMANAGER_HH

#include "OpticsFactory.hh"
#include "Config.hh"
#include "EVector.hh"
#include "EsafConfigurable.hh"
#include "VirtualDetectorTransportManager.hh"

class PhotonsOnPupil;
class EEvent;

class DetectorTransportManager: public VirtualDetectorTransportManager, public DetectorPhotonTransporter {
protected:
    Double_t EusoCylIntersection( Photon *p ) const;
    EVector InteractionPoint(Photon *p, Double_t radius = -1) const;
    EVector NewInteractionPoint(const Photon *p, double DZ) const;

    WallInteraction*   fWalls;
    Baffle*            fBaffle;

    Bool_t fEnableFocalPlane;


public:
    DetectorTransportManager();
    virtual ~DetectorTransportManager() {};

    virtual Photon* Transport(Photon *) const;
    virtual void Go( PhotonsOnPupil* );

    inline WallInteraction*   GetWalls() { return fWalls; }
    inline Baffle*            GetBaffle() { return fBaffle; }

    inline void EnableFocalPlane(Bool_t b=kTRUE){ fEnableFocalPlane=b;}

    virtual void Reset();

    EsafConfigClass(Optics,DetectorTransportManager)

    ClassDef(DetectorTransportManager,0)
};


inline EVector DetectorTransportManager::NewInteractionPoint(const Photon *p, double DZ) const {
    EVector dir=p->dir.Unit();
    EVector newPoint=p->pos+dir*(DZ/dir[Z]);
#ifdef DEBUG
#endif /* DEBUG */
    return newPoint;
}

#endif /* TRANSPORTMANAGER_HH */
