/* -------------------------------------------------------------------------
 *   trace_optF1v4.h
 *
 *   --- header file for trace_optF1v4.h
 *
 * Copyright (c) 2000-2007 N.Sakaki, Y.Takizawa, Y.Kawasaki
 * All rights reserved.
 * $Id: Ltrace_optF1v4.hh,v 1.0 2007/11/13 Dmitry V.Naumov adapted for ESAF
 * -------------------------------------------------------------------------
 */
#ifndef _LTRACE_OPTF1V4_H
#define _LTRACE_OPTF1V4_H

#include <stdio.h>
#include "Ltracemain_optF1v4.hh"
#include "Rtypes.h"


#define CHECK_TIMING
#include <TH1D.h>
static TH1D* thehisto;
void CreateHisto();
void DeleteHisto();


/* the following numbers are related to the data files. */
#define LAcryl_N 71
#define LCytop_N 71
#define LBg3_N   22


/* error codes */

#define RT_NO_SPINTERSECTION       -2
#define RT_HIT_SIDE_WALL           -3
#define RT_NO_INTERSECTION1        -4
#define RT_NO_INTERSECTION2        -5
#define RT_OUT_OF_SURFACE1         -6
#define RT_OUT_OF_SURFACE2         -7
#define RT_REFLECTION_DISCARDED    -8
#define RT_NO_REFRACTED_LIGHT      -9
#define RT_REFLECTED_IN_REF       -10
#define RT_ABSORBED               -11
#define RT_OUT_OF_IRIS            -12
#define RT_STRAY_LIGHT            -13
#define RT_ESCAPE_FROM_ENTRANCE   -14
#define RT_EXCEED_ITER_MAX        -15
#define RT_OUT_OF_DOES            -16

#define RT_REFRACTED                1
#define RT_REFLECTED                2

#define SURFACE0                    0
#define SURFACE1                -1000
#define SURFACE2                -2000
#define SURFACE3                -3000
#define SURFACE4                -4000
#define SURFACED                -5000
#define SURFACE5                -5000
#define SURFACE6                -6000
#define SURFACE7                -7000
#define SURFACE_FS             -8000

/* speed of light in mm/ns */
//#define C0 299.792458
/* the current optics is designed to be put in air.
Maybe little change for vacuum.  */
#define LREFRACTIVE_IDX_MEDIA 1.0003   /* air */



Int_t LReadLensData(tel_param *param, FILE *fplog);
Double_t LGet_n_Acryl(Double_t lambda);
Double_t LGet_n_Cytop(Double_t lambda);
Double_t LGet_k_Acryl(Double_t lambda);
Double_t LGet_k_Cytop(Double_t lambda);
Int_t Lrefract(Double_t in_ph[3],Double_t nnn[3],Double_t n1,Double_t n2);
Int_t Ltrace_lens(tel_param *param, Double_t ph_in[], Double_t ph_out[], FILE *fplog);
Int_t LCalSpheVec(Double_t aX, Double_t aY, Double_t aZ, Double_t aCZ, Double_t nnnDIFF[3]);
Int_t LReadDiffractData(char *dirname, FILE *fplog);
Double_t LCal_D(Double_t r);
Int_t Ldiffract(Int_t ID, Double_t in_phdir[3],
	     Double_t x3, Double_t y3, Double_t z3, Double_t Zc,
	     Double_t n1, Double_t n2, Double_t wl, Double_t wl0, Int_t m);

#endif
