/* -------------------------------------------------------------------------
 *   trace_lenses.h
 *
 *   --- header file for trace_lenses.c
 *
 * Copyright (c) 2000-2010 N.Sakaki, Y.Takizawa, Y.Kawasaki
 * All rights reserved.
 * $Id$
 * -------------------------------------------------------------------------
 */
#ifndef _NTRACE_LENSES_H
#define _NTRACE_LENSES_H

#include <stdio.h>
#include "Ntrace_optics.hh"
#include "Nphoton.hh"


#ifdef NTRACE
#define NSURFPTS 65536
#define NMATPTS 128

/* alias */
#define X      0
#define Y      1
#define Z      2
#define PX     3
#define PY     4
#define PZ     5
#define L      6
#define T      7

/* error codes */

#define RT_NO_SPINTERSECTION       -2
#define RT_HIT_SIDE_WALL           -3
#define RT_NO_INTERSECTION1        -4
#define RT_NO_INTERSECTION2        -5
#define RT_OUT_OF_SURFACE1         -6
#define RT_OUT_OF_SURFACE2         -7
#define RT_REFLECTION_DISCARDED    -8
#define RT_NO_REFRACTED_LIGHT      -9
#define RT_REFLECTED_IN_REF       -10
#define RT_ABSORBED               -11
#define RT_OUT_OF_IRIS            -12
#define RT_STRAY_LIGHT            -13
#define RT_ESCAPE_FROM_ENTRANCE   -14
#define RT_EXCEED_ITER_MAX        -15
#define RT_OUT_OF_DOES            -16

#define RT_REFRACTED                1
#define RT_REFLECTED                2

#define SURFACE0                    0
#define SURFACE1                -1000
#define SURFACE2                -2000
#define SURFACE3                -3000
#define SURFACE4                -4000
#define SURFACED                -5000
#define SURFACE5                -5000
#define SURFACE6                -6000
#define SURFACE7                -7000
#define SURFACE_FS              -8000
#endif


namespace NTraceLens {
int ReadLensData(tel_param *p, FILE *fplog);
double Get_n(int matid, tel_param *p, double lambda);
double Get_k(int matid, tel_param *p, double lambda);
int refract(NPhoton *photon, double nnn[3],double n1,double n2);
int trace_lens(tel_param *para, NPhoton *photon, FILE *fplog);
int CalSpheVec(double aX, double aY, double aZ, double aCZ, double nnnDIFF[3]);
int ReadDiffractData(tel_param *p, FILE *fplog);
double Cal_D(double r, tel_param *p);
int diffract(int ID, tel_param *p, NPhoton *photon, double Zc,
	     double n1, double n2, double wl, double wl0, int m);
int CheckRcut(tel_param *p, NPhoton *photon, int flag);
double differential_r(int i, double *lens_r,double *lens_z, int n_lens_elem);
void SetRunGen(int i);
void SetEffFactor(double ef);

};

#endif /* _TRACE_LENSES_H */
