/* -------------------------------------------------------------------------
 *   trace_optics.h
 *
 *   --- header file for trace_optics.h
 *
 * Copyright (c) 2000-2010 N.Sakaki, Y.Takizawa, Y.Kawasaki
 * All rights reserved.
 * $Id$
 * -------------------------------------------------------------------------
 */
#ifndef _NTRACE_OPTICS_H
#define _NTRACE_OPTICS_H

#include <stdio.h>
#include "Nphoton.hh"

#define FS_TOLERANCE   0.01  /* mm */
#define FS_DEF_STEP    0.1   /* mm */
#define FSLOOP_MAX      100

#define NSURFACES 9
#define SURFDOE NSURFACES
#define NMATERIALS 8

namespace NTraceLens{

typedef struct _MatData{
  char *name,*filename;
  int ndata,size;
  double *wl;
  double *n,*zn;
  double *k,*zk;
} MatData;

typedef struct _SurfData{
  int ndata,size,matid;
  char *filename;
  double r_cut,r_lens;
  double Rc,Sz,Zc;
  double *r;
  double *z;
} SurfData;

typedef struct _tel_param{
  char *name;
  int nmat;
  int flag_printray;
  double r_wall, r_fs, r_cut_fs;
  MatData material[NMATERIALS];
  SurfData surface[NSURFACES+1];
  double FS_C, FS_K, FS[4], FS_OFFSET; /* for FS */
  double lambda0; /* nm */
  char lens_dir[256];
} tel_param;



int read_tel_param(char *filename, tel_param *param, FILE *fplog);
int print_param(tel_param *p);
int trace_main(tel_param *param, NPhoton *photon, FILE *fplog);
double FocalSurface(double r, tel_param *p);
};
#endif /* _TRACE_OPTICS_H */
