// $Id: OpticalSystem.hh 2878 2011-04-28 13:15:36Z naumov $
// Author: Daniel De Marco

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: OpticalSystem                                                        *
 *  Package: Optics                                                          *
 *  Coordinator: Alessandro.Thea                                             *
 *                                                                           *
 *****************************************************************************/

#ifndef OPTICALSYSTEM_HH
#define OPTICALSYSTEM_HH

#include "DetectorPhotonTransporter.hh"
#include "VirtualTelParm.hh"
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// OpticalSystem                                                              //
//                                                                            //
// Abstract base class providing the interface of a generic optical system.   //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class OpticalSystem: public DetectorPhotonTransporter, public VirtualTelParm    {
protected:
    OpticalSystem() : DetectorPhotonTransporter(),fFirstLensDZ(0),fSecondLensDZ(0){}

    // Focal Distance
    Double_t fFocalDistance;

    // Lower lens height (vertex/base)
    Double_t fFirstLensDZ;

    // Upper lens height (vertex/base)
    Double_t fSecondLensDZ;

public:
    virtual Photon *Transport(Photon *) const = 0;

    // checks if the photon hits the lateral surface of the optics
    virtual Bool_t HitSide(Photon *p) const;

    // height of the vertex of the upper lens
    virtual Double_t SecondLensTop() const { return Top(); }

    // height of the base of the upper lens
    virtual Double_t SecondLensBottom() const { return Top()-fSecondLensDZ; }

    // height of the vertex of the lower lens
    virtual Double_t FirstLensTop() const { return Bottom()+fFirstLensDZ; }

    // height of the base of the lower lens
    virtual Double_t FirstLensBottom() const { return Bottom(); }

    EsafConfigClass(Optics,OpticalSystem)

    ClassDef(OpticalSystem,0)
};


#include "Interpolate.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// TestOpticalSystem                                                          //
//                                                                            //
// A simple implementation of a perfect lens with psf and absorption          //
// (wavelength dependent)                                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class TestOpticalSystem: public OpticalSystem {
protected:
    Interpolate *psf, *tr;

    virtual EVector getPSF(Photon *p, double thetain) const;

public:
    TestOpticalSystem();
    virtual Photon *Transport(Photon *p) const;
    EsafConfigClass(Optics,TestOpticalSystem)

    ClassDef(TestOpticalSystem,0)
};

#endif /* OPTICALSYSTEM_HH */
