// VirtualDetectorTransportManager
// copied from $Id: DetectorTransportManager.hh 2201 2005-10-15 10:12:59Z thea $

#ifndef VIRTUALDETECTORTRANSPORTMANAGER_HH
#define VIRTUALDETECTORTRANSPORTMANAGER_HH

#include "OpticsFactory.hh"
#include "Config.hh"
#include "EsafConfigurable.hh"

class PhotonsOnPupil;
class EEvent;

class VirtualDetectorTransportManager {
protected:
    OpticsFactory *of;

    OpticalSystem*     fOptics;
    IdealFocalSurface* fIdealFocalSurface;
    FocalPlane*        fFocalPlane;

    UInt_t fNout;
    UInt_t fNbaffle;
    UInt_t fNoptics;
    UInt_t fNwalls;
    UInt_t fNfocalplane;
    UInt_t fNreflected;
    UInt_t fNlost;

    Int_t fMaxIterations;
    Bool_t fDebug;

    Double_t fTimeFirstPhoton;   // arrival time of the first photon
    Double_t fTimeLastPhoton;    // arrival time of the last photon
    Double_t fInnerRadius;       // radius of the walls of the det.
    Double_t fOuterRadius;       // external detector's radius
    Double_t fScaleFactor;       // detector size scale factor

public:
    VirtualDetectorTransportManager(): fDebug(kFALSE), fScaleFactor(1.) {};
    virtual ~VirtualDetectorTransportManager() {};

    virtual void Go( PhotonsOnPupil* )=0;
    virtual Photon *Transport(Photon *) const = 0;

    inline OpticalSystem*     GetOptics() { return fOptics; }
    inline IdealFocalSurface* GetIdealFocalSurface() { return fIdealFocalSurface; }
    inline FocalPlane*        GetFocalPlane() { return fFocalPlane; }

    inline Double_t GetTimeFirstPhoton() const {return fTimeFirstPhoton;}
    inline Double_t GetTimeLastPhoton() const {return fTimeLastPhoton;}

    inline Double_t GetOuterRadius() { return fOuterRadius; }

    virtual Double_t GetScaleFactor() const {return fScaleFactor; }
    virtual void SetScaleFactor( Double_t scale ) { // set the detector size scaling factor
                                                    if ( scale > 0 ) fScaleFactor = scale; }

    virtual void Reset()=0;

    EsafConfigClass(Optics,VirtualDetectorTransportManager)

    ClassDef(VirtualDetectorTransportManager,0)
};

#endif /* VIRTUALDETECTORTRANSPORTMANAGER_HH */
