/* -------------------------------------------------------------------------
 *   spline.c
 *
 *   --- cubic spline interpolation for non-periodic functions
 *        "The latest algorithm collections in C" (1991) (in Japanese)
 *        Haruhiko Okumura, Professor (Computer Science), Ph.D.
 *        Matsusaka University
 *
 *        modified by N.Sakaki
 *
 * $Id: Lspline.cc 2784 2007-11-16 10:19:08Z naumov $
 * $Id: Lspline.cc,v 1.0 2007/11/13 Dmitry V.Naumov adapted for ESAF
 * -------------------------------------------------------------------------
 */
/*
 */
#include <stdlib.h>
#include "Lspline.hh"

Int_t Lcsp_maketable(Double_t x[], Double_t y[], Double_t z[], size_t size)
{
    Int_t i;
    Double_t t;
    Double_t *h, *d;

    h=(Double_t *)malloc(sizeof(Double_t)*size);
    d=(Double_t *)malloc(sizeof(Double_t)*size);
    if(h==NULL || d==NULL)
      return -1;

    z[0] = 0;  z[size-1] = 0;  /* y"(x) / 6 at both end points */
    for(i=0; i<(Int_t)(size-1); i++)
      {
        h[i  ] =  x[i+1] - x[i];
        d[i+1] = (y[i+1] - y[i])/h[i];
      }

    z[1] = d[2] - d[1] - h[0]*z[0];
    d[1] = 2.0*(x[2]-x[0]);

    for (i=1; i<(Int_t)(size-2); i++)
      {
        t = h[i]/d[i];
        z[i+1] = d[i+2] - d[i+1] - z[i]*t;
        d[i+1] = 2.0*(x[i+2]-x[i]) - h[i]*t;
    }

    z[size-2] -= h[size-2]*z[size-1];
    for (i=size-2; i>0; i--)
        z[i] = (z[i] - h[i]*z[i+1]) /d[i];

    free(h);
    free(d);

    return 0;
}

Double_t Lcspline(Double_t t, Double_t x[], Double_t y[], Double_t z[], size_t size)
{
    Int_t i, j, k;
    Double_t d, h;

    i = 0;  j = size - 1;
    while (i<j)
      {
        k = (i+j)/2;
        if (x[k]<t)
	  i = k+1;
	else
	  j = k;
      }

    if (i>0)
      i--;
    h = x[i+1] - x[i];
    d = t - x[i];

    return (( (z[i+1] - z[i])*d/h + z[i]*3.0) * d
	    + ( (y[i+1] - y[i])/h - (z[i]*2. + z[i+1])* h) )*d + y[i];
}

