/* -------------------------------------------------------------------------
 *   NPhoton.c
 *
 *   ---
 *
 * Copyright (c) 2010 N.Sakaki, Y.Takizawa, Y.Kawasaki
 * All rights reserved.
 * $Id$
 * -------------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include "Nphoton.hh"
#include "EConst.hh"

using namespace NTraceLens;

int NTraceLens::StorePhotonHistory(NPhoton *p){
  PhPosition *pos,*temp;

  pos=(PhPosition *)malloc(sizeof(PhPosition));
  if(!pos)
    {
#ifdef DEBUG
      /* stderr to be replaced with fplog */
      fprintf(stderr,"memory allocation error in StorePhotonHistory()\n");
#endif
      return -1;
    }
  pos->x[0]=p->x[0];
  pos->x[1]=p->x[1];
  pos->x[2]=p->x[2];
  pos->next=NULL;

  temp=p->history;
  if(temp)
    {
      while(temp->next){
	temp=temp->next;
      }
      temp->next=pos;
    }
  else
    p->history=pos;
    p->id=0;

  return 0;
}

int NTraceLens::ShowPhotonHistory(NPhoton *p){
  PhPosition *pos;

  if(p)
    {
      pos=p->history;
      while(pos){
	printf("[HIST] %f %f %f\n",pos->x[0],pos->x[1],pos->x[2]);
	pos=pos->next;
      }
    }

  return 0;
}

int NTraceLens::ClearPhotonHistory(NPhoton *p)
{
  PhPosition *temp,*history;

  if(p)
    {
      history=p->history;

      while(history!=NULL)
	{
	  temp=history->next;
	  free(history);
	  history=temp;
	}
    }

  return 0;
}

int NTraceLens::DeletePhoton(NPhoton *p)
{
  PhPosition *temp,*history;

  if(p)
    {
      ClearPhotonHistory(p);
      free(p);
    }

  return 0;
}

int NTraceLens::SetPhotonDirection(NPhoton *p, double ex, double ey, double ez)
{
  p->px[0]=ex;
  p->px[1]=ey;
  p->px[2]=ez;
  return 0;
}

int NTraceLens::SetPhotonPosition(NPhoton *p, double x, double y, double z, double t)
{
  p->x[0]=x;
  p->x[1]=y;
  p->x[2]=z;
  return 0;
}

int NTraceLens::MovePhoton(NPhoton *p, double dist, double rindex)
{
  int i;

  for(i=0;i<3;i++)
    p->x[i]+=p->px[i]*dist*rindex/EConst::Clight();

  p->t+=dist*rindex/EConst::Clight();
  return 0;
}
