#include <stdexcept>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <math.h>
#include "utils.hh"

static const double Rc1=4098.87779648404011129753;
static const double Sz1= 80.0;
static const double Zc1=4178.87779648404011129753;

static const double Rc2= 4098.87779648404011129753;
static const double Sz2= 100.0        /* Sz1 + 20.0 */;
static const double Zc2= 4198.87779648404011129753;

static const double Zc3= 694.526780535      /* Sz2 + 594.526780535 */;
static const double Zc4= 694.526780535       /* Zc3 + 0.0    */;
static const double Rc3= 915.148864746;

static const double Rc5= 2555.41062802899818454576;
static const double Sz5= 1889.696341785      /*  Zc4 + 1195.16956125 */;
static const double Zc5= -665.71428624399818454576;

static const double Rc6= 2555.41062802899818454576;
static const double Sz6= 1909.696341785      /*  Sz5 + 20.0 */;
static const double Zc6= -645.71428624399818454576;


static const int S_DATA =35000;
static const int Acryl_N=71;
static const int Bg3_N  =22;

static double sr1[S_DATA],sz1[S_DATA];
static double sr2[S_DATA],sz2[S_DATA];
static double sr5[S_DATA],sz5[S_DATA];
static double sr6[S_DATA],sz6[S_DATA];
static double ac_la[Acryl_N],ac_n[Acryl_N],ac_k[Acryl_N];
static double bg3_la[Bg3_N],bg3_n[Bg3_N],bg3_k[Bg3_N];

#define C0 2997.92458

static const int NR_END=1;

static double *make_vector(long nl, long nh)
     /* allocate a double vector with subscript range v[nl..nh] */
{
  double *v;

#if 0
  v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
  if (!v) {
    fprintf(stderr,"allocation failure in make_vector()");
    exit(-200);
  }
  return v-nl+NR_END;
#endif

  v=new double[nh-nl+1+NR_END];
  if(!v) throw runtime_error("Out of memory");
  return v-nl+NR_END;
}

static void free_vector(double *v, long nl, long nh)
     /* free a double vector allocated with vector() */
{
#if 0
  free((char*) (v+nl-NR_END));
#endif
  delete[] (v+nl-NR_END);
}


static void polint(double xa[], double ya[], int n, double x, double *y, double *dy)
{
  int i,m,ns=1;
  double den,dif,dift,ho,hp,w;
  double *c,*d;

  dif=fabs(x-xa[1]);
  c=make_vector(1,n);
  d=make_vector(1,n);
  for (i=1;i<=n;i++) {
    if ( (dift=fabs(x-xa[i])) < dif) {
      ns=i;
      dif=dift;
    }
    c[i]=ya[i];
    d[i]=ya[i];
  }
  *y=ya[ns--];
  for (m=1;m<n;m++) {
    for (i=1;i<=n-m;i++) {
      ho=xa[i]-x;
      hp=xa[i+m]-x;
      w=c[i+1]-d[i];
      if ( (den=ho-hp) == 0.0) {
	fprintf(stderr,"Error in routine polint");
	exit(-200);
      }
      den=w/den;
      d[i]=hp*den;
      c[i]=ho*den;
    }
    *y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
  }
  free_vector(d,1,n);
  free_vector(c,1,n);
}

static double ref(double n1, double n2, double rti, double rtt)
{
  double rp,rs;

  rp=(n2*cos(rti)-n1*cos(rtt))/(n2*cos(rti)+n1*cos(rtt));
  rs=(n1*cos(rti)-n2*cos(rtt))/(n1*cos(rti)+n2*cos(rtt));

  return((rp*rp+rs*rs)/2.0);
}



void read_para(void)
{
  int i,j;
  float dum1,dum2;
  FILE *fp;

  fprintf(stderr,"Reading Surface Data....  ");
  /* Read S1 surface */
  fp=fopen("config/Optics/JOpticalSystem/s1_e2.dat","r");
  i=0;
  while(fscanf(fp,"%lf %f %f %lf",&sr1[i],&dum1,&dum2,&sz1[i])!=EOF)
    {
      sz1[i]=Sz1-sz1[i];
      i++;
    }
  fclose(fp);

  /* Read S2 surface */
  fp=fopen("config/Optics/JOpticalSystem/s2_e2.dat","r");
  i=0;
  while(fscanf(fp,"%lf %f %f %lf",&sr2[i],&dum1,&dum2,&sz2[i])!=EOF)
    {
      sz2[i]=Sz2-sz2[i];
      i++;
    }
  fclose(fp);

  /* Read S5 surface */
  fp=fopen("config/Optics/JOpticalSystem/s5_e2.dat","r");
  i=0;
  while(fscanf(fp,"%lf %f %f %lf",&sr5[i],&dum1,&dum2,&sz5[i])!=EOF)
    {
      sz5[i]=Sz5-sz5[i];
      i++;
    }
  fclose(fp);

  /* Read S6 surface */
  fp=fopen("config/Optics/JOpticalSystem/s6_e2.dat","r");
  i=0;
  while(fscanf(fp,"%lf %f %f %lf",&sr6[i],&dum1,&dum2,&sz6[i])!=EOF)
    {
      sz6[i]=Sz6-sz6[i];
      i++;
    }
  fclose(fp);
  fprintf(stderr,"Done.\n");

  fprintf(stderr,"Reading Optical Data....   ");

  /* Read Acryl data */
  fp=fopen("config/Optics/JOpticalSystem/acryl.dat","r");
  i=0;
  while(fscanf(fp,"%d %lf %lf",&j,&ac_n[i],&ac_k[i])!=EOF)
    {
      ac_la[i]=(double)j;
      ac_n[i]=1.52565;
      i++;
    }
  fclose(fp);

  /* Read BG3 data */
  fp=fopen("config/Optics/JOpticalSystem/bg3.dat","r");
  i=0;
  while(fscanf(fp,"%d %lf %lf",&j,&bg3_n[i],&bg3_k[i])!=EOF)
    {
      bg3_la[i]=(double)j;
      i++;
    }
  fclose(fp);

  fprintf(stderr,"Done.\n");


}

static int refract(double in_ph[3],double nnn[3],double n1,double n2,double out_ph[3])
{
  int i;
  double rti, rtt;
  double dbuf1,dbuf2,dbuf3;
  double ct1,ct2;

  ct1=0.0;
  for(i=0;i<3;i++)
    {
      ct1-=in_ph[i]*nnn[i];
    }

  dbuf1=n1/n2;
  dbuf2=n2*n2/(n1*n1)+ct1*ct1-1.0;
  if(dbuf2<0.0) return(-81);

  dbuf3=sqrt(dbuf2)-ct1;
  for(i=0;i<3;i++)
    {
      out_ph[i]=dbuf1*(in_ph[i]-nnn[i]*dbuf3);
    }

  /* check hansya */

  rti=acos(ct1);
  ct2=0.0;
  for(i=0;i<3;i++)
    {
      ct2-=out_ph[i]*nnn[i];
    }
  rtt=acos(ct2);

  if(drand48() < ref(n1,n2,rti,rtt))
    { 
      //      fprintf(stderr,"*");
      return(-9);
    }

  return(0);
}


/*******************************************/
/*         main routine                    */
/*******************************************/

int trace(double ph_in[], double ph_out[])
     /* ph_in[0] : x0 mm */
     /* ph_in[1] : y0 mm */
     /* ph_in[2] : z0 mm */
     /* ph_in[3] : l     */
     /* ph_in[4] : m     */
     /* ph_in[5] : n     */
     /* ph_in[6] : lambda nm */
     /* ph_in[7] : time ns */
{
  int i,j,k;
  double dbuf1,dbuf2;
  double a0,a00;
  double a,b,c;
  double z0,z1,r1,z2,r2,x3,y3,z3,r3;
  double nr,nnn[3];
  double nn,kk;
  double out_ph[3];
  double dis;
  double dt=0.0;

  // Attention: we change photon's wavelength to 357 nm as suggested by Y.Takahashi
  // Dmitry Naumov, 21.05.2003
  ph_out[6]=ph_in[6];
  ph_in[6] = 357.;

  /* Opt. index  for this lambda */ 
  i=0;
  while(ph_in[6]>ac_la[i]) i++;

  polint(&ac_la[i-1],&ac_n[i-1],6,ph_in[6],&nn,&dbuf1); 
  polint(&ac_la[i-1],&ac_k[i-1],6,ph_in[6],&kk,&dbuf1);

  /**********************************/
  /*          S1                    */
  /**********************************/

  /* Check S1 */
  dbuf1=ph_in[2]-Zc1;
  a=ph_in[3]*ph_in[3]+ph_in[4]*ph_in[4]+ph_in[5]*ph_in[5];
  b=ph_in[3]*ph_in[0]+ph_in[4]*ph_in[1]+ph_in[5]*dbuf1;
  c=ph_in[0]*ph_in[0]+ph_in[1]*ph_in[1]+dbuf1*dbuf1-Rc1*Rc1;
  /* Ƚ̼ */
  dbuf2=b*b-a*c;
  if (dbuf2 <= 0.0)
    {
      return(-2);
    }
  dbuf1=(-b-sqrt(dbuf2))/a;
  x3=ph_in[3]*dbuf1+ph_in[0];
  y3=ph_in[4]*dbuf1+ph_in[1];
  z3=ph_in[5]*dbuf1+ph_in[2];
  
  /* s1  */
  j=0;
  do{
    k=0;
    r3=sqrt(x3*x3+y3*y3);
    if(r3>1240.0)
      {
	return(-3);
      }

    i=0;
    while(r3>sr1[i]) i++;

    r1=sr1[i-1];
    z1=sz1[i-1];
    r2=sr1[i];
    z2=sz1[i];

    a00=(z2-z1)/(r2-r1);
    z0=z1-a00*r1;
    a0=a00*a00;

    dbuf1=ph_in[2]-z0;
    a=a0*ph_in[3]*ph_in[3]+a0*ph_in[4]*ph_in[4]-ph_in[5]*ph_in[5];
    b=a0*ph_in[3]*ph_in[0]+a0*ph_in[4]*ph_in[1]-ph_in[5]*dbuf1;
    c=a0*ph_in[0]*ph_in[0]+a0*ph_in[1]*ph_in[1]-dbuf1*dbuf1;
    /* Ƚ̼ */
    dbuf2=b*b-a*c;
    if (dbuf2 <= 0.0)
      {
	return(-4);
      }
    dbuf1=(-b+sqrt(dbuf2))/a;
    x3=ph_in[3]*dbuf1+ph_in[0];
    y3=ph_in[4]*dbuf1+ph_in[1];
    z3=ph_in[5]*dbuf1+ph_in[2];
    r3=sqrt(x3*x3+y3*y3);
    if(a00 * r3 + z0 -z3 > 0.1)
      {
        dbuf1=(-b-sqrt(dbuf2))/a;
        x3=ph_in[3]*dbuf1+ph_in[0];
        y3=ph_in[4]*dbuf1+ph_in[1];
        z3=ph_in[5]*dbuf1+ph_in[2];
	r3=sqrt(x3*x3+y3*y3);
      }

    
    if(r3<r1)
      {
	j++;
        i--;
        k=-1;
	if(i<1) return(-5);
      }
    if(r3>r2)
      {
	j++;
        i++;
        k=-1;
	if(i>S_DATA) return(-6);
      }
    if(j>100) return(-j);
  }
  while(k<0);

  /* normal vector */
  dbuf1=atan2(r1-r2,z2-z1);
  nr=cos(dbuf1);
  nnn[2]=sin(dbuf1);
  
  dbuf1=atan2(y3,x3);
  nnn[0]=nr*cos(dbuf1);
  nnn[1]=nr*sin(dbuf1);


  /* refraction */
  k=refract(&ph_in[3],nnn,1.0003,nn,out_ph);
  if(k<0) {  return(-7);}
  
  /* distance between Start and S1 */
  dbuf1=ph_in[0]-x3;
  dis=dbuf1*dbuf1;
  dbuf1=ph_in[1]-y3;
  dis+=dbuf1*dbuf1;
  dbuf1=ph_in[2]-z3;
  dis+=dbuf1*dbuf1;
  dis=sqrt(dis);

  dt+=dis/C0;  

  ph_in[0]=x3;
  ph_in[1]=y3;
  ph_in[2]=z3;
  ph_in[3]=out_ph[0];
  ph_in[4]=out_ph[1];
  ph_in[5]=out_ph[2];


#ifdef ROOT
  printf("%d %f %f R\n",event,sqrt(x3*x3+y3*y3),z3);
#endif

   
  /***********************************/
  /*          S2                     */
  /***********************************/

  /* Check S2 */
  dbuf1=ph_in[2]-Zc2;
  a=ph_in[3]*ph_in[3]+ph_in[4]*ph_in[4]+ph_in[5]*ph_in[5];
  b=ph_in[3]*ph_in[0]+ph_in[4]*ph_in[1]+ph_in[5]*dbuf1;
  c=ph_in[0]*ph_in[0]+ph_in[1]*ph_in[1]+dbuf1*dbuf1-Rc2*Rc2;
  /* Ƚ̼ */
  dbuf2=b*b-a*c;
  if (dbuf2 <= 0.0)
    {
      return(-2);
    }
  dbuf1=(-b-sqrt(dbuf2))/a;
  x3=ph_in[3]*dbuf1+ph_in[0];
  y3=ph_in[4]*dbuf1+ph_in[1];
  z3=ph_in[5]*dbuf1+ph_in[2];

  /* s2  */
  j=0;
  do{
    k=0;
    r3=sqrt(x3*x3+y3*y3);
    if(r3>1240.0)
      {
	return(-3);
      }
    i=0;
    while(r3>sr2[i]) i++;

    r1=sr2[i-1];
    z1=sz2[i-1];
    r2=sr2[i];
    z2=sz2[i];

    a00=(z2-z1)/(r2-r1);
    z0=z1-a00*r1;
    a0=a00*a00;

    dbuf1=ph_in[2]-z0;
    a=a0*ph_in[3]*ph_in[3]+a0*ph_in[4]*ph_in[4]-ph_in[5]*ph_in[5];
    b=a0*ph_in[3]*ph_in[0]+a0*ph_in[4]*ph_in[1]-ph_in[5]*dbuf1;
    c=a0*ph_in[0]*ph_in[0]+a0*ph_in[1]*ph_in[1]-dbuf1*dbuf1;
    /* Ƚ̼ */
    dbuf2=b*b-a*c;
    if (dbuf2 <= 0.0)
      {
	return(-4);
      }
    dbuf1=(-b+sqrt(dbuf2))/a;
    x3=ph_in[3]*dbuf1+ph_in[0];
    y3=ph_in[4]*dbuf1+ph_in[1];
    z3=ph_in[5]*dbuf1+ph_in[2];

    r3=sqrt(x3*x3+y3*y3);
    if(a00 * r3 + z0 -z3 > 0.1)
      {
        dbuf1=(-b-sqrt(dbuf2))/a;
        x3=ph_in[3]*dbuf1+ph_in[0];
        y3=ph_in[4]*dbuf1+ph_in[1];
        z3=ph_in[5]*dbuf1+ph_in[2];
	r3=sqrt(x3*x3+y3*y3);
      }

    if(r3<r1)
      {
	j++;
        i--;
        k=-1;
        if(i<1) return(-5);
      }
    if(r3>r2)
      {
	j++;
        i++;
        k=-1;
        if(i>S_DATA) return(-6);
      }
    if(j>100) return(-j);
  }
  while(k<0);

  /* absorption between S1 and S2 */
  dbuf1=ph_in[0]-x3;
  dis=dbuf1*dbuf1;
  dbuf1=ph_in[1]-y3;
  dis+=dbuf1*dbuf1;
  dbuf1=ph_in[2]-z3;
  dis+=dbuf1*dbuf1;
  dis=sqrt(dis);

  dt+=nn*dis/C0;

  dbuf1=kk*M_PI*4.0/(ph_in[6]*1.e-6);
  dbuf2=exp(-dbuf1*dis);
  if(drand48()>dbuf2) 
    {
      return(-82);
    }

  /* normal vector */
  dbuf1=atan2(r1-r2,z2-z1);
  nr=cos(dbuf1);
  nnn[2]=sin(dbuf1);
  
  dbuf1=atan2(y3,x3);
  nnn[0]=nr*cos(dbuf1);
  nnn[1]=nr*sin(dbuf1);

  /* refraction */
  k=refract(&ph_in[3],nnn,nn,1.0003,out_ph);
  if(k<0) {  return(-7);}


  ph_in[0]=x3;
  ph_in[1]=y3;
  ph_in[2]=z3;
  ph_in[3]=out_ph[0];
  ph_in[4]=out_ph[1];
  ph_in[5]=out_ph[2];

#ifdef ROOT
  printf("%d %f %f R\n",event,sqrt(x3*x3+y3*y3),z3);
#endif


  /*********************/
  /*       IRIS (S3)    */
  /*********************/

  /* S3 surface */
  dbuf1=(Zc3-ph_in[2])/ph_in[5];
  x3=ph_in[3]*dbuf1+ph_in[0];
  y3=ph_in[4]*dbuf1+ph_in[1];

  dbuf2=sqrt(x3*x3+y3*y3);
  if(dbuf2>Rc3) {  return(-3);}

#ifdef ROOT
  printf("%d %f %f R\n",event,sqrt(x3*x3+y3*y3),z3);
#endif


  /***********************************/
  /*          S5                     */
  /***********************************/
  
  /* Check S5 */
  dbuf1=ph_in[2]-Zc5;
  a=ph_in[3]*ph_in[3]+ph_in[4]*ph_in[4]+ph_in[5]*ph_in[5];
  b=ph_in[3]*ph_in[0]+ph_in[4]*ph_in[1]+ph_in[5]*dbuf1;
  c=ph_in[0]*ph_in[0]+ph_in[1]*ph_in[1]+dbuf1*dbuf1-Rc5*Rc5;
  /* Ƚ̼ */
  dbuf2=b*b-a*c;
  if (dbuf2 <= 0.0)
    {
      return(-2);
    }
  dbuf1=(-b+sqrt(dbuf2))/a;
  x3=ph_in[3]*dbuf1+ph_in[0];
  y3=ph_in[4]*dbuf1+ph_in[1];
  z3=ph_in[5]*dbuf1+ph_in[2];

  /* S5  */
  j=0;
  do{
    k=0;
    r3=sqrt(x3*x3+y3*y3);
    if(r3>1240.0)
      {
	return(-3);
      }

    i=0;
    while(r3>sr5[i]) i++;

    r1=sr5[i-1];
    z1=sz5[i-1];
    r2=sr5[i];
    z2=sz5[i];

    a00=(z2-z1)/(r2-r1);
    z0=z1-a00*r1;
    a0=a00*a00;

    dbuf1=ph_in[2]-z0;
    a=a0*ph_in[3]*ph_in[3]+a0*ph_in[4]*ph_in[4]-ph_in[5]*ph_in[5];
    b=a0*ph_in[3]*ph_in[0]+a0*ph_in[4]*ph_in[1]-ph_in[5]*dbuf1;
    c=a0*ph_in[0]*ph_in[0]+a0*ph_in[1]*ph_in[1]-dbuf1*dbuf1;
    /* Ƚ̼ */
    dbuf2=b*b-a*c;
    if (dbuf2 <= 0.0)
      {
	return(-4);
      }
    dbuf1=(-b+sqrt(dbuf2))/a;

    x3=ph_in[3]*dbuf1+ph_in[0];
    y3=ph_in[4]*dbuf1+ph_in[1];
    z3=ph_in[5]*dbuf1+ph_in[2];

    r3=sqrt(x3*x3+y3*y3);
    if(a00 * r3 + z0 - z3 > 0.1)
      {
	dbuf1=(-b-sqrt(dbuf2))/a;
	x3=ph_in[3]*dbuf1+ph_in[0];
	y3=ph_in[4]*dbuf1+ph_in[1];
	z3=ph_in[5]*dbuf1+ph_in[2];
	r3=sqrt(x3*x3+y3*y3);
      }


    if(r3<r1)
      {
	j++;
        i--;
        k=-1;
        if(i<1) return(-5);
      }
    if(r3>r2)
      {
	j++;
        i++;
        k=-1;
        if(i>S_DATA) return(-6);
      }
    if(j>100) return(-j);
  }
  while(k<0);

 
  /* normal vector */
  dbuf1=atan2(r1-r2,z2-z1);
  nr=cos(dbuf1);
  nnn[2]=sin(dbuf1);

  dbuf1=atan2(y3,x3);
  nnn[0]=nr*cos(dbuf1);
  nnn[1]=nr*sin(dbuf1);

  /* refraction */
  k=refract(&ph_in[3],nnn,1.0003,nn,out_ph);
  if(k<0) {  return(-7);}

  /* distance between S2 and S5 */
  dbuf1=ph_in[0]-x3;
  dis=dbuf1*dbuf1;
  dbuf1=ph_in[1]-y3;
  dis+=dbuf1*dbuf1;
  dbuf1=ph_in[2]-z3;
  dis+=dbuf1*dbuf1;
  dis=sqrt(dis);

  dt+=dis/C0;


  ph_in[0]=x3;
  ph_in[1]=y3;
  ph_in[2]=z3;
  ph_in[3]=out_ph[0];
  ph_in[4]=out_ph[1];
  ph_in[5]=out_ph[2];

#ifdef ROOT
  printf("%d %f %f R\n",event,sqrt(x3*x3+y3*y3),z3);
#endif


  /***********************************/
  /*          S6                     */
  /***********************************/

  /* Check S6 */
  dbuf1=ph_in[2]-Zc6;
  a=ph_in[3]*ph_in[3]+ph_in[4]*ph_in[4]+ph_in[5]*ph_in[5];
  b=ph_in[3]*ph_in[0]+ph_in[4]*ph_in[1]+ph_in[5]*dbuf1;
  c=ph_in[0]*ph_in[0]+ph_in[1]*ph_in[1]+dbuf1*dbuf1-Rc6*Rc6;
  /* Ƚ̼ */
  dbuf2=b*b-a*c;
  if (dbuf2 <= 0.0)
    {
      return(-2);
    }
  dbuf1=(-b+sqrt(dbuf2))/a;
  x3=ph_in[3]*dbuf1+ph_in[0];
  y3=ph_in[4]*dbuf1+ph_in[1];
  z3=ph_in[5]*dbuf1+ph_in[2];

  /* S6  */
  j=0;
  do{
    k=0;
    r3=sqrt(x3*x3+y3*y3);
    if(r3>1240.0)
      {
	return(-3);
      }

    i=0;
    while(r3>sr6[i]) i++;

    r1=sr6[i-1];
    z1=sz6[i-1];
    r2=sr6[i];
    z2=sz6[i];

    a00=(z2-z1)/(r2-r1);
    z0=z1-a00*r1;
    a0=a00*a00;

    dbuf1=ph_in[2]-z0;
    a=a0*ph_in[3]*ph_in[3]+a0*ph_in[4]*ph_in[4]-ph_in[5]*ph_in[5];
    b=a0*ph_in[3]*ph_in[0]+a0*ph_in[4]*ph_in[1]-ph_in[5]*dbuf1;
    c=a0*ph_in[0]*ph_in[0]+a0*ph_in[1]*ph_in[1]-dbuf1*dbuf1;
    /* Ƚ̼ */
    dbuf2=b*b-a*c;
    if (dbuf2 <= 0.0)
      {
	return(-4);
      }
    dbuf1=(-b+sqrt(dbuf2))/a;
    x3=ph_in[3]*dbuf1+ph_in[0];
    y3=ph_in[4]*dbuf1+ph_in[1];
    z3=ph_in[5]*dbuf1+ph_in[2];
    r3=sqrt(x3*x3+y3*y3);
    if(a00 * r3 + z0 -z3 > 0.1)
      {
        dbuf1=(-b-sqrt(dbuf2))/a;
        x3=ph_in[3]*dbuf1+ph_in[0];
        y3=ph_in[4]*dbuf1+ph_in[1];
        z3=ph_in[5]*dbuf1+ph_in[2];
        r3=sqrt(x3*x3+y3*y3);
	r3=sqrt(x3*x3+y3*y3);
      }

    if(r3<r1)
      {
	j++;
        i--;
        k=-1;
        if(i<1) return(-5);
      }
    if(r3>r2)
      {
	j++;
        i++;
        k=-1;
        if(i>S_DATA) return(-6);
      }
    if(j>100) return(-j);
  }
  while(k<0);

  /* absorption between S5 and S6 */
  dbuf1=ph_in[0]-x3;
  dis=dbuf1*dbuf1;
  dbuf1=ph_in[1]-y3;
  dis+=dbuf1*dbuf1;
  dbuf1=ph_in[2]-z3;
  dis+=dbuf1*dbuf1;
  dis=sqrt(dis);


  dbuf1=kk*M_PI*4.0/(ph_in[6]*1.e-6);
  dbuf2=exp(-dbuf1*dis);

  if(drand48()>dbuf2) 
    {
      return(-83);
    }

  dt+=nn*dis/C0;


  /* normal vector */
  dbuf1=atan2(r1-r2,z2-z1);
  nr=cos(dbuf1);
  nnn[2]=sin(dbuf1);
  
  dbuf1=atan2(y3,x3);
  nnn[0]=nr*cos(dbuf1);
  nnn[1]=nr*sin(dbuf1);

  /* refraction */
  k=refract(&ph_in[3],nnn,nn,1.0003,out_ph);
  if(k<0) {  return(-7);}

  ph_out[0]=x3;
  ph_out[1]=y3;
  ph_out[2]=z3;
  ph_out[3]=out_ph[0];
  ph_out[4]=out_ph[1];
  ph_out[5]=out_ph[2];
  ph_out[7]=ph_in[7]+dt;

#ifdef ROOT
  printf("%d %f %f R\n",event,sqrt(x3*x3+y3*y3),z3);
#endif


  return(0);


}
