*     ... THIS IS THE ENERGY DISTRIBUTION FUNCTION OF ELECTRONS
*     ... IN A SHOWER DERIVED FROM A PAPER OF A.M.HILLAS
*     ... J.Phys.G.:Nucl.Phys., 8 (1982) 1461-1473
*     ... INPUT: S  (shower age)
*                ET (electron energy in eV)
*     ... OUTPUT: PROBABILITY TO HAVE AN ELECTRON WITH ET AND S
*     ... DATE: 29.05.2002
*     ... AUTHOR: D.V.NAUMOV      
      REAL FUNCTION ENERGY_SPECTRUM_HILLAS(S,ET)
      IMPLICIT NONE
#include "shower.inc"
#include "event.inc"
      REAL S,ET,f1,f2,E,E0
*
      E = ET                              ! MeV
      IF(S.GE.0.4) E0 = 44. - 17*(S-1.46) ! MeV
      IF(S.LT.0.4) E0 = 26.               ! MeV
      f1 = 2.e-4*S*((0.89*E0-1.2)/(E+E0))**S/(1.+1.e-4*E*S)**3
      f2 = (0.89*E0-1.2)*((0.89*E0-1.2)/(E+E0))**(S-1.)*S/(E+E0)**2
      f2 = f2/(1.+1.e-4*E*S)**2
      ENERGY_SPECTRUM_HILLAS = f1+f2
      END

