// ESAF : Euso Simulation and Analysis Framework
// $Id: Meteoroid.hh 1002 2004-09-13 16:44:02Z thea $
// P. Scarsi created Dec,  4 2003
// description of a meteoroid at atmophere entrance
// spherical shape is assumed at the entrance
// 

#ifndef __METEOROID_HH_
#define __METEOROID_HH_

#include "euso.hh"
#include "EVector.hh"
#include "TMath.h"

class Meteoroid {
public:
    // ctor
    Meteoroid( double, double, const EVector&, const EVector&, double, double );

    // dtor
    virtual ~Meteoroid();

    // get mass
    inline double GetMass() const { return fMass;}

    // get density
    inline double GetDensity() const { return fDensity;}

    // get radius
    double GetRadius() const; 

private:
    // meteoroid initial mass
    double fMass;

    // density
    double fDensity;

    // initial velocity
    EVector fVelocity;

    // initial position
    EVector fPosition;

    // heat ablation
    double fHeatAbla;

    // heat transfer coefficient
    double fHeatTransf;
    ClassDef(Meteoroid,0)
};

#endif  /* __METEOROID_HH_ */

