// $Id$
// Author: kenjikry   2009/03/04

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ConexFileGenerator                                                   *
 *  Package: <generators/shower>                                             *
 *  Coordinator: <Dmitry.Naumov>                                             *
 *                                                                           *
 *****************************************************************************/

#ifndef __CONEXFILEGENERATOR_HH__
#define __CONEXFILEGENERATOR_HH__

#include "euso.hh"
#include "EventGenerator.hh"
#include "EsafMsgSource.hh"
#include "ShowerStep.hh"
#include "EarthVector.hh"
#include <TString.h>

class PhysicsData;
class MCTruth;
class ShowerTrack;
class TTree;
class TFile;
class TLorentzVector;
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// ConexFileGenerator                                                         //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class ConexFileShowerGenerator : public EventGenerator, public EsafMsgSource {
public:
    ConexFileShowerGenerator();
    virtual ~ConexFileShowerGenerator();

    // get a new event
    PhysicsData *Get();

    // get pointer to MonteCarlo Truth
    MCTruth *GetTruth();

    virtual void Reset();                                       // Reset important variables for the next event
 
    EsafConfigClass(Generators, ConexFileShowerGenerator)

















private:
    // open conex root file
    void OpenFile();
    // close conex root file
    void CloseFile();
    // init trees
    void InitTrees();
    // load shower track
    Bool_t LoadTrack();
    const ShowerStep& GetShowerStep(Int_t); 

    ShowerTrack          *fTrack;                          // Shower Track object
    ShowerStep            fStep;                           // current ShowerStep object    
    MCTruth              *fTruth;                          // MC Truth information
    Int_t                 fCurrentEvent;                   // current event
    TFile                *fInputFile;                      // Conex input Root File
    TString               fFileName;                       // Conex input root file name
    Int_t                 fFirstEvent;                     // first event to start froms
    TTree                *fConexHeader;                    // Conex header tree
    TTree                *fConexShower;                    // Conex Shower tree
    TTree                *fConexFirstInteraction;          // Conex first interaction tree
    Float_t X[4000], H[4000],D[4000],N[4000],Electrons[4000],eN[4000],dEdX[4000];
    Int_t nX, nPart, nInt, PrimaryIndex,IdInt1[100000];
    Double_t depth[100000], px[100000],py[100000],pz[100000],Energy[100000];                       // first depth of the current track and momentum
    Double_t Xmax;
    Double_t height[100];
    Float_t fitpars[11];
    Float_t zenith,azimuth,lgE;
    Float_t delX;
    Float_t Hfirst;

// temporary setting (hand-in input)
  Float_t core_x,core_y,core_z;
  Float_t cnx_l,cnx_m,cnx_n;  // Theta,phi direction cosine with respect to shower landing posistion
  Float_t geo_l,geo_m,geo_n; // Shower direction cosines with respect to geocentric coordinates (Ng=1; geocentre->EUSO)
  



  
  Double_t prevdepth;
  EarthVector prevpoint, currentpoint;
  TLorentzVector       *pMom;                          // UHECR 4 momentum


  ClassDef(ConexFileShowerGenerator,0);

  

};

#endif  /* __CONEXFILEGENERATOR_HH__ */

