// $Id$
// Author: kenjikry   2009/03/31

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ConexFileGenerator                                                   *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// ConexFileGenerator
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "ConexFileShowerGenerator.hh"
#include <TFile.h>
#include <TTree.h>
#include "MCTruth.hh"
#include "ShowerTrack.hh"
#include "Atmosphere.hh"
#include <TMath.h>
#include <TSystem.h>
#include <TLorentzVector.h>
#include <iostream>
using namespace sou;
using namespace std;
using TMath::RadToDeg;

ClassImp(ConexFileShowerGenerator)

//_____________________________________________________________________________
ConexFileShowerGenerator::ConexFileShowerGenerator(): EventGenerator("conex"), EsafMsgSource(){
  //
  // Constructor
  //
  fFirstEvent = 0;
  fFileName = "";
  fTruth = NULL;
  fTrack = NULL; 
  fConexHeader = NULL;
  fConexShower = NULL;
  fConexFirstInteraction = NULL;
  pMom = NULL;
}

//_____________________________________________________________________________
ConexFileShowerGenerator::~ConexFileShowerGenerator() {
  //
  // Destructor
  //
}



//_____________________________________________________________________________
void ConexFileShowerGenerator::OpenFile() {
  //
  // open the input file
  //
  if ( Conf()->IsDefined( "ConexFileShowerGenerator.FileName" ) ) {
    fFileName = Conf()->GetStrPath("ConexFileShowerGenerator.FileName");
    fFirstEvent = (Int_t)Conf()->GetNum("ConexFileShowerGenerator.FirstEvent") ;
  }
  else {
    Msg(EsafMsg::Panic) << "Unknown input file name in ConexFileShowerGenerator" << MsgDispatch;
  }
  
  fInputFile = TFile::Open(fFileName);
  fConexHeader = (TTree*)fInputFile->Get("Header");
  fConexShower = (TTree*)fInputFile->Get("Shower");
  fConexFirstInteraction = (TTree*)fInputFile->Get("FirstInteraction");
  fCurrentEvent=fFirstEvent;
  InitTrees();
}

//_____________________________________________________________________________
void ConexFileShowerGenerator::CloseFile() {
  //
  // close the input file
  //
  fInputFile->Close();
  fConexHeader = NULL;
  fConexShower = NULL;
  fConexFirstInteraction = NULL;
}

//_____________________________________________________________________________
void ConexFileShowerGenerator::InitTrees() {
  //
  // initialize the trees in conex file
  //
  fConexShower->SetBranchAddress("nX",&nX); // number of points in slant depth 
  fConexShower->SetBranchAddress("N",N);     // particles(X)      
  fConexShower->SetBranchAddress("dEdX",dEdX); // Energy deposit(X)     
  fConexShower->SetBranchAddress("Electrons",Electrons);
  fConexHeader->SetBranchAddress("X",X);       // slant depth    
  fConexHeader->SetBranchAddress("H",H);       // Height    
  fConexHeader->SetBranchAddress("D",D);       // Height    
  fConexHeader->SetBranchAddress("delX",&delX);// depth step  
      
  fConexFirstInteraction->SetBranchAddress("depth",depth);
  
  fConexShower->SetBranchAddress("lgE",&fitpars[0]);
  fConexShower->SetBranchAddress("zenith",&fitpars[1]);
  fConexShower->SetBranchAddress("azimuth",&fitpars[10]);
  fConexShower->SetBranchAddress("Xmax",&fitpars[9]);
    
  fConexFirstInteraction->SetBranchAddress("height",height);
  fConexFirstInteraction->SetBranchAddress("idInt",IdInt1);
  fConexFirstInteraction->SetBranchAddress("nPart",&nPart);
  fConexFirstInteraction->SetBranchAddress("nInt",&nInt);

}

//_____________________________________________________________________________
Bool_t ConexFileShowerGenerator::LoadTrack() {
  //
  // load the shower track. Conert it from Conex format to ESAF
  //
  fConexHeader->GetEntry(0);
  fConexShower->GetEntry(fCurrentEvent);
  fConexFirstInteraction->GetEntry(fCurrentEvent);
  
  if(!fTrack) fTrack = new ShowerTrack();
  else fTrack->Reset();
  
  if (!pMom) pMom = new TLorentzVector;
  else       pMom->SetXYZT(0,0,0,0);
  
  fTrack->fEnergy = pow(10.,fitpars[0]) *eV; 
  fTrack->fTheta = fitpars[1] * TMath::DegToRad();
  fTrack->fPhi = fitpars[10] * TMath::DegToRad();  // Definition phi needs to be checked.
  fTrack->fX1 = depth[0]/(cm2/g);  // Need to be checked 


// KS (090331)
//	Temporarily theta and phi angles to behand-in inputted 

//  fTrack->fTheta  =  45. * TMath::DegToRad();  // Commented out so that ONEX Theta is not overwritten
  fTrack->fPhi    =  23. * TMath::DegToRad();

//	Temporarily theta and phi angles to behand-in inputted 
 // core_x = 1.852*600.*km;   // 
  core_x = 100.*km;
  core_y = 0*km;
  core_z = TMath::Sqrt((6371.*km)*(6371.*km)-core_x*core_x-core_y*core_y)-6371.*km;  







// Note that the Earth's radii in CONEX is 6378.14 km.     


  // Shower axis with respect to core
  cnx_l = TMath::Sin(fTrack->fTheta)*TMath::Cos(fTrack->fPhi);
  cnx_m = TMath::Sin(fTrack->fTheta)*TMath::Sin(fTrack->fPhi);
  cnx_n = TMath::Cos(fTrack->fTheta);

    //

  cout << core_x << " " << core_y << " " << core_z << " km" << endl;
  


  // Vector of zenith direction with respect to the core location
  TVector3 core_zenith(core_x/6371./km,core_y/6371./km,(core_z+6371.*km)/6371./km);
  
  cout << core_x/6371./km << " " << core_y/6371./km << " " << (core_z+6371.*km)/6371./km << "  corepos_vector " << endl;
  
  Float_t rotate_x,rotate_y;  
  
  rotate_y = TMath::ATan2(core_x,core_z+6371.*km);
  rotate_x = TMath::ATan2(core_y,core_z+6371.*km);
  
  //  tmp_lat = TMath::ASin(Core_y/6371./km);

  cout << rotate_x * TMath::RadToDeg() << " " << rotate_y * TMath::RadToDeg() <<  " rotation " << endl;
  
  
  // Primary particle trajectory vector (skyward)
  geo_l = (cnx_l * TMath::Cos(rotate_y) + cnx_n * TMath::Sin(rotate_y));
  geo_m = (cnx_m) * TMath::Cos(rotate_x) - (- cnx_l * TMath::Sin(rotate_y) + cnx_n * TMath::Cos(rotate_y)) * TMath::Sin(rotate_x);
  geo_n = (cnx_m) * TMath::Sin(rotate_x) + (- cnx_l * TMath::Sin(rotate_y) + cnx_n * TMath::Cos(rotate_y)) * TMath::Cos(rotate_x);
  
#ifndef DEBUG
  cout << cnx_l << " " << cnx_m << " " << cnx_n << " " << cnx_l*cnx_l+cnx_m*cnx_m+cnx_n*cnx_n << " " << fTrack->fTheta*TMath::RadToDeg()  << " deg " << fTrack->fPhi*TMath::RadToDeg() << " deg " << endl;
  
  
  cout << geo_l << " " << geo_m << " " << geo_n << " " << geo_l*geo_l+geo_m*geo_m+geo_n*geo_n << " " << TMath::ACos(geo_n)*TMath::RadToDeg() << " " << TMath::ATan2(geo_m,geo_l)*TMath::RadToDeg()  << " final_vector " << endl;
  
  cout << TMath::ACos((geo_l*core_x+geo_m*core_y+geo_n*(core_z+6371.*km))/(6371.*km))*TMath::RadToDeg() << " " << (geo_l*core_x+geo_m*core_y+geo_n*(core_z+6371.*km))/(6371.*km) << " inner prod" << endl;
  
  //exit(-1);
  #endif

  
  TVector3 dir(geo_l,geo_m,geo_n);
  
  fTrack->fDirVers = -dir;
    
  fTrack->fEthrEl = 0;
  

//  pMom->SetXYZT(-fTrack->fEnergy*TMath::Sin(fTrack->fTheta)*TMath::Cos(fTrack->fPhi),-fTrack->fEnergy*TMath::Sin(fTrack->fTheta)*TMath::Sin(fTrack->fPhi),-fTrack->fEnergy*TMath::Cos(fTrack->fTheta),fTrack->fEnergy);
  
  pMom->SetXYZT(-geo_l*fTrack->fEnergy,-geo_m*fTrack->fEnergy,-geo_n*fTrack->fEnergy,fTrack->fEnergy);
  




  //  cout << fitpars[9] << " " << fitpars[1] << " " << fitpars[10] << " " << fitpars[0] << endl;
  cout << geo_l << " " <<  geo_m << " " << geo_n <<  " " <<  " ve" << endl;
  cout << core_x << " " <<  core_y << " " << core_z <<  " " <<  " ve" << endl;


   

  // Age parameter setting (not yet fully working)
  for (int i = 0 ; i < nX ; i++ ) {
    ShowerStep s = GetShowerStep(i);
    fTrack->Add(s);
  }
  return kTRUE;
}

//_____________________________________________________________________________
const ShowerStep& ConexFileShowerGenerator::GetShowerStep(Int_t i) {
  fStep.Clear();
  
  if (i==0) {

#ifdef ZERO_ASSUMPTION    
    prevdepth = X[0];
    
    Double_t L = D[0]*m; // from CONEX data (distance from the core on Earth along shower axis)
    
#endif

    
    prevdepth = depth[0];

    cout << depth[0] << endl;

    //re-compute 3D initial position
    Double_t ct = - TMath::Cos(fTrack->GetTheta());
    Double_t y1 = height[0]/6378140.; // height/EarthRadius
    Double_t L = (ct + TMath::Sqrt(ct*ct + 2*y1 + y1*y1))*6378140.*m; // track length
    
//    Double_t x = L*TMath::Sin(fTrack->GetTheta())*TMath::Cos(fTrack->GetPhi());
//    Double_t y = L*TMath::Sin(fTrack->GetTheta())*TMath::Sin(fTrack->GetPhi());
//    Double_t z = L*TMath::Cos(fTrack->GetTheta());


    Double_t x = core_x + L * geo_l;
    Double_t y = core_y + L * geo_m;
    Double_t z = core_z + L * geo_n;
    
    cout << x << " " << y << " " << z << endl;
    cout << geo_l << " " <<  geo_m << " " << geo_n <<  " " <<  L << endl;

    prevpoint.SetXYZ(x,y,z);
    fTrack->fInitPos.SetXYZ(x,y,z);
    
    // cerr << "showerstep " << y1 << " " << prevdepth << " " <<  height[nInt-1] << " " << i << " " <<currentpoint.Zv()/km << "  " << X[i-1] << height[i] << " " << L <<  " " << ct << endl;
  }
  else{
    prevdepth = X[i-1];
  }
    
  currentpoint = prevpoint;
  
//  if (currentpoint.Zv()/km < 0) return fStep;   // In case core is not at (0,0)
  TVector3 dir = pMom->Vect().Unit(); 
  
  Double_t depth_step=0;
  while (kTRUE) {
    Double_t step = 10*m;
    currentpoint += step*dir;
    if (currentpoint.Zv()/km<0) break;
    depth_step += Atmosphere::Get()->Air_Density(currentpoint)*step;
    if (depth_step>=delX*g/cm2) break;
  }
  
  fStep.fXi = prevdepth;
  fStep.fXf = X[i];
  fStep.fXYZi = prevpoint;
  fStep.fXYZf = currentpoint;
  fStep.fNcharged = N[i];
  fStep.fNelectrons = Electrons[i];
  
  //  cout << X[i] << endl;
  
  //  fStep.fEnergyLoss = dEdX[i] * Xdel * GeV;
  
  // Temporary age parameter approximation
  
  if ( fStep.fXi > depth[0] ) fStep.fAgei =  2. / (1 + (fitpars[9]-depth[0]) /(fStep.fXi-depth[0]) );
  else fStep.fAgei = 0.;
  
  if ( fStep.fXf > depth[0] ) fStep.fAgef =  2. / (1 + (fitpars[9]-depth[0]) /(fStep.fXf-depth[0]) );
  else fStep.fAgef = 0.;
  
  prevpoint = currentpoint;
  return fStep;
}

//_____________________________________________________________________________
void ConexFileShowerGenerator::Reset() {
  
}

//______________________________________________________________________________
PhysicsData* ConexFileShowerGenerator::Get() {

  //
  // Get a new event
  //
  
  
  // build and return a track
  
  if ( !fInputFile ) OpenFile();  
  
  while (1) {
    
    if ( fTrack ) {
      delete fTrack;
      fTrack=NULL;
    }
    if ( fTruth ) {
      delete fTruth;
      fTruth=NULL;
    }
    
    if (fCurrentEvent >= fFirstEvent && fCurrentEvent<100) {
      if ( LoadTrack() ){
	
	MsgForm(EsafMsg::Info,"ConexFile event %d", fCurrentEvent);
	MsgForm(EsafMsg::Info,
		"ShowerTrack produced : Energy = %.2E MeV, Theta= %.2f deg, Phi= %.2f deg",
		fTrack->GetEnergy(), fTrack->GetTheta()*RadToDeg(),fTrack->GetPhi()*RadToDeg());
	MsgForm(EsafMsg::Info,
		"         First interaction X1 = %.3f at : (%.2f,%.2f,%.2f) km gives a shower with %d steps",
		fTrack->GetX1()*cm2/g,fTrack->GetInitPos().X()/km,fTrack->GetInitPos().Y()/km,
		fTrack->GetInitPos().Z()/km, fTrack->GetNumStep());
      } else {
	Msg(EsafMsg::Warning) << "ConexFileShowerGenerator: End of file "
			      << fFileName << " reached? " << MsgDispatch;
	fTrack = (ShowerTrack*)NULL;
      }
      fCurrentEvent++;
      break;
    } 
    
    fCurrentEvent++;
    
  }
  fTrack->CheckTrack();
  return fTrack;
  
}

//KS
//______________________________________________________________________________
MCTruth* ConexFileShowerGenerator::GetTruth() {
  //
  // Method returning the Truth object 
  //
  
  // init
  if(!fTruth) fTruth = new MCTruth();
  
  // can be filled even if showertrack has no steps
  fTruth->SetEnergy(fTrack->GetEnergy());
  fTruth->SetThetaPhi(fTrack->GetTheta(),fTrack->GetPhi());
  
// temporarily 
  Int_t particode = Int_t(floor(1+0.5)); // proton (default in CONEX)
  fTruth->SetParticle(particode);
  
  //    fTruth->SetTOAImpact(fTOAImpact);
  //    fTruth->SetEarthImpact(fEarthImpact);
  
  // filled only if steps exist    
  if (fTrack->Size() > 0) {
    const ShowerStep& s = fTrack->GetLastStep();
    fTruth->SetEarthAge(s.GetAgef());
    // get the shower maximum
    Float_t MaxElectrons(0);
    Int_t MaxIndex(-1);
    for (Int_t i(0); i < Int_t(fTrack->Size()) ; i++) {
      const ShowerStep& s = fTrack->GetStep(i);
      if (MaxElectrons < s.GetNelectrons()) {
	MaxIndex = i;
	MaxElectrons = s.GetNelectrons();
      }
    }
    if (MaxIndex != -1) {
      const ShowerStep& s = fTrack->GetStep(MaxIndex);
      fTruth->SetShowerMax(0.5*(s.GetXYZi() + s.GetXYZf()),0.5*(s.GetXi() + s.GetXf())*(g/cm2));
    }
  }
  return fTruth;
}


