// ESAF : Euso Simulation and Analysis Framework
// $Id: LightSource.hh 2676 2006-04-28 15:22:49Z moreggia $
// D. De Marco - M. Pallavicini created Jan, 20 2002
//
// IMPORTANT : EVERY child of this generic class MUST CONTAIN and HANDLE
//             creation and destruction of the PhotonsInAtmosphere* pointer
//             (the object must be in the HEAP)
//

#ifndef __LIGHTSOURCE_HH_
#define __LIGHTSOURCE_HH_

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "ConfigFileParser.hh"

class PhotonsInAtmosphere;
class MCTruth;
class PhysicsData;

class LightSource : public EsafConfigurable {
public:
    // ctor
    LightSource() {}
    LightSource(const string& name);

    // dtor
    virtual ~LightSource();

    virtual PhotonsInAtmosphere* Get( const PhysicsData* ) = 0;
    virtual MCTruth* Truth() = 0;
    
    // get ready for next event
    virtual void Reset() = 0;

    // release all the mem hold in the buffers
    virtual Bool_t ClearMemory() = 0;
    
    // Set/Get wavelength bounds
    inline void SetWlRange(Double_t min, Double_t max) {fLambdaMin = min; fLambdaMax = max;}
    inline Double_t GetWlRangeMin() {return fLambdaMin;}
    inline Double_t GetWlRangeMax() {return fLambdaMax;}
 
    EsafConfigClass(LightSource,LightSource)

protected:
    string fName;
    Double_t fLambdaMin;
    Double_t fLambdaMax;

    ClassDef(LightSource,0)
};

#endif  /* __LIGHTSOURCE_HH_ */

