// ESAF : Euso Simulation and Analysis Framework
// $Id: ShowerLightSource.hh 2604 2006-03-20 21:51:35Z thea $
// Alessandro Thea created Nov, 24 2003

#ifndef __SHOWERLIGHTSOURCE_HH_
#define __SHOWERLIGHTSOURCE_HH_

#include "euso.hh"
#include "LightSource.hh"
#include "BunchOfPhotons.hh"
#include "ShowerTrack.hh"
#include <TF2.h>
#include <TF12.h>

class FluoCalculator;
class CrkCalculator;
class ListPhotonsInAtmosphere;
class PhysicsData;
 
class ShowerLightSource : public LightSource, public EsafMsgSource {
public:
    // ctor
    ShowerLightSource();
  
    // dtor
    virtual ~ShowerLightSource();
  
    // generate photons
    virtual PhotonsInAtmosphere* Get( const PhysicsData* );

    MCTruth* Truth();
    
    // get ready for next event
    virtual void Reset();

    // release all the mem hold in the buffers
    virtual Bool_t ClearMemory();

    EsafConfigClass(LightSource,ShowerLightSource)
    
private:
    BunchOfPhotons* MakeFluoStep(const ShowerStep& step,TF2* EnergyDistribution,
                                 TF2* LateralDistribution,TF2* AngularDistribution);

    BunchOfPhotons* MakeCerenkovStep(const ShowerStep& step, TF2* EnergyDistribution, 
                                     TF2* LateralDistribution,TF2* AngularDistribution);

    void Configure();

    ListPhotonsInAtmosphere* fPh_in_atmo;
    FluoCalculator* fFluocalcul;
    CrkCalculator* fCrkcalcul;
    Double_t fShowerNemax;
    string fEnergyDistributionType;
    string fEnergyDistributionName;
    string fLateralDistributionName;
    string fAngularDistributionName;
    Bool_t fUseAngDev;                  // (config) if true -> Electrons angular deviation used for Yield calculation

    ClassDef(ShowerLightSource,0)
};

#endif  /* __SHOWERLIGHTSOURCE_HH_ */

