// ESAF : Euso Simulation and Analysis Framework
// $Id: LightToEuso.hh 2604 2006-03-20 21:51:35Z thea $
// Marco Pallavicini created Mar, 13 2002
// Abstract class for all possible simulation chains up to
// Euso Detector Simulation (excluded)
// Basically, it brings a PhotonsOnPupil at the Euso entrance pupil
// 
#ifndef __LIGHTTOEUSO_HH_
#define __LIGHTTOEUSO_HH_

#include "euso.hh"
#include "EsafConfigurable.hh"
#include "EsafMsgSource.hh"

class EventGenerator;
class LightSource;
class RadiativeTransfer;
class PhotonsOnPupil;
class MCTruth;
class PhysicsData;
class EEvent;
class DetectorGeometry;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  LightToEuso                                                               //
//                                                                            //
//  Abstract interface for all the light propagators                          //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////


class LightToEuso : public EsafConfigurable, public EsafMsgSource {
public:
    LightToEuso( const string& name );
    // ctor

    virtual ~LightToEuso();
    // dtor

    // returns the photons on the pupil
    virtual PhotonsOnPupil *Get(const DetectorGeometry*) = 0;
    
    virtual void Configure() = 0;
    // configure
    
    virtual void Reset();
    // get ready for next event

    virtual MCTruth* GetTruth() = 0;
    // get pointer to MonteCarlo Truth
    
    virtual PhysicsData* GetPhysics() = 0;
    // get pointer to shower object

    virtual void ReplaceInputFile( const char* ) = 0;
    // change the input file name when meaningful

    inline const string& GetName() const { return fName; }
    // get the light to euso name
    
    virtual Bool_t ClearMemory() = 0;
    // physically release the memory allocated by the arrays of this object
    
    EsafConfigClass(LightToEuso,LightToEuso)

protected:
    EventGenerator* fGenerator;          // event generator 
    LightSource* fLight;                 // light generator in atmosphere
    RadiativeTransfer* fRadiative;       // radiative transport to euso    
    
    inline EventGenerator* GetGenerator() { return fGenerator; }
    inline RadiativeTransfer* GetRadiative() { return fRadiative; }
    inline LightSource* GetLightSource() { return fLight; }

private:
    string fName;

    ClassDef(LightToEuso,0)
};

#endif  /* __LIGHTTOEUSO_HH_ */

