// $Id: PhPRootFileLightToEuso.hh 2604 2006-03-20 21:51:35Z thea $
// Author: Alessandro Thea   2005/02/14

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: PhPRootFileLightToEuso                                               *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __PHPROOTFILELIGHTTOEUSO_HH__
#define __PHPROOTFILELIGHTTOEUSO_HH__

#include "euso.hh"
#include "LightToEuso.hh"

class TFile;
class TTree;
class TClonesArray;
class ETruth;
class EGeometry;
class MCTruth;
class ListPhotonsOnPupil;


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// PhPRootFileLightToEuso                                                     //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class PhPRootFileLightToEuso : public LightToEuso {
public:
    PhPRootFileLightToEuso();
    virtual ~PhPRootFileLightToEuso();

    // returns the photons on the pupil
    virtual PhotonsOnPupil *Get(const DetectorGeometry*);
    
    // configure
    virtual void Configure() { return; }
    
    // get ready for next event
    virtual void Reset() {};

    // get pointer to MonteCarlo Truth
    virtual MCTruth* GetTruth() { return fMCTruth; }
    
    // get pointer to shower object
    virtual PhysicsData* GetPhysics() { return 0; };

    // change the input file name when meaningful
    virtual void ReplaceInputFile( const char* ) {};

    virtual Bool_t ClearMemory();
    // releas all the memory hold in the buffers
private:

    void Init();

    string fFileName;
    TFile *fFile;

    MCTruth* fMCTruth;
    ETruth* fRootTruth;
    EGeometry* fRootGeometry;
    TTree *fTree;
    Int_t fNumPhotons;
    TClonesArray* fPhotons;

    ListPhotonsOnPupil *fPhotonsOnPupil;
    Long64_t fFirstEvent;
    Long64_t fEvCounter;

    EsafConfigClass(LightToEuso,PhPRootFileLightToEuso)

    ClassDef(PhPRootFileLightToEuso,0)
};

#endif  /* __PHPROOTFILELIGHTTOEUSO_HH__ */

