// $Id: SinglePhotonRootFileLightToEuso.hh 2779 2006-12-21 15:23:54Z moreggia $
// Author: Sylvain Moreggia   2006/06/01

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SinglePhotonRootFileLightToEuso                                      *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __SINGLEPHOTONROOTFILELIGHTTOEUSO_HH__
#define __SINGLEPHOTONROOTFILELIGHTTOEUSO_HH__

#include "euso.hh"
#include "LightToEuso.hh"

class SinglePhoton;
class TFile;
class TTree;
class EAtmosphere;
class ETruth;
class EGeometry;
class MCTruth;
class ListPhotonsOnPupil;
class EarthVector;
class EEvent;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// SinglePhotonRootFileLightToEuso                                            //
//                                                                            //  
// Read input root file holding SinglePhoton information, to prepare detector //  
// simulation                                                                 //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class SinglePhotonRootFileLightToEuso : public LightToEuso {
public:
    SinglePhotonRootFileLightToEuso();
    virtual ~SinglePhotonRootFileLightToEuso();

    // returns the photons on the pupil
    virtual PhotonsOnPupil *Get(const DetectorGeometry*);
    
    // configure
    virtual void Configure() { return; }
    
    // get ready for next event
    virtual void Reset() {};

    // get pointer to MonteCarlo Truth
    virtual MCTruth* GetTruth() { return fMCTruth; }
    
    // get pointer to shower object
    virtual PhysicsData* GetPhysics() { return 0; };

    // change the input file name when meaningful
    virtual void ReplaceInputFile( const char* ) {};

    virtual Bool_t ClearMemory();
    // releas all the memory hold in the buffers

private:

    void Init();
    
    // convert SinglePhoton into ParentPhoton, then add it to fPhotonsOnPupil
    void ConvertThenAdd(const SinglePhoton&,const EarthVector&,const EarthVector&);


    string fFileName;
    TFile *fFile;
    Bool_t fNewPhFormat;   // to handle new photons format (yes : shower age contained in singlephoton)
    Bool_t fSeveralEvents; // false : a single detector simu for each input event

    MCTruth* fMCTruth;
    TTree *fTree;
    EEvent* fEv;
    ETruth* fRootTruth;
    Int_t fNumPhotons;
    EGeometry* fRootGeometry;
    EAtmosphere* fAtmosphere;

    ListPhotonsOnPupil *fPhotonsOnPupil;
    Long64_t fFirstEvent;
    Long64_t fEvCounter;

    EsafConfigClass(LightToEuso,SinglePhotonRootFileLightToEuso)


    ClassDef(SinglePhotonRootFileLightToEuso,0)
};

#endif  /* __SINGLEPHOTONROOTFILELIGHTTOEUSO_HH__ */

