// ESAF : Euso Simulation and Analysis Framework
// $Id: SlastLightToEuso.hh 2807 2008-10-12 17:13:07Z naumov $
// Dmitry Naumov created Oct, 28 2002
// This is an ESAF interface to Fortran program SLAST
// This class invokes the SLAST subroutine for the complete
// simulation on one event, from shower simulation to transport
// of the photons to Euso in one shot
// Another interface to SLAST is foreseen, splitting the process
// in to steps ( see classes SlastLightSource and LowTranRadiativeTransfer )
// SlastShowerGenerator is implemented 
#ifndef __SLASTLIGHTTOEUSO_HH_
#define __SLASTLIGHTTOEUSO_HH_

#include "euso.hh"
#include "LightToEuso.hh"
#include "ListPhotonsOnPupil.hh"
#include "MCTruth.hh"
#include "EEvent.hh"

class PhysicsData;

class SlastLightToEuso : public LightToEuso {
public:
    SlastLightToEuso();
    virtual void Init();
    virtual ~SlastLightToEuso();

    // returns the appropriate PhotonsOnPupil object for one event
    virtual PhotonsOnPupil *Get(const DetectorGeometry*);

    // re-configure SLAST
    virtual void Configure();

    // returns Montecarlo truth informations for this event
    virtual MCTruth* GetTruth();

    // returns simulated shower infos for this event
    virtual PhysicsData* GetPhysics();

    
    // replace input file
    virtual void ReplaceInputFile( const char* fil) {}

    virtual Bool_t ClearMemory();

    EsafConfigClass(LightToEuso, SlastLightToEuso )


private:
    void PrepareDataCards();
    ListPhotonsOnPupil *fPhotons;
    MCTruth* fTruth;

    ClassDef(SlastLightToEuso,0)

};

#endif  /* __SLASTLIGHTTOEUSO_HH_ */
