// ESAF : Euso Simulation and Analysis Framework
// $Id: TestLightToEuso.hh 2604 2006-03-20 21:51:35Z thea $
// M. Pallavicini created Sep, 16 2002

#ifndef __TEST_LIGHTTOEUSO_HH__
#define __TEST_LIGHTTOEUSO_HH__

#include "euso.hh"
#include "LightToEuso.hh"
#include "ParentPhoton.hh"
#include "Config.hh"

class ListPhotonsOnPupil;
class PhysicsData;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// TestLightToEuso                                                            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class TestLightToEuso : public LightToEuso {
public:
    // ctor
    TestLightToEuso();

    // dtor
    virtual ~TestLightToEuso();

    // returns the list of fPhotons for the pupil
    virtual PhotonsOnPupil *Get(const DetectorGeometry*);
    
    // next functions not meaningful in this case
    virtual void Configure() { return; }
    virtual MCTruth* GetTruth() { return fTruth; }
    virtual PhysicsData* GetPhysics() { return NULL; }
    virtual void ReplaceInputFile(const char*) { return; }

    virtual Bool_t ClearMemory();

    EsafConfigClass(LightToEuso,TestLightToEuso)

private:
    PhotonsOnPupil *MakeSpot( Double_t th, Double_t ph, Int_t nPhotons, 
		          Double_t tm);

    PhotonsOnPupil *MakeCircle( Double_t ph1, Double_t ph2, Double_t th, 
		           Int_t nPhotons, Double_t tm);

    PhotonsOnPupil *MakeRadius( Double_t ph, Double_t th1, Double_t th2, 
		           Int_t nPhotons, Double_t tm);

    PhotonsOnPupil *MakeTrack( Double_t ph1, Double_t  ph2, Double_t th1, 
		           Double_t th2, Int_t nPhotons,  Double_t  tm);
    
    PhotonsOnPupil *MakeExtSpot( Double_t th1, Double_t  ph1, Double_t radius,
		           Int_t nPhotons,  Double_t  tm);

    PhotonsOnPupil *MakeExtCircle( Double_t ph1, Double_t ph2, Double_t th1,
                   Double_t th2, Double_t radius, Int_t nPhotons, Double_t tm);

    PhotonsOnPupil *MakeExtRadius( Double_t ph, Double_t th1, Double_t  th2,
                   Double_t radius, Int_t nPhotons, Double_t tm);
        
    PhotonsOnPupil *MakeDiffuse( Double_t th1, Double_t th2, Double_t radius,
		           Int_t nPhotons,  Double_t tm);

    PhotonsOnPupil *MakeCFluxDiffuse( Double_t th1, Double_t th2, Double_t radius,
		           Int_t nPhotons, Double_t tm);

    PhotonsOnPupil *MakeShowerTrack(const DetectorGeometry*);
    
    void Reset();
    ListPhotonsOnPupil* fPhOnPupil;
    MCTruth* fTruth;

    ClassDef(TestLightToEuso,0)
};

#endif  /* __TEST_LIGHTTOEUSO_HH__ */

