// $Id: Ground.hh 2672 2006-04-18 12:26:23Z moreggia $
// Author: S.Moreggia   Oct, 27 2003

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: Ground                                                               *
 *  Package: RadiativeTransfer                                               *
 *  Coordinator: Sylvain Moreggia                                            *
 *                                                                           *
 *****************************************************************************/


#ifndef __GROUND_HH__
#define __GROUND_HH__

#include "EsafConfigurable.hh"
#include "EarthVector.hh"
#include "BunchOfPhotons.hh"
#include "EsafMsgSource.hh"

class ListPhotonsInAtmosphere;


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Ground                                                                     //
//                                                                            //
// Interface for ground description                                           //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

class Ground : public EsafConfigurable, public EsafMsgSource {
public:
    
    // ctor
    Ground();

    // dtor
    virtual ~Ground();
        
    virtual EarthVector GetImpact(const EarthVector&, const EarthVector&) const = 0;

    // Get an impact form starting position and direction in atmosphere
    // NB : if no impact,               return (0,0,HUGE)
    //      if track is underground,    return (0,0,-HUGE)
    inline EarthVector GetImpact(const BunchOfPhotons& b) const {return GetImpact(b.GetPos(),b.GetDir());}

    // Phase function of the outgoing light (earth sphericity taken into account)
    // - pos      = position on ground (MES frame)
    // - incomdir = incident direction (MES frame)
    // - outgodir = outgoing position  (MES frame)
    virtual Double_t Outgoing_phase_function(const EarthVector& pos, const EarthVector& incomdir, const EarthVector& outgodir) const = 0;
    
    virtual Double_t GetMaxOutgoing_phase_function() const = 0;
    
    // change incom_dir to the outgoing direction of scattering, sampling Outgoing phase function
    virtual void RandomDir(const EarthVector& pos, EarthVector& incom_dir) const = 0;

    // Get albedo for a given position
    virtual Double_t Albedo(const EarthVector&) const = 0;
    
    // Get ground altitude at the vertical of the given point
    virtual Double_t Altitude(const EarthVector&) const = 0;
    
    // Get ground specular component intensity at given position
    virtual Double_t Specular(const EarthVector&) const = 0;
    
    // to know if position is underground
    Bool_t IsUnderGround(const EarthVector& pos) const;
    
    // Configure
    virtual void Configure() = 0;
    
    // get ready for next event
    virtual void Reset() = 0;
    
    EsafConfigClass(RadiativeTransfer,Ground)

protected:
    
    ClassDef(Ground,0)
}; 

#endif /* __GROUND_HH__ */
