// $Id: PureMCRadiativeTransfer.hh 2681 2006-05-03 10:58:33Z moreggia $
// Author: Sylvain Moreggia   2006/03/09

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: PureMCRadiativeTransfer                                              *
 *  Package: RadiativeTransfer                                               *
 *  Coordinator: S. Moreggia                                                 *
 *                                                                           *
 *****************************************************************************/

#ifndef __PUREMCRADIATIVETRANSFER_HH__
#define __PUREMCRADIATIVETRANSFER_HH__

#include "euso.hh"
#include "RadiativeTransfer.hh"
#include "DetectorGeometry.hh"
#include "EarthVector.hh"
#include "SinglePhotonPropagator.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// PureMCRadiativeTransfer                                                    //
//                                                                            //  
// Pure Monte Carlo propagation of photons in the atmosphere                  //
// MUST NOT BE USED TO SIMULATE SIGNAL ON DETECTOR PUPIL :                    //
// indeed present simulation is dedicated to pure radiative transfer studies  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class PhotonsInAtmosphere;
class ListPhotonsInAtmosphere;
class ListPhotonsOnPupil;
class BunchOfPhotons;
class Ground;
class ClearSkyPropagator;
class InCloudsPropagator;
class SinglePhoton;

class EEvent;

class PureMCRadiativeTransfer : public RadiativeTransfer {
public:
    PureMCRadiativeTransfer();
    virtual ~PureMCRadiativeTransfer();

    // Transfer photons in atmosphere, return a list of photons on Euso pupil
    // NO PHOTONSONPUPIL produced in this mode
    virtual PhotonsOnPupil* Get(PhotonsInAtmosphere*, const DetectorGeometry* dg);
    
    // get ready for next event
    void Reset();
        
    EsafConfigClass(RadiativeTransfer,PureMCRadiativeTransfer)
    

private:

    // Scan fTempList before filling fTotalList, applying relevant cuts
    void ScanTempList();
    
    // convert the bunch of created photons into single photons to be propagated
    Double_t ConvertIntoSingles(const BunchOfPhotons&, Double_t);
           
    ListPhotonsInAtmosphere* fTotalList;  // pointer on the list created by LightSource module
					  // STORE SINGLEPHOTONS AFTER THEIR SCATTERING SIMU
                                          // once inside a photon MUST NOT BE DELETED before conversion into ParentPhoton
					  // _must not be reset nor deleted_ by the present class

    SinglePhotonPropagator* fPropagator;               // propagator for scattering simulation
    vector<SinglePhoton*> fTempList;                   // photons kept here during scattering simu. Then copied in fTotalList

    
    size_t fNbBunch;                      // number of bunches to handle
    Double_t fNbTot;                      // total number of photons created by LightSource
    Double_t fNbSingles;                  // total nb of photons to propagate
    Double_t fMaxPhNb;                    // max nb of photons to be transported --> if nb of photons is larger, simu is splitted in several steps
    Int_t fMaxScatOrder;                  // if photon needs more scattering, it is lost

    virtual Bool_t ClearMemory();
    // release all the mem hold in the buffers
    
    // copy of detector geometry defined in Detector part 
    inline virtual void CopyDetectorGeometry(const DetectorGeometry* dg);

    ClassDef(PureMCRadiativeTransfer,0)
};

//____________________________________________________________________________________________
inline void PureMCRadiativeTransfer::CopyDetectorGeometry(const DetectorGeometry* dg) {
    //
    // set DetectorGeometry for the propagator
    //
    fDetGeom = dg;
    fPropagator->CopyDetectorGeometry(fDetGeom,fDecoupled);
}

#endif  /* __PUREMCRADIATIVETRANSFER_HH__ */

