// $Id: SinglePhotonPropagator.hh 2681 2006-05-03 10:58:33Z moreggia $
// Author:    2005/08/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SinglePhotonPropagator                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __SINGLEPHOTONPROPAGATOR_HH__
#define __SINGLEPHOTONPROPAGATOR_HH__

#include "euso.hh"
#include "DetectorGeometry.hh"
#include "EarthVector.hh"
#include "EsafMsgSource.hh"
#include "BunchPropagator.hh"


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// SinglePhotonPropagator                                                     //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class FastRadiativeProcessesCalculator;
class SinglePhoton;
class Ground;

class SinglePhotonPropagator : public EsafMsgSource {
public:
    
    SinglePhotonPropagator();
    SinglePhotonPropagator(const Ground*,Double_t,Double_t);
    
    virtual ~SinglePhotonPropagator();
    
    // MC based code for scattering simu
    void Go(const vector<SinglePhoton*>&,Int_t scatorder) const;
    
    // Pure MC code for scattering simu : for atmosphere propagation studies ONLY (not to be used with a detector)
    void GoForPureMC(const vector<SinglePhoton*>&) const;
    
    // check if photons would scatter at next order
    // if not it is KEPT, otherwise it is flagged as "LostByCut"
    void CheckIfScatAgain(const vector<SinglePhoton*>&) const;

    // One photon treatment
    void DoOnePhoton(SinglePhoton&,Int_t scatorder) const;
    
    // One photon treatment
    void DoOnePhotonPureMC(SinglePhoton&) const;
    
    // copy of detector geometry defined in Detector part 
    inline void CopyDetectorGeometry(const DetectorGeometry* dg, Bool_t decoupled) {fDetGeom = dg; fDecoupled = decoupled;}
   
    // Copy the RadiativeTransfer ground description object
    inline void CopyGround(const Ground* g) {fGround = g;}
    
    // get radiative calculator
    inline const FastRadiativeProcessesCalculator* GetRadiativeCalculator() const {return fCalcul;}
    
    // reset method
    void Reset();
    
    // set methods
    inline void SetOmegaMax(Double_t omegamax) {fOmegaMax = omegamax;}
    inline void SetMaxPhaseFunction(Double_t maxphfunc) {fMaxPhaseFunction = maxphfunc;}


private:

    FastRadiativeProcessesCalculator* fCalcul;         // interaction length, transmission etc.. WITHOUT Lowtran

    const Ground* fGround;                             // copy of the RadativeTransfer ground description object    
    const DetectorGeometry* fDetGeom;                  // copy of detector geometry built in RT part
                                                       // both _must not be modified_ by the present class
    
    Bool_t fDecoupled;                                 // in case detector geometry is decoupled from RT simu
    Double_t fMaxPhaseFunction;                        // to normalize all the phase function
    Double_t fOmegaMax;                                // factor defining 'reduced solid angle' used in algo
    Double_t fTofCut;                                  // maximum TOF treated by scattering simu
    Double_t fTOA;                                     // usefull if fTOA < Atmosphere::fTOA  --> used to optimize CPU time

    // get impact on next medium
    // returns type of next medium
    Medium GetNextImpact(const EarthVector&,const EarthVector&,EarthVector&,Medium without=NONE) const;
   
    // return detector position
    inline EarthVector EUSO() const {return fDetGeom->GetPos();}
    inline Double_t EusoOmega(const EarthVector& pos) const;

    ClassDef(SinglePhotonPropagator,0)
};

//_______________________________________________________________________________________________________________________
inline Double_t SinglePhotonPropagator::EusoOmega(const EarthVector& pos) const {
    if(fDecoupled) return fDetGeom->MaxSolidAngle(pos);
    else return fDetGeom->SolidAngle(pos);
}


#endif  /* __SINGLEPHOTONPROPAGATOR_HH__ */

