// ESAF : Euso Simulation and Analysis Framework
// $Id: TestGround.hh 2681 2006-05-03 10:58:33Z moreggia $
// S. Moreggia created 27 October 2003

// A test class for ground description
// Albedo = (nb of outgoing photons) / (nb of incoming photons) is used to get nb of outgoing photons
// then they are distributed assuming ground is lambertian, i.e. BRDF is independant of outgoing angles
//                                                          i.e. outgoing phase function follows a cos(theta) law

#ifndef __TESTGROUND_HH__
#define __TESTGROUND_HH__

#include "Ground.hh"
class TF2;

class TestGround : public Ground {
public:
    // ctor
    TestGround();
    //dtor
    virtual ~TestGround();
        
    // Get an impact form starting position and direction in atmosphere
    virtual EarthVector GetImpact(const EarthVector&,const EarthVector&) const;

    // Phase function of the outgoing light (earth sphericity taken into account)
    // - pos      = position on ground (MES frame)
    // - incomdir = incident direction (MES frame)
    // - outgodir = outgoing position  (MES frame)
    virtual Double_t Outgoing_phase_function(const EarthVector& pos, const EarthVector& incomdir, const EarthVector& outgodir) const;
    
    // maximum value of outgoing phase function
    virtual Double_t GetMaxOutgoing_phase_function() const;
        
    // Get albedo for a given position    
    inline virtual Double_t Albedo(const EarthVector&) const {return fAlbedo;}
    
    // Get ground specular component intensity at given position
    inline virtual Double_t Specular(const EarthVector&) const {return fSpec;}
    
    // Get ground altitude at the vertical of the given point
    inline virtual Double_t Altitude(const EarthVector&) const {return fAltitude;}
    
    // Configure data members
    virtual void Configure();
    
    // change incom_dir to the outgoing direction of scattering, sampling Outgoing phase function
    virtual void RandomDir(const EarthVector& pos, EarthVector& incom_dir) const;
    
    // get ready for next event
    void Reset() {}
    
    EsafConfigClass(RadiativeTransfer,TestGround)

private:

    string fType;                     // lambertian or isotropic
    Double_t fSpec;                   // gaussian specular compoent (1 means only specular)
    Double_t fSigma;                  // gaussian width for specular component
    Double_t fWindSpeed;              // determine sigma (if fSigma = -1 is configured)
    Double_t fAltitude;               // from sea level
    Double_t fAlbedo;                 // between 0..1
    TF2* Specular_distrib;            // compute specular 2D-phase function values using theta-phi values
    Double_t NORM[91];                // specular component normalization table (faster than compute it each time)
    Double_t NORMmax;                 // maximum value in NORM table

    ClassDef(TestGround,0)
};

#endif /* __TESTGROUND_HH__ */
