// $Id: SinglePhotonPropagator.cc 2699 2006-05-28 18:19:32Z moreggia $
// Author: S. Moreggia    2005/08/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SinglePhotonPropagator                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// SinglePhotonPropagator
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "SinglePhotonPropagator.hh"
#include "SinglePhoton.hh"
#include "FastRadiativeProcessesCalculator.hh"
#include "Clouds.hh"
#include "Atmosphere.hh"
#include "EsafRandom.hh"
#include "Ground.hh"
#include "EConst.hh"

using namespace sou;
using namespace TMath;
using namespace EConst;

ClassImp(SinglePhotonPropagator)

//_____________________________________________________________________________
SinglePhotonPropagator::SinglePhotonPropagator() : EsafMsgSource() {
    //
    // Constructor
    //
    Msg(EsafMsg::Panic) <<"This constructor SHOULD not be used : ground must be given in argument" << MsgDispatch;

}
//_____________________________________________________________________________
SinglePhotonPropagator::SinglePhotonPropagator(const Ground* g, Double_t maxtof,Double_t TOA) {
    //
    // Constructor which copy RT ground description
    //
    CopyGround(g);
    fCalcul = new FastRadiativeProcessesCalculator();
    fDetGeom = 0;
    fMaxPhaseFunction = 1.;
    fTofCut = maxtof;
    fTOA = TOA;
}

//_____________________________________________________________________________
SinglePhotonPropagator::~SinglePhotonPropagator() {
    //
    // Destructor
    //
    SafeDelete(fCalcul);
}

//_____________________________________________________________________________
void SinglePhotonPropagator::Go(const vector<SinglePhoton*>& list,Int_t scatorder) const {
    //
    // MC code for photon scattering simulation
    //

    size_t nbPh = list.size();
    Msg(EsafMsg::Info) <<"Nb of SinglePhoton to propagate = " << nbPh << MsgDispatch;
    // for future info dumping
    Bool_t next(0),subnext(0);
    Double_t nbTracked(0), fraction(0), left(0),right(10),subleft(0),subright(2.5);
    Msg(EsafMsg::Info) << "Scattering Simulation: 0%" << MsgFlush;
    
    // loop over the list
    SinglePhoton* s = 0;
    for(size_t i=0; i<nbPh; i++) {
        s = list[i];
	
	DoOnePhoton(*s,scatorder);
	nbTracked++;

	// dump info
        fraction = 100*nbTracked/Double_t(nbPh);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
}

//_____________________________________________________________________________
void SinglePhotonPropagator::GoForPureMC(const vector<SinglePhoton*>& list) const {
    //
    // MC code for photon scattering simulation
    //

    size_t nbPh = list.size();
    Msg(EsafMsg::Info) <<"Nb of SinglePhoton to propagate = " << nbPh << MsgDispatch;
    // for future info dumping
    Bool_t next(0),subnext(0);
    Double_t nbTracked(0), fraction(0), left(0),right(10),subleft(0),subright(2.5);
    Msg(EsafMsg::Info) << "Scattering Simulation: 0%" << MsgFlush;
    
    // loop over the list
    SinglePhoton* s = 0;
    Int_t lost = 0;
    for(size_t i=0; i<nbPh; i++) {
        s = list[i];
	
	DoOnePhotonPureMC(*s);
	PhotonStatus status = s->Status();
	if(status == LostByCut) Msg(EsafMsg::Info) << "Photon is lost by cut ("<<++lost<<")" << MsgDispatch;
	else if(!(status == OutAtmo || status == ScatAbsorb ||  status == LostByCut)) {
	    Msg(EsafMsg::Warning) << "Photon status MUST be absorbed or outgoing, here it is = "<< status<<". Status set to LostByCut"<< MsgDispatch;
	    s->SetStatus(LostByCut);
	}
	    
	nbTracked++;

	// dump info
        fraction = 100*nbTracked/Double_t(nbPh);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
}

//_____________________________________________________________________________
void SinglePhotonPropagator::CheckIfScatAgain(const vector<SinglePhoton*>& list) const {
    //
    // check if photons "successfully scattered until present order" would scatter at next order
    // if not they are KEPT, otherwise they are flagged as "LostByCut"
    //

    size_t nbPh = list.size();
    Msg(EsafMsg::Info) <<"Nb of SinglePhoton to CHECK at next order = " << nbPh << MsgDispatch;
    // for future info dumping
    Bool_t next(0),subnext(0);
    Double_t nbTracked(0), fraction(0), left(0),right(10),subleft(0),subright(2.5);
    Msg(EsafMsg::Info) << "Scattering Simulation: 0%" << MsgFlush;
    
    // loop over the list
    SinglePhoton* ss = 0;
    for(size_t i=0; i<nbPh; i++) {
        ss = list[i];
	if(ss->Status() <= 4) {
	    SinglePhoton s(*ss);
	    DoOnePhoton(s,1);
	    if( s.Status() <= 4 ) ss->SetStatus(LostByCut);
	    else if(s.Status() == LostByCut) Msg(EsafMsg::Warning) << "<CheckIfScatAgain()> LostByCut should not occur here" << MsgDispatch;
	}

	nbTracked++;
	// dump info
        fraction = 100*nbTracked/Double_t(nbPh);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
}

//_____________________________________________________________________________
void SinglePhotonPropagator::DoOnePhoton(SinglePhoton& s,Int_t scatorder) const {
    //
    // MC propagation : handle rayleigh, clouds and ground (aerosols to come) for fluo and cerenkov
    // Each scattering order simulation is independent from another
    // NB : Ozone treatment optimized --> if wl > 350nm, Ozone not calculated
    //

    const Atmosphere* atmo = Atmosphere::Get();
    const Clouds* clouds = atmo->GetClouds();
    const Aerosol* aerosol = atmo->GetAerosol();
    Int_t nbInter = 0;
    Double_t TOF = 0.;
    Int_t invert_status(100);       // for Atmosphere::InvertGrammage() method
    Int_t Ozinvert_status(100);     // for Atmosphere::InvertOzoneAmount() method
    EarthVector dir(1.);
    Int_t count_debug(0); // to stop simu if infinite loop
    Int_t MediumInter = 0;  // 0 : air, 1 : clouds, 2 : aerosols, 3 : ozone
    
    static Int_t Lost_by_optimization = 0;
    static Int_t Total_photon_simulated = 0;
    
    // init
    TRandom* rndm = EsafRandom::Get();
    EarthVector scatpos_rayl(0.,0,HUGE), scatpos_clouds(0,0,HUGE), scatpos_aerosol(0,0,HUGE), absorbpos_ozone(0,0,HUGE);
    EarthVector nextimpact(0,0,HUGE), nextimpact_tempcloud(0,0,HUGE), nextimpact_tempaerosol(0,0,HUGE);
    Medium nextmedium = CLEARSKY;
    Medium nextmedium_tempcloud = CLEARSKY;
    Medium nextmedium_tempaerosol = CLEARSKY;
    Bool_t AeroScattered = kTRUE;  // true means scattering, false means absorption

    // propagation stops when :
    // - photon is absorbed
    // - photon is out of atmosphere
    // - relevant scattering order is reached
    while(true) {
        
	// flags and cuts
	// if weird situations appear
	if(nbInter == scatorder) Msg(EsafMsg::Warning) <<"<DoOnePhoton> Loop should have stopped before"<<MsgDispatch;
        if(count_debug++ > 100) {
            Msg(EsafMsg::Warning) <<"<DoOnePhoton> infinite loop broken by hand "<<MsgDispatch;
            break;
        }
	// cut in time of flight defined here
	if(TOF > fTofCut) {
	    s.SetStatus(LostByCut);
#ifdef DEBUG
            //Msg(EsafMsg::Debug) <<"<DoOnePhoton> TOF cut reached : photon is lost"<<MsgDispatch;
#endif
	    break;
	}
	
	
		
	// if photon reached ground
	if(nextmedium == GROUND) {
	    s.AddInteraction();
	    s.AddHistory(Reflected);
	    nbInter++;
	    // 1. if not absorbed at ground
	    if(rndm->Rndm() < fGround->Albedo(s.Pos())) {
		
	        // 1.1. if photon has come to its order of scattering
	        // look if it points toward detector and stop its simu
	        if(scatorder == nbInter) {
		    if(rndm->Rndm() < (fGround->Outgoing_phase_function(s.Pos(),s.Dir(),(EUSO() - s.Pos()).Unit()) * EusoOmega(s.Pos()) / (fMaxPhaseFunction*fOmegaMax)) ) {
		        s.SetStatus(Reflected);
		        dir = s.Dir();
		        fGround->RandomDir(s.Pos(),dir);
		        s.SetDir(dir);       // for MScattAnalysis
		        s.SetMScatDir(dir);  // for MScattAnalysis
		    }
		    else s.SetStatus(LostByCut);
		    break;
		}
		
		// 1.2. else get a new direction and go to next step
		else {
		    dir = s.Dir();
		    fGround->RandomDir(s.Pos(),dir);
		    s.SetDir(dir);
		    s.SetStatus(LostByCut);
		    nextmedium = CLEARSKY;
		}
	    }
	    
	    // 2. else photon is absorbed at ground
	    else {
	        s.SetAbsorbed();
		s.SetStatus(ScatAbsorb);
		break;
	    }
	}
	
	
	// if TOA reached
	else if(nextmedium == TOA) {
	    s.AddHistory(OutAtmo);
	    s.SetStatus(OutAtmo);
	    break;
	}
	
	else if(nextmedium == NONE) Msg(EsafMsg::Panic) <<"<DoOnePhoton> next medium is NONE -> SHOULD NOT" <<MsgDispatch;



/////////////////////  if photons is in the atmosphere  /////////////////////////
	// INITS
	nextimpact_tempcloud.SetXYZ(0,0,HUGE);
	nextimpact_tempaerosol.SetXYZ(0,0,HUGE);
	nextmedium_tempcloud = nextmedium;
	nextmedium_tempaerosol = nextmedium;
	
		
	// 1. CLOUDS treatment
	//    ONLY SINGLE LAYER clouds handled sor far
	//     - single scattering albedo ~ 1  ->  no absorption by water droplets or ice crystals (cf. Liou)
	//     - scattering cross-section is ~ lambda independent in UV band (cf. Liou)
	if(clouds->IsInClouds(s.Pos()) || (nextmedium == CLOUDY) ) {
	    // 1.1. get next medium impact (clouds excluded)
	    nextmedium_tempcloud = GetNextImpact(s.Pos(),s.Dir(),nextimpact_tempcloud,CLOUDY);
	    
	    // 1.2. get random clouds scattering position,
	    // check it is within clouds.             If not  ->  scatpos_clouds = (0,0,HUGE)
	    // check it is above ground or below TOA. If not  ->  scatpos_clouds = (0,0,HUGE)
	    scatpos_clouds = clouds->RandomScatPos(s.Pos(),s.Dir());
	    if(scatpos_clouds.Z() == HUGE)                  scatpos_clouds.SetXYZ(0,0,HUGE);  // occurs if fOD=0
	    else if(!clouds->IsInClouds(scatpos_clouds))    scatpos_clouds.SetXYZ(0,0,HUGE);
	}
	    
	// 1. AEROSOL treatment
	if(aerosol->IsInAerosol(s.Pos()) || (nextmedium == AEROSOLS) ) {
	    // 1.1. get next medium impact (aerosol excluded)
	    nextmedium_tempaerosol = GetNextImpact(s.Pos(),s.Dir(),nextimpact_tempaerosol,AEROSOLS);
	    
	    // 1.2. get random aerosol interaction position (absorption or scattering)
	    AeroScattered = aerosol->RandomScatPos(s.Pos(),s.Dir(),s.Wl(),scatpos_aerosol);
	}
	    
	
	// 2. Chose the nearest next impact
	Double_t dist_cloud = (nextimpact_tempcloud - s.Pos()).Mag();    // can be zero if s.Pos() is in clouds
	Double_t dist_aero  = (nextimpact_tempaerosol - s.Pos()).Mag();  // can be zero if s.Pos() is in aerosol
	// 2.1. if s.Pos() not in clouds and aerosol in the same time
	if((dist_cloud + dist_aero) > 0) {
            if(dist_cloud <= dist_aero) {
	        nextimpact = nextimpact_tempcloud;
	        nextmedium = nextmedium_tempcloud;
	    }
	    else  {
	        nextimpact = nextimpact_tempaerosol;
	        nextmedium = nextmedium_tempaerosol;
	    }
	}
	// 2.2. if s.Pos() is in clouds AND aerosol : next impact still needs to be determined
	else nextimpact.SetXYZ(0,0,HUGE);
	    
	    
	// 3. CLEARSKY treatment : rayleigh scattering and ozone absorption
	// RAYLEIGH
	// 3.1. if no medium encountered (no clouds, no aerosols etc..) : get next medium type, and impact position upon it
	if(nextimpact.Z() == HUGE) nextmedium = GetNextImpact(s.Pos(),s.Dir(),nextimpact);
	// 3.2. get random rayleigh scattering grammage
	Double_t grammage_rayl = fCalcul->RandomGrammage_RaylScat(s.Wl());
	// OZONE
	// 3.3. get position of interaction with ozone (optimized, if wl too high, not calculated)
	if(s.Wl() < 350*nm) Ozinvert_status = fCalcul->RandomOzoneAbsorbPosition(s.Pos(),s.Dir(),s.Wl(),absorbpos_ozone,fTOA);
	
	
	// 4. check if any interaction position occur before nextimpact. If not :
	//    rayleigh ->  grammage_rayl = -1
	//    clouds   ->  scatpos_clouds.Z = HUGE
	//    aerosol  ->  scatpos_aero.Z = HUGE
	//    ozone    ->  absorbpos_ozone.Z = HUGE
	// 4.1. rayleigh
	if(atmo->Grammage(s.Pos(),nextimpact) < grammage_rayl)                    grammage_rayl = -1;
	// 4.2. clouds
	if(fGround->IsUnderGround(scatpos_clouds))                                scatpos_clouds.SetXYZ(0,0,HUGE);
	else if((scatpos_clouds - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag())  scatpos_clouds.SetXYZ(0,0,HUGE);
	// 4.2. aerosol
	if(fGround->IsUnderGround(scatpos_aerosol))                               scatpos_aerosol.SetXYZ(0,0,HUGE);
	else if((scatpos_aerosol - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag()) scatpos_aerosol.SetXYZ(0,0,HUGE);
	// 4.3. ozone
	if(Ozinvert_status > 0)                                                   absorbpos_ozone.SetXYZ(0,0,HUGE);
	else if(fGround->IsUnderGround(absorbpos_ozone))                          absorbpos_ozone.SetXYZ(0,0,HUGE);
	else if((absorbpos_ozone - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag()) absorbpos_ozone.SetXYZ(0,0,HUGE);


	// 5. if next medium reached before any scattering,
	//    position is set to impact and goes to next step
	if((scatpos_clouds.Z() == HUGE) && (grammage_rayl == -1.) && (scatpos_aerosol.Z() == HUGE) && (absorbpos_ozone.Z() == HUGE)) {
	    TOF += (nextimpact - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(nextimpact - s.Pos());
	    continue;
	}
	    
	    
	    
	// ELSE, PHOTON INTERACTS WITH ATMOSPHERE
	s.AddInteraction();
	nbInter++;

	// 6. chose the nearest position of interaction
	MediumInter = 0;
	EarthVector Closest(0,0,HUGE);
	// 6.1. clouds vs/ aerosol --> closest_1 found
	if(scatpos_clouds.Z() == HUGE && scatpos_aerosol.Z() == HUGE) MediumInter = 0;
	else if((scatpos_clouds - s.Pos()).Mag() < (scatpos_aerosol - s.Pos()).Mag()) {
	    Closest = scatpos_clouds;
	    MediumInter = 1;
	}
	else {
	    Closest = scatpos_aerosol;
	    MediumInter = 2;
	}
	// 6.2. ozone vs/ closest_1  --> closest_2 found
	if(MediumInter == 0) {
	    if(absorbpos_ozone.Z() != HUGE) {
	        MediumInter = 3;
		Closest = absorbpos_ozone;
	    }
	}
	else if((absorbpos_ozone - s.Pos()).Mag() < (Closest - s.Pos()).Mag()) {
	    MediumInter = 3;
            Closest = absorbpos_ozone;
	}
	// 6.3. rayleigh vs/ closest_2
	if(MediumInter > 0) {
	    if(Closest.Z() == HUGE) MediumInter = 0;
	    else if((atmo->Grammage(s.Pos(),Closest) > grammage_rayl) && (grammage_rayl =! -1.))
	        MediumInter = 0;
	}
	


        // 7. the corresponding type of interaction is applied
	// 7.1. clouds scattering	    
	if(MediumInter == 1) {
	    TOF += (scatpos_clouds - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_clouds - s.Pos());
	    s.AddHistory(CloudScat);
	    // 7.1.1. if photon has come to its order of scattering
	    // look if it points toward detector and stop its simu
	    if(scatorder == nbInter) {
		Double_t phfunc = clouds->TruncatedPhaseFunction(s.Dir(),(EUSO() - s.Pos()).Unit());
		Total_photon_simulated++;
		if(phfunc == 0.) {
		    Lost_by_optimization++;
		    Msg(EsafMsg::Info) <<"<DoOnePhoton> Photons lost via TRUNCATED phase function ("<<Float_t(Lost_by_optimization) / Float_t(Total_photon_simulated)<<" \% photons)"<<MsgDispatch;
		    s.SetStatus(LostByCut);
		}
	        else if(rndm->Rndm() < (phfunc * EusoOmega(s.Pos()) / (fMaxPhaseFunction*fOmegaMax)) ) {
	            s.SetStatus(CloudScat);
		    dir = s.Dir();
		    clouds->RandomDir(dir);
		    s.SetDir(dir);       // for MScattAnalysis
		    s.SetMScatDir(dir);  // for MScattAnalysis
		}
		else s.SetStatus(LostByCut);
		break;
	    }
	    // 7.1.2. else chose a new direction, and goes to next step
	    else {
		dir = s.Dir();
		clouds->RandomDir(dir);
		s.SetDir(dir);
		s.SetStatus(LostByCut);
		nextmedium = CLOUDY;
	    }
	}


	// 7.2 aerosol interaction (scat or absorb)
	else if(MediumInter == 2) {
	    TOF += (scatpos_aerosol - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_aerosol - s.Pos());
	    // 7.2.1. if photon is absorbed
	    if(!AeroScattered) {
	        s.SetAbsorbed();
		s.SetStatus(ScatAbsorb);
		break;
	    }
	    // 7.2.2. else photon is scattered
	    s.AddHistory(AeroScat);
	    // 7.2.2.1. if photon has come to its order of scattering
	    // look if it points toward detector and stop its simu
	    if(scatorder == nbInter) {
		Double_t phfunc = aerosol->PhaseFunction(s.Dir(),(EUSO() - s.Pos()).Unit(),s.Wl());
	        if(rndm->Rndm() < (phfunc * EusoOmega(s.Pos()) / (fMaxPhaseFunction*fOmegaMax)) ) {
	            s.SetStatus(AeroScat);
		    dir = s.Dir();
		    aerosol->RandomDir(dir,s.Wl());
		    s.SetDir(dir);       // for MScattAnalysis
		    s.SetMScatDir(dir);  // for MScattAnalysis
		}
		else s.SetStatus(LostByCut);
		break;
	    }
	    // 7.2.2.2. else chose a new direction, and goes to next step
	    else {
		dir = s.Dir();
		aerosol->RandomDir(dir,s.Wl());
		s.SetDir(dir);
		s.SetStatus(LostByCut);
		nextmedium = AEROSOLS;
	    }
	}


	// 7.3. rayleigh scattering
	else if(MediumInter == 0) {
	    if(grammage_rayl > -1.) invert_status = atmo->InvertGrammage(s.Pos(),s.Dir(),grammage_rayl,scatpos_rayl,fTOA,fTofCut - TOF);
	    else Msg(EsafMsg::Warning) <<"<DoOnePhoton> This situation SHOULD NOT occur"<<MsgDispatch;
	    if(invert_status > 1) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhoton> Atmo::InvertGrammage() returns status = " <<invert_status<<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
            else if(invert_status == 1) {
	        s.SetStatus(LostByCut);
		break;
	    }
            else if(invert_status == -1) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhoton> Atmo::InvertGrammage() returns status = -1. It SHOULD NOT OCCUR" <<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
	    TOF += (scatpos_rayl - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_rayl - s.Pos());
	    s.AddHistory(RaylScat);
	    // 7.3.1. if photon has come to its order of scattering
	    // look if it points toward detector and stop its simu
	    if(scatorder == nbInter) {
	        if(rndm->Rndm() < (fCalcul->RayleighPhaseFunction(s.Dir(),(EUSO() - s.Pos()).Unit()) * EusoOmega(s.Pos()) / (fMaxPhaseFunction*fOmegaMax)) ) {
	            s.SetStatus(RaylScat);
		    dir = s.Dir();
		    fCalcul->RandomDir("rayleigh",dir);
		    s.SetDir(dir);       // for MScattAnalysis
		    s.SetMScatDir(dir);  // for MScattAnalysis
		}
		else s.SetStatus(LostByCut);
		break;
	    }
	    // 7.3.2. else chose a new direction, and goes to next step
	    else {
		dir = s.Dir();
		fCalcul->RandomDir("rayleigh",dir);
		s.SetDir(dir);
		s.SetStatus(LostByCut);
		nextmedium = CLEARSKY;
	    }
	}
	
	// 7.4. ozone absorption
	else if(MediumInter == 3) {
	    if(Ozinvert_status != 0) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhoton> Atmosphere::InvertOzoneAmount() returns status = " <<Ozinvert_status<<MsgDispatch;
	        Msg(EsafMsg::Warning) <<"<DoOnePhoton> It SHOULD NOT OCCUR" <<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
	    TOF += (absorbpos_ozone - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(absorbpos_ozone - s.Pos());
	    s.SetAbsorbed();
            s.SetStatus(ScatAbsorb);
	    break;
	}
	
	// 7.5. PB !
	else {
	    Msg(EsafMsg::Warning) <<"<DoOnePhoton> Choice of scattering process FAILED : it should not"<<MsgDispatch;
	    return;
	}	
    }
    
}

//_____________________________________________________________________________
void SinglePhotonPropagator::DoOnePhotonPureMC(SinglePhoton& s) const {
    //
    // MC propagation : handle rayleigh, clouds and ground (aerosols to come) for fluo and cerenkov
    // Photons is propagated until it goes out atmosphere of it is absorbed (by ground only so far)
    // NB : Ozone treatment optimized --> if wl > 350nm, Ozone not calculated
    //


    const Atmosphere* atmo = Atmosphere::Get();
    const Clouds* clouds = atmo->GetClouds();
    const Aerosol* aerosol = atmo->GetAerosol();
    Int_t nbInter = 0;
    Double_t TOF = 0.;
    Int_t invert_status(100);     // for Atmosphere::InvertGrammage() method
    Int_t Ozinvert_status(100);   // for Atmosphere::InvertOzoneAmount() method
    EarthVector dir(1.);
    Int_t count_debug(0); // to stop simu if infinite loop
    Int_t MediumInter = 0;  // 0 : air, 1 : clouds, 2 : aerosols, 3 : ozone
    
    // init
    TRandom* rndm = EsafRandom::Get();
    EarthVector scatpos_rayl(0.,0,HUGE), scatpos_clouds(0,0,HUGE), scatpos_aerosol(0,0,HUGE), absorbpos_ozone(0,0,HUGE);
    EarthVector nextimpact(0,0,HUGE), nextimpact_tempcloud(0,0,HUGE), nextimpact_tempaerosol(0,0,HUGE);
    Medium nextmedium = CLEARSKY;
    Medium nextmedium_tempcloud = CLEARSKY;
    Medium nextmedium_tempaerosol = CLEARSKY;
    Bool_t AeroScattered = kTRUE;  // true means scattering, false means absorption

    // propagation stops when :
    // - photon is absorbed
    // - photon is out of atmosphere
    while(true) {
        
	// flags and cuts
	// if weird situations appear
        if(count_debug++ > 1000000) {
            Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> infinite loop broken by hand "<<MsgDispatch;
            break;
        }
	// cut in time of flight defined here
	if(TOF > fTofCut) {
	    s.SetStatus(LostByCut);
#ifdef DEBUG
            //Msg(EsafMsg::Debug) <<"<DoOnePhotonPureMC> TOF cut reached : photon is lost"<<MsgDispatch;
#endif
	    break;
	}
	
	
		
	// if photon reached ground
	if(nextmedium == GROUND) {
	    s.AddInteraction();
	    s.AddHistory(Reflected);
	    nbInter++;
	    // 1. if not absorbed at ground
	    if(rndm->Rndm() < fGround->Albedo(s.Pos())) {
	        // get a new direction and go to next step
		dir = s.Dir();
		fGround->RandomDir(s.Pos(),dir);
		s.SetDir(dir);
	        s.SetPosInAtmo(s.Pos());
		s.SetStatus(LostByCut);
		nextmedium = CLEARSKY;
	    }
	    
	    // 2. else photon is absorbed at ground
	    else {
	        s.SetAbsorbed();
		s.SetMScatDir(s.Dir());
		s.SetStatus(ScatAbsorb);
		break;
	    }
	}
	
	
	// if TOA reached
	else if(nextmedium == TOA) {
	    s.AddHistory(OutAtmo);
	    s.SetMScatDir(s.Dir());
	    s.SetStatus(OutAtmo);
	    break;
	}
	
	else if(nextmedium == NONE) Msg(EsafMsg::Panic) <<"DoOnePhotonPureMC next medium is NONE -> SHOULD NOT" <<MsgDispatch;



/////////////////////  if photons is in the atmosphere  /////////////////////////
	// INITS
	nextimpact_tempcloud.SetXYZ(0,0,HUGE);
	nextimpact_tempaerosol.SetXYZ(0,0,HUGE);
	nextmedium_tempcloud = nextmedium;
	nextmedium_tempaerosol = nextmedium;
	
		
	// 1. CLOUDS treatment
	//    ONLY SINGLE LAYER clouds handled sor far
	//     - single scattering albedo ~ 1  ->  no absorption by water droplets or ice crystals (cf. Liou)
	//     - scattering cross-section is ~ lambda independent in UV band (cf. Liou)
	if(clouds->IsInClouds(s.Pos()) || (nextmedium == CLOUDY) ) {
	    // 1.1. get next medium impact (clouds excluded)
	    nextmedium_tempcloud = GetNextImpact(s.Pos(),s.Dir(),nextimpact_tempcloud,CLOUDY);
	    
	    // 1.2. get random clouds scattering position,
	    // check it is within clouds.             If not  ->  scatpos_clouds = (0,0,HUGE)
	    // check it is above ground or below TOA. If not  ->  scatpos_clouds = (0,0,HUGE)
	    scatpos_clouds = clouds->RandomScatPos(s.Pos(),s.Dir());
	    if(scatpos_clouds.Z() == HUGE)                  scatpos_clouds.SetXYZ(0,0,HUGE);  // occurs if fOD=0
	    else if(!clouds->IsInClouds(scatpos_clouds))    scatpos_clouds.SetXYZ(0,0,HUGE);
	}
	    
	// 1. AEROSOL treatment
	if(aerosol->IsInAerosol(s.Pos()) || (nextmedium == AEROSOLS) ) {
	    // 1.1. get next medium impact (aerosol excluded)
	    nextmedium_tempaerosol = GetNextImpact(s.Pos(),s.Dir(),nextimpact_tempaerosol,AEROSOLS);
	    
	    // 1.2. get random aerosol interaction position (absorption or scattering)
	    AeroScattered = aerosol->RandomScatPos(s.Pos(),s.Dir(),s.Wl(),scatpos_aerosol);
	}
	    
	
	// 2. Chose the nearest next impact
	Double_t dist_cloud = (nextimpact_tempcloud - s.Pos()).Mag();    // can be zero if s.Pos() is in clouds
	Double_t dist_aero  = (nextimpact_tempaerosol - s.Pos()).Mag();  // can be zero if s.Pos() is in aerosol
	// 2.1. if s.Pos() not in clouds and aerosol in the same time
	if((dist_cloud + dist_aero) > 0) {
            if(dist_cloud <= dist_aero) {
	        nextimpact = nextimpact_tempcloud;
	        nextmedium = nextmedium_tempcloud;
	    }
	    else  {
	        nextimpact = nextimpact_tempaerosol;
	        nextmedium = nextmedium_tempaerosol;
	    }
	}
	// 2.2. if s.Pos() is in clouds AND aerosol : next impact still needs to be determined
	else nextimpact.SetXYZ(0,0,HUGE);
	    
	    
	// 3. CLEARSKY treatment : rayleigh scattering and ozone absorption
	// RAYLEIGH
	// 3.1. if no medium encountered (no clouds, no aerosols etc..) : get next medium type, and impact position upon it
	if(nextimpact.Z() == HUGE) nextmedium = GetNextImpact(s.Pos(),s.Dir(),nextimpact);
	// 3.2. get random rayleigh scattering grammage
	Double_t grammage_rayl = fCalcul->RandomGrammage_RaylScat(s.Wl());
	// OZONE
	// 3.3. get position of interaction with ozone
	if(s.Wl() < 350*nm) Ozinvert_status = fCalcul->RandomOzoneAbsorbPosition(s.Pos(),s.Dir(),s.Wl(),absorbpos_ozone,fTOA);
	
	
	// 4. check if any scattering position occur before nextimpact. If not :
	//    rayleigh ->  grammage_rayl = -1
	//    clouds   ->  scatpos_clouds.Z = HUGE
	//    aerosol  ->  scatpos_aero.Z = HUGE
	//    ozone    ->  absorbpos_ozone.Z = HUGE
	// 4.1. rayleigh
	if(atmo->Grammage(s.Pos(),nextimpact) < grammage_rayl)                    grammage_rayl = -1;
	// 4.2. clouds
	if(fGround->IsUnderGround(scatpos_clouds))                                scatpos_clouds.SetXYZ(0,0,HUGE);
	else if((scatpos_clouds - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag())  scatpos_clouds.SetXYZ(0,0,HUGE);
	// 4.2. aerosol
	if(fGround->IsUnderGround(scatpos_aerosol))                               scatpos_aerosol.SetXYZ(0,0,HUGE);
	else if((scatpos_aerosol - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag()) scatpos_aerosol.SetXYZ(0,0,HUGE);
	// 4.3. ozone
	if(Ozinvert_status > 0)                                                   absorbpos_ozone.SetXYZ(0,0,HUGE);
	else if(fGround->IsUnderGround(absorbpos_ozone))                          absorbpos_ozone.SetXYZ(0,0,HUGE);
	else if((absorbpos_ozone - s.Pos()).Mag() > (nextimpact - s.Pos()).Mag()) absorbpos_ozone.SetXYZ(0,0,HUGE);


	// 5. if next medium reached before any scattering,
	//    position is set to impact and goes to next step
	if((scatpos_clouds.Z() == HUGE) && (grammage_rayl == -1) && (scatpos_aerosol.Z() == HUGE) && (absorbpos_ozone.Z() == HUGE)) {
	    TOF += (nextimpact - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(nextimpact - s.Pos());
	    continue;
	}
	    
	    
	    
	// ELSE, PHOTON INTERACTS WITH ATMOSPHERE
	s.AddInteraction();
	nbInter++;

	// 6. chose the nearest position of interaction
	MediumInter = 0;
	EarthVector Closest(0,0,HUGE);
	// 6.1. clouds vs/ aerosol --> closest_1 found
	if(scatpos_clouds.Z() == HUGE && scatpos_aerosol.Z() == HUGE) MediumInter = 0;
	else if((scatpos_clouds - s.Pos()).Mag() < (scatpos_aerosol - s.Pos()).Mag()) {
	    Closest = scatpos_clouds;
	    MediumInter = 1;
	}
	else {
	    Closest = scatpos_aerosol;
	    MediumInter = 2;
	}
	// 6.2. ozone vs/ closest_1  --> closest_2 found
	if(MediumInter == 0) {
	    if(absorbpos_ozone.Z() != HUGE) {
	        MediumInter = 3;
		Closest = absorbpos_ozone;
	    }
	}
	else if((absorbpos_ozone - s.Pos()).Mag() < (Closest - s.Pos()).Mag()) {
	    MediumInter = 3;
            Closest = absorbpos_ozone;
	}
	// 6.3. rayleigh vs/ closest_2
	if(MediumInter > 0) {
	    if(Closest.Z() == HUGE) MediumInter = 0;
	    else if((atmo->Grammage(s.Pos(),Closest) > grammage_rayl) && (grammage_rayl =! -1.))
	        MediumInter = 0;
	}



        // 7. the corresponding type of interaction is applied
	// 7.1. clouds scattering	    
	if(MediumInter == 1) {
	    TOF += (scatpos_clouds - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_clouds - s.Pos());
	    s.AddHistory(CloudScat);
	    // chose a new direction, and goes to next step
	    dir = s.Dir();
	    clouds->RandomDir(dir);
	    s.SetDir(dir);
	    s.SetStatus(LostByCut);
	    nextmedium = CLOUDY;
	}


	// 7.2 aerosol interaction (scat or absorb)
	else if(MediumInter == 2) {
	    TOF += (scatpos_aerosol - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_aerosol - s.Pos());
	    // 7.2.1. if photon is absorbed
	    if(!AeroScattered) {
	        s.SetAbsorbed();
		s.SetStatus(ScatAbsorb);
		break;
	    }
	    // 7.2.2. else photon is scattered
	    s.AddHistory(AeroScat);
	    // chose a new direction, and goes to next step
	    dir = s.Dir();
	    aerosol->RandomDir(dir,s.Wl());
	    s.SetDir(dir);
	    s.SetStatus(LostByCut);
	    nextmedium = AEROSOLS;
	}


	// 7.3. rayleigh scattering
	else if(MediumInter == 0) {
	    if(grammage_rayl > -1.) invert_status = atmo->InvertGrammage(s.Pos(),s.Dir(),grammage_rayl,scatpos_rayl,fTOA,fTofCut - TOF);
	    else Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> This situation SHOULD NOT occur"<<MsgDispatch;
	    if(invert_status > 1) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> Atmo::InvertGrammage() returns status = " <<invert_status<<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
            else if(invert_status == 1) {
	        s.SetStatus(LostByCut);
		break;
	    }
            else if(invert_status == -1) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> Atmo::InvertGrammage() returns status = -1. It SHOULD NOT OCCUR" <<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
	    TOF += (scatpos_rayl - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(scatpos_rayl - s.Pos());
	    s.AddHistory(RaylScat);
	    // chose a new direction, and goes to next step
	    dir = s.Dir();
	    fCalcul->RandomDir("rayleigh",dir);
	    s.SetDir(dir);
	    s.SetPosInAtmo(s.Pos());
	    s.SetStatus(LostByCut);
	    nextmedium = CLEARSKY;
	}
	
	// 7.4. ozone absorption
	else if(MediumInter == 3) {
	    if(Ozinvert_status != 0) {
	        Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> Atmosphere::InvertOzoneAmount() returns status = " <<Ozinvert_status<<MsgDispatch;
	        Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> It SHOULD NOT OCCUR" <<MsgDispatch;
	        s.SetStatus(LostByCut);
		break;
	    }
	    TOF += (absorbpos_ozone - s.Pos()).Mag() / Clight();
	    s.AddToPosTof(absorbpos_ozone - s.Pos());
	    s.SetAbsorbed();
            s.SetStatus(ScatAbsorb);
	    break;
	}
	
	// 7.5. PB !
	else {
	    Msg(EsafMsg::Warning) <<"<DoOnePhotonPureMC> Choice of scattering process FAILED : it should not"<<MsgDispatch;
	    return;
	}	
    }
    
}

//_____________________________________________________________________________
Medium SinglePhotonPropagator::GetNextImpact(const EarthVector& pos,const EarthVector& dir,EarthVector& res, Medium without_medium) const {
    //
    // Calculate the impact upon the next medium according to the given track
    // Return type of the next encountered medium
    // OUT OF TOA is handled
    //
    // - If pos is underground, NONE medium is returned and impact set to (0,0,-HUGE)
    // - without_medium : allow to find next impact without considering the specified medium
    //
    // SHOULD NEVER RETURN Z=HUGE, cause impact always exists !!
    //
    
    Medium rtn = GROUND;
    EarthVector gd_imp, cloud_imp, aero_imp;
    
    // 1. get impact on ground
    gd_imp = fGround->GetImpact(pos,dir);
    res = gd_imp;
    // if pos is underground
    if(gd_imp.Z() == -HUGE) return NONE;
  
    // 2. get impact on clouds, then make appropriate changes
    cloud_imp = Atmosphere::Get()->GetClouds()->GetCloudImpact(pos,dir);
    if( ((cloud_imp - pos).Mag() < (res - pos).Mag()) && without_medium != CLOUDY ) {
	res = cloud_imp;
	rtn = CLOUDY;
    }
    
    // 3. get impact on aerosol layer, then make appropriate changes
    aero_imp = Atmosphere::Get()->GetAerosol()->GetImpact(pos,dir);
    if( ((aero_imp - pos).Mag() < (res - pos).Mag()) && without_medium != AEROSOLS ) {
	res = aero_imp;
	rtn = AEROSOLS;
    }

    // 4. if no impact, means top of atmosphere reached
    if(res.Z() == HUGE) {
        rtn = TOA;
	res = Atmosphere::Get()->ImpactAtTOA(pos,dir,fTOA);
    }
    
      
    return rtn;
}

//_____________________________________________________________________________
void SinglePhotonPropagator::Reset() {
    //
    // ready for next event
    //
    
    fCalcul->Reset();
}



