// main.cc
// $Id: simu_main.cc 2804 2008-10-09 12:10:06Z biktem $
// Description:
// Main program for ESAF: Euso Simulation and Analysis Framework
// It can be used both in batch mode and in interactive mode through a GUI
// Based on root (http::/root.cern.ch/)
// M. Pallavicini - created 29-12-2001
//

#include <iostream>
#include <exception>
#include <TROOT.h>
#include "SimuApplication.hh"
#include "EusoMainFrame.hh"
#include "EsafMsg.hh"

// the root object
TROOT root("ESAF-Simu","ESAF-Simu");

// main program
int main(int argc, char** argv) {

    try {

        // build a root application for Euso
        SimuApplication* App = new SimuApplication(argc,argv);

        // batch mode ( no GUI ) follows a different path
        if ( gROOT->IsBatch()) {
            cout << "ESAF running in batch mode\n";
            App->DoAll();
            delete App;

            cout << "Simu completed." << endl;
            return 0;
        }

        // build main window
        EusoMainFrame window(App,gClient->GetRoot(), 400, 200 );

        // start SimuApplication main loop
        App->Run(0);

        return 0;
    } catch (exception &e) {
        cerr << "ESAF ERROR: " << e.what() << endl;
        exit(999);
    } catch (EsafMsg& msg) {
        cerr << "ESAF ERROR: " << msg.str() << endl;
        exit(999);
    }
}

