#!/usr/bin/perl
# $Id: CreateNewClass.pl 1004 2004-09-13 22:07:40Z thea $
# simple tool to create an empty new class with right headers
# Author: D. DeMarco, M. Pallavicini, A.Thea
#

use Getopt::Std;
getopt('npus');

use POSIX qw(strftime);
$now_string = strftime "%Y/%m/%d", gmtime;


$singleton = $opt_s;
$classname = $opt_n;
$parent = $opt_p;

$classname or die "CreateNewClass: no name defined\n\t usage: CreateNewClass.pl -n name [-s ] [-u <username>] [-p <parent>]";

$uid = $<;
$user = (getpwuid($uid))[6];
$user=~s/,.*$//;

if ($opt_u) {
  $user = $opt_u;
}

$aa = $classname;
$aa=~y/[a-z]/[A-Z]/;

$ifd = "__".$aa."_HH__";
$inc = "";
if ( $parent ) {
  $inc = "\#include \"$parent.hh\"";
  $classparent = " : public $parent";
}

print "creating $classname.hh\n";
open(HH,">include/$classname.hh");
if ( !$singleton) {
print HH <<pippo;
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

\#ifndef $ifd
\#define $ifd

\#include "euso.hh"
$inc

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// $classname                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class $classname$classparent {
public:
    $classname();
    virtual ~$classname();

private:

    ClassDef($classname,0)
};

\#endif  /* $ifd */

pippo

}

else 

{
print HH <<pippo;
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

\#ifndef $ifd
\#define $ifd

\#include "euso.hh"
$inc

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// $classname                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class $classname$classparent {
public:
    virtual ~$classname();

    // instance 
    static $classname *Get();
 
private:
    $classname();

    // unique instance of $classname
    static $classname *fMe;

    ClassDef($classname,0)
};

\#endif  /* $ifd */

pippo

}
close(HH);

print "creating $classname.cc\n";
open(CC,">src/$classname.cc");
if ( !$singleton) {
print CC <<pippo;
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// $classname
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

\#include "$classname.hh"

ClassImp($classname)

//_____________________________________________________________________________
$classname\:\:$classname() {
    //
    // Constructor
    //

}

//_____________________________________________________________________________
$classname\:\:~$classname() {
    //
    // Destructor
    //
}

pippo

}
else {
print CC <<pippo;
// \$Id\$
// Author: $user   $now_string

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: $classname                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
//   $classname
//   ==========
//
//   <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

\#include "$classname.hh"

ClassImp($classname)

$classname *$classname\:\:fMe = 0;

//_____________________________________________________________________________
$classname\:\:$classname() {
    //
    // Constructor
    //
}

//_____________________________________________________________________________
$classname\:\:~$classname() {
    //
    // Destructor
    //
}

//_____________________________________________________________________________
$classname* $classname\:\:Get() {
    // instance
    if ( fMe == 0 )
        fMe = new $classname();
    return fMe;
}

pippo

}

