#!/bin/sh
# sh script used to check if the basic requirements needed to compile
# and run ESAF are satisfied
#
# $Id: checkRequirements.sh 1657 2005-04-07 12:35:38Z thea $
# 2002/01/23 D. De Marco - created
#
if [ -f .requirements.checked.successfully ]; then
        echo "requirements already satisfied"
        exit 0
fi

OUR_ROOT=40200

error() {
        echo -e "FAILED\n$1 not found" >&2
        exit 1
}

root_old() {
        DUMMY=`echo $ROOT_VERSION|sed 's!\(.\)\(..\)\(..\)!\1.\2/\3!'`
        echo -e "FAILED\nroot version installed on this system is: $DUMMY" >&2
        DUMMY=`echo $OUR_ROOT|sed 's!\(.\)\(..\)\(..\)!\1.\2/\3!'`
        echo "required version is: $DUMMY" >&2
        exit 1
}

g++ -v >/dev/null 2>&1
[ $? -eq 127 ] && error g++


sed -v >/dev/null 2>&1
[ $? -eq 127 ] && error sed

uname >/dev/null 2>&1
[ $? -eq 127 ] && error uname

ar -V>/dev/null 2>&1
[ $? -eq 127 ] && error ar

ranlib -V>/dev/null 2>&1
[ $? -eq 127 ] && error ranlib

mkid --version>/dev/null 2>&1
[ $? -eq 127 ] && echo "mkid not found (but optional)" 

ctags -V>/dev/null 2>&1
[ $? -eq 127 ] && echo "ctags not found (but optional)"

root-config --version>/dev/null 2>&1
[ $? -eq 127 ] && error root

ROOT_VERSION=$(root-config --version|sed 's!^\(.\)\.\(..\)/\(..\)$!\1\2\3!')

if [ $ROOT_VERSION -lt $OUR_ROOT ]; then
        # root too old
        root_old;
fi

echo "requirements checked successfully"
exit 0


